@@ -18,15 +18,16 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaRdpt.c,v 1.5 2001/07/23 17:49:49 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/calma/CalmaRdpt.c,v 1.2 2005/05/11 17:25:06 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>                 /* for abs() */
 #include <sys/types.h>
 
 #include <netinet/in.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/utils.h"
@@ -34,14 +35,14 @@ static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaRdpt.c,v 1.5 2001
 #include "database/database.h"
 #include "database/databaseInt.h"
 #include "utils/malloc.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "cif/cif.h"
 #include "cif/CIFint.h"
 #include "cif/CIFread.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "textio/textio.h"
 #include "calma/calmaInt.h"
 #include "calma/calma.h"
@@ -205,6 +206,7 @@ calmaElementBoundary()
     CIFPath *pathheadp;
     LinkedRect *rp;
     Plane *plane;
+    CellUse *use;
 #ifdef NONMANHATTAN
     CellDef *savedef, *newdef = NULL;
 #endif
@@ -227,6 +229,8 @@ calmaElementBoundary()
 	plane = NULL;
 	calmaLayerError("Unknown layer/datatype in boundary", layer, dt);
     }
+    else
+	plane = cifCurReadPlanes[ciftype];
 
     /* Read the path itself, building up a path structure */
     if (!calmaReadPath(&pathheadp, (plane == NULL) ? 0 : 1)) 
@@ -237,8 +241,8 @@ calmaElementBoundary()
     }
 
     /* Note that calmaReadPath() may reallocate planes of cifCurReadPlanes */
-
-    plane = cifCurReadPlanes[ciftype];
+    /* so we need to set it again.					   */
+    if (ciftype >= 0) plane = cifCurReadPlanes[ciftype];
 
     /* Convert the polygon to rectangles. */
 
@@ -296,8 +300,6 @@ calmaElementBoundary()
 #ifdef NONMANHATTAN
     if (cifCurReadPlanes == cifEditCellPlanes)
     {
-	CellUse *use;
-
 	CIFPaintCurrent();
 	DBReComputeBbox(cifReadCellDef);
 	DRCCheckThis(cifReadCellDef, TT_CHECKPAINT, &cifReadCellDef->cd_bbox);
