@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaWrite.c,v 1.4 2001/07/23 17:49:49 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic/calma/CalmaWrite.c,v 1.3 2005/05/31 21:41:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -29,33 +29,39 @@ static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaWrite.c,v 1.4 200
 #else
 #include <sys/time.h>
 #endif
-#include "misc/magic.h"
+
+#include "utils/magic.h"
+#include "utils/malloc.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/utils.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "database/databaseInt.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "cif/cif.h"
 #include "cif/CIFint.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "textio/textio.h"
 #include "calma/calmaInt.h"
+#include "utils/main.h"		/* for Path and CellLibPath */
 
     /* Exports */
-bool CalmaDoLabels = TRUE;	/* If FALSE, don't output labels with GDS-II */
-bool CalmaDoLower = TRUE;	/* If TRUE, allow lowercase labels. */
-bool CalmaFlattenArrays = FALSE;/* If TRUE, output arrays as individual uses */
+bool CalmaDoLabels = TRUE;	 /* If FALSE, don't output labels with GDS-II */
+bool CalmaDoLower = TRUE;	 /* If TRUE, allow lowercase labels. */
+bool CalmaFlattenArrays = FALSE; /* If TRUE, output arrays as individual uses */
+bool CalmaContactArrays = FALSE; /* If TRUE, output contacts as subcell arrays */
 
     /* Forward declarations */
 extern int calmaWriteInitFunc();
 extern int calmaWriteMarkFunc();
 extern int calmaWritePaintFunc();
 extern int calmaWriteUseFunc();
+extern void calmaWriteContacts();
+extern void calmaDelContacts();
 extern void calmaOutFunc();
 extern void calmaOutStructName();
 extern void calmaWriteLabelFunc();
@@ -65,6 +71,13 @@ extern void calmaOutStringRecord();
 extern void calmaOut8();
 extern void calmaOutR8();
 
+/* Structure used by calmaWritePaintFunc() */
+
+typedef struct {
+   FILE *f;		/* File stream for output		*/
+   Rect *area;		/* Clipping area, in GDS coordinates	*/
+} calmaOutputStruct;
+
 /* Number assigned to each cell */
 int calmaCellNum;
 
@@ -83,6 +96,7 @@ int calmaPaintLayerType;
 
 /* Imports */
 extern time_t time();
+
 
 /* -------------------------------------------------------------------- */
 
@@ -141,18 +155,18 @@ static char calmaMapTableStrict[] =
       0,    0,    0,    0,    0,    0,    0,    0,	/* BS  - SI  */
       0,    0,    0,    0,    0,    0,    0,    0,	/* DLE - ETB */
       0,    0,    0,    0,    0,    0,    0,    0,	/* CAN - US  */
-      0,    0,    0,    0,  '$',    0,    0,    0,	/* SP  - '   */
-      0,    0,    0,    0,    0,    0,    0,    0,	/* (   - /   */
+    '_',  '_',  '_',  '_',  '$',  '_',  '_',  '_',	/* SP  - '   */
+    '_',  '_',  '_',  '_',  '_',  '_',  '_',  '_',	/* (   - /   */
     '0',  '1',  '2',  '3',  '4',  '5',  '6',  '7',	/* 0   - 7   */
-    '8',  '9',    0,    0,    0,    0,    0,    0,	/* 8   - ?   */
-      0,  'A',  'B',  'C',  'D',  'E',  'F',  'G',	/* @   - G   */
+    '8',  '9',  '_',  '_',  '_',  '_',  '_',  '_',	/* 8   - ?   */
+    '_',  'A',  'B',  'C',  'D',  'E',  'F',  'G',	/* @   - G   */
     'H',  'I',  'J',  'K',  'L',  'M',  'N',  'O',	/* H   - O   */
     'P',  'Q',  'R',  'S',  'T',  'U',  'V',  'W',	/* P   - W   */
-    'X',  'Y',  'Z',    0,    0,    0,    0,  '_',	/* X   - _   */
-      0,  'a',  'b',  'c',  'd',  'e',  'f',  'g',	/* `   - g   */
+    'X',  'Y',  'Z',  '_',  '_',  '_',  '_',  '_',	/* X   - _   */
+    '_',  'a',  'b',  'c',  'd',  'e',  'f',  'g',	/* `   - g   */
     'h',  'i',  'j',  'k',  'l',  'm',  'n',  'o',	/* h   - o   */
     'p',  'q',  'r',  's',  't',  'u',  'v',  'w',	/* p   - w   */
-    'x',  'y',  'z',    0,    0,    0,    0,    0,	/* x   - DEL */
+    'x',  'y',  'z',  '_',  '_',  '_',  '_',  0,	/* x   - DEL */
 };
 
 static char calmaMapTablePermissive[] =
@@ -161,7 +175,7 @@ static char calmaMapTablePermissive[] =
       0,    0,    0,    0,    0,    0,    0,    0,	/* BS  - SI  */
       0,    0,    0,    0,    0,    0,    0,    0,	/* DLE - ETB */
       0,    0,    0,    0,    0,    0,    0,    0,	/* CAN - US  */
-      0,  '!',  '"',  '#',  '$',  '&',  '%', '\'',	/* SP  - '   */
+    '_',  '!',  '"',  '#',  '$',  '&',  '%', '\'',	/* SP  - '   */
     '(',  ')',  '*',  '+',  ',',  '-',  '.',  '/',	/* (   - /   */
     '0',  '1',  '2',  '3',  '4',  '5',  '6',  '7',	/* 0   - 7   */
     '8',  '9',  ':',  ';',  '<',  '=',  '>',  '?',	/* 8   - ?   */
@@ -252,6 +266,11 @@ CalmaWrite(rootDef, f)
     /* Output the header, identifying this file */
     calmaOutHeader(rootDef, f);
 
+    /*
+     * Write all contact cell definitions first
+     */
+    if (CalmaContactArrays) calmaWriteContacts(f);
+
     /*
      * We perform a post-order traversal of the tree rooted at 'rootDef',
      * to insure that each child cell is output before it is used.  The
@@ -268,6 +287,11 @@ CalmaWrite(rootDef, f)
     if (problems = (DBWFeedbackCount - oldCount))
 	TxPrintf("%d problems occurred.  See feedback entries.\n", problems);
 
+    /*
+     * Destroy all contact cell definitions 
+     */
+    if (CalmaContactArrays) calmaDelContacts();
+
     return (good);
 }
 
@@ -295,7 +319,7 @@ calmaWriteInitFunc(def)
     def->cd_client = (ClientData) 0;
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -333,7 +357,7 @@ calmaProcessDef(def, outf)
     FILE *outf;		/* Stream file */
 {
     char *filename;
-    bool isReadOnly;
+    bool isReadOnly, oldStyle;
 
     /* Skip if already output */
     if ((int) def->cd_client > 0)
@@ -374,7 +398,10 @@ calmaProcessDef(def, outf)
 	off_t cellstart, cellend;
 	FILE *fi;
 
-	fi = fopen(filename, "r");
+	/* Use PaOpen() so the paths searched are the same as were	*/
+	/* searched to find the .mag file that indicated this GDS file.	*/
+
+	fi = PaOpen(filename, "r", "", Path, CellLibPath, (char **)NULL);
 	if (fi == NULL)
 	{
 	    TxError("Calma output error:  Can't find GDS file for vendor cell."
@@ -383,11 +410,27 @@ calmaProcessDef(def, outf)
 	}
 	else
 	{
-	    offptr = (char *)DBPropGet(def, "GDS_BEGIN", NULL);
-	    sscanf(offptr, "%d", &cellstart);
-	    fseek(fi, cellstart, SEEK_SET);
 	    offptr = (char *)DBPropGet(def, "GDS_END", NULL);
 	    sscanf(offptr, "%d", &cellend);
+	    offptr = (char *)DBPropGet(def, "GDS_BEGIN", &oldStyle);
+	    if (!oldStyle)
+	    {
+		offptr = (char *)DBPropGet(def, "GDS_START", NULL);
+
+		/* Write our own header and string name, to ensure	*/
+		/* that the magic cell name and GDS name match.		*/
+
+		/* Output structure header */
+		calmaOutRH(28, CALMA_BGNSTR, CALMA_I2, outf);
+		calmaOutDate(def->cd_timestamp, outf);
+		calmaOutDate(time((time_t *) 0), outf);
+
+		/* Name structure the same as the magic cellname */
+		calmaOutStructName(CALMA_STRNAME, def, outf);
+	    }
+
+	    sscanf(offptr, "%d", &cellstart);
+	    fseek(fi, cellstart, SEEK_SET);
 	    defsize = (size_t)(cellend - cellstart);
 
 	    buffer = (char *)mallocMagic(defsize);
@@ -410,12 +453,18 @@ calmaProcessDef(def, outf)
 	    }
 	    freeMagic(buffer);
 	    fclose(fi);
+
+	    /* Mark the definition as vendor GDS so that magic doesn't	*/
+	    /* try to generate subcell interaction or array interaction	*/
+	    /* paint for it.						*/
+
+	    def->cd_flags |= CDVENDORGDS;
 	}
     }
 
     /* Output this cell */
     if (!isReadOnly)
-	calmaOutFunc(def, outf);
+	calmaOutFunc(def, outf, &TiPlaneRect);
 
     return (0);
 }
@@ -440,14 +489,22 @@ calmaProcessDef(def, outf)
  */
 
 void
-calmaOutFunc(def, f)
+calmaOutFunc(def, f, cliprect)
     CellDef *def;	/* Pointer to cell def to be written */
     FILE *f;		/* Open output file */
+    Rect *cliprect;	/* Area to clip to (used for contact cells),
+			 * in CIF/GDS coordinates.
+			 */
 {
     Label *lab;
     CIFLayer *layer;
     Rect bigArea;
     int type;
+    int dbunits;
+    calmaOutputStruct cos;
+
+    cos.f = f;
+    cos.area = (cliprect == &TiPlaneRect) ? NULL : cliprect;
 
     /* Output structure begin */
     calmaOutRH(28, CALMA_BGNSTR, CALMA_I2, f);
@@ -457,21 +514,32 @@ calmaOutFunc(def, f)
     /* Output structure name */
     calmaOutStructName(CALMA_STRNAME, def, f);
 
-    /* Since Calma database units are millimicrons, multiply all units by 10,
+    /* Since Calma database units are nanometers, multiply all units by 10,
      * modified by the scale multiplier.
      */
 
-    if ((10 % CIFCurStyle->cs_expander) == 0)
+    dbunits = (CIFCurStyle->cs_flags & CWF_ANGSTROMS) ? 100 : 10;
+    if ((dbunits % CIFCurStyle->cs_expander) == 0)
     {
-	calmaWriteScale = CIFCurStyle->cs_scaleFactor * 10 / CIFCurStyle->cs_expander;
-	calmaPaintScale = 10 / CIFCurStyle->cs_expander;
+	calmaWriteScale = CIFCurStyle->cs_scaleFactor * dbunits
+		/ CIFCurStyle->cs_expander;
+	calmaPaintScale = dbunits / CIFCurStyle->cs_expander;
     }
     else
     {
 	TxError("Calma output error:  Output scale units are %2.1f nanometers.\n",
-		(float)10 / (float)CIFCurStyle->cs_expander);
-	TxError("Magic Calma output supports values in integer nanometers or larger.\n");
+		(float)dbunits / (float)CIFCurStyle->cs_expander);
 	TxError("Magic Calma output will be scaled incorrectly!\n");
+	if ((dbunits == 10) && ((100 % CIFCurStyle->cs_expander) == 0))
+	{
+	    TxError("Please add \"units angstroms\" to the cifoutput section"
+			" of the techfile.\n");
+	}
+	else
+	{
+	    TxError("Magic GDS output is limited to a minimum dimension of"
+			" 1 angstrom.\n");
+	}
 	/* Set expander to 10 so output scales are not zero. */
 	calmaWriteScale = CIFCurStyle->cs_scaleFactor;
 	calmaPaintScale = 1;
@@ -487,7 +555,7 @@ calmaOutFunc(def, f)
     /* Output all the tiles associated with this cell; skip temporary layers */
     GEO_EXPAND(&def->cd_bbox, CIFCurStyle->cs_radius, &bigArea);
     CIFErrorDef = def;
-    CIFGen(def, &bigArea, CIFPlanes, &DBAllTypeBits, TRUE, TRUE);
+    CIFGen(def, &bigArea, CIFPlanes, &DBAllTypeBits, TRUE, TRUE, (ClientData) f);
     CIFGenSubcells(def, &bigArea, CIFPlanes);
     CIFGenArrays(def, &bigArea, CIFPlanes);
 
@@ -499,9 +567,10 @@ calmaOutFunc(def, f)
 	calmaPaintLayerNumber = layer->cl_calmanum;
 	calmaPaintLayerType = layer->cl_calmatype;
 
+
 	(void) DBSrPaintArea((Tile *) NULL, CIFPlanes[type],
-	    &TiPlaneRect, &CIFSolidBits, calmaWritePaintFunc,
-	    (ClientData) f);
+	    cliprect, &CIFSolidBits, calmaWritePaintFunc,
+	    (ClientData) &cos);
     }
 
     /* Output labels */
@@ -803,7 +872,7 @@ calmaOutStructName(type, def, f)
     int calmanum;
     char *table;
 
-    if(CIFCurStyle->cs_flags & CWF_PERMISSIVE_LABELS) 
+    if (CIFCurStyle->cs_flags & CWF_PERMISSIVE_LABELS) 
     {
 	table = calmaMapTablePermissive;
     } else {
@@ -812,8 +881,16 @@ calmaOutStructName(type, def, f)
 
     /* Is the def name a legal Calma name? */
     for (cp = def->cd_name; c = (unsigned char) *cp; cp++)
-	if (c > 127 || table[c] == 0)
+    {
+	if ((c > 127) || (table[c] == 0))
 	    goto bad;
+	else if ((unsigned char)table[c] != c)
+	{
+	    TxError("Warning: character \'%c\' changed to \'%c\' in"
+		" name %s\n", (char)c, table[c], def->cd_name);
+	}
+	/* We really should ensure that the new name is unique. . . */
+    }
     if (cp <= def->cd_name + CALMANAMELENGTH)
     {
 	/* Yes, it's legal: use it */
@@ -826,11 +903,255 @@ bad:
 	calmanum = (int) def->cd_client;
 	if (calmanum < 0) calmanum = -calmanum;
 	(void) sprintf(defname, "XXXXX%d", calmanum);
+	TxError("Warning: string in output unprintable; changed to \'%s\'\n",
+		 defname);
     }
 
     calmaOutStringRecord(type, defname, f);
 }
-
+
+/* Added by NP 8/21/2004 */
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * calmaGetContactCell --
+ *
+ *   This routine creates [if it hasn't been created yet] a cell definition
+ *   containing the given TileType.  Cellname is "$$" + layer1_name + "_" +
+ *   layer2_name... + "$$". Cellname contains the short name of all the
+ *   residues of the layer "type".
+ *
+ * Results:
+ *   Returns new celldef it doesn't exist else created one.
+ *
+ * Side effects:
+ *	 New celldef created specially for contact type if it does not exist.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+CellDef *
+calmaGetContactCell(type, lookOnly)
+    TileType type;		/* magic contact tile type */
+    bool lookOnly;		/* if true, don't generate any new cells */
+{
+    TileType j;
+    char contactCellName[100];
+    TileTypeBitMask *rMask = DBResidueMask(type);
+    CellDef *def;
+    bool first = TRUE;
+
+    strcpy(contactCellName, "$$");
+    for (j = TT_SPACE + 1; j < DBNumUserLayers; j++)
+	if (TTMaskHasType(rMask, j))
+	{
+            /* Cellname starts with "$$" to make it diffrent from
+             * other database cells, and to be compatible with a
+	     * number of other EDA tools.
+             */
+	    if (!first)
+		strcat(contactCellName, "_");
+	    else
+		first = FALSE;
+            strcat(contactCellName, DBTypeShortName(j));
+        }
+    strcat(contactCellName, "$$");
+
+    def = DBCellLookDef(contactCellName);
+    if ((def == (CellDef *) NULL) && (lookOnly == FALSE))
+    {
+	def = DBCellNewDef(contactCellName, (char *) NULL);
+       	def->cd_flags &= ~(CDMODIFIED|CDGETNEWSTAMP);
+        def->cd_flags |= CDAVAILABLE;
+    }
+    return def;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CalmaGenerateArray --
+ *
+ *	This routine
+ *
+ * Results:
+ *	TRUE on success, FALSE if no contact cell could be found.
+ *
+ * Side effects:
+ *	Writes an AREF record to the GDS stream output.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+CalmaGenerateArray(f, type, llx, lly, pitch, cols, rows)
+    FILE *f;		/* GDS output file */
+    TileType type;	/* Magic tile type of contact */
+    int llx, lly;	/* Lower-left hand coordinate of the array
+			 * (centered on contact cut)
+			 */
+    int pitch;		/* Pitch of the array elements */
+    int cols, rows;	/* Number of array elements in X and Y */
+{
+    CellDef *child;	/* Cell definition of the contact cell */
+    int xxlate, yxlate;
+
+    child = calmaGetContactCell(type, TRUE);
+    if (child == NULL) return FALSE;
+
+    /* Structure reference */
+    calmaOutRH(4, CALMA_AREF, CALMA_NODATA, f);
+    calmaOutStructName(CALMA_SNAME, child, f);
+
+    /* Transformation flags */
+    calmaOutRH(6, CALMA_STRANS, CALMA_BITARRAY, f);
+    calmaOutI2(0, f);
+
+    /* Number of columns and rows in the array */
+    calmaOutRH(8, CALMA_COLROW, CALMA_I2, f);
+    calmaOutI2(cols, f);
+    calmaOutI2(rows, f);
+
+    /* Translation */
+    xxlate = llx * calmaPaintScale;
+    yxlate = lly * calmaPaintScale;
+    calmaOutRH(28, CALMA_XY, CALMA_I4, f);
+    calmaOutI4(xxlate, f);
+    calmaOutI4(yxlate, f);
+
+    /* Column reference point */
+    calmaOutI4(xxlate + pitch * cols * calmaPaintScale, f);
+    calmaOutI4(yxlate, f);
+
+    /* Row reference point */
+    calmaOutI4(xxlate, f);
+    calmaOutI4(yxlate + pitch * rows * calmaPaintScale, f);
+
+    /* End of AREF element */
+    calmaOutRH(4, CALMA_ENDEL, CALMA_NODATA, f);
+
+    return TRUE;
+}
+
+/* Added by NP 8/22/2004 */
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * calmaWriteContacts --
+ *
+ *  This routine creates a new cellDef for each contact type and writes to
+ *  the GDS output stream file. It is called before processing all cell
+ *  definitions while writing GDS output.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Writes contact cell definition to the open Calma output file.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+calmaWriteContacts(f)
+    FILE *f;
+{
+    TileType type;
+    TileTypeBitMask tMask;
+    CellDef *def, *cellDef;
+    Rect area, cliprect;
+    int halfwidth, halfsize;
+    CIFOp *op;
+    HashSearch hs;
+    HashEntry *entry;
+
+    /* Turn off generation of contact arrays for the duration of this	*/
+    /* subroutine, so that the contact definitions themselves will get	*/
+    /* the proper contact cut drawn.  It is turned on again at the end	*/
+    /* of the routine.  Note that this routine is not called unless	*/
+    /* CalmaContactArrays is TRUE.					*/
+    
+    CalmaContactArrays = FALSE;
+
+    DBEnumerateTypes(&tMask);
+
+    /* Decompose stacking types */
+    for (type = DBNumUserLayers; type < DBNumTypes; type++)
+	if (TTMaskHasType(&tMask, type))
+	    TTMaskSetMask(&tMask, DBResidueMask(type));
+
+    for (type = TT_SPACE + 1; type < DBNumUserLayers; type++)
+    {
+	/* We need to create cell array only for contact types */
+	if (DBIsContact(type) && TTMaskHasType(&tMask, type))
+	{
+            /* Write definition of cell to GDS stream.	*/
+	    /* Get cell definition for Tiletype type */
+            def = calmaGetContactCell(type, FALSE);
+
+            /* Get clip bounds, so that residue surround is	*/
+	    /* minimum.  Note that these values are in CIF/GDS	*/
+	    /* units, and the clipping rectangle passed to	*/
+	    /* calmaOutFunc is also in CIF/GDS units.		*/
+
+	    halfsize = CIFGetContactSize(type, NULL, NULL, NULL) >> 1;
+
+            /* Get minimum width for layer by rounding halfsize	*/
+	    /* to the nearest lambda value.			*/
+            halfwidth = halfsize / CIFCurStyle->cs_scaleFactor;
+	    if ((halfsize % CIFCurStyle->cs_scaleFactor) != 0)
+		halfwidth++;
+
+            area.r_xbot = area.r_ybot = -halfwidth;
+            area.r_xtop = area.r_ytop = halfwidth;
+       	    UndoDisable();
+       	    DBPaint(def, &area, type);
+       	    DBReComputeBbox(def);
+    	    TTMaskSetType(&def->cd_types, type);
+
+	    /* Clip output to the bounds of "cliprect"	*/
+	    cliprect.r_xbot = cliprect.r_ybot = -halfsize;
+	    cliprect.r_xtop = cliprect.r_ytop = halfsize;
+
+            calmaOutFunc(def, f, &cliprect);
+            UndoEnable();
+	}
+    }
+    CalmaContactArrays = TRUE;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * calmaDelContacts --
+ *
+ *  This routine removes all cell definitions generated by
+ *  calmaWriteContacts().
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Removes contact cell defs from the database.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+calmaDelContacts()
+{
+    TileType type;
+    CellDef *def;
+
+    for (type = TT_SPACE + 1; type < DBNumUserLayers; type++)
+	if (DBIsContact(type))
+	{
+            def = calmaGetContactCell(type, TRUE);
+	    if (def != (CellDef *)NULL)
+		 DBCellDeleteDef(def);
+	}
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -853,13 +1174,18 @@ bad:
  */
 
 int
-calmaWritePaintFunc(tile, f)
-    Tile *tile;		/* Tile to be written out. */
-    FILE *f;				/* File in which to write. */
+calmaWritePaintFunc(tile, cos)
+    Tile *tile;			/* Tile to be written out. */
+    calmaOutputStruct *cos;	/* File for output and clipping area */
 {
-    Rect r;
+    FILE *f = cos->f;
+    Rect *clipArea = cos->area;
+    Rect r, r2;
 
     TiToRect(tile, &r);
+    if (clipArea != NULL)
+	GeoClip(&r, clipArea);
+
     r.r_xbot *= calmaPaintScale;
     r.r_ybot *= calmaPaintScale;
     r.r_xtop *= calmaPaintScale;
@@ -1045,7 +1371,10 @@ calmaOutHeader(rootDef, f)
      */
 
     calmaOutRH(20, CALMA_UNITS, CALMA_R8, f);
+    if (CIFCurStyle->cs_flags & CWF_ANGSTROMS) useru = 0.0001;
     calmaOutR8(useru, f);	/* User units per database unit */
+
+    if (CIFCurStyle->cs_flags & CWF_ANGSTROMS) mum = 1e-10;
     calmaOutR8(mum, f);		/* Meters per database unit */
 }
 
@@ -1110,7 +1439,7 @@ calmaOutStringRecord(type, str, f)
 {
     int len;
     unsigned char c;
-    char *table;
+    char *table, *locstr, *origstr = NULL;
 
     if(CIFCurStyle->cs_flags & CWF_PERMISSIVE_LABELS) 
     {
@@ -1120,6 +1449,7 @@ calmaOutStringRecord(type, str, f)
     }
 
     len = strlen(str);
+    locstr = str;
 
     /*
      * Make sure length is even.
@@ -1134,18 +1464,35 @@ calmaOutStringRecord(type, str, f)
     /* Output the string itself */
     while (len--)
     {
-	c = (unsigned char) *str++;
+	c = (unsigned char) *locstr++;
 	if (c == 0) putc('\0', f);
 	else
 	{
-	    if (c > 127 || table[c] == 0) c = 'X';
-	    else c = table[c];
+	    if ((c > 127) || (c == 0))
+	    {
+		TxError("Warning: Unprintable character changed "
+			"to \'X\' in label.\n");
+		c = 'X';
+	    }
+	    else
+	    {
+		if (((unsigned char)table[c] != c) && (origstr == NULL))
+		    origstr = StrDup(NULL, locstr);
+
+		c = table[c];
+	    }
 	    if (!CalmaDoLower && islower(c))
 		(void) putc(toupper(c), f);
 	    else
 		(void) putc(c, f);
 	}
     }
+    if (origstr != NULL)
+    {
+	TxError("Warning: characters changed in string \'%s\'; "
+		"modified string is \'%s\'\n", origstr, locstr);
+	freeMagic(origstr);
+    }
 }
 
 /*
