@@ -22,11 +22,12 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFhier.c,v 1.3 2001/03/29 19:50:04 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/cif/CIFhier.c,v 1.2 2005/05/31 21:41:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
@@ -35,9 +36,9 @@ static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFhier.c,v 1.3 2001/0
 #include "cif/cif.h"
 #include "drc/drc.h"
 #include "textio/textio.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "utils/malloc.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 
 /* To compute CIF where there are interaction areas we do two things:
  * 1. Compute the CIF by combining all the material in all the interacting
@@ -117,7 +118,7 @@ CIFInitCells()
     }
     CIFTotalUse = DBCellNewUse (CIFTotalDef, (char *) NULL);
     DBSetTrans (CIFTotalUse, &GeoIdentityTransform);
-    CIFTotalUse->cu_expandMask = -1;	/* This is always expanded. */
+    CIFTotalUse->cu_expandMask = CU_DESCEND_SPECIAL;	/* This is always expanded. */
 
     CIFComponentDef = DBCellLookDef("__CIF2__");
     if (CIFComponentDef == (CellDef *) NULL)
@@ -129,7 +130,7 @@ CIFInitCells()
     }
     CIFComponentUse = DBCellNewUse (CIFComponentDef, (char *) NULL);
     DBSetTrans (CIFComponentUse, &GeoIdentityTransform);
-    CIFComponentUse->cu_expandMask = -1;	/* This is always expanded. */
+    CIFComponentUse->cu_expandMask = CU_DESCEND_SPECIAL; /* This is always expanded. */
 
     /* Clear out the planes used to collect hierarchical CIF. */
 
@@ -229,26 +230,31 @@ cifHierCopyFunc(tile, cxp)
 				 * transform and client data.
 				 */
 {
-    TileType type;
+    TileType type = TiGetTypeExact(tile);
     Rect sourceRect, targetRect;
     int pNum;
     CellDef *def = (CellDef *) cxp->tc_filter->tf_arg;
 
+#ifdef NONMANHATTAN
+    int dinfo = 0;
+#endif
+
+    /* Ignore tiles in vendor GDS */
+
+    if (cxp->tc_scx->scx_use->cu_def->cd_flags & CDVENDORGDS) return 0;
+
     /* Ignore space tiles, since they won't do anything anyway. */
 
 #ifdef NONMANHATTAN
-    int loctype = TiGetTypeExact(tile);
    
     if (IsSplit(tile))
     {
-	if (SplitSide(tile))
-	    type = SplitRightType(tile);
-	else
-	    type = SplitLeftType(tile);
+	dinfo = DBTransformDiagonal(type, &cxp->tc_scx->scx_trans);
+	type = (SplitSide(tile)) ? SplitRightType(tile) :
+		SplitLeftType(tile);
     }
-    else 
 #endif
-    type = TiGetTypeExact(tile);
+
     if (type == TT_SPACE) return 0;
 
     /* Get the rectangular area, and transform to final coords. */
@@ -256,30 +262,12 @@ cifHierCopyFunc(tile, cxp)
     TiToRect(tile, &sourceRect);
     GeoTransRect(&cxp->tc_scx->scx_trans, &sourceRect, &targetRect);
 
-#ifdef NONMANHATTAN
-    if (loctype & TT_DIAGONAL)
-    {
-	int o1, o2, o3, dir, side;
-
-	o1 = ((cxp->tc_scx->scx_trans.t_e > 0) ||
-		(cxp->tc_scx->scx_trans.t_d > 0)) ? 1 : 0;
-	o2 = ((cxp->tc_scx->scx_trans.t_a > 0) ||
-		(cxp->tc_scx->scx_trans.t_b > 0)) ? 1 : 0;
-	o3 = (cxp->tc_scx->scx_trans.t_a != 0) ? 1 : 0;
-	dir = ((loctype & TT_DIRECTION) ? 1 : 0);
-	side = ((loctype & TT_SIDE) ? 1 : 0) ^ o2 ^ (dir | o3);
-	dir ^= o1 ^ o2;
-	loctype &= ~(TT_DIRECTION | TT_SIDE);
-	loctype |= ((dir) ? TT_DIRECTION : 0) | ((side) ? TT_SIDE : 0);
-    }
-#endif
-
     for (pNum = PL_SELECTBASE; pNum < DBNumPlanes; pNum++)
     {
 	if (DBPaintOnPlane(type, pNum))
 	{
 #ifdef NONMANHATTAN
-	    DBNMPaintPlane(def->cd_planes[pNum], loctype, &targetRect,
+	    DBNMPaintPlane(def->cd_planes[pNum], dinfo, &targetRect,
 		DBStdPaintTbl(type, pNum), (PaintUndoInfo *) NULL);
 #else
 	    DBPaintPlane(def->cd_planes[pNum], &targetRect,
@@ -316,6 +304,10 @@ cifHierCellFunc(scx)
     SearchContext newscx;
     Rect rootArea;
 
+    /* Do not call this function on cells that are vendor GDS */
+    /* (handled in cifHierCopyFunc())	*/
+    /* if (scx->scx_use->cu_def->cd_flags & CDVENDORGDS) return 0; */
+
     /* In order to generate CIF safely in the interaction area, we
      * have to yank material in a larger area:  bloats and shrinks
      * may cause this material to affect CIF in the interaction area.
@@ -450,7 +442,13 @@ cifHierPaintFunc(tile, plane)
     Rect area;
 
     TiToRect(tile, &area);
-    if (CIFCurStyle->cs_growSlivers == TRUE) cifGrowSliver(tile,&area);
+    if (CIFCurStyle->cs_flags & CWF_GROW_SLIVERS) cifGrowSliver(tile, &area);
+#ifdef NONMANHATTAN
+    if (IsSplit(tile))
+	DBNMPaintPlane(plane, TiGetTypeExact(tile), &area, CIFPaintTable,
+		(PaintUndoInfo *) NULL);
+    else
+#endif  
     DBPaintPlane(plane, &area, CIFPaintTable, (PaintUndoInfo *) NULL);
     CIFTileOps += 1;
     return 0;
@@ -775,7 +773,7 @@ cifHierPaintArrayFunc(tile)
     int i, j, xbot, xtop;
 
     TiToRect(tile, &area);
-    if (CIFCurStyle->cs_growSlivers == TRUE) cifGrowSliver(tile,&area);
+    if (CIFCurStyle->cs_flags & CWF_GROW_SLIVERS) cifGrowSliver(tile, &area);
     xbot = area.r_xbot;
     xtop = area.r_xtop;
     for (i=0; i<cifHierYCount; i++)
