@@ -19,24 +19,25 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFrdcl.c,v 1.4 2001/03/29 19:50:04 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/cif/CIFrdcl.c,v 1.2 2005/05/13 19:31:58 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
+#include "utils/malloc.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "database/database.h"
 #include "cif/CIFint.h"
 #include "cif/CIFread.h"
 #include "utils/utils.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "drc/drc.h"
 
 /* The following variable is made available to the outside world,
@@ -418,10 +419,8 @@ CIFPaintCurrent()
 
     /* Now go through all the current planes and zero them out. */
 
-    for (i = 0; i < MAXCIFRLAYERS; i += 1)
-    {
+    for (i = 0; i < MAXCIFRLAYERS; i++)
 	DBClearPaintPlane(cifCurReadPlanes[i]);
-    }
 }
 
 /* Below is the search function invoked for each CIF tile type
@@ -442,7 +441,7 @@ cifPaintCurrentFunc(tile, type)
     /* snapped to the nearest magic coordinate if the result is a	*/
     /* fractional magic coordinate.					*/
 
-    if (DBConnPlanes[type] != 0) snap_type = COORD_HALF_U;
+    if (DBIsContact(type)) snap_type = COORD_HALF_U;
 
     /* Compute the area of the CIF tile, then scale it into
      * Magic coordinates.
@@ -473,6 +472,10 @@ cifPaintCurrentFunc(tile, type)
 	area.r_xbot *= (savescale / cifCurReadStyle->crs_scaleFactor);
     }
 
+    /* Check for degenerate areas (from rescale limiting) before painting */
+    if ((area.r_xbot == area.r_xtop) || (area.r_ybot == area.r_ytop))
+	return 0;
+
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (DBPaintOnPlane(type, pNum))
 	{
@@ -993,6 +996,7 @@ CIFReadCellCleanup()
     HashEntry *h;
     HashSearch hs;
     CellDef *def;
+    MagWindow *window;
 
     if (cifSubcellBeingRead)
     {
@@ -1016,6 +1020,10 @@ CIFReadCellCleanup()
 	{
 	    CIFReadError("cell %s was used but not defined.\n", def->cd_name);
         }
+	if (def->cd_flags & CDPROCESSEDGDS)
+	{
+	    def->cd_flags &= ~CDPROCESSEDGDS;
+	}
 	if (def->cd_flags & CDFLATGDS)
 	{
 	    /* These cells have been flattened and are no longer needed. */
