@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFrdutils.c,v 1.6 2001/06/15 17:53:20 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/cif/CIFrdutils.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -27,7 +27,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFrdutils.c,v 1.6 200
 #include <stdarg.h>
 #include <ctype.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
@@ -36,8 +36,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFrdutils.c,v 1.6 200
 #include "cif/CIFread.h"
 #include "cif/cif.h"
 #include "textio/textio.h"
-#include "signals/signals.h"
-#include "undo/undo.h"
+#include "utils/signals.h"
+#include "utils/undo.h"
 #include "utils/malloc.h"
 
 /* The following variables are used to provide one character of
@@ -775,8 +775,9 @@ seg_intersect(tstart, bf, bs, respt)
 	    return FALSE;
 
 	respt->p_x = afx;
-        respt->p_y = bf->p_y + ((bs->p_y - bf->p_y) * (afx - bf->p_x)) /
-			(bs->p_x - bf->p_x);
+        respt->p_y = bf->p_y + (int)
+			(((dlong)(bs->p_y - bf->p_y) * (dlong)(afx - bf->p_x)) /
+			(dlong)(bs->p_x - bf->p_x));
 	if (((respt->p_y > afy) && (respt->p_y < asy)) ||
 		((respt->p_y < afy) && (respt->p_y > asy)))
 	    return TRUE;
@@ -794,8 +795,9 @@ seg_intersect(tstart, bf, bs, respt)
 	    return FALSE;
 
 	respt->p_y = afy;
-        respt->p_x = bf->p_x + ((bs->p_x - bf->p_x) * (afy - bf->p_y)) /
-			(bs->p_y - bf->p_y);
+        respt->p_x = bf->p_x + (int)
+			(((dlong)(bs->p_x - bf->p_x) * (dlong)(afy - bf->p_y)) /
+			(dlong)(bs->p_y - bf->p_y));
 	if (((respt->p_x > afx) && (respt->p_x < asx)) ||
 		((respt->p_x < afx) && (respt->p_x > asx)))
 	    return TRUE;
@@ -911,7 +913,7 @@ is_clockwise(pathHead)
 {
     CIFPath *path, *midx, *last;
     Point *p1, *p2, *p3;
-    long sdir;
+    dlong sdir;
     int minx = INFINITY;
 
     /* Find out if this is a clockwise or counterclockwise path by	*/
@@ -960,9 +962,10 @@ is_clockwise(pathHead)
 
     /* Find which side p3 falls on relative to the line p1-p2.  This	*/
     /* determines whether the path is clockwise or counterclockwise.	*/
+    /* Use type dlong to avoid integer overflow.			*/
 
-    sdir = ((p2->p_x - p1->p_x) * (p3->p_y - p1->p_y) -
-		(p2->p_y - p1->p_y) * (p3->p_x - p1->p_x));
+    sdir = ((dlong)(p2->p_x - p1->p_x) * (dlong)(p3->p_y - p1->p_y) -
+		(dlong)(p2->p_y - p1->p_y) * (dlong)(p3->p_x - p1->p_x));
 
     return (sdir < 0) ? TRUE : FALSE;
 }
@@ -1066,19 +1069,24 @@ CIFMakeManhattanPath(pathHead, plane, resultTbl, ui)
 	    new->cifp_next = new2;
 	    new2->cifp_next = next;
 
+	    /* Use double long for the multiplication and	*/
+	    /* division, or else integer overflow can occur.	*/
+
 	    if (path->cifp_x == new->cifp_x)	/* vertical line */
 	    {
 		new2->cifp_y = new->cifp_y;
-		new2->cifp_x = path->cifp_x + (new2->cifp_y - path->cifp_y)
-			* (next->cifp_x - path->cifp_x)
-			/ (next->cifp_y - path->cifp_y);
+		new2->cifp_x = path->cifp_x + (int)
+			 ((dlong)(new2->cifp_y - path->cifp_y)
+			* (dlong)(next->cifp_x - path->cifp_x)
+			/ (dlong)(next->cifp_y - path->cifp_y));
 	    }
 	    else
 	    {
 		new2->cifp_x = new->cifp_x;
-		new2->cifp_y = path->cifp_y + (new2->cifp_x - path->cifp_x)
-			* (next->cifp_y - path->cifp_y)
-			/ (next->cifp_x - path->cifp_x);
+		new2->cifp_y = path->cifp_y + (int)
+			 ((dlong)(new2->cifp_x - path->cifp_x)
+			* (dlong)(next->cifp_y - path->cifp_y)
+			/ (dlong)(next->cifp_x - path->cifp_x));
 	    }
 	}
 
@@ -1096,7 +1104,9 @@ CIFMakeManhattanPath(pathHead, plane, resultTbl, ui)
 //		(type & TT_SIDE) ? "right" : "left", (type & TT_DIRECTION)
 //		? '\\' : '/', tt.r_xbot, tt.r_ybot, plane);
 
-        if (plane)
+	/* Final check---ensure that rectangle is not degenerate */
+
+        if (plane && (tr.r_xtop - tr.r_xbot > 0) && (tr.r_ytop - tr.r_ybot > 0))
             DBNMPaintPlane(plane, type, &tr, resultTbl, ui);
     }
 }
