@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFtech.c,v 1.4 2001/07/23 17:49:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/cif/CIFtech.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -26,14 +26,15 @@ static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFtech.c,v 1.4 2001/0
 #ifdef SYSV
 #include <string.h>
 #endif
-#include <stdlib.h>	/* for atof() */
+#include <stdlib.h>	/* for atof()	*/
+#include <math.h>	/* for pow()	*/
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "utils/utils.h"
 #include "cif/CIFint.h"
 #include "calma/calmaInt.h"
@@ -105,8 +106,11 @@ cifTechNewStyle()
 	    {
 		for (op = layer->cl_ops; op != NULL; op = op->co_next)
 		{
-		    if (op->co_bloats != NULL)
-			freeMagic((char *)op->co_bloats);
+		    if (op->co_client != (ClientData)NULL)
+		    {
+			if ((op->co_opcode != CIFOP_OR) && (op->co_opcode != CIFOP_BBOX))
+			    freeMagic((char *)op->co_client);
+		    }
 		    freeMagic((char *)op);
 		}
 		freeMagic((char *)layer);
@@ -142,7 +146,7 @@ cifTechStyleInit()
     CIFCurStyle->cs_expander = 1;
     CIFCurStyle->cs_yankLayers = DBZeroTypeBits;
     CIFCurStyle->cs_hierLayers = DBZeroTypeBits;
-    CIFCurStyle->cs_growSlivers = FALSE;
+    CIFCurStyle->cs_flags = 0;
     for (i=0;  i<TT_MAXTYPES;  i+=1)
 	CIFCurStyle->cs_labelLayer[i] = -1;
     for (i = 0; i < MAXCIFLAYERS; i++)
@@ -216,7 +220,7 @@ cifParseLayers(string, style, paintMask, cifMask,spaceOK)
 
 	if (paintMask != NULL)
 	{
-	    paintType = DBTechNameTypes(curLayer,&curPaintMask);
+	    paintType = DBTechNameTypes(curLayer, &curPaintMask);
 	    if (paintType >= 0) goto okpaint;
 	}
 	else paintType = -2;
@@ -273,7 +277,20 @@ okpaint:
 	{
 	    TTMaskSetMask(cifMask, &curCifMask);
 	}
-	else TechError("Unrecognized layer (type) \"%s\".\n", curLayer);
+	else
+	{
+	    HashEntry *he;
+	    TileTypeBitMask *amask;
+
+	    he = HashLookOnly(&DBTypeAliasTable, curLayer);
+	    if (he != NULL)
+	    {
+		amask = (TileTypeBitMask *)HashGetValue(he);
+		TTMaskSetMask(paintMask, amask);
+	    }
+	    else
+		TechError("Unrecognized layer (type) \"%s\".\n", curLayer);
+	}
     }
 }
 
@@ -425,7 +442,9 @@ CIFTechLine(sectionName, argc, argv)
     CIFOp *newOp = NULL;
     CIFKeep *newStyle, *p;
     char **bloatArg;
-    int local_bloats[TT_MAXTYPES];
+    BloatData *bloats;
+    SquaresData *squares;
+    SlotsData *slots;
 
     if (argc <= 0) return TRUE;
     else if (argc >= 2) l = strlen(argv[1]);
@@ -456,7 +475,7 @@ CIFTechLine(sectionName, argc, argv)
 	    /* Here we're only establishing existance;		*/
 	    /* break on the first variant found.		*/
 
-	    if (!strcmp(newStyle->cs_name, argv[1]))
+	    if (!strncmp(newStyle->cs_name, argv[1], l))
 		break;
 	}
 	if (newStyle == NULL)
@@ -584,8 +603,12 @@ CIFTechLine(sectionName, argc, argv)
 	 */
 
 	if (argc >= 3)
+	{
 	    if (strcmp(argv[argc - 1], "nanometers") == 0)
 		CIFCurStyle->cs_expander *= 10;
+	    else if (strncmp(argv[argc - 1], "angstr", 6) == 0)
+		CIFCurStyle->cs_expander *= 100;
+	}
 
 	CIFCurStyle->cs_reducer = 1;	/* initial value only */
 
@@ -598,6 +621,19 @@ CIFTechLine(sectionName, argc, argv)
 	return TRUE;
     }
 
+    /* New for magic-7.3.100---allow GDS database units to be other	*/
+    /* than nanometers.  Really, there is only one option here, which	*/
+    /* is "units angstroms".   This option does not affect CIF output,	*/
+    /* whose units are fixed at centimicrons.				*/
+
+    if (strcmp(argv[0], "units") == 0)
+    {
+	if (argc != 2) goto wrongNumArgs;
+	if (!strncmp(argv[1], "angstr", 6))
+	    CIFCurStyle->cs_flags |= CWF_ANGSTROMS;
+	return TRUE;
+    }
+
     if (strcmp(argv[0], "stepsize") == 0)
     {
 	if (argc != 2) goto wrongNumArgs;
@@ -627,7 +663,12 @@ CIFTechLine(sectionName, argc, argv)
 	while (*tptr != '\0')
 	{
 	    cptr = strchr(tptr, ',');
-	    if (cptr != NULL) *cptr = '\0';
+	    if (cptr != NULL) 
+	    {
+		*cptr = '\0';
+		for (j = 1; isspace(*(cptr - j)); j++)
+		    *(cptr - j) = '\0';
+	    }
 
 	    if (*tptr == '*')
 	    {
@@ -708,7 +749,7 @@ CIFTechLine(sectionName, argc, argv)
 		&cifCurOp->co_cifMask,FALSE);
 	    cifCurOp->co_distance = 0;
 	    cifCurOp->co_next = NULL;
-	    cifCurOp->co_bloats = (int *)NULL;
+	    cifCurOp->co_client = (ClientData)NULL;
 	    cifCurLayer->cl_ops = cifCurOp;
 	}
 	return TRUE;
@@ -760,7 +801,7 @@ CIFTechLine(sectionName, argc, argv)
 	}
 	if (argc != 2) goto wrongNumArgs;
 	cifCurLayer->min_width = atoi(argv[1]);
-	CIFCurStyle->cs_growSlivers = TRUE;
+	CIFCurStyle->cs_flags |= CWF_GROW_SLIVERS;
 	return TRUE;
     }
 
@@ -842,10 +883,7 @@ CIFTechLine(sectionName, argc, argv)
     newOp->co_opcode = 0;
     newOp->co_distance = 0;
     newOp->co_next = NULL;
-    newOp->co_bloats = (int *)NULL;
-
-    for (i = 0; i < TT_MAXTYPES; i++)
-	local_bloats[i] = 0;
+    newOp->co_client = (ClientData)NULL;
 
     if (strcmp(argv[0], "and") == 0)
 	newOp->co_opcode = CIFOP_AND;
@@ -855,6 +893,8 @@ CIFTechLine(sectionName, argc, argv)
 	newOp->co_opcode = CIFOP_OR;
     else if (strcmp(argv[0], "grow") == 0)
 	newOp->co_opcode = CIFOP_GROW;
+    else if (strcmp(argv[0], "grow-grid") == 0)
+	newOp->co_opcode = CIFOP_GROW_G;
     else if (strcmp(argv[0], "shrink") == 0)
 	newOp->co_opcode = CIFOP_SHRINK;
     else if (strcmp(argv[0], "bloat-or") == 0)
@@ -863,10 +903,16 @@ CIFTechLine(sectionName, argc, argv)
 	newOp->co_opcode = CIFOP_BLOATMAX;
     else if (strcmp(argv[0], "bloat-min") == 0)
 	newOp->co_opcode = CIFOP_BLOATMIN;
+    else if (strcmp(argv[0], "bloat-all") == 0)
+	newOp->co_opcode = CIFOP_BLOATALL;
     else if (strcmp(argv[0], "squares") == 0)
 	newOp->co_opcode = CIFOP_SQUARES;
     else if (strcmp(argv[0], "squares-grid") == 0)
 	newOp->co_opcode = CIFOP_SQUARES_G;
+    else if (strcmp(argv[0], "slots") == 0)
+	newOp->co_opcode = CIFOP_SLOTS;
+    else if (strcmp(argv[0], "bbox") == 0)
+	newOp->co_opcode = CIFOP_BBOX;
     else
     {
 	TechError("Unknown statement \"%s\".\n", argv[0]);
@@ -884,6 +930,7 @@ CIFTechLine(sectionName, argc, argv)
 	    break;
 	
 	case CIFOP_GROW:
+	case CIFOP_GROW_G:
 	case CIFOP_SHRINK:
 	    if (argc != 2) goto wrongNumArgs;
 	    newOp->co_distance = atoi(argv[1]);
@@ -894,6 +941,26 @@ CIFTechLine(sectionName, argc, argv)
 	    }
 	    break;
 	
+	case CIFOP_BLOATALL:
+	    if (argc != 3) goto wrongNumArgs;
+	    cifParseLayers(argv[1], CIFCurStyle, &newOp->co_paintMask,
+		(TileTypeBitMask *)NULL, FALSE);
+	    bloatLayers = newOp->co_paintMask;
+	    bloats = (BloatData *)mallocMagic(sizeof(BloatData));
+	    for (i = 0; i < TT_MAXTYPES; i++)
+		bloats->bl_distance[i] = 0;
+	    newOp->co_client = (ClientData)bloats;
+
+	    cifParseLayers(argv[2], CIFCurStyle, &mask, &tempMask, TRUE);
+	    TTMaskSetMask(&bloatLayers, &mask);
+	    if (!TTMaskEqual(&tempMask, &DBZeroTypeBits))
+		TechError("Can't use templayers in bloat statement.\n");
+
+	    for (i = 0; i < TT_MAXTYPES;  i++)
+		if (TTMaskHasType(&mask, i))
+		    bloats->bl_distance[i] = 1;
+	    goto bloatCheck;
+		
 	case CIFOP_BLOAT:
 	case CIFOP_BLOATMIN:
 	case CIFOP_BLOATMAX:
@@ -903,6 +970,11 @@ CIFTechLine(sectionName, argc, argv)
 	    argc -= 2;
 	    bloatArg = argv + 2;
 	    bloatLayers = newOp->co_paintMask;
+	    bloats = (BloatData *)mallocMagic(sizeof(BloatData));
+	    for (i = 0; i < TT_MAXTYPES; i++)
+		bloats->bl_distance[i] = 0;
+	    newOp->co_client = (ClientData)bloats;
+
 	    while (argc > 0)
 	    {
 		if (argc == 1) goto wrongNumArgs;
@@ -918,6 +990,7 @@ CIFTechLine(sectionName, argc, argv)
 		}
 		if (!TTMaskEqual(&tempMask, &DBZeroTypeBits))
 		    TechError("Can't use templayers in bloat statement.\n");
+		
 		distance = atoi(bloatArg[1]);
 		if ((distance < 0) && (newOp->co_opcode == CIFOP_BLOAT))
 		{
@@ -926,19 +999,20 @@ CIFTechLine(sectionName, argc, argv)
 		}
 		for (i = 0; i < TT_MAXTYPES;  i++)
 		    if (TTMaskHasType(&mask, i))
-			local_bloats[i] = distance;
+			bloats->bl_distance[i] = distance;
 
 		argc -= 2;
 		bloatArg += 2;
 	    }
 
+bloatCheck:
 	    /* Don't do any bloating at boundaries between tiles of the
 	     * types being bloated.  Otherwise a bloat could pass right
 	     * through a skinny tile and out the other side.
 	     */
 	    for (i = 0; i < TT_MAXTYPES; i++)
 		if (TTMaskHasType(&newOp->co_paintMask, i))
-		    local_bloats[i] = 0;
+		    bloats->bl_distance[i] = 0;
 
 	    /* Make sure that all the layers specified in the statement
 	     * fall in a single plane.
@@ -948,98 +1022,176 @@ CIFTechLine(sectionName, argc, argv)
 	    {
 		tempMask = bloatLayers;
 		TTMaskAndMask(&tempMask, &DBPlaneTypes[i]);
-		if (TTMaskEqual(&tempMask, &bloatLayers))
+		if (TTMaskEqual(&tempMask, &bloatLayers)) {
+		    bloats->bl_plane = i;
 		    goto bloatDone;
+		}
 	    }
 	    TechError("Not all bloat layers fall in the same plane.\n");
 	    bloatDone: break;
-	
+
+	case CIFOP_BBOX:
+	    if (argc == 2)
+	    {
+		if (!strcmp(argv[1], "top"))
+		    newOp->co_client = (ClientData)1;
+		else
+		    TechError("BBox takes only one optional argument \"top\".\n");
+	    }
+	    else if (argc != 1)
+		goto wrongNumArgs;
+	    break;
+
 	case CIFOP_SQUARES_G:
-	    local_bloats[3] = 1; /* set default grid and fall through */
-	    local_bloats[4] = 1;
 	case CIFOP_SQUARES:
+	
+	    squares = (SquaresData *)mallocMagic(sizeof(SquaresData));
+	    newOp->co_client = (ClientData)squares;
+
 	    if (argc == 2)
 	    {
 		i = atoi(argv[1]);
-		local_bloats[0] = atoi(argv[1]);
+		squares->sq_border = atoi(argv[1]);
 		if ((i <= 0) || (i & 1))
 		{
 		    TechError("Squares must have positive even sizes.\n");
 		    goto errorReturn;
 		}
-		local_bloats[0] = i/2;
-		local_bloats[1] = i;
-		local_bloats[2] = i;
-		local_bloats[3] = 0;	/* placeholder; not used by SQUARES */
+		squares->sq_border = i/2;
+		squares->sq_size = i;
+		squares->sq_sep = i;
+		squares->sq_gridx = 1; /* set default grid */
+		squares->sq_gridy = 1; /* set default grid */
 	    }
 	    else if (argc == 4 || ((argc == 5 || argc == 6)
 			&& newOp->co_opcode==CIFOP_SQUARES_G))
 	    {
-		local_bloats[0] = atoi(argv[1]);
-		if (local_bloats[0] < 0)
+		squares->sq_border = atoi(argv[1]);
+		if (squares->sq_border < 0)
 		{
 		    TechError("Square border must not be negative.\n");
 		    goto errorReturn;
 		}
-		local_bloats[1] = atoi(argv[2]);
-		if (local_bloats[1] <= 0)
+		squares->sq_size = atoi(argv[2]);
+		if (squares->sq_size <= 0)
 		{
 		    TechError("Squares must have positive sizes.\n");
 		    goto errorReturn;
 		}
-		local_bloats[2] = atoi(argv[3]);
-		if (local_bloats[2] <= 0)
+		squares->sq_sep = atoi(argv[3]);
+		if (squares->sq_sep <= 0)
 		{
 		    TechError("Square separation must be positive.\n");
 		    goto errorReturn;
 		}
 		if (argc >= 5)
 		{
-		    local_bloats[3] = local_bloats[4] = atoi(argv[4]);
-		    if (local_bloats[3] <= 0)
+		    squares->sq_gridx = squares->sq_gridy = atoi(argv[4]);
+		    if (squares->sq_gridx <= 0)
 		    {
 		        TechError("Square grid must be strictly positive.\n");
-			local_bloats[3] = 1;
+			squares->sq_gridx = squares->sq_gridy = 1;
 		        goto errorReturn;
 		    }
 		}
+		else
+		{
+		    squares->sq_gridx = 1; /* set default grid */
+		    squares->sq_gridy = 1; /* set default grid */
+		}
 		if (argc == 6)
 		{
-		    local_bloats[4] = atoi(argv[5]);
-		    if (local_bloats[4] <= 0)
+		    squares->sq_gridy = atoi(argv[5]);
+		    if (squares->sq_gridy <= 0)
 		    {
 		        TechError("Square y-grid must be strictly positive.\n");
-			local_bloats[4] = 1;
+			squares->sq_gridy = 1;
 		        goto errorReturn;
 		    }
 		}
 	    }
 	    else goto wrongNumArgs;
 	    break;
-    }
 
-    /* Generate bloat rules list in newOp.				*/
-    /* There are so few bloat rules per CIF op, this is more efficient	*/
-    /* than an array of TT_MAXTYPES integers.				*/
+	case CIFOP_SLOTS:
+	
+	    slots = (SlotsData *)mallocMagic(sizeof(SlotsData));
+	    newOp->co_client = (ClientData)slots;
 
-    switch (newOp->co_opcode)
-    {
-	case CIFOP_SQUARES:
-	    newOp->co_bloats = (int *) mallocMagic(4 * sizeof(int));
-	    for (i = 0; i < 4; i++)
-		newOp->co_bloats[i] = local_bloats[i];
-	    break;
-	case CIFOP_SQUARES_G:
-	    newOp->co_bloats = (int *) mallocMagic(5 * sizeof(int));
-	    for (i = 0; i < 5; i++)
-		newOp->co_bloats[i] = local_bloats[i];
-	    break;
-	case CIFOP_BLOAT:
-	case CIFOP_BLOATMAX:
-	case CIFOP_BLOATMIN:
-	    newOp->co_bloats = (int *) mallocMagic(TT_MAXTYPES * sizeof(int));
-	    for (i = 0; i < TT_MAXTYPES; i++)
-		newOp->co_bloats[i] = local_bloats[i];
+	    if (argc >= 4)
+	    {
+		i = atoi(argv[1]);
+		slots->sl_sborder = i;
+		if (i < 0)
+		{
+		    TechError("Slot border must be non-negative.\n");
+		    goto errorReturn;
+		}
+		i = atoi(argv[2]);
+		slots->sl_ssize = i;
+		if (i <= 0)
+		{
+		    TechError("Slot short-side size must be strictly positive.\n");
+		    goto errorReturn;
+		}
+		i = atoi(argv[3]);
+		slots->sl_ssep = i;
+		if (i <= 0)
+		{
+		    TechError("Slot separation must be strictly positive.\n");
+		    goto errorReturn;
+		}
+		/* Initialize other values, in case they are not specified */
+		slots->sl_lborder = 0;
+		slots->sl_lsize = 0;
+		slots->sl_lsep = 0;
+		slots->sl_offset = 0;
+	    }
+	    if (argc >= 5)
+	    {
+		i = atoi(argv[4]);
+		slots->sl_lborder = i;
+		if (i < 0)
+		{
+		    TechError("Slot border must be non-negative.\n");
+		    goto errorReturn;
+		}
+	    }
+	    if (argc >= 6)
+	    {
+		i = atoi(argv[5]);
+		slots->sl_lsize = i;
+		if (i < 0)
+		{
+		    TechError("Slot long-side size must be positive or zero.\n");
+		    goto errorReturn;
+		}
+		i = atoi(argv[6]);
+		slots->sl_lsep = i;
+		if (i < 0)
+		{
+		    TechError("Slot long-side separation must be non-negative.\n");
+		    goto errorReturn;
+		}
+		else if (i == 0 && (slots->sl_lsize > 0))
+		{
+		    TechError("Slot long-side separation must be strictly positive"
+				" when long-side size is nonzero\n");
+		    goto errorReturn;
+		}
+	    }
+	    if (argc == 8)
+	    {
+		i = atoi(argv[7]);
+		slots->sl_offset = i;
+		if (i < 0)
+		{
+		    TechError("Slot offset must be non-negative.\n");
+		    goto errorReturn;
+		}
+	    }
+	    if ((argc < 4) || (argc == 6) || (argc == 7) || (argc > 8))
+		goto wrongNumArgs;
 	    break;
     }
 
@@ -1053,7 +1205,7 @@ CIFTechLine(sectionName, argc, argv)
 
     return TRUE;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1117,6 +1269,7 @@ cifComputeRadii(layer, des)
 {
     int i, grow, shrink, curGrow, curShrink;
     CIFOp *op;
+    BloatData *bloats;
 
     grow = shrink = 0;
 
@@ -1151,6 +1304,7 @@ cifComputeRadii(layer, des)
 	    case CIFOP_OR: break;
 
 	    case CIFOP_GROW:
+	    case CIFOP_GROW_G:
 		grow += op->co_distance;
 		break;
 	    
@@ -1164,12 +1318,13 @@ cifComputeRadii(layer, des)
 
 	    case CIFOP_BLOAT:
 		curGrow = curShrink = 0;
+		bloats = (BloatData *)op->co_client;
 		for (i = 0; i < TT_MAXTYPES; i++)
 		{
-		    if (op->co_bloats[i] > curGrow)
-			curGrow = op->co_bloats[i];
-		    else if ((-op->co_bloats[i]) > curShrink)
-			curShrink = -op->co_bloats[i];
+		    if (bloats->bl_distance[i] > curGrow)
+			curGrow = bloats->bl_distance[i];
+		    else if ((-bloats->bl_distance[i]) > curShrink)
+			curShrink = -bloats->bl_distance[i];
 		}
 		grow += curGrow;
 		shrink += curShrink;
@@ -1266,12 +1421,31 @@ CIFTechFinal()
 	return;
     }
 
+    /*
+     * Make sure that all contact layers include stacked contact types
+     */
+    for (i = 0; i < style->cs_nLayers; i++)
+	for (op = style->cs_layers[i]->cl_ops; op != NULL; op = op->co_next)
+	{
+	    TileType d, s;
+	    TileTypeBitMask *rMask;
+
+	    for (d = TT_TECHDEPBASE; d < DBNumUserLayers; d++)
+		if (TTMaskHasType(&op->co_paintMask, d) && DBIsContact(d))
+		    for (s = DBNumUserLayers; s < DBNumTypes; s++)
+		    {
+			rMask = DBResidueMask(s);
+			if (TTMaskHasType(rMask, d))
+			    TTMaskSetType(&op->co_paintMask, s);
+		    }
+	}
+
     /*
      * Find the largest reducer value which divides into all of the CIF values.
      */
 
     minReduce = style->cs_scaleFactor;
-    for (i=0; i<style->cs_nLayers; i++)
+    for (i = 0; i < style->cs_nLayers; i++)
     {
 	for (op = style->cs_layers[i]->cl_ops; op != NULL; op = op->co_next)
 	{
@@ -1281,13 +1455,51 @@ CIFTechFinal()
 		c = FindGCF(style->cs_scaleFactor, op->co_distance);
 		if (c < minReduce) minReduce = c;
 	    }
-	    if (op->co_bloats)
+	    if (op->co_client)
 	    {
+		BloatData *bloats;
+		SquaresData *squares;
+		SlotsData *slots;
+		if (op->co_opcode == CIFOP_SLOTS)
+		{
+		    slots = (SlotsData *)op->co_client;
+
+		    for (j = 0; j < 7; j++)
+		    {
+			switch (j) {
+			   case 0: bvalue = slots->sl_sborder; break;
+			   case 1: bvalue = slots->sl_ssize; break;
+			   case 2: bvalue = slots->sl_ssep; break;
+			   case 3: bvalue = slots->sl_lborder; break;
+			   case 4: bvalue = slots->sl_lsize; break;
+			   case 5: bvalue = slots->sl_lsep; break;
+			   case 6: bvalue = slots->sl_offset; break;
+			}
+			if (bvalue != 0)
+			{
+			    if ((j == 1) || (j == 2) || (j == 4) || (j == 5))
+			    {
+				if (bvalue & 0x1)
+				    TxError("Internal error: slot size/sep %d"
+						" cannot be halved.\n", bvalue);
+				bvalue >>= 1;
+			    }
+			    c = FindGCF(style->cs_scaleFactor, bvalue);
+			    if (c < minReduce) minReduce = c;
+			}
+		    }
+		}
 		if (op->co_opcode == CIFOP_SQUARES)
 		{
-		    for (j = 0; j < 4; j++)
+		    squares = (SquaresData *)op->co_client;
+
+		    for (j = 0; j < 3; j++)
 		    {
-			bvalue = op->co_bloats[j];
+			switch (j) {
+			   case 0: bvalue = squares->sq_border; break;
+			   case 1: bvalue = squares->sq_size; break;
+			   case 2: bvalue = squares->sq_sep; break;
+			}
 			if (bvalue != 0)
 			{
 			    if ((j == 1) || (j == 2))
@@ -1302,14 +1514,36 @@ CIFTechFinal()
 			}
 		    }
 		}
-		else
+		else if (op->co_opcode == CIFOP_SQUARES_G)
 		{
-		    for (j = 0; j < ((op->co_opcode == CIFOP_SQUARES_G)
-				? 5 : TT_MAXTYPES); j++)
+		    squares = (SquaresData *)op->co_client;
+		    for (j = 0; j < TT_MAXTYPES; j++)
 		    {
-			if (op->co_bloats[j] != 0)
+			switch (j) {
+			   case 0: bvalue = squares->sq_border; break;
+			   case 1: bvalue = squares->sq_size; break;
+			   case 2: bvalue = squares->sq_sep; break;
+			   case 3: bvalue = squares->sq_gridx; break;
+			   case 4: bvalue = squares->sq_gridy; break;
+			}
+			if (bvalue != 0)
 			{
-			    c = FindGCF(style->cs_scaleFactor, op->co_bloats[j]);
+			    c = FindGCF(style->cs_scaleFactor, bvalue);
+			    if (c < minReduce) minReduce = c;
+			}
+		    }
+		}
+		/* Presence of op->co_opcode in CIFOP_OR indicates a copy */
+		/* of the SquaresData pointer from a following operator	  */
+		/* CIFOP_BBOX uses the co_client field as a flag field.	  */
+		else if ((op->co_opcode != CIFOP_OR) && (op->co_opcode != CIFOP_BBOX))
+		{
+		    bloats = (BloatData *)op->co_client;
+		    for (j = 0; j < TT_MAXTYPES; j++)
+		    {
+			if (bloats->bl_distance[j] != 0)
+			{
+			    c = FindGCF(style->cs_scaleFactor, bloats->bl_distance[j]);
 			    if (c < minReduce) minReduce = c;
 			}
 		    }
@@ -1329,7 +1563,7 @@ CIFTechFinal()
      * and remember the largest.
      */
     maxGrow = maxShrink = 0;
-    for (i=0; i<style->cs_nLayers; i++)
+    for (i = 0; i < style->cs_nLayers; i++)
     {
 	cifComputeRadii(style->cs_layers[i], style);
 	if (style->cs_layers[i]->cl_growDist > maxGrow)
@@ -1374,6 +1608,8 @@ CIFTechFinal()
 	for (op = style->cs_layers[i]->cl_ops; op != NULL;
 		    op = op->co_next)
 	{
+	    BloatData *bloats;
+
 	    TTMaskSetMask(&ourDepend, &op->co_cifMask);
 	    TTMaskSetMask(&ourYank, &op->co_paintMask);
 	    switch (op->co_opcode)
@@ -1381,28 +1617,12 @@ CIFTechFinal()
 		case CIFOP_BLOAT:
 		case CIFOP_BLOATMAX:
 		case CIFOP_BLOATMIN:
+		case CIFOP_BLOATALL:
+		    bloats = (BloatData *)op->co_client;
 		    for (j = 0; j < TT_MAXTYPES; j++)
 		    {
-			if (op->co_bloats[j] != op->co_bloats[TT_SPACE])
-			{
-			    /* Careful!  For contact images, must yank
-			     * the MASTER image.
-			     */
-				
-			    if (j < DBNumUserLayers)
-				TTMaskSetType(&ourYank, j);
-			    else
-			    {
-				int k;
-				for (k = TT_SELECTBASE;
-					k < DBNumUserLayers; k += 1)
-				{
-				    if (TTMaskHasType(
-					    &DBLayerTypeMaskTbl[k], j))
-					    TTMaskSetType(&ourYank, k);
-				}
-			    }
-			}
+			if (bloats->bl_distance[j] != bloats->bl_distance[TT_SPACE])
+			    TTMaskSetType(&ourYank, j);
 		    }
 		    needThisLayer = TRUE;
 		    break;
@@ -1422,20 +1642,33 @@ CIFTechFinal()
 	    TTMaskSetMask(&style->cs_hierLayers, &ourDepend);
 	}
     }
-    /* make sure that all the yanklayers that are images have their
-     * base types set -dcs 4/6/90
-     */
-    for (i = DBNumUserLayers; i < DBNumTypes;i++)
+
+    /* Added by Tim, 10/18/04					*/
+
+    /* Go through the layer operators looking for those that	*/
+    /* contain only OR operators followed by a single SQUARES	*/
+    /* operator.  If found, set the clientdata record of the OR	*/
+    /* operator to be a copy of that for the SQUARES operator.	*/
+    /* This is used by the GDS generator when the "gds contact	*/
+    /* on" option is enabled (CalmaContactArrays is TRUE) to	*/
+    /* translate contact areas into contact subcell arrays.	*/
+
+    for (i = 0; i < style->cs_nLayers; i++)
     {
-	if (TTMaskHasType(&style->cs_yankLayers,i))
+	ClientData clientdata;
+	for (op = style->cs_layers[i]->cl_ops; (op != NULL) &&
+			(op->co_opcode == CIFOP_OR) &&
+			(TTMaskIsZero(&op->co_cifMask)); op = op->co_next);
+	if (op && (op->co_opcode == CIFOP_SQUARES) && (op->co_next == NULL))
 	{
-	    int k;
-	    for (k = TT_SELECTBASE; k <DBNumUserLayers; k++)
+	    clientdata = op->co_client;
+	    for (op = style->cs_layers[i]->cl_ops; op->co_opcode == CIFOP_OR;
+			op = op->co_next)
 	    {
-		if (TTMaskHasType(DBLayerTypeMaskTbl+k,i))
-		{
-		    TTMaskSetType(&style->cs_yankLayers,k);
-		}
+		/* Copy the client record from the CIFOP_SQUARES operator
+		 * into the CIFOP_OR operator.
+		 */
+		op->co_client = clientdata;
 	    }
 	}
     }
@@ -1494,6 +1727,63 @@ CIFLoadStyle(stylename)
     CIFTechOutputScale(DBLambda[0], DBLambda[1]);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CIFGetContactSize --
+ *
+ *   Return the smallest allowable contact size in lambda units corresponding
+ *   to the "squares" function operating on the indicated type.  This value
+ *   is computed as the (cut size) + 2 * (cut border).
+ *
+ * Results:
+ *	Contact minimum dimension, in CIF/GDS units
+ *
+ * Side effects:
+ *	If any of edge, border, or spacing is non-NULL, the appropriate
+ *	cut dimension is filled in.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+CIFGetContactSize(type, edge, spacing, border)
+    TileType type;
+    int *edge;
+    int *border;
+    int *spacing;
+{
+    CIFStyle *style = CIFCurStyle;
+    CIFOp *op, *sop;
+    int i;
+    SquaresData *squares;
+
+    for (i = 0; i < style->cs_nLayers; i++)
+    {
+	for (op = style->cs_layers[i]->cl_ops; (op != NULL) &&
+			(op->co_opcode == CIFOP_OR) &&
+			(TTMaskIsZero(&op->co_cifMask)); op = op->co_next)
+	    if (TTMaskHasType(&op->co_paintMask, type))
+		for (sop = op->co_next; sop != NULL; sop = sop->co_next)
+		{
+		    if (sop->co_opcode == CIFOP_SQUARES)
+		    {
+			squares = (SquaresData *)sop->co_client;
+			if (edge != NULL) *edge = squares->sq_size;
+			if (border != NULL) *border = squares->sq_border;
+			if (spacing != NULL) *spacing = squares->sq_sep;
+			return (squares->sq_size + (squares->sq_border << 1));
+		    }
+		    /* Anything other than an OR function will break	*/
+		    /* the relationship between magic layers and cuts.	*/
+		    else if (sop->co_opcode != CIFOP_OR)
+			break;
+		}
+    }
+    return 0;
+}
+
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1519,6 +1809,16 @@ CIFLoadStyle(stylename)
  *   we check for odd numbers in these positions.  If there are any, and
  *   "d" is also an odd number, then we multiply both "n" and "d" by 2.
  *
+ *   (Added 2/23/05)---The above is not sufficient!  If the contact size
+ *   is an odd number, then centering may not be possible even if all
+ *   the contact parameters are even numbers.  e.g., for size=22 in a 0.18
+ *   process (scalefactor=9), a 5x5 lambda contact is 45x45 centimicrons.
+ *   45 - 22 = 23, so border is 11.5 centimicrons.  Therefore, when
+ *   the scalefactor is even, the contact size must be even, but when the
+ *   scalefactor is odd, then we have to multiply both "n" and "d" by 2
+ *   regardless, since contact areas may be either odd or even (compare
+ *   the above example to one where the contact is drawn 6x6 lambda).
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -1530,26 +1830,53 @@ CIFTechOutputScale(n, d)
     CIFStyle *ostyle = CIFCurStyle;
     CIFLayer *cl;
     CIFOp *op;
+    SquaresData *squares;
+    SlotsData *slots;
+    BloatData *bloats;
+    bool has_odd_space;
 
     /* For contact half-grid centering, check for odd numbers. . . */
-    if (d & 0x1)
+
+    if (ostyle->cs_scaleFactor & 0x1)
     {
-	bool has_odd_space = FALSE;
+	n *= 2;
+	d *= 2;
+	has_odd_space = TRUE;
+    }
+    else if (d & 0x1)
+    {
+	has_odd_space = FALSE;
 	for (i = 0; i < ostyle->cs_nLayers; i++)
 	{
 	    cl = ostyle->cs_layers[i];
 	    for (op = cl->cl_ops; op != NULL; op = op->co_next)
+	    {
 		if (op->co_opcode == CIFOP_SQUARES)
-		    if ((op->co_bloats[1] & 0x01) || (op->co_bloats[2] & 0x01))
+		{
+		    squares = (SquaresData *)op->co_client;
+		    if ((squares->sq_size & 0x01) || (squares->sq_sep & 0x01))
 		    {
 			has_odd_space = TRUE;
 			break;
 		    }
+		}
+		else if (op->co_opcode == CIFOP_SLOTS)
+		{
+		    slots = (SlotsData *)op->co_client;
+		    if ((slots->sl_lsize & 0x01) || (slots->sl_lsep & 0x01)
+			|| (slots->sl_ssize & 0x01) || (slots->sl_ssep & 0x01))
+		    {
+			has_odd_space = TRUE;
+			break;
+		    }
+		}
+	    }
 
 	    if (has_odd_space)
 	    {
 		n *= 2;
 		d *= 2;
+		has_odd_space = FALSE;
 		break;
 	    }
 	}
@@ -1574,32 +1901,89 @@ CIFTechOutputScale(n, d)
 		lgcf = FindGCF(abs(op->co_distance), ostyle->cs_expander);
 		lexpand = FindGCF(lexpand, lgcf);
 	    }
-	    if (op->co_bloats)
+	    if (op->co_client)
 	    {
-		int bvalue;
+		int bvalue, *bptr;
 		if (op->co_opcode == CIFOP_SQUARES)
 		{
-		    for (j = 0; j < 4; j++)
+		    squares = (SquaresData *)op->co_client;
+		    for (j = 0; j < 3; j++)
 		    {
-			if (op->co_bloats[j] != 0)
+			switch (j) {
+			    case 0: bptr = &squares->sq_border; break;
+			    case 1: bptr = &squares->sq_size; break;
+			    case 2: bptr = &squares->sq_sep; break;
+			}
+			if (*bptr != 0)
 			{
-			    op->co_bloats[j] *= d;
-			    bvalue = abs(op->co_bloats[j]);
+			    (*bptr) *= d;
+			    bvalue = abs(*bptr);
 			    if ((j == 1) || (j == 2)) bvalue >>= 1;	/* half-grid */
+			    if (has_odd_space) bvalue >>= 1;  /* force half-grid */
 		            lgcf = FindGCF(bvalue, ostyle->cs_expander);
 			    lexpand = FindGCF(lexpand, lgcf);
 			}
 		    }
 		}
-		else
+		else if (op->co_opcode == CIFOP_SLOTS)
 		{
-		    for (j = 0; j < ((op->co_opcode == CIFOP_SQUARES_G)
-				? 5 : TT_MAXTYPES); j++)
+		    slots = (SlotsData *)op->co_client;
+		    for (j = 0; j < 7; j++)
 		    {
-			if (op->co_bloats[j] != 0)
+			switch (j) {
+			    case 0: bptr = &slots->sl_sborder; break;
+			    case 1: bptr = &slots->sl_ssize; break;
+			    case 2: bptr = &slots->sl_ssep; break;
+			    case 3: bptr = &slots->sl_lborder; break;
+			    case 4: bptr = &slots->sl_lsize; break;
+			    case 5: bptr = &slots->sl_lsep; break;
+			    case 6: bptr = &slots->sl_offset; break;
+			}
+			if (*bptr != 0)
 			{
-			    op->co_bloats[j] *= d;
-		            lgcf = FindGCF(abs(op->co_bloats[j]), ostyle->cs_expander);
+			    (*bptr) *= d;
+			    bvalue = abs(*bptr);
+			    if ((j == 1) || (j == 2) || (j == 4) || (j == 5))
+				bvalue >>= 1;	/* half-grid */
+			    if (has_odd_space) bvalue >>= 1;  /* force half-grid */
+		            lgcf = FindGCF(bvalue, ostyle->cs_expander);
+			    lexpand = FindGCF(lexpand, lgcf);
+			}
+		    }
+		}
+		else if (op->co_opcode == CIFOP_SQUARES_G)
+		{
+		    squares = (SquaresData *)op->co_client;
+		    for (j = 0; j < 5; j++)
+		    {
+			switch (j) {
+			    case 0: bptr = &squares->sq_border; break;
+			    case 1: bptr = &squares->sq_size; break;
+			    case 2: bptr = &squares->sq_sep; break;
+			    case 3: bptr = &squares->sq_gridx; break;
+			    case 4: bptr = &squares->sq_gridy; break;
+			}
+			if (*bptr != 0)
+			{
+			    (*bptr) *= d;
+		            lgcf = FindGCF(abs(*bptr), ostyle->cs_expander);
+			    lexpand = FindGCF(lexpand, lgcf);
+			}
+		    }
+		}
+		/* Presence of op->co_opcode in CIFOP_OR indicates a copy */
+		/* of the SquaresData pointer from a following operator	  */
+		/* CIFOP_BBOX uses the co_client field as a flag field.	  */
+		else if ((op->co_opcode != CIFOP_OR) && (op->co_opcode != CIFOP_BBOX))
+		{
+		    bloats = (BloatData *)op->co_client;
+		    for (j = 0; j < TT_MAXTYPES; j++)
+		    {
+			if (bloats->bl_distance[j] != 0)
+			{
+			    bloats->bl_distance[j] *= d;
+		            lgcf = FindGCF(abs(bloats->bl_distance[j]),
+					ostyle->cs_expander);
 			    lexpand = FindGCF(lexpand, lgcf);
 			}
 		    }
@@ -1636,24 +2020,57 @@ CIFTechOutputScale(n, d)
 	    if (op->co_distance)
 		op->co_distance /= lexpand;
 
-	    if (op->co_bloats)
+	    if (op->co_client)
 	    {
 		int nlayers;
 		switch (op->co_opcode)
 		{
-		    case CIFOP_SQUARES:
-			nlayers = 4;
+		    case CIFOP_SLOTS:
+			slots = (SlotsData *)op->co_client;
+			if (slots->sl_sborder != 0)
+			    slots->sl_sborder /= lexpand;
+			if (slots->sl_ssize != 0)
+			    slots->sl_ssize /= lexpand;
+			if (slots->sl_ssep != 0)
+			    slots->sl_ssep /= lexpand;
+			if (slots->sl_lborder != 0)
+			    slots->sl_lborder /= lexpand;
+			if (slots->sl_lsize != 0)
+			    slots->sl_lsize /= lexpand;
+			if (slots->sl_lsep != 0)
+			    slots->sl_lsep /= lexpand;
+			if (slots->sl_offset != 0)
+			    slots->sl_offset /= lexpand;
 			break;
 		    case CIFOP_SQUARES_G:
-			nlayers = 5;
+			squares = (SquaresData *)op->co_client;
+			if (squares->sq_gridx != 0)
+			    squares->sq_gridx /= lexpand;
+			if (squares->sq_gridy != 0)
+			    squares->sq_gridy /= lexpand;
+			/* (drop through) */
+		    case CIFOP_SQUARES:
+			squares = (SquaresData *)op->co_client;
+			if (squares->sq_border != 0)
+			    squares->sq_border /= lexpand;
+			if (squares->sq_size != 0)
+			    squares->sq_size /= lexpand;
+			if (squares->sq_sep != 0)
+			    squares->sq_sep /= lexpand;
+			break;
+		    case CIFOP_BLOAT:
+		    case CIFOP_BLOATMIN:
+		    case CIFOP_BLOATMAX:
+			bloats = (BloatData *)op->co_client;
+			for (j = 0; j < TT_MAXTYPES; j++)
+			    if (bloats->bl_distance[j] != 0)
+				bloats->bl_distance[j] /= lexpand;
 			break;
 		    default:
-			nlayers = TT_MAXTYPES;
+			/* op->co_opcode in CIFOP_OR is a pointer copy	*/
+			/* and in CIFOP_BBOX is a flag.			*/
 			break;
 		}
-		for (j = 0; j < nlayers; j++)
-		    if (op->co_bloats[j] != 0)
-			op->co_bloats[j] /= lexpand;
 	    }
 	}
     }
