@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFwrite.c,v 1.4 2001/03/29 19:50:04 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/cif/CIFwrite.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdlib.h>
@@ -25,24 +25,23 @@ static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFwrite.c,v 1.4 2001/
 #include <string.h>
 #include <time.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/utils.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "database/databaseInt.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "utils/stack.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "cif/cif.h"
 #include "cif/CIFint.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "textio/textio.h"
-#include "misc/patchlevel.h"
 
     /* Forward declarations */
 extern int cifWriteInitFunc();
@@ -264,10 +263,7 @@ cifOutPreamble(outf, cell)
     fprintf(outf,"( @@user : %s );\n", (t = getenv("USER")) ? t : "?");
     fprintf(outf,"( @@machine : %s );\n", (t = getenv("HOSTNAME")) ? t : "?");
     fprintf(outf,"( @@source : %s );\n",(cell->cd_file) ? cell->cd_file : "?");
-    fprintf(outf,"( @@tool : Magic %s );\n", MagicVersion);
-    /* PATCHLEVEL defined in "patchlevel.h" */
-    fprintf(outf,"( @@patch : %d );\n", PATCHLEVEL);
-    fprintf(outf,"( @@patchnames : %s );\n", PatchNames());
+    fprintf(outf,"( @@tool : Magic %s.%s );\n", MagicVersion, MagicRevision);
     fprintf(outf,"( @@compiled : %s );\n", MagicCompileTime);
     fprintf(outf,"( @@technology : %s );\n", DBTechName);
     if (DBTechVersion)
