@@ -17,7 +17,7 @@
  *     *********************************************************************
  *
  *
- * rcsid "$Header: /ufs/repository/magic/cif/cif.h,v 1.2 2001/01/12 22:12:15 jsolomon Exp $
+ * rcsid "$Header: /usr/cvsroot/magic/cif/cif.h,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $
  */
 
 #ifndef _CIF_H
@@ -57,6 +57,7 @@ extern void CIFReadTechFinal();
 
 extern float CIFGetOutputScale();
 extern float CIFGetInputScale();
+extern int CIFGetDefaultContactSize();
 
 extern void CIFSeeLayer();
 extern void CIFSeeHierLayer();
@@ -74,5 +75,6 @@ extern void CIFPrintReadStyle();
 extern int CIFOutputScaleFactor();
 
 extern void PaintWireList();
+extern void PaintPolygon();
 
 #endif /* _CIF_H */
