@@ -17,22 +17,22 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdAB.c,v 1.4 2001/01/12 22:12:16 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/commands/CmdAB.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 
-#include "magic/tclmagic.h"
-#include "misc/magic.h"
+#include "tcltk/tclmagic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "utils/utils.h"
 #include "textio/textio.h"
@@ -41,7 +41,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdAB.c,v 1.4 200
 #include "graphics/graphics.h"
 #include "textio/txcommands.h"
 #include "utils/malloc.h"
-#include "netlist/netlist.h"
+#include "utils/netlist.h"
 #include "select/select.h"
 
 
@@ -196,8 +196,12 @@ CmdArray(w, cmd)
 		}
 		break;
 	    }
+	    else if ((cmd->tx_argc != 4) && (cmd->tx_argc != 6))
+		goto badusage;
+
 	    if (!StrIsInt(cmd->tx_argv[2]) || !StrIsInt(cmd->tx_argv[3])) 
-		    goto badusage;
+		goto badusage;
+
 	    if (cmd->tx_argc == 4)
 	    {
 		a.ar_xlo = 0;
@@ -440,7 +444,9 @@ selGetArrayFunc(selUse, use, trans, arg)
  *	box position [llx lly] [-edit]
  *	box values [llx lly urx ury] [-edit]
  *
- *	box move <direction> <distance>
+ *	box <direction> <distance> | cursor
+ *
+ *	box move <direction> <distance> | cursor
  *	box grow <direction> <distance>
  *	box shrink <direction> <distance>
  *
@@ -466,7 +472,6 @@ selGetArrayFunc(selUse, use, trans, arg)
  *
  * ----------------------------------------------------------------------------
  */
- /*ARGSUSED*/
 
 #define BOX_WIDTH	0
 #define BOX_HEIGHT	1
@@ -476,9 +481,10 @@ selGetArrayFunc(selUse, use, trans, arg)
 #define BOX_MOVE	5
 #define BOX_GROW	6
 #define BOX_SHRINK	7
-#define BOX_EXISTS	8
-#define BOX_HELP	9
-#define BOX_DEFAULT	10
+#define BOX_CORNER	8
+#define BOX_EXISTS	9
+#define BOX_HELP	10
+#define BOX_DEFAULT	11
 
 void
 CmdBox(w, cmd)
@@ -491,9 +497,10 @@ CmdBox(w, cmd)
 	"size		[width height]		set or return box size",
 	"position		[llx lly] [-edit]	set or return box position",
 	"values		[llx lly urx ury] [-edit]	set or return box coordinates",
-	"move		<direction> <distance>	move box position",
+	"move		<direction> <distance> 	move box position",
 	"grow		<direction> <distance>	expand box size",
 	"shrink		<direction> <distance>  shrink box size",
+	"corner		<direction> <distance>	set box corner",
 	"exists					is the cursor box present?",
 	"help					print help information",
 	NULL
@@ -501,12 +508,15 @@ CmdBox(w, cmd)
 
     CellDef *rootBoxDef;
     Rect rootBox, editbox, *boxptr;
-    int option, direction, distance;
-    int width, height, area, llx, lly;
+    Point ll;
+    int option, direction, distancex, distancey;
+    int width, height, area;
     int argc;
+    int tcorner;
     float iscale, oscale;
     bool needBox = TRUE;		/* require that box be defined */
     bool refEdit = FALSE;		/* referenced to edit cell coordinates */
+    bool cursorRef = FALSE;		/* reference position is the cursor */
     char **msg;
 
     argc = cmd->tx_argc;
@@ -546,12 +556,22 @@ CmdBox(w, cmd)
     windCheckOnlyWindow(&w, DBWclientID);
 
     /*----------------------------------------------------------*/
-    /* Check for the two commands which do not require a box to */
-    /* be present.						*/
+    /* Check for the command options which do not require a box	*/
+    /* to be present.						*/
     /*----------------------------------------------------------*/
 
     switch (option)
     {
+	case BOX_MOVE:
+	case BOX_GROW:
+	case BOX_SHRINK:
+	case BOX_CORNER:
+	    if (!strncmp(cmd->tx_argv[argc - 1], "cursor", 6))
+	    {
+		needBox = FALSE;
+		cursorRef = TRUE;
+	    }
+	    break;
 	case BOX_DEFAULT:
 	    if (argc == 5) needBox = FALSE;
 	    break;
@@ -592,10 +612,23 @@ CmdBox(w, cmd)
     {
 	Rect r;
 
-	r.r_xbot = cmdParseCoord(cmd->tx_argv[argc - 4]);
-	r.r_ybot = cmdParseCoord(cmd->tx_argv[argc - 3]);
-	r.r_xtop = cmdParseCoord(cmd->tx_argv[argc - 2]);
-	r.r_ytop = cmdParseCoord(cmd->tx_argv[argc - 1]);
+	if (argc >= 5)
+	{
+	    r.r_xbot = cmdParseCoord(w, cmd->tx_argv[argc - 4], FALSE, TRUE);
+	    r.r_ybot = cmdParseCoord(w, cmd->tx_argv[argc - 3], FALSE, FALSE);
+	    r.r_xtop = cmdParseCoord(w, cmd->tx_argv[argc - 2], FALSE, TRUE);
+	    r.r_ytop = cmdParseCoord(w, cmd->tx_argv[argc - 1], FALSE, FALSE);
+	}
+	else
+	{
+	    /* If we got here, then we requested a box or corner move
+	     * to the cursor position, but there was no box present.
+	     * To keep users from becoming bewildered at this point,
+	     * we generate a zero-size box at the origin and work from
+	     * there.
+	     */
+	    r.r_xbot = r.r_ybot = r.r_xtop = r.r_ytop = 0;
+	}
 
 	rootBoxDef = ((CellUse *) w->w_surfaceID)->cu_def;
 	if (refEdit)
@@ -621,11 +654,62 @@ CmdBox(w, cmd)
 	case BOX_MOVE:
 	case BOX_GROW:
 	case BOX_SHRINK:
+	case BOX_CORNER:
 	    if (argc != 4) goto badusage;
 	    direction = GeoNameToPos(cmd->tx_argv[2], FALSE, TRUE);
 	    if (direction < 0) return;
-	    distance = cmdParseCoord(cmd->tx_argv[3]);
-	    if (distance == 0) return;
+	    else if (cursorRef)
+	    {
+		switch (direction)
+		{
+		    case GEO_SOUTHWEST:
+			tcorner = TOOL_BL;
+			break;
+		    case GEO_NORTHEAST:
+			tcorner = TOOL_TR;
+			break;
+		    case GEO_NORTHWEST:
+			tcorner = TOOL_TL;
+			break;
+		    case GEO_SOUTHEAST:
+			tcorner = TOOL_BR;
+			break;
+		}
+		switch(option)
+		{
+		    case BOX_MOVE:
+			ToolMoveBox(tcorner, &cmd->tx_p, TRUE, rootBoxDef);
+			break;
+		    case BOX_CORNER:
+			ToolMoveCorner(tcorner, &cmd->tx_p, TRUE, rootBoxDef);
+			break;
+		}
+		return;
+	    }
+	    else if (DBWSnapToGrid != DBW_SNAP_USER)
+	    {
+		distancex = cmdParseCoord(w, cmd->tx_argv[3], TRUE, FALSE);
+		distancey = distancex;
+	    }
+	    else
+	    {
+		switch (direction)
+		{
+		    case GEO_EAST: case GEO_WEST:
+			distancex = cmdParseCoord(w, cmd->tx_argv[3], TRUE, TRUE);
+			distancey = distancex;
+			break;
+		    case GEO_NORTH: case GEO_SOUTH:
+			distancey = cmdParseCoord(w, cmd->tx_argv[3], TRUE, FALSE);
+			distancex = distancey;
+			break;
+		    default:
+			distancex = cmdParseCoord(w, cmd->tx_argv[3], TRUE, TRUE);
+			distancey = cmdParseCoord(w, cmd->tx_argv[3], TRUE, FALSE);
+			break;
+		}
+	    }
+	    if ((distancex == 0) && (distancey == 0)) return;
 	    break;
     }
 
@@ -649,7 +733,7 @@ CmdBox(w, cmd)
 		return;
 	    }
 	    else if (argc != 3) goto badusage;
-	    width = cmdParseCoord(cmd->tx_argv[2]);
+	    width = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
 	    rootBox.r_xtop = rootBox.r_xbot + width;
 	    break;
 
@@ -669,7 +753,7 @@ CmdBox(w, cmd)
 		return;
 	    }
 	    else if (argc != 3) goto badusage;
-	    height = cmdParseCoord(cmd->tx_argv[2]);
+	    height = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
 	    rootBox.r_ytop = rootBox.r_ybot + height;
 	    break;
 
@@ -691,8 +775,8 @@ CmdBox(w, cmd)
 		return;
 	    }
 	    else if (argc != 4) goto badusage;
-	    width = cmdParseCoord(cmd->tx_argv[2]);
-	    height = cmdParseCoord(cmd->tx_argv[3]);
+	    width = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
+	    height = cmdParseCoord(w, cmd->tx_argv[3], TRUE, FALSE);
 	    rootBox.r_xtop = rootBox.r_xbot + width;
 	    rootBox.r_ytop = rootBox.r_ybot + height;
 	    break;
@@ -715,10 +799,10 @@ CmdBox(w, cmd)
 	    else if (argc != 4) goto badusage;
 	    width = rootBox.r_xtop - rootBox.r_xbot;
 	    height = rootBox.r_ytop - rootBox.r_ybot;
-	    llx = cmdParseCoord(cmd->tx_argv[2]);
-	    lly = cmdParseCoord(cmd->tx_argv[3]);
-	    boxptr->r_xbot = llx;
-	    boxptr->r_ybot = lly;
+	    ll.p_x = cmdParseCoord(w, cmd->tx_argv[2], FALSE, TRUE);
+	    ll.p_y = cmdParseCoord(w, cmd->tx_argv[3], FALSE, FALSE);
+	    boxptr->r_xbot = ll.p_x;
+	    boxptr->r_ybot = ll.p_y;
 	    boxptr->r_xtop = boxptr->r_xbot + width;
 	    boxptr->r_ytop = boxptr->r_ybot + height;
 	    break;
@@ -747,44 +831,44 @@ CmdBox(w, cmd)
 	    switch (direction)
 	    {
 		case GEO_NORTH:
-		    rootBox.r_ybot += distance;
-		    rootBox.r_ytop += distance;
+		    rootBox.r_ybot += distancey;
+		    rootBox.r_ytop += distancey;
 		    break;
 		case GEO_SOUTH:
-		    rootBox.r_ybot -= distance;
-		    rootBox.r_ytop -= distance;
+		    rootBox.r_ybot -= distancey;
+		    rootBox.r_ytop -= distancey;
 		    break;
 		case GEO_EAST:
-		    rootBox.r_xbot += distance;
-		    rootBox.r_xtop += distance;
+		    rootBox.r_xbot += distancex;
+		    rootBox.r_xtop += distancex;
 		    break;
 		case GEO_WEST:
-		    rootBox.r_xbot -= distance;
-		    rootBox.r_xtop -= distance;
+		    rootBox.r_xbot -= distancex;
+		    rootBox.r_xtop -= distancex;
 		    break;
 		case GEO_NORTHEAST:
-		    rootBox.r_ybot += distance;
-		    rootBox.r_ytop += distance;
-		    rootBox.r_xbot += distance;
-		    rootBox.r_xtop += distance;
+		    rootBox.r_ybot += distancey;
+		    rootBox.r_ytop += distancey;
+		    rootBox.r_xbot += distancex;
+		    rootBox.r_xtop += distancex;
 		    break;
 		case GEO_NORTHWEST:
-		    rootBox.r_ybot += distance;
-		    rootBox.r_ytop += distance;
-		    rootBox.r_xbot -= distance;
-		    rootBox.r_xtop -= distance;
+		    rootBox.r_ybot += distancey;
+		    rootBox.r_ytop += distancey;
+		    rootBox.r_xbot -= distancex;
+		    rootBox.r_xtop -= distancex;
 		    break;
 		case GEO_SOUTHEAST:
-		    rootBox.r_ybot -= distance;
-		    rootBox.r_ytop -= distance;
-		    rootBox.r_xbot += distance;
-		    rootBox.r_xtop += distance;
+		    rootBox.r_ybot -= distancey;
+		    rootBox.r_ytop -= distancey;
+		    rootBox.r_xbot += distancex;
+		    rootBox.r_xtop += distancex;
 		    break;
 		case GEO_SOUTHWEST:
-		    rootBox.r_ybot -= distance;
-		    rootBox.r_ytop -= distance;
-		    rootBox.r_xbot -= distance;
-		    rootBox.r_xtop -= distance;
+		    rootBox.r_ybot -= distancey;
+		    rootBox.r_ytop -= distancey;
+		    rootBox.r_xbot -= distancex;
+		    rootBox.r_xtop -= distancex;
 		    break;
 	    }
 	    break;
@@ -793,78 +877,99 @@ CmdBox(w, cmd)
 	    switch (direction)
 	    {
 		case GEO_NORTH:
-		    rootBox.r_ytop += distance;
+		    rootBox.r_ytop += distancey;
 		    break;
 		case GEO_SOUTH:
-		    rootBox.r_ybot -= distance;
+		    rootBox.r_ybot -= distancey;
 		    break;
 		case GEO_EAST:
-		    rootBox.r_xtop += distance;
+		    rootBox.r_xtop += distancex;
 		    break;
 		case GEO_WEST:
-		    rootBox.r_xbot -= distance;
+		    rootBox.r_xbot -= distancex;
 		    break;
 		case GEO_NORTHEAST:
-		    rootBox.r_ytop += distance;
-		    rootBox.r_xtop += distance;
+		    rootBox.r_ytop += distancey;
+		    rootBox.r_xtop += distancex;
 		    break;
 		case GEO_NORTHWEST:
-		    rootBox.r_ytop += distance;
-		    rootBox.r_xbot -= distance;
+		    rootBox.r_ytop += distancey;
+		    rootBox.r_xbot -= distancex;
 		    break;
 		case GEO_SOUTHEAST:
-		    rootBox.r_ybot -= distance;
-		    rootBox.r_xtop += distance;
+		    rootBox.r_ybot -= distancey;
+		    rootBox.r_xtop += distancex;
 		    break;
 		case GEO_SOUTHWEST:
-		    rootBox.r_ybot -= distance;
-		    rootBox.r_xbot -= distance;
+		    rootBox.r_ybot -= distancey;
+		    rootBox.r_xbot -= distancex;
 		    break;
 		case GEO_CENTER:
-		    rootBox.r_ytop += distance;
-		    rootBox.r_ybot -= distance;
-		    rootBox.r_xtop += distance;
-		    rootBox.r_xbot -= distance;
+		    rootBox.r_ytop += distancey;
+		    rootBox.r_ybot -= distancey;
+		    rootBox.r_xtop += distancex;
+		    rootBox.r_xbot -= distancex;
 		    break;
 	    }
 	    break;
 
+	case BOX_CORNER:
+	    ll.p_x = cmdParseCoord(w, cmd->tx_argv[2], FALSE, TRUE);
+	    ll.p_y = cmdParseCoord(w, cmd->tx_argv[3], FALSE, FALSE);
+	    switch (direction)
+	    {
+		case GEO_SOUTHWEST:
+		    tcorner = TOOL_BL;
+		    break;
+		case GEO_NORTHEAST:
+		    tcorner = TOOL_TR;
+		    break;
+		case GEO_NORTHWEST:
+		    tcorner = TOOL_TL;
+		    break;
+		case GEO_SOUTHEAST:
+		    tcorner = TOOL_BR;
+		    break;
+	    }
+	    ToolMoveCorner(tcorner, &ll, FALSE, rootBoxDef);
+	    return;
+
 	case BOX_SHRINK:
 	    switch (direction)
 	    {
 		case GEO_NORTH:
-		    rootBox.r_ytop -= distance;
+		    rootBox.r_ytop -= distancey;
 		    break;
 		case GEO_SOUTH:
-		    rootBox.r_ybot += distance;
+		    rootBox.r_ybot += distancey;
 		    break;
 		case GEO_EAST:
-		    rootBox.r_xtop -= distance;
+		    rootBox.r_xtop -= distancex;
 		    break;
 		case GEO_WEST:
-		    rootBox.r_xbot += distance;
+		    rootBox.r_xbot += distancex;
 		    break;
 		case GEO_NORTHEAST:
-		    rootBox.r_ytop -= distance;
-		    rootBox.r_xtop -= distance;
+		    rootBox.r_ytop -= distancey;
+		    rootBox.r_xtop -= distancex;
 		    break;
 		case GEO_NORTHWEST:
-		    rootBox.r_ytop -= distance;
-		    rootBox.r_xbot += distance;
+		    rootBox.r_ytop -= distancey;
+		    rootBox.r_xbot += distancex;
 		    break;
 		case GEO_SOUTHEAST:
-		    rootBox.r_ybot += distance;
-		    rootBox.r_xtop -= distance;
+		    rootBox.r_ybot += distancey;
+		    rootBox.r_xtop -= distancex;
 		    break;
 		case GEO_SOUTHWEST:
-		    rootBox.r_ybot += distance;
-		    rootBox.r_xbot += distance;
+		    rootBox.r_ybot += distancey;
+		    rootBox.r_xbot += distancex;
 		    break;
 		case GEO_CENTER:
-		    rootBox.r_ytop -= distance;
-		    rootBox.r_xtop -= distance;
-		    rootBox.r_ybot += distance;
-		    rootBox.r_xbot += distance;
+		    rootBox.r_ytop -= distancey;
+		    rootBox.r_xtop -= distancex;
+		    rootBox.r_ybot += distancey;
+		    rootBox.r_xbot += distancex;
 		    break;
 	    }
 	    break;
