@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdCD.c,v 1.5 2001/01/12 22:12:16 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/commands/CmdCD.c,v 1.2 2005/05/13 19:31:58 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -27,26 +27,27 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdCD.c,v 1.5 200
 #endif
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "utils/utils.h"
 #include "textio/textio.h"
 #include "drc/drc.h"
 #include "graphics/graphics.h"
 #include "textio/txcommands.h"
+#include "textio/textio.h"
 #include "cif/cif.h"
 #include "calma/calma.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "router/rtrDcmpose.h"
 #include "select/select.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "utils/malloc.h"
 #include "cif/CIFint.h"
 
@@ -94,16 +95,18 @@ bool cmdDumpParseArgs();
 
 #define CALMA_HELP	0
 #define CALMA_ARRAYS	1
-#define	CALMA_FLATTEN	2
-#define	CALMA_LABELS	3
-#define	CALMA_LOWER	4
-#define CALMA_READ	5
-#define CALMA_READONLY	6
-#define CALMA_RESCALE	7
-#define CALMA_WARNING	8
-#define CALMA_WRITE	9
+#define CALMA_CONTACTS	2
+#define	CALMA_FLATTEN	3
+#define CALMA_ORDERING	4
+#define	CALMA_LABELS	5
+#define	CALMA_LOWER	6
+#define CALMA_READ	7
+#define CALMA_READONLY	8
+#define CALMA_RESCALE	9
+#define CALMA_WARNING	10
+#define CALMA_WRITE	11
 #ifdef NONMANHATTAN
-#define CALMA_POLYS	10
+#define CALMA_POLYS	12
 #endif
 
 #define CALMA_WARN_HELP CIF_WARN_END	/* undefined by CIF module */
@@ -126,7 +129,9 @@ CmdCalma(w, cmd)
     {	
 	"help		print this help information",
 	"arrays [yes|no]	output arrays as individual subuses (like in CIF)",
+	"contacts [yes|no]	optimize output by arraying contacts as subcells",
 	"flatten [yes|no]	flatten simple cells (e.g., contacts) on input",
+	"ordering [on|off]	cause cells to be read in post-order",
 	"labels [yes|no]	cause labels to be output when writing GDS-II",
 	"lower [yes|no]		allow both upper and lower case in labels",
 	"read file		read Calma GDS-II format from \"file\"\n"
@@ -222,6 +227,28 @@ CmdCalma(w, cmd)
 	    CalmaDoLabels = (option < 3) ? FALSE : TRUE;
 	    return;
 
+	case CALMA_CONTACTS:
+	    if (cmd->tx_argc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CalmaContactArrays));
+#else
+		if (CalmaContactArrays)
+		    TxPrintf("Contact areas are arrayed as subcells.\n");
+		else
+		    TxPrintf("Contact areas are output as individual cuts.\n");
+#endif
+		return;
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto wrongNumArgs;
+
+	    option = Lookup(cmd->tx_argv[2], cmdCalmaYesNo);
+	    if (option < 0)
+		goto wrongNumArgs;
+	    CalmaContactArrays = (option < 3) ? FALSE : TRUE;
+	    return;
+
 	case CALMA_FLATTEN:
 	    if (cmd->tx_argc == 2)
 	    {
@@ -244,6 +271,29 @@ CmdCalma(w, cmd)
 	    CalmaFlattenUses = (option < 3) ? FALSE : TRUE;
 	    return;
 
+	case CALMA_ORDERING:
+	    if (cmd->tx_argc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CalmaPostOrder));
+#else
+		if (CalmaPostOrder)
+		    TxPrintf("GDS parser reads cells in post-order.\n");
+		else
+		    TxPrintf("GDS parser reads cells in the order "
+				"encountered in the stream file.\n");
+		return;
+#endif
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto wrongNumArgs;
+
+	    option = Lookup(cmd->tx_argv[2], cmdCalmaYesNo);
+	    if (option < 0)
+		goto wrongNumArgs;
+	    CalmaPostOrder = (option < 3) ? FALSE : TRUE;
+	    return;
+
 	case CALMA_ARRAYS:
 	    if (cmd->tx_argc == 2)
 	    {
@@ -445,11 +495,13 @@ outputCalma:
  *
  * Usage:
  *	cellname [list] children|parents|exists|instances|
- *			celldef|self|childinst|delete [name]
- * or
- *	cellname rename [name] [newname]
+ *			celldef|self|childinst [name]
  * or
  *	cellname [list] topcells|allcells|window
+ * or
+ *	cellname delete|lock|unlock [name]
+ * or
+ *	cellname rename [name] [newname]
  *
  * Results:
  *	None.
@@ -473,7 +525,7 @@ CmdCellname(w, cmd)
     int locargc = cmd->tx_argc;
     char *cellname = NULL;
     void (*func)();
-    CellDef *newDef;
+    CellDef *newDef, *cellDef;
 
     static char *cmdCellOption[] =
     {	
@@ -489,13 +541,17 @@ CmdCellname(w, cmd)
 	"window		list top-level cell of a layout window",
 	"create		create a new cell definition",
 	"delete		delete the named cell definition",
+	"flags		list option flags of the indicated cell definition",
+	"lock		lock the cell definition (make read-only)",
 	"rename		rename the indicated cell",
+	"unlock		unlock the cell definition (make writeable)",
 	NULL
     };
     typedef enum { IDX_CHILDREN, IDX_PARENTS, IDX_EXISTS, IDX_SELF,
 		   IDX_INSTANCE, IDX_CHILDINST, IDX_CELLDEF, IDX_ALLCELLS,
 		   IDX_TOPCELLS, IDX_IN_WINDOW, IDX_CREATE,
-		   IDX_DELETE, IDX_RENAME } optionType;
+		   IDX_DELETE, IDX_FLAGS, IDX_LOCK, IDX_RENAME,
+		   IDX_UNLOCK } optionType;
 
     if (strstr(cmd->tx_argv[0], "in"))
 	func = DBUsePrint;
@@ -535,7 +591,7 @@ CmdCellname(w, cmd)
 	    case IDX_TOPCELLS:
 		TxError("Instances do not have a top level.  Use \"cellname\"?\n");
 		return;
-	    case IDX_IN_WINDOW:
+	    case IDX_IN_WINDOW: case IDX_LOCK: case IDX_UNLOCK: case IDX_FLAGS:
 		TxError("Function unimplemented for instances.\n");
 		return;
 	    case IDX_DELETE:
@@ -581,10 +637,42 @@ CmdCellname(w, cmd)
 	    break;
 	case IDX_DELETE:
 	    /* Unload the cell definition and free memory */
-	    DBCellDelete(cellname);
+	    if ((locargc == 4) && !strcmp(cmd->tx_argv[3 + ((dolist) ? 1 : 0)],
+			"-noprompt"))
+	        DBCellDelete(cellname, TRUE);
+	    else
+	        DBCellDelete(cellname, FALSE);
+	    break;
+	case IDX_LOCK:
+	    cellDef = DBCellLookDef(cellname);
+	    cellDef->cd_flags |= CDNOEDIT;
+	    break;
+	case IDX_UNLOCK:
+	    cellDef = DBCellLookDef(cellname);
+	    cellDef->cd_flags &= ~CDNOEDIT;
+	    break;
+	case IDX_FLAGS:
+	    cellDef = DBCellLookDef(cellname);
+#ifdef MAGIC_WRAPPER
+	    if (cellDef->cd_flags & CDAVAILABLE)
+		Tcl_AppendElement(magicinterp, "available");
+	    if (cellDef->cd_flags & CDMODIFIED)
+		Tcl_AppendElement(magicinterp, "modified");
+	    if (cellDef->cd_flags & CDMODIFIED)
+		Tcl_AppendElement(magicinterp, "readonly");
+#else
+	    TxPrintf("Flag settings for cell %s:\n", cellname);
+	    TxPrintf("in database: %s\n", (cellDef->cd_flags & CDAVAILABLE) ?
+			"true" : "false");
+	    TxPrintf("modified: %s\n", (cellDef->cd_flags & CDMODIFIED) ?
+			"true" : "false");
+	    TxPrintf("readonly: %s\n", (cellDef->cd_flags & CDNOEDIT) ?
+			"true" : "false");
+#endif
 	    break;
 	case IDX_RENAME:
 	    /* Rename the cell and mark as modified.  Do not write to disk. */
+	    if (locargc != 4) goto badusage;
 	    DBCellRename(cellname, cmd->tx_argv[3 + ((dolist) ? 1 : 0)]);
 	    break;
 	case IDX_CREATE:
@@ -882,7 +970,7 @@ CmdCif(w, cmd)
     switch (option)
     {
 	case HELP: case ISTYLE: case OSTYLE: case PREFIX:
-	case AREALABELS: case WARNING:
+	case AREALABELS: case WARNING: case CIF_LIMIT:
 #ifdef NONMANHATTAN
 	case POLYGONS:
 #endif
@@ -1223,8 +1311,8 @@ CmdCif(w, cmd)
  * box clockwise around the point.
  *
  * Usage:
- *	clockwise [degrees]	or
- *	rotate [degrees]
+ *	clockwise [degrees] [-origin]	or
+ *	rotate [degrees] [-origin]
  *
  * Results:
  *	None.
@@ -1239,6 +1327,9 @@ CmdCif(w, cmd)
  *	compatibility but the command must contain at least "clockw" for
  *	the interpreter to uniquely identify it.
  *
+ *	The "-origin" option has been added to allow rotating stuff
+ *	relative to the origin, insteadd of the lower-left corner.
+ *
  * ----------------------------------------------------------------------------
  */
 
@@ -1248,13 +1339,21 @@ CmdClockwise(w, cmd)
     TxCommand *cmd;
 {
     Transform trans, t2;
-    int degrees;
+    int degrees, locargc;
     Rect rootBox,  bbox;
     CellDef *rootDef;
+    bool noAdjust = FALSE;
 
-    if (cmd->tx_argc == 1)
+    locargc = cmd->tx_argc;
+    if (!strncmp(cmd->tx_argv[locargc - 1], "-orig", 5))
+    {
+	noAdjust = TRUE;
+	locargc--;
+    }
+
+    if (locargc == 1)
 	degrees = 90;
-    else if (cmd->tx_argc == 2)
+    else if (locargc == 2)
     {
 	if (!StrIsInt(cmd->tx_argv[1])) goto badusage;
 	degrees = atoi(cmd->tx_argv[1]);
@@ -1285,9 +1384,17 @@ CmdClockwise(w, cmd)
      * that it used to be.
      */
     
-    GeoTransRect(&t2, &SelectDef->cd_bbox, &bbox);
-    GeoTranslateTrans(&t2, SelectDef->cd_bbox.r_xbot - bbox.r_xbot,
-	SelectDef->cd_bbox.r_ybot - bbox.r_ybot, &trans);
+    if (noAdjust)
+    {
+	GeoTransRect(&t2, &SelectDef->cd_bbox, &bbox);
+	trans = t2;
+    }
+    else
+    {
+	GeoTransRect(&t2, &SelectDef->cd_bbox, &bbox);
+	GeoTranslateTrans(&t2, SelectDef->cd_bbox.r_xbot - bbox.r_xbot,
+		SelectDef->cd_bbox.r_ybot - bbox.r_ybot, &trans);
+    }
 
     SelectTransform(&trans);
 
@@ -2399,6 +2506,50 @@ cmdBevelFunc(tile, cxp)
 }
 #endif  /* NONMANHATTAN */
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CmdCrosshair --
+ *
+ * Manipulate the crosshair highlight
+ *
+ * Usage:
+ *	crosshair <x> <y>
+ *	crosshair off
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The crosshair is enabled, moved, or disabled.
+ * ----------------------------------------------------------------------------
+ */
+void
+CmdCrosshair(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    Point pos;
+
+    if (cmd->tx_argc == 2)
+    {
+	if (!strcmp(cmd->tx_argv[1], "off"))
+	{
+	    /* Erase and turn off */
+	    pos.p_x = pos.p_y = MINFINITY;
+	    DBWSetCrosshair(w, &pos);
+	}
+    }
+    else if (cmd->tx_argc == 3)
+    {
+	pos.p_x = cmdParseCoord(w, cmd->tx_argv[1], FALSE, TRUE);
+	pos.p_y = cmdParseCoord(w, cmd->tx_argv[2], FALSE, FALSE);
+	DBWSetCrosshair(w, &pos);
+    }
+    else
+	TxError("Usage: %s off|x y \n", cmd->tx_argv[0]);
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -2425,7 +2576,7 @@ CmdDelete(w, cmd)
     TxCommand *cmd;
 {
     if (cmd->tx_argc != 1) goto badusage;
-    SelectDelete("deleted");
+    SelectDelete("deleted", TRUE);
     return;
 
     badusage:
@@ -2568,20 +2719,22 @@ cmdDownEnumFunc(selUse, use, transform, area)
  */
 
 #define FLATCHECK	0
-#define SHOWINT		1
-#define CATCHUP		2
-#define CHECK		3
-#define COUNT		4
-#define EUCLIDEAN	5
-#define FIND		6
-#undef HELP
-#define HELP		7
-#define OFF		8
-#define ON		9
-#define PRINTRULES	10
-#define RULESTATS	11
-#define STATISTICS	12
-#define WHY		13
+#define DRC_HALO	1
+#define SHOWINT		2
+#define DRC_STEPSIZE	3
+#define CATCHUP		4
+#define CHECK		5
+#define COUNT		6
+#define EUCLIDEAN	7
+#define FIND		8
+#define DRC_HELP	9
+#define DRC_OFF		10
+#define DRC_ON		11
+#define DRC_STATUS	12
+#define PRINTRULES	13
+#define RULESTATS	14
+#define STATISTICS	15
+#define WHY		16
 
 void
 CmdDrc(w, cmd)
@@ -2589,27 +2742,34 @@ CmdDrc(w, cmd)
     TxCommand *cmd;
 {
     FILE        * fp;
-    int		  option, nth, result, radius;
+    static int	drc_nth = 1;
+    int		  option, result, radius;
     Rect	  rootArea, area;
     CellUse	* rootUse, *use;
     CellDef	* rootDef;
     Transform	  trans;
     MagWindow	* window;
     char 	**msg;
-    bool	  wizardHelp;
+    bool	wizardHelp;
+    bool	incremental;
+    int		count_total;
+    DRCCountList *dcl, *dclsrch;
 
     static char *cmdDrcOption[] =
     {	
 	"*flatcheck             check box area by flattening",
+	"*halo [d]		limit error checking to areas of d lambda",
 	"*showint radius        show interaction area under box",
+	"*stepsize [d]		change drc step size to d lambda",
 	"catchup                run checker and wait for it to complete",
 	"check                  recheck area under box in all cells",
 	"count                  count error tiles in each cell under box",
 	"euclidean on|off	enable/disable Euclidean geometry checking",
-	"find [nth]             locate next (or nth) error in current cell",
+	"find [nth]      	locate next (or nth) error in the layout",
 	"help                   print this help information",
 	"off                    turn off background checker",
 	"on                     reenable background checker",
+	"status			report if the drc checker is on or off",
 	"printrules [file]      print out design rules in file or on tty",
 	"rulestats              print out stats about design rule database",
 	"statistics             print out statistics gathered by checker",
@@ -2620,7 +2780,7 @@ CmdDrc(w, cmd)
     if (cmd->tx_argc < 2)
     {
 	TxError("No option given in \":drc\" command.\n");
-	option = HELP;
+	option = DRC_HELP;
     }
     else
     {
@@ -2628,11 +2788,12 @@ CmdDrc(w, cmd)
 	if (option < 0)
 	{
 	    TxError("%s isn't a valid drc option.\n", cmd->tx_argv[1]);
-	    option = HELP;
+	    option = DRC_HELP;
 	    cmd->tx_argc = 2;
 	}
 	if ((cmd->tx_argc > 2) && (option != PRINTRULES) && (option != FIND)
-	    && (option != SHOWINT) && (option != HELP) && (option != EUCLIDEAN))
+	    && (option != SHOWINT) && (option != DRC_HELP) && (option != EUCLIDEAN)
+	    && (option != DRC_STEPSIZE) && (option != DRC_HALO) && (option != COUNT))
 	{
 	    badusage:
 	    TxError("Wrong arguments in \"drc %s\" command:\n",
@@ -2684,10 +2845,44 @@ CmdDrc(w, cmd)
 	    break;
 	
 	case COUNT:
-	    window = ToolGetBoxWindow(&rootArea, (int *) NULL);
-	    if (window == NULL) return;
+	    count_total = -1;
+	    if (cmd->tx_argc == 3)
+		if (!strncmp(cmd->tx_argv[2], "total", 5))
+		    count_total = 0;
+
+	    if ((window = w) == NULL)
+	    {
+		window = ToolGetBoxWindow(&rootArea, (int *) NULL);
+		if (window == NULL) return;
+	    }
+	    else
+		rootArea = w->w_surfaceArea;
+
 	    rootUse = (CellUse *) window->w_surfaceID;
-	    DRCCount(rootUse, &rootArea);
+	    dcl = DRCCount(rootUse, &rootArea);
+	    while (dcl != NULL)
+	    {
+		if (count_total >= 0)
+		    count_total += dcl->dcl_count;
+		else
+		{
+		   
+		    if (dcl->dcl_count > 1)
+			TxPrintf("Cell %s has %d error tiles.\n",
+				dcl->dcl_def->cd_name, dcl->dcl_count);
+		    else if (dcl->dcl_count == 1)
+			TxPrintf("Cell %s has just one error tile.\n",
+				dcl->dcl_def->cd_name);
+		}
+		freeMagic((char *)dcl);
+		dcl = dcl->dcl_next;
+	    }
+	    if (count_total >= 0)
+#ifdef MAGIC_WRAPPER
+	        Tcl_SetObjResult(magicinterp, Tcl_NewIntObj(count_total));
+#else
+		TxPrintf("Total DRC errors found: %d\n", count_total);
+#endif
 	    break;
 
 	case EUCLIDEAN:
@@ -2698,38 +2893,75 @@ CmdDrc(w, cmd)
 	    break;
 	
 	case FIND:
-	    if (cmd->tx_argc > 2)
+	    if ((window = w) == NULL)
 	    {
-		if (cmd->tx_argc > 3) goto badusage;
-		nth = atoi(cmd->tx_argv[2]);
+		window = ToolGetBoxWindow(&rootArea, (int *) NULL);
+		if (window == NULL) return;
 	    }
-	    else nth = 0;
-	    use = CmdGetSelectedCell(&trans);
-	    rootDef = SelectRootDef;
-	    if (use == NULL)
+	    else
+		rootArea = w->w_surfaceArea;
+	    if (cmd->tx_argc > 3) goto badusage;
+
+	    rootUse = (CellUse *) window->w_surfaceID;
+	    rootDef = rootUse->cu_def;
+
+	    incremental = FALSE;
+	    if (cmd->tx_argc == 3 && StrIsInt(cmd->tx_argv[2]))
 	    {
-		use = EditCellUse;
-		rootDef = EditRootDef;
-		trans = EditToRootTransform;
-	    }
-	    result = DRCFind(use->cu_def, &area, nth);
-	    if (result != 0)
-	    {
-		GeoTransRect(&trans, &area, &rootArea);
-		ToolMoveBox(TOOL_BL, &rootArea.r_ll, FALSE, rootDef);
-		ToolMoveCorner(TOOL_TR, &rootArea.r_ur, FALSE, rootDef);
-		TxPrintf("Error area #%d:\n", result);
-		DRCWhy(use, &area);
+		drc_nth = atoi(cmd->tx_argv[2]);
+		if (drc_nth <= 0) drc_nth = 1;
 	    }
 	    else
 	    {
-		if (nth > 1) TxPrintf("There aren't that many errors");
+	        incremental = TRUE;
+		drc_nth++;
+	    }
+
+	    result = DRCFind(rootUse, &rootArea, &area, drc_nth);
+	    if (incremental && (result < 0))
+	    {
+		/* 2nd pass, if we exceeded the total number of errors */
+		drc_nth = 1;
+		result = DRCFind(rootUse, &rootArea, &area, drc_nth);
+	    }
+
+	    if (result > 0)
+	    {
+		ToolMoveBox(TOOL_BL, &area.r_ll, FALSE, rootDef);
+		ToolMoveCorner(TOOL_TR, &area.r_ur, FALSE, rootDef);
+		TxPrintf("Error area #%d:\n", result);
+		DRCWhy(rootUse, &area);
+	    }
+	    else
+	    {
+		if (result < 0) TxPrintf("There aren't that many errors");
 		else TxPrintf("There are no errors");
-		TxPrintf(" in %s.\n", use->cu_def->cd_name);
+		TxPrintf(" in %s.\n", rootDef->cd_name);
 	    }
 	    break;
 	
-	case HELP:
+	case DRC_HALO:
+	    if (cmd->tx_argc == 3)
+	    {
+		TechHalo = atoi(cmd->tx_argv[2]);
+		if (TechHalo > MaxHalo)
+		    TechHalo = MaxHalo;
+
+		if (TechHalo < MaxHalo)
+		    TxPrintf("Warning: rulechecking limited to halo of %d lambda.\n",
+				TechHalo);
+		else
+		    TxPrintf("DRC checks all rules (halo of %d lambda)\n", TechHalo);
+	    }
+	    else
+	    {
+		TxPrintf("DRC halo is %d lambda\n", TechHalo);
+		if (TechHalo != MaxHalo)
+		    TxPrintf("Maximum rule distance is %d lambda\n", MaxHalo);
+	    }
+	    break;
+
+	case DRC_HELP:
 	    if ((cmd->tx_argc == 3)
 		    && (strcmp(cmd->tx_argv[2], "wizard") == 0))
 		wizardHelp = TRUE;
@@ -2743,13 +2975,25 @@ CmdDrc(w, cmd)
 	    }
 	    break;
 	
-	case OFF:
+	case DRC_OFF:
 	    DRCBackGround = FALSE;
+#ifdef MAGIC_WRAPPER
+	    if (TxInputRedirect != TX_INPUT_REDIRECTED)
+#endif
+		TxSetPrompt('%');		/* Return prompt to "normal" */
 	    break;
 	
-	case ON:
+	case DRC_ON:
 	    DRCBackGround = TRUE;
 	    break;
+
+	case DRC_STATUS:
+#ifdef MAGIC_WRAPPER
+	    Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(DRCBackGround));
+#else
+	    TxPrintf("DRC checker is %s.\n", (DRCBackGround) ? "on" : "off");
+#endif
+	    break;
 	
 	case PRINTRULES:
 	    if (cmd->tx_argc > 3) goto badusage;
@@ -2773,6 +3017,20 @@ CmdDrc(w, cmd)
 	    DRCPrintStats();
 	    break;
 
+	case DRC_STEPSIZE:
+	    if (cmd->tx_argc == 3)
+	    {
+		DRCStepSize = atoi(cmd->tx_argv[2]);
+		TxPrintf("DRC step size is now %d lambda)\n", DRCStepSize);
+	    }
+	    else
+	    {
+		TxPrintf("DRC step size is %d lambda\n", DRCStepSize);
+		if (DRCStepSize != (16 * MaxHalo))
+		    TxPrintf("Recommended step size is %d lambda\n", 16 * MaxHalo);
+	    }
+	    break;
+
 	case WHY:
 	    window = ToolGetBoxWindow(&rootArea, (int *) NULL);
 	    if (window == NULL) return;
@@ -2866,7 +3124,7 @@ cmdDumpParseArgs(cmdName, w, cmd, dummy, scx)
     Rect rootBox;
     Transform *tx_cell, trans_cell;
     char **av;
-    int ac;
+    int ac, clen;
 
     if (cmd->tx_argc < 2)
     {
@@ -2877,6 +3135,12 @@ cmdDumpParseArgs(cmdName, w, cmd, dummy, scx)
     /* Locate the cell specified by the command */
     if (CmdIllegalChars(cmd->tx_argv[1], "", "Cell name"))
 	return (FALSE);
+
+    /* If the name still has ".mag" attached, then strip it. */
+    clen = strlen(cmd->tx_argv[1]);
+    if ((clen > 4) && !strcmp(cmd->tx_argv[1] + clen - 4, ".mag"))
+	*(cmd->tx_argv[1] - 4) = '\0';
+
     def = DBCellLookDef(cmd->tx_argv[1]);
     if (def == (CellDef *) NULL)
 	def = DBCellNewDef(cmd->tx_argv[1], (char *) NULL);
@@ -2894,7 +3158,7 @@ cmdDumpParseArgs(cmdName, w, cmd, dummy, scx)
     DBReComputeBbox(def);
     dummy->cu_def = def;
     dummy->cu_transform = GeoIdentityTransform;
-    dummy->cu_expandMask = -1;
+    dummy->cu_expandMask = CU_DESCEND_SPECIAL;
     if (DBIsAncestor(def, EditCellUse->cu_def))
     {
 	TxError("The edit cell is already a desecendant of \"%s\",\n",
