@@ -17,18 +17,16 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdE.c,v 1.4 2001/01/12 22:12:16 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/commands/CmdE.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef MAGIC_WRAPPER
-#include "magic/tclmagic.h"
-#endif
-#include "misc/magic.h"
-#include "misc/styles.h"
+#include "tcltk/tclmagic.h"
+#include "utils/magic.h"
+#include "utils/styles.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
@@ -37,14 +35,15 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdE.c,v 1.4 2001
 #include "windows/windows.h"
 #include "graphics/graphics.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "textio/textio.h"
-#include "macros/macros.h"
+#include "utils/macros.h"
 #include "drc/drc.h"
 #include "textio/txcommands.h"
 #include "extract/extract.h"
 #include "select/select.h"
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -276,9 +275,9 @@ CmdElement(w, cmd)
     bool getopt;
 
     static char *cmdElementOption[] = {
-	"add		create a new element",
+	"add			create a new element",
 	"delete		delete an existing element",
-	"configure	configure or query an existing element",
+	"configure		configure or query an existing element",
 	"names		print names of all elements",
 	"inbox		print name of element nearest the box",
 	"help		print help information",
@@ -286,16 +285,16 @@ CmdElement(w, cmd)
     };
 
     static char *cmdElementType[] = {
-	"line		style x1 y1 x2 y2",
-	"rectangle	style llx lly urx ury",
-	"text		style cx cy label",
+	"line		name style x1 y1 x2 y2",
+	"rectangle		name style llx lly urx ury",
+	"text		name style cx cy label",
 	NULL
     };
 
     static char *cmdConfigureType[] = {
-	"text		get or replace <string>",
-	"style		get or add <style> or remove <style>",
-	"position	get or <point> or <rect>",
+	"text		get (or) replace <string>",
+	"style		get (or) add <style> (or) remove <style>",
+	"position		get (or) <point> (or) <rect>",
 	"flags		set element flags",
 	NULL
     };
@@ -318,7 +317,7 @@ CmdElement(w, cmd)
     switch (option)
     {
 	case ELEMENT_ADD:
-	    if (cmd->tx_argc < 7)
+	    if (cmd->tx_argc < 3)
 	    {
 		TxError("Usage:  element add <type> <name> <values...>\n");
 		return;
@@ -326,14 +325,13 @@ CmdElement(w, cmd)
 	    type = Lookup(cmd->tx_argv[2], cmdElementType);
 	    if (type < 0)
 	    {
-		TxError("Usage:  element add <type> <name>, where type is one of:\n");
+		TxError("Usage:  element add <type>..., where type is one of:\n");
 		for (msg = &(cmdElementType[0]); *msg != NULL; msg++)
-		{
 		    TxError("    %s\n", *msg);
-		}
+
 		return;
 	    }
-	    else
+	    else if (cmd->tx_argc >= 7)
 	    {
 		if (StrIsInt(cmd->tx_argv[4]))
 		    style = atoi(cmd->tx_argv[4]);
@@ -358,7 +356,8 @@ CmdElement(w, cmd)
 		    case ELEMENT_LINE:
 			if (cmd->tx_argc != 9)
 			{
-			    TxError("Wrong number of arguments for line.\n");
+			    TxError("Usage:  element add %s\n",
+					cmdElementType[ELEMENT_LINE]);
 			    return;
 			}
 			if (!StrIsInt(cmd->tx_argv[7])) goto badusage;
@@ -370,7 +369,8 @@ CmdElement(w, cmd)
 		    case ELEMENT_RECT:
 			if (cmd->tx_argc != 9)
 			{
-			    TxError("Wrong number of arguments for line.\n");
+			    TxError("Usage:  element add %s\n",
+					cmdElementType[ELEMENT_RECT]);
 			    return;
 			}
 			if (!StrIsInt(cmd->tx_argv[7])) goto badusage;
@@ -382,7 +382,8 @@ CmdElement(w, cmd)
 		    case ELEMENT_TEXT:
 			if (cmd->tx_argc != 8)
 			{
-			    TxError("Wrong number of arguments for line.\n");
+			    TxError("Usage:  element add %s\n",
+					cmdElementType[ELEMENT_TEXT]);
 			    return;
 			}
 			DBWElementAddText(w, cmd->tx_argv[3], area.r_xbot,
@@ -390,6 +391,11 @@ CmdElement(w, cmd)
 			break;
 		}
 	    }
+	    else
+	    {
+		TxError("Usage:  element add %s\n", cmdElementType[type]);
+		return;
+	    }
 	    break;
 	case ELEMENT_DELETE:
 	    if (cmd->tx_argc != 3)
@@ -400,7 +406,7 @@ CmdElement(w, cmd)
 	    DBWElementDelete(w, cmd->tx_argv[2]);
 	    break;
 	case ELEMENT_CONFIG:
-	    if (cmd->tx_argc < 3)
+	    if (cmd->tx_argc < 4)
 	    {
 		TxError("Usage:  element configure <name> <option...>\n");
 		return;
@@ -564,7 +570,7 @@ badusage:
  * EditCellUse->cu_def.
  *
  * Usage:
- *	erase [layers]
+ *	erase [layers | cursor]
  *
  * Results:
  *	None.
@@ -598,7 +604,7 @@ CmdErase(w, cmd)
 
     if (cmd->tx_argc > 2)
     {
-	TxError("Usage: %s [layers]\n", cmd->tx_argv[0]);
+	TxError("Usage: %s [<layers> | cursor]\n", cmd->tx_argv[0]);
 	return;
     }
 
@@ -612,6 +618,11 @@ CmdErase(w, cmd)
 
     if (cmd->tx_argc == 1)
 	(void) CmdParseLayers("*,label", &mask);
+    else if (!strncmp(cmd->tx_argv[1], "cursor", 6))
+    {
+	CmdPaintEraseButton(w, &cmd->tx_p, FALSE);
+	return;
+    }
     else if (!CmdParseLayers(cmd->tx_argv[1], &mask))
 	return;
 
@@ -713,8 +724,10 @@ CmdExpand(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    int windowMask, boxMask;
+    int windowMask, boxMask, d;
     Rect rootRect;
+    CellUse *rootBoxUse;
+    CellDef *rootBoxDef;
     int cmdExpandFunc();		/* Forward reference. */
 
     if (cmd->tx_argc > 2 || (cmd->tx_argc == 2 
@@ -731,20 +744,45 @@ CmdExpand(w, cmd)
 	return;
     }
     windowMask = ((DBWclientRec *) w->w_clientData)->dbw_bitmask;
+    d = DBLambda[1];
+    rootBoxUse = (CellUse *)w->w_surfaceID;
+    rootBoxDef = rootBoxUse->cu_def;
 
-    if (cmd->tx_argc == 2)
-	SelectExpand(windowMask);
-    else
+    /* Check for possible rescaling during the expansion, and adjust    */
+    /* the cursor box and window to restore the original view.          */
+
+    d = DBLambda[1];
+    do 
     {
-	(void) ToolGetBoxWindow(&rootRect, &boxMask);
-	if ((boxMask & windowMask) != windowMask)
+	if (d != DBLambda[1])
 	{
-	    TxError("The box isn't in the same window as the cursor.\n");
-	    return;
+            d = DBLambda[1] / d;
+            DBScalePoint(&rootRect.r_ll, d, 1);
+            DBScalePoint(&rootRect.r_ur, d, 1);
+            ToolMoveBox(TOOL_BL, &rootRect.r_ll, FALSE, rootBoxDef);
+            ToolMoveCorner(TOOL_TR, &rootRect.r_ur, FALSE, rootBoxDef);
+
+            /* Adjust all window viewing scales and positions and redraw */
+
+            WindScale(d, 1);
+	    TxPrintf("expand: rescaled by %d\n", d);
+	    d = DBLambda[1];
 	}
-	DBExpandAll(((CellUse *) w->w_surfaceID), &rootRect, windowMask,
-		TRUE, cmdExpandFunc, (ClientData) windowMask);
-    }
+
+	if (cmd->tx_argc == 2)
+	    SelectExpand(windowMask);
+	else
+	{
+	    (void) ToolGetBoxWindow(&rootRect, &boxMask);
+	    if ((boxMask & windowMask) != windowMask)
+	    {
+		TxError("The box isn't in the same window as the cursor.\n");
+		return;
+	    }
+	    DBExpandAll(rootBoxUse, &rootRect, windowMask,
+			TRUE, cmdExpandFunc, (ClientData) windowMask);
+	}
+    } while (d != DBLambda[1]);
 }
 
 /* This function is called for each cell whose expansion status changed.
@@ -948,7 +986,7 @@ CmdExtract(w, cmd)
 		TxError("No cell selected\n");
 		return;
 	    }
-	    ExtCell(selectedUse->cu_def, namep);
+	    ExtCell(selectedUse->cu_def, namep, FALSE);
 	    return;
 
 	case EXTPARENTS:
