@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdFI.c,v 1.4 2001/12/10 16:34:16 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/commands/CmdFI.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -35,22 +35,22 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdFI.c,v 1.4 200
 #endif
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "textio/textio.h"
-#include "macros/macros.h"
+#include "utils/macros.h"
 #include "drc/drc.h"
 #include "textio/txcommands.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "extract/extract.h"
 #include "utils/malloc.h"
 #include "select/select.h"
@@ -623,9 +623,10 @@ CmdFindLabel(w, cmd)
     }
     else
     {
-	/* Exact-match label search */
+	/* Exact-match label search (corrected by Nishit, 10/14/04) */
 
-	found = DBSrLabelLoc(EditCellUse, labname, cmdFindLabelFunc, (ClientData) 0);
+	found = DBSrLabelLoc(EditCellUse, labname, cmdFindLabelFunc,
+		(ClientData) &cmdFindLabelRect);
 	if (found) {
 	    if (cmdFindLabelRect.r_xbot == cmdFindLabelRect.r_xtop) 
 		cmdFindLabelRect.r_xtop++;
@@ -1008,11 +1009,67 @@ CmdGrid(w, cmd)
 	return;
     }
 
-    if ((cmd->tx_argc == 2) &&
-	    ((strcmp(cmd->tx_argv[1], "off") == 0)
-	    || (strcmp(cmd->tx_argv[1], "0") == 0)))
-	crec->dbw_flags &= ~DBW_GRID;
-    else if (cmd->tx_argc >= 2)
+    /* Parse various options with two arguments */
+
+    if (cmd->tx_argc == 2)
+    {
+	if (!strcmp(cmd->tx_argv[1], "box")
+		|| !strcmp(cmd->tx_argv[1], "what"))
+	{
+#ifdef MAGIC_WRAPPER
+	    char *boxvalues = (char *)Tcl_Alloc(50);
+	    sprintf(boxvalues, "%d %d %d %d",
+		crec->dbw_gridRect.r_xbot, crec->dbw_gridRect.r_ybot,
+		crec->dbw_gridRect.r_xtop, crec->dbw_gridRect.r_ytop);
+	    Tcl_SetResult(magicinterp, boxvalues, TCL_DYNAMIC);
+	
+#else
+	    TxPrintf("Grid unit box is (%d, %d) to (%d, %d)\n", 
+		crec->dbw_gridRect.r_xbot, crec->dbw_gridRect.r_ybot,
+		crec->dbw_gridRect.r_xtop, crec->dbw_gridRect.r_ytop);
+#endif
+	    return;
+	}
+	else if (!strcmp(cmd->tx_argv[1], "state"))
+	{
+#ifdef MAGIC_WRAPPER
+	    Tcl_SetObjResult(magicinterp,
+			Tcl_NewBooleanObj(crec->dbw_flags & DBW_GRID));
+#else
+	    TxPrintf("Grid is %s\n", 
+			(crec->dbw_flags & DBW_GRID) ? "on" : "off");
+#endif
+	    return;
+	}
+	else if (!strcmp(cmd->tx_argv[1], "help"))
+	{
+	    TxPrintf("Usage: grid [xSpacing [ySpacing [xOrig yOrig]]]]\n");
+	    TxPrintf("or     grid <option>\n");
+	    TxPrintf("where <option> is one of: on, off, state, box, or help.\n");
+	    return;
+	}
+	else if (!strcmp(cmd->tx_argv[1], "off")
+		|| !strcmp(cmd->tx_argv[1], "0"))
+	{
+	    if (crec->dbw_flags & DBW_GRID)
+	    {
+		crec->dbw_flags &= ~DBW_GRID;
+		WindAreaChanged(w, (Rect *) NULL);
+	    }
+	    return;
+	}
+	else if (!strcmp(cmd->tx_argv[1], "on"))
+	{
+	    if (!(crec->dbw_flags & DBW_GRID))
+	    {
+		crec->dbw_flags |= DBW_GRID;
+		WindAreaChanged(w, (Rect *) NULL);
+	    }
+	    return;
+	}
+    }
+
+    if (cmd->tx_argc >= 2)
     {
 	xSpacing = cmdParseCoord(w, cmd->tx_argv[1], TRUE, TRUE);
 	if (xSpacing <= 0)
@@ -1192,7 +1249,6 @@ cmdIdFunc(selUse, use, transform, newId)
     return 1;
 }
 
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1471,6 +1527,7 @@ flatCopyAllLabels(scx, lab, tpath, targetUse)
     int targetPos;
     CellDef *def;
     char	labelname[1024];
+    char *n, *f, c;
 
     def = targetUse->cu_def;
     if (!GEO_LABEL_IN_AREA(&lab->lab_rect, &(scx->scx_area))) return 0;
@@ -1481,18 +1538,28 @@ flatCopyAllLabels(scx, lab, tpath, targetUse)
      * in deciding on duplicates:  if text and position match, it's a
      * duplicate.
      */
+    /* 9/12/05---The utility of eliminating duplicate labels has been
+     * put into question by observing the O(n^2) behavior causing
+     * significant flattening times for layouts with many labels.
+     * Label text will not be duplicated anyway, because each label
+     * contains the heirarchical names.
+     */
+
+    /* (void) DBEraseLabelsByContent(def, &labTargetRect, targetPos,
+        -1, lab->lab_text); */
+
+    /* (Added 9/10/04) Make sure that the target flattened label is
+     * not a port; possibly we should retain ports taken from the
+     * top level cell, but certainly not any others.
+     */
+
+    n = tpath->tp_next;
+    f = tpath->tp_first;
+    c = *n;
+    strcpy(n, lab->lab_text);
+    DBPutLabel(def, &labTargetRect, targetPos, f, lab->lab_type, 0);
+    *n = c;
 
-    (void) DBEraseLabelsByContent(def, &labTargetRect, targetPos,
-        -1, lab->lab_text);
-    {
-    	 char *n = tpath->tp_next;
-    	 char *f = tpath->tp_first;
-	 char c = *n;
-	 strcpy(n,lab->lab_text);
-         (void) DBPutLabel(def, &labTargetRect, targetPos,
-			f, lab->lab_type, lab->lab_flags);
-	 *n = c;
-    }
     return 0;
 }
 
@@ -1504,7 +1571,7 @@ flatCopyAllLabels(scx, lab, tpath, targetUse)
  * Implements flatten command
  *
  * Usage:
- *	flatten destname
+ *	flatten [-<option>] destname
  *
  * Results:
  *	creates new cell def with all the desired info.
@@ -1518,16 +1585,56 @@ CmdFlatten(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-     char		*destname = cmd->tx_argv[1];
+     int		rval, xMask;
+     bool		dolabels;
+     char		*destname;
      CellDef		*newdef;
      CellUse		*newuse;
      SearchContext	scx;
-     CellUse	*flatDestUse;
+     CellUse		*flatDestUse;
      
-    if (cmd->tx_argc != 2)
+    destname = cmd->tx_argv[cmd->tx_argc - 1];
+    xMask = CU_DESCEND_ALL;
+    dolabels = TRUE;
+
+    rval = 0;
+    if (cmd->tx_argc > 2)
     {
-     	  TxError("usage: flatten destcell\n");
-	  return;
+	int i;
+	for (i = 1; i < (cmd->tx_argc - 1); i++)
+	{
+	    if (strncmp(cmd->tx_argv[i], "-no", 3))
+	    {
+	        rval = -1;
+		break;
+	    }
+	    else if (strlen(cmd->tx_argv[i]) > 3)
+	    {
+		switch(cmd->tx_argv[1][3])
+		{
+		    case 'l':
+			dolabels = FALSE;
+			break;
+		    case 's':
+			xMask = CU_DESCEND_NO_SUBCKT;
+			break;
+		    case 'v':
+			xMask = CU_DESCEND_NO_VENDOR;
+			break;
+		    default:
+			TxError("options are: -nolabels, -nosubcircuits -novendor\n");
+			break;
+		}
+	    }
+	}
+    }
+    else if (cmd->tx_argc != 2)
+	rval = -1;
+
+    if (rval != 0)
+    {
+     	TxError("usage: flatten [-<option>...] destcell\n");
+	return;
     }
     /* create the new def */
     if (newdef = DBCellLookDef(destname))
@@ -1541,17 +1648,22 @@ CmdFlatten(w, cmd)
     newuse = DBCellNewUse(newdef, (char *) NULL);
     (void) StrDup(&(newuse->cu_id), "Flattened cell");
     DBSetTrans(newuse, &GeoIdentityTransform);
-    newuse->cu_expandMask = -1;
+    newuse->cu_expandMask = CU_DESCEND_SPECIAL;
     UndoDisable();
     flatDestUse = newuse;
     
     scx.scx_use  = EditCellUse;
     scx.scx_area = EditCellUse->cu_def->cd_bbox;
     scx.scx_trans = GeoIdentityTransform;
-    DBCellCopyAllPaint(&scx,&DBAllButSpaceAndDRCBits,0,flatDestUse);
-    FlatCopyAllLabels(&scx,&DBAllButSpaceAndDRCBits,0,flatDestUse);
-    /* DBCellEnum(EditCellUse->cu_def,flattenyank,&GeoIdentityTransform); */
+
+    DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, xMask, flatDestUse);
+    if (dolabels)
+	FlatCopyAllLabels(&scx, &DBAllTypeBits, xMask, flatDestUse);
+
+    if (xMask != CU_DESCEND_ALL)
+	DBCellCopyAllCells(&scx, xMask, flatDestUse, (Rect *)NULL);
     
     UndoEnable();
 }
 
+
