@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdLQ.c,v 1.11 2001/07/23 20:37:00 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/commands/CmdLQ.c,v 1.2 2005/05/13 19:31:58 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -28,7 +28,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdLQ.c,v 1.11 20
 #include <ctype.h>
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
+#include "utils/malloc.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
@@ -36,14 +37,19 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdLQ.c,v 1.11 20
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "textio/textio.h"
 #include "graphics/graphics.h"
 #include "drc/drc.h"
 #include "textio/txcommands.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "select/select.h"
+#include "netmenu/netmenu.h"
+
+/* Forward declarations */
+
+void CmdPaintEraseButton();
 
 
 /*
@@ -271,6 +277,14 @@ CmdLoad(w, cmd)
     {
 	if (CmdIllegalChars(cmd->tx_argv[1], "[],", "Cell name"))
 	    return;
+#ifdef MAGIC_WRAPPER
+	/* Names that have been list-braced in Tcl need to be modified */
+	if (cmd->tx_argv[1][0] == '{')
+	{
+	    cmd->tx_argv[1]++;
+	    *(cmd->tx_argv[1] + strlen(cmd->tx_argv[1]) - 1) = '\0';
+	}
+#endif
 	DBWloadWindow(w, cmd->tx_argv[1], ignoreTech, FALSE);
 
 	if ((n > 1) || (d > 1))
@@ -320,7 +334,6 @@ keepGoing(use, clientdata)
     return 0;	/* keep the search going */
 }
 
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -503,7 +516,7 @@ moveToPoint:
  * Paint the specified layers underneath the box in EditCellUse->cu_def.
  *
  * Usage:
- *	paint [layers]
+ *	paint <layers> | cursor
  *
  * Results:
  *	None.
@@ -531,14 +544,19 @@ CmdPaint(w, cmd)
 
     if (cmd->tx_argc != 2)
     {
-	TxError("Usage: %s layers\n", cmd->tx_argv[0]);
+	TxError("Usage: %s <layers> | cursor\n", cmd->tx_argv[0]);
 	return;
     }
 
-    if (!ToolGetEditBox(&editRect)) return;
-
-    if (!CmdParseLayers(cmd->tx_argv[1], &mask))
+    if (!strncmp(cmd->tx_argv[1], "cursor", 6))
+    {
+	CmdPaintEraseButton(w, &cmd->tx_p, TRUE);
 	return;
+    }
+    else if (!CmdParseLayers(cmd->tx_argv[1], &mask))
+	return;
+
+    if (!ToolGetEditBox(&editRect)) return;
 
     if (TTMaskHasType(&mask, L_LABEL))
     {
@@ -563,10 +581,10 @@ CmdPaint(w, cmd)
 /*
  * ----------------------------------------------------------------------------
  *
- * CmdPaintButton --
+ * CmdPaintEraseButton --
  *
- * This is a first crack at an implementation of the "paint" button.
  * Paint the specified layers underneath the box in EditCellUse->cu_def.
+ * Implements the traditional "box tool" Button2 function as a command.
  *
  * Results:
  *	None.
@@ -578,9 +596,10 @@ CmdPaint(w, cmd)
  */
 
 void
-CmdPaintButton(w, butPoint)
+CmdPaintEraseButton(w, butPoint, isPaint)
     MagWindow *w;
     Point *butPoint;	/* Screen location at which button was raised */
+    bool isPaint;	/* True for paint, False for erase.	*/
 {
     Rect rootRect, editRect;
     TileTypeBitMask mask;
@@ -619,9 +638,12 @@ CmdPaintButton(w, butPoint)
 	DBEraseMask(EditCellUse->cu_def, &editRect, &crec->dbw_visibleLayers);
 	(void) DBEraseLabel(EditCellUse->cu_def, &editRect, &mask);
     }
+    else if (isPaint)
+	DBPaintMask(EditCellUse->cu_def, &editRect, &mask);
     else
     {
-	DBPaintMask(EditCellUse->cu_def, &editRect, &mask);
+	DBEraseMask(EditCellUse->cu_def, &editRect, &mask);
+	DBEraseLabel(EditCellUse->cu_def, &editRect, &mask);
     }
     SelectClear();
     DBAdjustLabels(EditCellUse->cu_def, &editRect);
@@ -631,7 +653,6 @@ CmdPaintButton(w, butPoint)
     DBReComputeBbox(EditCellUse->cu_def);
     UndoNext();
 }
-
 
 /*
  * ----------------------------------------------------------------------------
@@ -765,6 +786,95 @@ usage:
     TxError("Usage: %s [search|cell|sys] [[+]path]\n", cmd->tx_argv[0]);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CmdPolygon --
+ *
+ * Implement the "polygon" command.  Draws a polygon based on the
+ * point pairs supplied.
+ *
+ * Usage:
+ *	polygon tiletype x1 y1 x2 y2 [x3 y3 ...] xn yn
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Paints into the database.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+CmdPolygon(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    TileType type;
+    int points, i, j, pNum;
+    Point *plist;
+    Rect r;
+    CellDef *def = (CellDef *)NULL;
+    PaintUndoInfo ui;
+
+    if (EditCellUse != NULL)
+	def = EditCellUse->cu_def;
+
+    if (def == NULL)
+    {
+	TxError("No cell being edited\n");
+	return;
+    }
+
+    if (cmd->tx_argc < 8)
+    {
+	TxError("Usage:  polygon tiletype x1 y1 x2 y2 [x3 y3 ...] xn yn\n");
+	return;
+    }
+
+    type = DBTechNoisyNameType(cmd->tx_argv[1]);
+    if (type < 0)
+	return;
+
+    if (cmd->tx_argc & 1)
+    {
+	TxError("Unpaired coordinate value\n");
+	return;
+    }
+    points = (cmd->tx_argc - 2) >> 1;
+    plist = (Point *)mallocMagic(points * sizeof(Point));
+    for (i = 0, j = 2; i < points; i++)
+    {
+	plist[i].p_x = cmdParseCoord(w, cmd->tx_argv[j++], FALSE, TRUE);
+	plist[i].p_y = cmdParseCoord(w, cmd->tx_argv[j++], FALSE, FALSE);
+    }
+
+    def->cd_flags |= CDMODIFIED | CDGETNEWSTAMP;
+    ui.pu_def = def;
+    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+    {
+	if (DBPaintOnPlane(type, pNum))
+	{
+	    ui.pu_pNum = pNum;
+	    PaintPolygon(plist, points, def->cd_planes[pNum],
+			DBStdPaintTbl(type, pNum), &ui);
+	}
+    }
+
+    /* Get the bounding box of the polygon and update the cell bbox */
+
+    r.r_xbot = r.r_xtop = plist[0].p_x;
+    r.r_ybot = r.r_ytop = plist[0].p_y;
+    for (i = 1; i < points; i++)
+	GeoIncludePoint(plist + i, &r);
+
+    DBWAreaChanged(def, &r, DBW_ALLWINDOWS, &DBAllButSpaceBits);
+    DBReComputeBbox(def);
+    DRCCheckThis(def, TT_CHECKPAINT, &r);
+    freeMagic(plist);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -895,6 +1005,9 @@ CmdPort(w, cmd)
 	return;
     }
 
+    option = Lookup(cmd->tx_argv[1], cmdPortOption);
+    if (option < 0 || option == PORT_HELP) goto portWrongNumArgs;
+
     /*
      * Check for unique label in box area
      */
@@ -938,10 +1051,9 @@ CmdPort(w, cmd)
     }
 
     posstart = 1;
-    option = Lookup(cmd->tx_argv[1], cmdPortOption);
     if (option >= 0)
     {
-	if ((option != PORT_HELP) && (option != PORT_MAKE))
+	if (option != PORT_MAKE)
 	{
 	    /* label "lab" must already be a port */
 	    if (!(lab->lab_flags & PORT_DIR_MASK))
@@ -1325,3 +1437,85 @@ printPropertiesFunc(name, value)
 
     return 0;	/* keep the search alive */
 }
+
+/* Added by NP 10/28/2004 */
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CmdNetlist --
+ *
+ * Usage:
+ *	netlist option
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Performs the functions previously handled by the button
+ *	handler mechanism, which has been replaced with a command-line
+ *	command plus macro method.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+#define NLIST_HELP	0
+#define NLIST_SELECT	1
+#define NLIST_JOIN	2
+#define NLIST_TERMINAL	3
+
+void
+CmdNetlist(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    int option;
+    char **msg, *lastargv;
+    Point cursor;
+    static char *cmdNetlistOption[] =
+    {	
+	"help           print this help information",
+	"select		select the net nearest the cursor",
+	"join           join current net and net containing terminal nearest the cursor",
+	"terminal	toggle the terminal nearest the cursor into/out of current net",
+	NULL
+    };
+
+    if (cmd->tx_argc < 2)
+    {
+	option = NLIST_HELP;
+    }
+    else
+    {
+	option = Lookup(cmd->tx_argv[1], cmdNetlistOption);
+	if (option < 0)
+	{
+	    TxError("\"%s\" isn't a valid netlist option.\n", cmd->tx_argv[1]);
+	    option = NLIST_HELP;
+	}
+    }
+
+    switch (option)
+    {
+	case NLIST_HELP:
+	    TxPrintf("Netlist commands have the form \":netlist option\",");
+	    TxPrintf(" where option is one of:\n");
+	    for (msg = &(cmdNetlistOption[0]); *msg != NULL; msg++)
+	    {
+		TxPrintf("    %s\n", *msg);
+	    }
+	    break;
+
+	case NLIST_SELECT:
+	    NMButtonLeft(w, cmd);
+	    break;
+
+	case NLIST_JOIN:
+	    NMButtonMiddle(w, cmd);
+	    break;
+
+	case NLIST_TERMINAL:
+	    NMButtonRight(w, cmd);
+	    break;
+    }
+}
