@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdRS.c,v 1.8 2001/03/29 19:50:05 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/commands/CmdRS.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -28,7 +28,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdRS.c,v 1.8 200
 #endif
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/stack.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
@@ -37,18 +37,18 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdRS.c,v 1.8 200
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "textio/textio.h"
 #include "graphics/graphics.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "drc/drc.h"
 #include "textio/txcommands.h"
 #include "utils/malloc.h"
-#include "netlist/netlist.h"
+#include "utils/netlist.h"
 #include "netmenu/netmenu.h"
 #include "select/select.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "sim/sim.h"
 
 extern void DisplayWindow();
@@ -758,7 +758,6 @@ CmdSelect(w, cmd)
     SearchContext scx;
     DBWclientRec *crec;
     MagWindow *window;
-    bool samePlace;
 
     /* The two tables below define the allowable selection options, and
      * also the message printed out by ":select help" to describe the
@@ -767,17 +766,20 @@ CmdSelect(w, cmd)
      * second table, due to a help message for ":select" with no arguments.
      */
 
-#define AREA	 0
-#define VISIBLE	 1
-#define CELL	 2
-#define SCLEAR	 3
-#define HELP	 4
-#define SAVE	 5
-#define SBOX	 6
-#define CHUNK	 7
-#define REGION   8
-#define NET	 9
-#define DEFAULT	10 
+#define SEL_AREA	 0
+#define SEL_VISIBLE	 1
+#define SEL_CELL	 2
+#define SEL_CLEAR	 3
+#define SEL_HELP	 4
+#define SEL_KEEP	 5
+#define SEL_MOVE	 6
+#define SEL_PICK	 7
+#define SEL_SAVE	 8
+#define SEL_BOX		 9
+#define SEL_CHUNK	10
+#define SEL_REGION	11
+#define SEL_NET		12
+#define SEL_DEFAULT	13
 
     static char *cmdSelectOption[] =
     {
@@ -786,6 +788,9 @@ CmdSelect(w, cmd)
 	"cell",
 	"clear",
 	"help",
+	"keep",
+	"move",
+	"pick",
 	"save",
 	"box",
 	"chunk",
@@ -795,7 +800,8 @@ CmdSelect(w, cmd)
     };
     static char *cmdSelectMsg[] =
     {
-	"[more | less] [layers]          [de]select paint chunk/region/net under\n"
+	"[more | less | nocycle] [layers]\n"
+        "                                [de]select paint chunk/region/net under\n"
  	"                                cursor, or [de]select subcell if cursor\n"
 	"	                         over space",
 	"[more | less] area [layers]     [de]select all info under box in layers",
@@ -803,9 +809,12 @@ CmdSelect(w, cmd)
 	"[more | less | top] cell [name] [de]select cell under cursor, or \"name\"",
 	"clear                           clear selection",
 	"help                            print this message",
+	"keep                            copy the selection to the layout",
+	"move                            move selection to a location in the layout",
+	"pick                            delete selection from layout",
 	"save file                       save selection on disk in file.mag",
 	"[more | less] box | chunk | region | net [layers]\n"
-	"				 [de]select chunk/region/net specified by"
+	"				 [de]select chunk/region/net specified by\n"
 	"				 the lower left corner of the current box",
 	NULL
     };
@@ -842,11 +851,13 @@ CmdSelect(w, cmd)
     char path[200], *printPath, **msg, **optionArgs;
     TerminalPath tpath;
     CellUse *use;
+    CellDef *rootBoxDef;
     Transform trans, rootTrans, tmp1;
-    Point p;
-    Rect r;
-    bool more, less, layerspec;
+    Point p, rootPoint;
+    Rect r, selarea;
     int option;
+    bool layerspec;
+    bool more = FALSE, less = FALSE, samePlace = TRUE;
 #ifdef MAGIC_WRAPPER
     char *tclstr;
 #endif
@@ -864,58 +875,63 @@ CmdSelect(w, cmd)
     }
 
     /* See if "more" was given.  If so, just strip off the "more" from
-     * the argument list and set the "more" flag.
+     * the argument list and set the "more" flag.  Similarly for options
+     * "less", "nocycle", "top", and "cell".
      */
     
-    if ((cmd->tx_argc >= 2)
-	    && (strncmp(cmd->tx_argv[1], "more", strlen(cmd->tx_argv[1]))
-	    == 0))
+    if (cmd->tx_argc >= 2)
     {
-	more = TRUE;
-	less = FALSE;
-	optionArgs = &cmd->tx_argv[2];
-	cmd->tx_argc--;
-    }
-    else if ((cmd->tx_argc >= 2)
-	     && (strncmp(cmd->tx_argv[1], "less", strlen(cmd->tx_argv[1])) == 0))
-    {
-	more = FALSE;
-	less = TRUE;
-	optionArgs = &cmd->tx_argv[2];
-	cmd->tx_argc--;
-    }
-    else if ((cmd->tx_argc >= 2)
-	    && (strncmp(cmd->tx_argv[1], "top", strlen(cmd->tx_argv[1])) == 0)
-	    && (strncmp(cmd->tx_argv[2], "cell", strlen(cmd->tx_argv[2])) == 0))
-    {
-	more = FALSE;
-	less = FALSE;
-	optionArgs = &cmd->tx_argv[2];
-    }
-    else
-    {
-	more = FALSE;
-	less = FALSE;
-	if (cmd->tx_argc >= 2)
-	    optionArgs = &cmd->tx_argv[1];
+	int arg1len = strlen(cmd->tx_argv[1]);
+
+	optionArgs = &cmd->tx_argv[1];
+	if (!strncmp(cmd->tx_argv[1], "more", arg1len))
+	{
+	    more = TRUE;
+	    less = FALSE;
+	    optionArgs = &cmd->tx_argv[2];
+	    cmd->tx_argc--;
+	}
+	else if (!strncmp(cmd->tx_argv[1], "less", arg1len))
+	{
+	    more = FALSE;
+	    less = TRUE;
+	    optionArgs = &cmd->tx_argv[2];
+	    cmd->tx_argc--;
+	}
+	else if (!strncmp(cmd->tx_argv[1], "nocycle", arg1len))
+	{
+	    samePlace = FALSE;
+	    more = FALSE;
+	    less = FALSE;
+	    type = TT_SELECTBASE - 1;	   /* avoid cycling between types */
+	    optionArgs = &cmd->tx_argv[2];
+	    cmd->tx_argc--;
+	}
+
+	else if (!strncmp(cmd->tx_argv[1], "top", arg1len)) 
+	{
+	    if ((cmd->tx_argc >= 3) && !strncmp(cmd->tx_argv[2],
+			"cell", strlen(cmd->tx_argv[2])))
+		optionArgs = &cmd->tx_argv[2];
+	}
     }
 
     /* Check the option for validity. */
 
     if (cmd->tx_argc == 1)
-	option = DEFAULT;
+	option = SEL_DEFAULT;
     else
     {
 	option = Lookup(optionArgs[0], cmdSelectOption);
 	if (option < 0 && cmd->tx_argc != 2)
 	{
 	    TxError("\"%s\" isn't a valid select option.\n", cmd->tx_argv[1]);
-	    option = HELP;
+	    option = SEL_HELP;
 	    cmd->tx_argc = 2;
 	}
 	else if (option < 0)
 	{
-	    option = DEFAULT;
+	    option = SEL_DEFAULT;
 	    if (more || less)
 		optionArgs = &cmd->tx_argv[1];
 	    else
@@ -935,7 +951,7 @@ CmdSelect(w, cmd)
 	 *--------------------------------------------------------------------
 	 */
 
-	case AREA:
+	case SEL_AREA:
 	    if (cmd->tx_argc > 3)
 	    {
 		usageError:
@@ -955,7 +971,7 @@ CmdSelect(w, cmd)
 	 *--------------------------------------------------------------------
 	 */
 
-	case VISIBLE:
+	case SEL_VISIBLE:
 	    if (cmd->tx_argc > 3) goto usageError;
 	    if (!(more || less)) SelectClear();
 	    if (cmd->tx_argc == 3)
@@ -968,7 +984,7 @@ CmdSelect(w, cmd)
 	 *--------------------------------------------------------------------
 	 */
 
-	case SCLEAR:
+	case SEL_CLEAR:
 	    if ((more) || (less) || (cmd->tx_argc > 2)) goto usageError;
 	    SelectClear();
 	    return;
@@ -978,18 +994,66 @@ CmdSelect(w, cmd)
 	 *--------------------------------------------------------------------
 	 */
 
-	case HELP:
+	case SEL_HELP:
 	    TxPrintf("Selection commands are:\n");
 	    for (msg = &(cmdSelectMsg[0]); *msg != NULL; msg++)
 		TxPrintf("    select %s\n", *msg);
 	    return;
 
+	/*--------------------------------------------------------------------
+	 * Make a copy of the selection at its present loction but do not
+	 * clear the selection.
+	 *--------------------------------------------------------------------
+	 */
+
+	 case SEL_KEEP:
+	    if ((more) || (less) || (cmd->tx_argc > 2)) goto usageError;
+	    SelectAndCopy1();
+	    GeoTransRect(&SelectUse->cu_transform, &SelectDef->cd_bbox, &selarea);
+	    DBWHLRedraw(SelectRootDef, &selarea, FALSE);
+	    return;
+
+	/*--------------------------------------------------------------------
+	 * Move the selection relative to the cell def
+	 *--------------------------------------------------------------------
+	 */
+
+	 case SEL_MOVE:
+	    if ((more) || (less) || (cmd->tx_argc != 4)) goto usageError;
+
+	    p.p_x = cmdParseCoord(w, cmd->tx_argv[2], FALSE, TRUE);
+	    p.p_y = cmdParseCoord(w, cmd->tx_argv[3], FALSE, FALSE);
+
+	    /* Erase first, then recompute the transform */
+	    GeoTransRect(&SelectUse->cu_transform, &SelectDef->cd_bbox, &selarea);
+	    DBWHLRedraw(SelectRootDef, &selarea, TRUE);
+
+	    GeoTransTranslate(p.p_x, p.p_y,
+			&GeoIdentityTransform, &SelectUse->cu_transform);
+
+	    GeoTransRect(&SelectUse->cu_transform, &SelectDef->cd_bbox, &selarea);
+	    DBWHLRedraw(SelectRootDef, &selarea, FALSE);
+	    return;
+
+	/*--------------------------------------------------------------------
+	 * Delete the selection from the layout, but don't clear the selection
+	 * cell.  This allows the selection to be dragged around independently
+	 * of the layout.
+	 *--------------------------------------------------------------------
+	 */
+
+	 case SEL_PICK:
+	    if ((more) || (less) || (cmd->tx_argc > 2)) goto usageError;
+	    SelectDelete("picked", FALSE);
+	    DBWHLRedraw(SelectRootDef, &selarea, FALSE);
+	    return;
+
 	/*--------------------------------------------------------------------
 	 * Save the selection as a new Magic cell on disk.
 	 *--------------------------------------------------------------------
 	 */
 
-	 case SAVE:
+	 case SEL_SAVE:
 	    if (cmd->tx_argc != 3) goto usageError;
 
 	    /* Be sure to paint DRC check information into the cell before
@@ -1007,7 +1071,7 @@ CmdSelect(w, cmd)
 	    cmdSaveCell(SelectDef, cmd->tx_argv[2], FALSE, FALSE);
 	    return;
 	
-	case SBOX: case CHUNK: case REGION: case NET:
+	case SEL_BOX: case SEL_CHUNK: case SEL_REGION: case SEL_NET:
 	    if (cmd->tx_argc > 3) goto usageError;
 	    if (cmd->tx_argc == 3)
 		layerspec = TRUE;
@@ -1019,26 +1083,25 @@ CmdSelect(w, cmd)
 	 * The default case (no args):  see what's under the cursor.  Select
 	 * paint if there is any, else select a cell.  In both cases,
 	 * multiple clicks cycle through larger and larger selections.  The
-	 * CELL option also comes here (to share initialization code) but
+	 * SEL_CELL option also comes here (to share initialization code) but
 	 * quickly branches away.
 	 *--------------------------------------------------------------------
 	 */
 
-	case DEFAULT:
+	case SEL_DEFAULT:
 	    if (cmd->tx_argc > 2) goto usageError;
 	    if (cmd->tx_argc == 2)
 		layerspec = TRUE;
 	    else
 		layerspec = FALSE;
 	    goto Okay;
-	case CELL:
+	case SEL_CELL:
 	    layerspec = FALSE;
 Okay:
 	    if (!(more || less)) SelectClear();
-	    if (option == SBOX || option == CHUNK || option == REGION
-			|| option == NET)
+	    if (option == SEL_BOX || option == SEL_CHUNK || option == SEL_REGION
+			|| option == SEL_NET)
 	    {
-		CellDef *rootBoxDef;
 		int windowMask, xMask;
 
 		if (!ToolGetBox (&rootBoxDef, NULL)) {
@@ -1078,10 +1141,10 @@ Okay:
 	     * another selection comand.
 	     */
 	
-	    if (GEO_ENCLOSE(&cmd->tx_p, &lastArea)
-		    && (lastCommand+1 == TxCommandNumber))
-		samePlace = TRUE;
-	    else samePlace = FALSE;
+	    if (!GEO_ENCLOSE(&cmd->tx_p, &lastArea)
+		    || ((lastCommand + 1) != TxCommandNumber))
+		samePlace = FALSE;
+
 	    lastArea.r_xbot = cmd->tx_p.p_x - MARGIN;
 	    lastArea.r_ybot = cmd->tx_p.p_y - MARGIN;
 	    lastArea.r_xtop = cmd->tx_p.p_x + MARGIN;
@@ -1094,7 +1157,7 @@ Okay:
 	     * then a whole net.
 	     */
 
-	    if (!TTMaskIsZero(&mask) && (option != CELL))
+	    if (!TTMaskIsZero(&mask) && (option != SEL_CELL))
 	    {
 		if (layerspec == TRUE)
 		{
@@ -1126,7 +1189,7 @@ Okay:
 
 		/* Set connectivity searching level */
 
-		if (option == CHUNK || option == REGION || option == NET)
+		if (option == SEL_CHUNK || option == SEL_REGION || option == SEL_NET)
 		{
 		    samePlace = TRUE;
 		    level = option;
@@ -1136,22 +1199,22 @@ Okay:
 		    if (samePlace && lessCycle == less)
 		    {
 			level += 1;
-			if (level > NET) level = CHUNK;
+			if (level > SEL_NET) level = SEL_CHUNK;
 		    }
-		    else level = CHUNK;
+		    else level = SEL_CHUNK;
 
-		    if (level != CHUNK && !TTMaskHasType (&mask, type)) {
+		    if (level != SEL_CHUNK && !TTMaskHasType (&mask, type)) {
 			/* User specified a new mask, and the current tile
 			 * type being expanded is not in the set of types
 			 * which the user wants us to use => reset level
 			 */
-			level = CHUNK;
+			level = SEL_CHUNK;
 		    }
 		}
 
 		lessCycle = less;
 
-		if (level == CHUNK)
+		if (level == SEL_CHUNK)
 		{
 		    /* Pick a tile type to use for selection.  If there are
 		     * several different types under the cursor, pick one of
@@ -1171,7 +1234,7 @@ Okay:
 		    if (!less)
 		      DBWSetBox(scx.scx_use->cu_def, &chunkSelection);
 		}
-		if (level == REGION)
+		if (level == SEL_REGION)
 		{
 		    /* If a region has the same size as the preceding chunk,
 		     * then we haven't added anything to the selection, so
@@ -1182,16 +1245,16 @@ Okay:
 
 		    SelectRegion(&scx, type, crec->dbw_bitmask, &area, less);
 		    if (GEO_SURROUND(&chunkSelection, &area))
-			level = NET;
+			level = SEL_NET;
 		}
-		if (level == NET)
+		if (level == SEL_NET)
 		    SelectNet(&scx, type, crec->dbw_bitmask, (Rect *) NULL, less);
 		return;
 	    }
 
 	/*--------------------------------------------------------------------
-	 * We get here either if the CELL option is requested, or under
-	 * the DEFAULT case where there's no paint under the mouse.  In
+	 * We get here either if the SEL_CELL option is requested, or under
+	 * the SEL_DEFAULT case where there's no paint under the mouse.  In
 	 * this case, select a subcell.
 	 *--------------------------------------------------------------------
 	 */
@@ -1664,7 +1727,7 @@ CmdSplit(w, cmd)
 {
     Rect editRect;
     TileTypeBitMask mask1, mask2, *cmask;
-    TileType t, dir, side;
+    TileType t, dir, side, diag;
     int pNum, direction;
     PaintUndoInfo ui;
 
@@ -1709,6 +1772,7 @@ CmdSplit(w, cmd)
 	for (cmask = &mask1; cmask != NULL; cmask = ((cmask == &mask1) ? &mask2 : NULL))
 	{
 	    if (cmask == &mask2) side = (side) ? 0 : TT_SIDE;
+	    diag = DBTransformDiagonal(TT_DIAGONAL | dir | side, &RootToEditTransform);
 
 	    if (TTMaskHasType(cmask, t))
 	    {
@@ -1719,8 +1783,7 @@ CmdSplit(w, cmd)
 		    {
 			ui.pu_pNum = pNum;
 			DBNMPaintPlane(EditCellUse->cu_def->cd_planes[pNum],
-				TT_DIAGONAL | dir | side,
-				&editRect, DBStdPaintTbl(t, pNum), &ui);
+				diag, &editRect, DBStdPaintTbl(t, pNum), &ui);
 		    }
 	    }
 	}
@@ -1756,7 +1819,7 @@ CmdSplitErase(w, cmd)
 {
     Rect editRect;
     TileTypeBitMask mask;
-    TileType t, dir, side;
+    TileType t, dir, side, diag;
     int pNum, direction;
     PaintUndoInfo ui;
 
@@ -1795,6 +1858,8 @@ CmdSplitErase(w, cmd)
     for (t = TT_SPACE + 1; t < DBNumTypes; t++)
     {
 	side = (direction & 0x2) ? 0 : TT_SIDE;
+	diag = DBTransformDiagonal(TT_DIAGONAL | dir | side, &RootToEditTransform);
+
 	if (TTMaskHasType(&mask, t))
 	{
 	    EditCellUse->cu_def->cd_flags |= CDMODIFIED|CDGETNEWSTAMP;
@@ -1804,8 +1869,7 @@ CmdSplitErase(w, cmd)
 		{
 		    ui.pu_pNum = pNum;
 		    DBNMPaintPlane(EditCellUse->cu_def->cd_planes[pNum],
-				TT_DIAGONAL | dir | side,
-				&editRect, DBStdEraseTbl(t, pNum), &ui);
+				diag, &editRect, DBStdEraseTbl(t, pNum), &ui);
 		}
 	}
     }
