@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdSubrs.c,v 1.5 2001/10/19 13:41:06 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/commands/CmdSubrs.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -26,7 +26,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdSubrs.c,v 1.5
 #include <string.h>
 #include <ctype.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
@@ -34,17 +34,21 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdSubrs.c,v 1.5
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "textio/textio.h"
 #include "cif/cif.h"
 #include "drc/drc.h"
 #include "textio/txcommands.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
+#include "utils/macros.h"
+#include "sim/sim.h"
+#include "select/select.h"
 #ifdef SCHEME_INTERPRETER
 #include "lisp/lisp.h"
 #endif
-    /* Forward declarations */
+
+/* Forward declarations */
 
 extern char *cmdCheckNewName();
 extern int cmdSaveWindSet();
@@ -422,7 +426,7 @@ CmdParseLayers(s, mask)
 	}
 	else
 	{
-	    type = DBTechNameType(name);
+	    type = DBTechNameTypes(name, &newmask);
 	    switch (type)
 	    {
 		case -2:
@@ -439,7 +443,6 @@ CmdParseLayers(s, mask)
 		    TxError("Ambiguous layer: %s\n", name);
 		    goto printTypes;
 	    }
-	    TTMaskSetType(&newmask, type);
 	}
 
 	if (adding)
@@ -1111,3 +1114,4 @@ CmdIllegalChars(string, illegal, msg)
     }
     return FALSE;
 }
+
