@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdTZ.c,v 1.7 2001/03/29 19:50:05 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/commands/CmdTZ.c,v 1.2 2005/05/31 21:41:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -27,7 +27,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdTZ.c,v 1.7 200
 #include <math.h>
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
+#include "utils/malloc.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
@@ -35,18 +36,19 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdTZ.c,v 1.7 200
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "textio/textio.h"
 #include "textio/txcommands.h"
-#include "signals/signals.h"
-#include "undo/undo.h"
+#include "utils/signals.h"
+#include "utils/undo.h"
 #include "select/select.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "wiring/wiring.h"
-#include "netlist/netlist.h"
+#include "utils/netlist.h"
 #include "netmenu/netmenu.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
+#include "drc/drc.h"
 
 #ifdef	LLNL
 #include "yacr.h"
@@ -265,7 +267,8 @@ CmdTech(w, cmd)
 		    /* Disallow the "tech load" command from overriding */
 		    /* a technology specified on the command line.	*/
 		    /* This is used when "tech load" sets up a specific	*/
-		    /* project technology from a .magic startup script.	*/
+		    /* project technology from a .magicrc startup	*/
+		    /* script.						*/
 
 		    if (TechOverridesDefault)
 			return;
@@ -609,7 +612,7 @@ CmdUpsidedown(w, cmd)
  * 	Print out information about what's selected.
  *
  * Usage:
- *	what
+ *	what [-list]
  *
  * Results:
  *	None.
@@ -617,6 +620,8 @@ CmdUpsidedown(w, cmd)
  * Side effects:
  *	Information gets printed to identify the kinds of paint, plus
  *	labels and subcells, that are selected.
+ *	In the TCL version, the "-list" option puts the result in a
+ *	nested TCL list.
  *
  * ----------------------------------------------------------------------------
  */
@@ -635,17 +640,43 @@ CmdWhat(w, cmd)
     MagWindow *w;		/* Window in which command was invoked. */
     TxCommand *cmd;		/* Information about the command. */
 {
-    int i;
+    int i, locargc;
     bool foundAny;
+    bool doList = FALSE;
     TileTypeBitMask layers;
+
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj, *paintobj, *labelobj, *cellobj;
+    extern int cmdWhatCellListFunc();
+#endif
+
     extern int cmdWhatPaintFunc(), cmdWhatLabelFunc(), cmdWhatCellFunc();
     extern int cmdWhatLabelPreFunc(), orderLabelFunc();
 
-    if (cmd->tx_argc > 1)
+    locargc = cmd->tx_argc;
+    
+#ifdef MAGIC_WRAPPER
+    if ((locargc == 2) && !strncmp(cmd->tx_argv[locargc - 1], "-list", 5))
+    {
+	doList = TRUE;
+	locargc--;
+	lobj = Tcl_NewListObj(0, NULL);
+	paintobj = Tcl_NewListObj(0, NULL);
+	labelobj = Tcl_NewListObj(0, NULL);
+	cellobj = Tcl_NewListObj(0, NULL);
+    }	
+    if (locargc > 1)
+    {
+	TxError("Usage: what [-list]\n");
+	return;
+    }
+#else
+    if (locargc > 1)
     {
 	TxError("Usage: what\n");
 	return;
     }
+#endif
 
     /* Find all the selected paint and print out the layer names. */
 
@@ -669,12 +700,26 @@ CmdWhat(w, cmd)
 
     if (!TTMaskIsZero(&layers))
     {
-	TxPrintf("Selected mask layers:\n");
-	for (i = TT_SELECTBASE; i < DBNumUserLayers; i++)
+#ifdef MAGIC_WRAPPER
+	if (doList)
 	{
-	    if (TTMaskHasType(&layers, i))
-		TxPrintf("    %s\n", DBTypeLongName(i));
+	    for (i = TT_SELECTBASE; i < DBNumUserLayers; i++)
+		if (TTMaskHasType(&layers, i))
+		    Tcl_ListObjAppendElement(magicinterp, paintobj,
+				Tcl_NewStringObj(DBTypeLongName(i), -1));
 	}
+	else
+	{
+#endif
+	    TxPrintf("Selected mask layers:\n");
+	    for (i = TT_SELECTBASE; i < DBNumUserLayers; i++)
+	    {
+		if (TTMaskHasType(&layers, i))
+		    TxPrintf("    %s\n", DBTypeLongName(i));
+	    }
+#ifdef MAGIC_WRAPPER
+	}
+#endif
     }
 
     /* Enumerate all of the selected labels. */
@@ -688,21 +733,60 @@ CmdWhat(w, cmd)
     if (labelBlockTop)
     {
 	qsort(labelBlockTop, labelEntryCount, sizeof(LabelStore), orderLabelFunc);
-    		/* now print them out */
-	for (labelEntry = labelBlockTop; labelEntryCount-- > 0; labelEntry++)
-            i = cmdWhatLabelFunc(labelEntry, &foundAny);
-	if (i > 1)
+
+#ifdef MAGIC_WRAPPER
+	if (doList)
+	{
+	    Tcl_Obj *newtriple;
+	    for (labelEntry = labelBlockTop; labelEntryCount-- > 0; labelEntry++)
+	    {
+		newtriple = Tcl_NewListObj(0, NULL);
+		Tcl_ListObjAppendElement(magicinterp, newtriple,
+			Tcl_NewStringObj(labelEntry->lab_text, -1));
+		Tcl_ListObjAppendElement(magicinterp, newtriple,
+			Tcl_NewStringObj(DBTypeLongName(labelEntry->lab_type), -1));
+		Tcl_ListObjAppendElement(magicinterp, newtriple,
+			Tcl_NewStringObj(labelEntry->cell_name, -1));
+		Tcl_ListObjAppendElement(magicinterp, labelobj, newtriple);
+	    }
+	}
+	else
+	{
+#endif
+    	    /* now print them out */
+	    for (labelEntry = labelBlockTop; labelEntryCount-- > 0; labelEntry++)
+        	i = cmdWhatLabelFunc(labelEntry, &foundAny);
+	    if (i > 1)
 		TxPrintf(" (%i instances)", i);
-        TxPrintf("\n");
+	    TxPrintf("\n");
+
+#ifdef MAGIC_WRAPPER
+	}
+#endif
 	freeMagic(labelBlockTop);
     }
-    
 
     /* Enumerate all of the selected subcells. */
 
     foundAny = FALSE;
-    (void) SelEnumCells(FALSE, (bool *) NULL, (SearchContext *) NULL,
-	    cmdWhatCellFunc, (ClientData) &foundAny);
+#ifdef MAGIC_WRAPPER
+    if (doList)
+	SelEnumCells(FALSE, (bool *) NULL, (SearchContext *) NULL,
+		cmdWhatCellListFunc, (ClientData) cellobj);
+    else
+#endif
+	SelEnumCells(FALSE, (bool *) NULL, (SearchContext *) NULL,
+		cmdWhatCellFunc, (ClientData) &foundAny);
+
+#ifdef MAGIC_WRAPPER
+    if (doList)
+    {
+	Tcl_ListObjAppendElement(magicinterp, lobj, paintobj);
+	Tcl_ListObjAppendElement(magicinterp, lobj, labelobj);
+	Tcl_ListObjAppendElement(magicinterp, lobj, cellobj);
+ 	Tcl_SetObjResult(magicinterp, lobj);
+    }
+#endif
 }
 
 /* Search function invoked for each paint tile in the selection:
@@ -845,7 +929,31 @@ cmdWhatCellFunc(selUse, realUse, transform, foundAny)
 	    realUse->cu_def->cd_name);
     return 0;
 }
-
+
+#ifdef MAGIC_WRAPPER
+
+/* Same search function as above, but appends use names to a Tcl list */
+
+int
+cmdWhatCellListFunc(selUse, realUse, transform, newobj)
+    CellUse *selUse;		/* Not used. */
+    CellUse *realUse;		/* Selected cell use. */
+    Transform *transform;	/* Not used. */
+    Tcl_Obj *newobj;		/* Tcl list object holding use names */
+{
+    Tcl_Obj *tuple;
+
+    tuple = Tcl_NewListObj(0, NULL);
+    Tcl_ListObjAppendElement(magicinterp, tuple,
+		Tcl_NewStringObj(realUse->cu_id, -1));
+    Tcl_ListObjAppendElement(magicinterp, tuple,
+		Tcl_NewStringObj(realUse->cu_def->cd_name, -1));
+    Tcl_ListObjAppendElement(magicinterp, newobj, tuple);
+    return 0;
+}
+
+#endif
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -866,13 +974,16 @@ cmdWhatCellFunc(selUse, realUse, transform, foundAny)
  * ----------------------------------------------------------------------------
  */
 
-#define HELP		0
-#define HORIZONTAL	1
-#define LEG		2
-#define SWITCH		3
-#define TYPE		4
-#define VERTICAL	5
-#define SEGMENT		6
+#define DECREMENT	0
+#define HELP		1
+#define HORIZONTAL	2
+#define INCREMENT	3
+#define LEG		4
+#define SHOW		5
+#define SWITCH		6
+#define TYPE		7
+#define VERTICAL	8
+#define SEGMENT		9
 
 void
 CmdWire(w, cmd)
@@ -886,9 +997,12 @@ CmdWire(w, cmd)
 
     static char *cmdWireOption[] =
     {	
+	"decrement layer|width 	decrement the wire layer or width",
 	"help                   print this help information",
 	"horizontal             add a new horizontal wire leg",
+	"increment layer|width 	increment the wire layer or width",
 	"leg                    add a new horizontal or vertical leg",
+	"show			show next wire segment as a selection",
 	"switch [layer width]   place contact and switch layers",
 	"type [layer width]     select the type and size of wires",
 	"vertical               add a new vertical wire leg",
@@ -916,6 +1030,77 @@ CmdWire(w, cmd)
 
     switch (option)
     {
+	case DECREMENT:
+	    if (cmd->tx_argc != 3)
+		goto badargs;
+	    if (!strcmp(cmd->tx_argv[2], "type"))
+	    {
+		Contact *contact;
+		type = TT_SPACE;
+		for (contact = WireContacts; contact != NULL;
+			contact = contact->con_next)
+		{
+		    if (contact->con_layer2 == WireType)
+		    {
+			if (type != TT_SPACE)
+			{
+			    TxError("Ambiguous directive---multiple routing types\n");
+			    break;
+			}
+			else
+			    type = contact->con_layer1;
+		    }
+		}
+		if (type == TT_SPACE)
+		    TxError("No routing layer defined\n");
+		else
+		{
+		    width = DRCGetDefaultLayerWidth(type);
+		    WireAddContact(type, width);
+		}
+	    }
+	    else if (!strcmp(cmd->tx_argv[2], "width"))
+		WirePickType(WireType, WireWidth - 1);
+	    else
+		goto badargs;
+	    return;
+
+	case INCREMENT:
+	    if (cmd->tx_argc != 3)
+		goto badargs;
+	    if (!strcmp(cmd->tx_argv[2], "type"))
+	    {
+		Contact *contact;
+		type = TT_SPACE;
+		for (contact = WireContacts; contact != NULL;
+			contact = contact->con_next)
+		{
+		    if (contact->con_layer1 == WireType)
+		    {
+			if (type != TT_SPACE)
+			{
+			    TxError("Ambiguous directive---multiple routing types\n");
+			    break;
+			}
+			else
+			    type = contact->con_layer2;
+		    }
+		}
+		if (type == TT_SPACE)
+		    TxError("No routing layer defined\n");
+		else
+		{
+		    width = DRCGetDefaultLayerWidth(type);
+		    WireAddContact(type, width);
+		}
+	    }
+	    else if (!strcmp(cmd->tx_argv[2], "width"))
+		WirePickType(WireType, WireWidth + 1);
+	    else
+		goto badargs;
+	    return;
+
+
 	case HELP:
 	    TxPrintf("Wiring commands have the form \":wire option\",");
 	    TxPrintf(" where option is one of:\n");
@@ -933,6 +1118,10 @@ CmdWire(w, cmd)
 	    WireAddLeg((Rect *) NULL, (Point *) NULL, WIRE_CHOOSE);
 	    return;
 
+	case SHOW:
+	    WireShowLeg();
+	    return;
+
 	case SWITCH:
 	    if (locargc == 2)
 		WireAddContact(-1, 0);
@@ -1027,13 +1216,16 @@ CmdWire(w, cmd)
 		    return;
 		}
 		type = DBTechNoisyNameType(cmd->tx_argv[2]);
-		if (!StrIsInt(cmd->tx_argv[3]))
+		if (type < 0)
+		    return;
+
+		if (!StrIsNumeric(cmd->tx_argv[3]))
 		{
-		    TxError("Route segment width must be an integer value\n");
+		    TxError("Route segment width must be a numeric value\n");
 		    return;
 		}
 		else
-		    width = atoi(cmd->tx_argv[3]);
+		    width = cmdParseCoord(w, cmd->tx_argv[3], FALSE, TRUE);
 
 		/* By doubling DBLambda[1], we can get back the	*/
 		/* coordinates in 2x internal units, which is	*/
@@ -1096,7 +1288,7 @@ CmdWire(w, cmd)
  * Write out all modified cells to disk.
  *
  * Usage:
- *	writeall [force]
+ *	writeall [force [cellname...]]
  *
  * Results:
  *	None.
@@ -1113,29 +1305,26 @@ CmdWire(w, cmd)
  * ----------------------------------------------------------------------------
  */
 
-static bool cmdWriteAllForce;  	/* If true, the user specified 'force' */
-
 void
 CmdWriteall(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
     int cmdWriteallFunc();
-    bool autoWrite;
     static char *force[] = { "force", 0 };
+    int argc;
 
-    if ((cmd->tx_argc > 2) || 
-	((cmd->tx_argc == 2) && (Lookup(cmd->tx_argv[1], force) < 0)))
+    if ((cmd->tx_argc >= 2) && (Lookup(cmd->tx_argv[1], force) < 0))
     {
-	TxError("Usage: %s [force]\n", cmd->tx_argv[0]);
+	TxError("Usage: %s [force [cellname ...]]\n", cmd->tx_argv[0]);
 	return;
     }
 
-    cmdWriteAllForce = autoWrite = (cmd->tx_argc == 2);
-
     DBUpdateStamps();
+    argc = cmd->tx_argc;
     (void) DBCellSrDefs(CDMODIFIED|CDBOXESCHANGED|CDSTAMPSCHANGED,
-	cmdWriteallFunc, (ClientData) &autoWrite);
+		cmdWriteallFunc, (ClientData)cmd);
+    cmd->tx_argc = argc;
 }
 
 /*
@@ -1146,18 +1335,20 @@ CmdWriteall(w, cmd)
 
     /*ARGSUSED*/
 int
-cmdWriteallFunc(def, autoWrite)
+cmdWriteallFunc(def, cmd)
     CellDef *def;	/* Pointer to CellDef to be saved.  This def might
 			 * be an internal buffer; if so, we ignore it.
 			 */
-    bool *autoWrite;	/* Client data passed to DBCellSrDefs; pointer to
-			 * flag:  if flag true, then write without asking.
-			 * Otherwise, ask user whether or not to write
-			 * dirty cells.
+    TxCommand *cmd;	/* Client data passed to DBCellSrDefs, a pointer
+			 * to the command structure.  If cmd->tx_argc == 1,
+			 * then prompt for each action.  If cmd->tx_argc
+			 * == 2, then write all cells without asking.  If
+			 * cmd->tx_argc > 2, then the arguments from 3 to
+			 * argc is a list of cells to write.
 			 */
 {
-    char *prompt;
-    int action, cidx = 0;
+    char *prompt, *argv;
+    int i, action, cidx = 0;
     static char *actionNames[] =
         { "write", "flush", "skip", "abort", "autowrite", 0 };
     static char *explain[] =
@@ -1166,11 +1357,21 @@ cmdWriteallFunc(def, autoWrite)
     if (def->cd_flags & CDINTERNAL) return 0;
     if (SigInterruptPending) return 1;
 
-    if (*autoWrite)
+    if (cmd->tx_argc == 2)
     {
 	action = 4;
     }
-    else
+    else if (cmd->tx_argc > 2)
+    {
+	action = 2;		/* skip */
+	for (i = 2; i < cmd->tx_argc; i++)
+	    if (!strcmp(cmd->tx_argv[i], def->cd_name))
+	    {
+		action = 0;	/* write */
+		break;
+	    }
+    }
+    else	/* cmd->tx_argc == 1 */
     {
 	if (!(def->cd_flags & CDMODIFIED))
 	{
@@ -1188,7 +1389,7 @@ cmdWriteallFunc(def, autoWrite)
     switch (action)
     {
 	case 0:		/* Write */
-	    cmdSaveCell(def, (char *) NULL, cmdWriteAllForce, TRUE);
+	    cmdSaveCell(def, (char *) NULL, FALSE, TRUE);
 	    break;
 	case 1:		/* Flush */
 	    cmdFlushCell(def);
@@ -1198,9 +1399,9 @@ cmdWriteallFunc(def, autoWrite)
 	case 3:		/* Abort command */
 	    return 1;
 	case 4:		/* Automatically write everything */
-	    *autoWrite = TRUE;
+	    cmd->tx_argc = 2;
 	    TxPrintf("Writing '%s'\n", def->cd_name);
-	    cmdSaveCell(def, (char *) NULL, cmdWriteAllForce, TRUE);
+	    cmdSaveCell(def, (char *) NULL, TRUE, TRUE);
 	    break;
     }
     return 0;
