@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdWizard.c,v 1.2 2001/01/12 22:12:17 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/commands/CmdWizard.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -29,7 +29,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdWizard.c,v 1.2
 #include <sys/types.h>
 #include <sys/times.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/malloc.h"
 #include "tiles/tile.h"
@@ -37,12 +37,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdWizard.c,v 1.2
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "utils/runstats.h"
 #include "textio/textio.h"
 #include "graphics/graphics.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "utils/utils.h"
 #include "textio/txcommands.h"
 
