@@ -17,11 +17,11 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellcopy.c,v 1.6 2001/07/30 17:16:21 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBcellcopy.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "utils/malloc.h"
@@ -287,7 +287,7 @@ DBCellCopyPaint(scx, mask, xMask, targetUse)
     struct copyAllArg arg;
     int dbCopyAllPaint();
 
-    if (!DBIsExpand(scx->scx_use, xMask))
+    if (!DBDescendSubcell(scx->scx_use, xMask))
 	return;
 
     arg.caa_mask = mask;
@@ -305,6 +305,7 @@ DBCellCopyPaint(scx, mask, xMask, targetUse)
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (PlaneMaskHasPlane(planeMask, pNum))
 	{
+	    cxp.tc_plane = pNum; /* not used? */
 	    (void) DBSrPaintArea((Tile *) NULL,
 		scx->scx_use->cu_def->cd_planes[pNum], &scx->scx_area,
 		mask, dbCopyAllPaint, (ClientData) &cxp);
@@ -360,7 +361,7 @@ DBCellCopyLabels(scx, mask, xMask, targetUse, pArea)
 	pArea->r_xtop = -1;
     }
 
-    if (!DBIsExpand(sourceUse, xMask))
+    if (!DBDescendSubcell(sourceUse, xMask))
 	return;
 
     for (lab = sourceUse->cu_def->cd_labels; lab; lab = lab->lab_next)
@@ -603,6 +604,7 @@ topbottom:
 
 splitdone:
 
+    /* Should this just be replaced with DBPaint()? */
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (DBPaintOnPlane(type, pNum))
 	{
