@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellname.c,v 1.4 2001/05/23 20:57:42 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBcellname.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -26,7 +26,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellname.c,v 1.
 #endif
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/hash.h"
 #include "utils/utils.h"
 #include "utils/geometry.h"
@@ -34,13 +34,13 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellname.c,v 1.
 #include "database/database.h"
 #include "database/databaseInt.h"
 #include "select/select.h"
-#include "signals/signals.h"
-#include "undo/undo.h"
+#include "utils/signals.h"
+#include "utils/undo.h"
 #include "utils/malloc.h"
 #include "windows/windows.h"
 #include "textio/textio.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "drc/drc.h"
 
 /*
@@ -68,7 +68,6 @@ extern void DBUnLinkCell();
  * Routines from other database modules
  */
 
-extern void dbComputeBbox();
 extern void DBSetUseIdHash();
 
 /*
@@ -124,6 +123,45 @@ DBCellRename(cellname, newname)
     return result;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBEnumerateTypes
+ *
+ *	Generate a mask of all known types in all known cells.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Returns the OR of the layer mask of each cell definition in
+ *	the pointer to rMask.
+ *
+ * Notes:
+ *	cd_types needs to be properly handled through paint/erase
+ *	operations, or else we should not rely on cd_types.
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBEnumerateTypes(rMask)
+    TileTypeBitMask *rMask;
+{
+    HashSearch hs;
+    HashEntry *entry;
+    CellDef *cellDef;
+    
+    TTMaskZero(rMask);
+    HashStartSearch(&hs);
+    while ((entry = HashNext(&dbCellDefTable, &hs)) != NULL)
+    {
+	cellDef = (CellDef *) HashGetValue(entry);
+	if (cellDef != (CellDef *)NULL)
+	    if ((cellDef->cd_flags & CDINTERNAL) != CDINTERNAL)
+		TTMaskSetMask(rMask, &cellDef->cd_types);
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -142,8 +180,9 @@ DBCellRename(cellname, newname)
  * ----------------------------------------------------------------------------
  */
 bool
-DBCellDelete(cellname)
+DBCellDelete(cellname, force)
     char *cellname;
+    bool force;
 {
     HashEntry *entry;
     CellDef *celldef;
@@ -185,7 +224,8 @@ DBCellDelete(cellname)
     /* Cleared to delete. . . now prompt user if the cell has changes. */
     /* Last chance to save! */
 
-    if (celldef->cd_flags & (CDMODIFIED|CDBOXESCHANGED|CDSTAMPSCHANGED))
+    if ((force == FALSE) &&
+	(celldef->cd_flags & (CDMODIFIED|CDBOXESCHANGED|CDSTAMPSCHANGED)))
     {
 	static char *yesno[] = { "no", "yes", 0 };
 	int code;
@@ -358,7 +398,7 @@ dbCellPrintInfo(StartDef, who, dolist)
 	    {
 	        if (dolist)
 #ifdef MAGIC_WRAPPER
-		    Tcl_AppendResult(magicinterp, "1", (char *)NULL);
+		    Tcl_AppendElement(magicinterp, "1");
 #else
 		    TxPrintf("TRUE\n");
 #endif
@@ -369,7 +409,7 @@ dbCellPrintInfo(StartDef, who, dolist)
 	    {
 		if (dolist)
 #ifdef MAGIC_WRAPPER
-		    Tcl_AppendResult(magicinterp, StartDef->cd_name, (char *)NULL);
+		    Tcl_AppendElement(magicinterp, StartDef->cd_name);
 #else
 		    TxPrintf("%s\n", StartDef->cd_name);
 #endif
@@ -549,7 +589,7 @@ DBTopPrint(mw, dolist)
 	if (celldef == NULL) return;
 #ifdef MAGIC_WRAPPER
 	if (dolist)
-	    Tcl_AppendResult(magicinterp, celldef->cd_name, (char *)NULL);
+	    Tcl_AppendElement(magicinterp, celldef->cd_name);
 	else
 #endif
 	TxPrintf("Top-level cell in the window is: %s\n", celldef->cd_name);
@@ -708,7 +748,7 @@ DBCellPrint(CellName, who, dolist)
 		{
 		    if (dolist)
 #ifdef MAGIC_WRAPPER
-			Tcl_AppendResult(magicinterp, "0", (char *)NULL);
+			Tcl_AppendElement(magicinterp, "0");
 #else
 			TxPrintf("Not loaded\n");
 #endif
@@ -763,7 +803,7 @@ dbUsePrintInfo(StartUse, who, dolist)
 	    {
 	        if (dolist)
 #ifdef MAGIC_WRAPPER
-		    Tcl_AppendResult(magicinterp, "1", (char *)NULL);
+		    Tcl_AppendElement(magicinterp, "1");
 #else
 		    TxPrintf("TRUE\n");
 #endif
@@ -775,7 +815,7 @@ dbUsePrintInfo(StartUse, who, dolist)
 		cu_name = dbGetUseName(StartUse);
 		if (dolist)
 #ifdef MAGIC_WRAPPER
-		    Tcl_AppendResult(magicinterp, cu_name, (char *)NULL);
+		    Tcl_AppendElement(magicinterp, cu_name);
 #else
 		    TxPrintf("%s\n", cu_name);
 #endif
@@ -790,7 +830,7 @@ dbUsePrintInfo(StartUse, who, dolist)
 	    {
 	        if (dolist)
 #ifdef MAGIC_WRAPPER
-		    Tcl_AppendResult(magicinterp, "0", (char *)NULL);
+		    Tcl_AppendElement(magicinterp, "0");
 #else
 		    TxPrintf("FALSE\n");
 #endif
@@ -801,8 +841,7 @@ dbUsePrintInfo(StartUse, who, dolist)
 	    {
 	        if (dolist)
 #ifdef MAGIC_WRAPPER
-		    Tcl_AppendResult(magicinterp, StartUse->cu_def->cd_name,
-				(char *)NULL);
+		    Tcl_AppendElement(magicinterp, StartUse->cu_def->cd_name);
 #else
 		    TxPrintf("%s\n", StartUse->cu_def->cd_name);
 #endif
@@ -942,7 +981,6 @@ DBUsePrint(CellName, who, dolist)
 			    {
 				dbUsePrintInfo(celluse, who, dolist);
 				found = 1;
-				break;
 			    }
 			}
 		    }
@@ -999,7 +1037,7 @@ dbCellUsePrintFunc(cellUse, dolist)
 	cu_name = dbGetUseName(cellUse);
 	if (*dolist)
 #ifdef MAGIC_WRAPPER
-            Tcl_AppendResult(magicinterp, cu_name, " ", (char *)NULL);
+            Tcl_AppendElement(magicinterp, cu_name);
 #else
 	    TxPrintf("%s ", cu_name);
 #endif
@@ -1185,7 +1223,6 @@ DBCellNewUse(cellDef, useName)
     cellUse->cu_yhi = 0;
     cellUse->cu_xsep = 0;
     cellUse->cu_ysep = 0;
-    cellUse->cu_delta = 0;		/* For plowing */
     cellUse->cu_nextuse = cellDef->cd_parents;
 
     /* Initial client field */
@@ -1195,7 +1232,7 @@ DBCellNewUse(cellDef, useName)
     cellUse->cu_client = (ClientData) MINFINITY;
 
     cellDef->cd_parents = cellUse;
-    dbComputeBbox(cellUse);
+    DBComputeUseBbox(cellUse);
     return (cellUse);
 }
 
@@ -1893,6 +1930,6 @@ DBNewYank(yname, pyuse, pydef)
     }
     *pyuse = DBCellNewUse(*pydef, (char *) NULL);
     DBSetTrans(*pyuse, &GeoIdentityTransform);
-    (*pyuse)->cu_expandMask = (~0);	/* This is always expanded. */
+    (*pyuse)->cu_expandMask = CU_DESCEND_SPECIAL; /* This is always expanded. */
 }
 
