@@ -17,12 +17,13 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellsrch.c,v 1.3 2001/03/29 19:50:05 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBcellsrch.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
+#include "utils/malloc.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
@@ -30,9 +31,9 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellsrch.c,v 1.
 #include "database/database.h"
 #include "database/databaseInt.h"
 #include "textio/textio.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "windows/windows.h"
-#include "main/main.h"
+#include "utils/main.h"
 
 /* Quick hack for dbScalePlanes() access to the CIF/GDS paint table */
 
@@ -189,7 +190,7 @@ dbCellTileSrFunc(scx, fp)
     int pNum;
 
     ASSERT(def != (CellDef *) NULL, "dbCellTileSrFunc");
-    if (!DBIsExpand(scx->scx_use, fp->tf_xmask))
+    if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask))
 	return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
 	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
@@ -205,6 +206,7 @@ dbCellTileSrFunc(scx, fp)
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (PlaneMaskHasPlane(fp->tf_planes, pNum))
 	{
+	    context.tc_plane = pNum;
 #ifdef NONMANHATTAN
 	    if (fp->tf_dinfo & TT_DIAGONAL)
 	    {
@@ -309,7 +311,7 @@ dbCellUniqueTileSrFunc(scx, fp)
     int pNum;
 
     ASSERT(def != (CellDef *) NULL, "dbCellUniqueTileSrFunc");
-    if (!DBIsExpand(scx->scx_use, fp->tf_xmask))
+    if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask))
 	return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
 	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
@@ -327,11 +329,11 @@ dbCellUniqueTileSrFunc(scx, fp)
 	{
 	    uMask = DBHomePlaneTypes(pNum);
 	    TTMaskAndMask(&uMask, fp->tf_mask);
-
-	    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
-		    &scx->scx_area, &uMask,
-		    fp->tf_func, (ClientData) &context))
-		return 1;
+	    if (!TTMaskIsZero(&uMask))
+	        if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
+		    	&scx->scx_area, &uMask, fp->tf_func,
+			(ClientData) &context))
+		    return 1;
 	}
 
     /*
@@ -416,7 +418,7 @@ DBNoTreeSrTiles(scx, mask, xMask, func, cdarg)
     int pNum;
 
     ASSERT(def != (CellDef *) NULL, "DBNoTreeSrTiles");
-    if (!DBIsExpand(cellUse, xMask))
+    if (!DBDescendSubcell(cellUse, xMask))
 	return 0;
 
     if ((def->cd_flags & CDAVAILABLE) == 0)
@@ -520,7 +522,7 @@ DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
     int dbCellLabelSrFunc();
 
     ASSERT(def != (CellDef *) NULL, "DBTreeSrLabels");
-    if (!DBIsExpand(cellUse, xMask)) return 0;
+    if (!DBDescendSubcell(cellUse, xMask)) return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
 	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
 
@@ -578,7 +580,7 @@ dbCellLabelSrFunc(scx, fp)
     int result;
 
     ASSERT(def != (CellDef *) NULL, "dbCellLabelSrFunc");
-    if (!DBIsExpand(scx->scx_use, fp->tf_xmask)) return 0;
+    if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask)) return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
 	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
     
@@ -683,7 +685,7 @@ DBTreeSrCells(scx, xMask, func, cdarg)
     TreeContext context;
     TreeFilter filter;
 
-    if (!DBIsExpand(cellUse, xMask))
+    if (!DBDescendSubcell(cellUse, xMask))
 	return 0;
     if ((cellUse->cu_def->cd_flags & CDAVAILABLE) == 0)
 	if (!DBCellRead(cellUse->cu_def, (char *) NULL, TRUE))
@@ -720,11 +722,11 @@ dbTreeCellSrFunc(scx, fp)
     CellUse *use = scx->scx_use;
     int result;
 
-    /* DBIsExpand treats a zero expand mask as "expanded everywhere",
+    /* DBDescendSubcell treats a zero expand mask as "expanded everywhere",
      * whereas we want it to mean "expanded nowhere".  Handle specially.
      */
 
-    if ((!DBIsExpand(use, fp->tf_xmask)) || (fp->tf_xmask == 0))
+    if ((!DBDescendSubcell(use, fp->tf_xmask)) || (fp->tf_xmask == CU_DESCEND_ALL))
 	result = (*fp->tf_func)(scx, fp->tf_arg);
     else
     {
@@ -765,15 +767,13 @@ DBSeeTypesAll(rootUse, rootRect, xMask, mask)
 {
     int dbSeeTypesAllSrFunc();
     SearchContext scontext;
-    TileTypeBitMask visitMask;
 
     scontext.scx_use = rootUse;
     scontext.scx_trans = GeoIdentityTransform;
     scontext.scx_area = *rootRect;
 
     TTMaskZero(mask);
-    TTMaskSetMask3(&visitMask, &DBUserLayerBits, &DBBuiltinLayerBits);
-    (void) DBTreeSrTiles(&scontext, &visitMask, xMask,
+    (void) DBTreeSrTiles(&scontext, &DBAllTypeBits, xMask,
 		dbSeeTypesAllSrFunc, (ClientData) mask);
 }
 
@@ -805,72 +805,7 @@ dbSeeTypesAllSrFunc(tile, cxp)
     }
     return 0;
 }
-
-/*
- *-----------------------------------------------------------------------------
- *
- * DBSeeTypes --
- *
- * Set a TileTypeBitMask to all tiles beneath the given rectangle in the
- * definition of the given CellUse.  "Beneath" in this case means
- * "completely containing".
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	Sets the TileTypeBitMask pointed to by 'mask' to all types beneath
- *	the rectangle.
- *
- *-----------------------------------------------------------------------------
- */
 
-void
-DBSeeTypes(rootUse, rootRect, mask)
-    CellUse *rootUse;	/* CellUse to search */
-    Rect *rootRect;	/* Clipping rectangle in coordinates of CellUse's def */
-    TileTypeBitMask *mask;	/* Mask to be set */
-{
-    int dbSeeTypesSrFunc();
-    TileTypeBitMask visitMask;
-    struct seeTypesArg arg;
-    CellDef *def;
-    int pNum;
-
-    TTMaskZero(mask);
-    arg.saa_mask = mask;
-    arg.saa_rect = rootRect;
-    def = rootUse->cu_def;
-    if ((def->cd_flags & CDAVAILABLE) == 0)
-	if (!DBCellRead(def, (char *) NULL, TRUE))
-	    return;
-
-    TTMaskSetMask3(&visitMask, &DBUserLayerBits, &DBBuiltinLayerBits);
-    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
-	(void) DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
-		    rootRect, &visitMask,
-		    dbSeeTypesSrFunc, (ClientData) &arg);
-}
-
-/*
- * dbSeeTypesSrFunc --
- *
- * Filter procedure applied to tiles by DBSeeTypes() above.
- */
-
-int
-dbSeeTypesSrFunc(tile, arg)
-    Tile *tile;			/* Tile found */
-    struct seeTypesArg *arg;
-{
-    Rect tileRect;
-
-    TiToRect(tile, &tileRect);
-    if (GEO_OVERLAP((&tileRect), (arg->saa_rect)))
-	TTMaskSetType(arg->saa_mask, TiGetType(tile));
-    return 0;
-}
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1613,7 +1548,7 @@ dbScaleCell(cellDef, scalen, scaled)
     /* DBCellEnum() attempts to read unavailable celldefs.  We don't	*/
     /* want to do that here, so check CDAVAILABLE flag first.	  	*/
 
-    if ((cellDef->cd_flags & CDAVAILABLE) == 0) return 0;
+    if ((cellDef->cd_flags & CDAVAILABLE) == 0) goto donecell;
 
     /* Enumerate all unique cell uses, and scale their position,	*/
     /* transform, and array information.				*/
@@ -1708,11 +1643,14 @@ dbScaleCell(cellDef, scalen, scaled)
 	}
     }
 
+donecell:
+
     /* The cellDef bounding box gets expanded to match the new scale. */
 
     DBScalePoint(&cellDef->cd_bbox.r_ll, scalen, scaled);
     DBScalePoint(&cellDef->cd_bbox.r_ur, scalen, scaled);
 
+    /* Perhaps we should not do this if CD_AVAILABLE is not set? */
     cellDef->cd_flags &= ~(CDMODIFIED|CDBOXESCHANGED|CDSTAMPSCHANGED);
 
     return 0;
