@@ -20,17 +20,17 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBconnect.c,v 1.7 2001/11/29 20:08:51 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBconnect.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "database/databaseInt.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "utils/malloc.h"
 
 /* General note for DBSrConnect:
@@ -471,6 +471,7 @@ donesides:
      */
     
     planes = DBConnPlanes[loctype];
+    planes &= ~(PlaneNumToMaskBit(csa->csa_plane));
     if (planes != 0)
     {
         struct conSrArg newcsa;
@@ -638,7 +639,7 @@ dbcConnectFunc(tile, cx)
 #ifdef NONMANHATTAN
     TileType dinfo = 0;
 #endif
-    int pNum;
+    int pNum = cx->tc_plane;
     CellDef *def;
 
     TiToRect(tile, &tileArea);
@@ -681,7 +682,6 @@ dbcConnectFunc(tile, cx)
      * since any processing that is needed was already done before.
      */
 	
-    pNum = DBPlane(loctype);
     connectMask = &csa2->csa2_connect[loctype];
 
     /* In the case of contact bits, the types underneath
