@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBio.c,v 1.5 2001/06/07 21:20:19 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBio.c,v 1.2 2005/05/31 21:41:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -33,7 +33,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBio.c,v 1.5 2001
 #include <sys/stat.h>
 #include <time.h>
 #include <sys/time.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/utils.h"
@@ -42,12 +43,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBio.c,v 1.5 2001
 #include "database/databaseInt.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "textio/textio.h"
 #include "drc/drc.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "utils/malloc.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 
 extern char *Path;
 
@@ -254,8 +255,8 @@ DBCellRead(cellDef, name, ignoreTech)
     FILE *f;
     Rect *rp;
     int c;
-    TileType type, rtype;
-    TileTypeBitMask *rmask;
+    TileType type, rtype, loctype;
+    TileTypeBitMask *rmask, typemask;
     Plane *plane;
     Rect r;
     int n = 1, d = 1;
@@ -377,10 +378,12 @@ DBCellRead(cellDef, name, ignoreTech)
 #endif
 	DBScaleEverything(d, 1);
 	DBLambda[1] *= d;
-	TxPrintf("Scaled magic internal geometry by factor of %d\n", d);
+	TxPrintf("Input cell %s scales magic internal geometry by factor of %d\n",
+			cellDef->cd_name, d);
     }
     if (n > 1)
-	TxPrintf("Scaled magic input file geometry by factor of %d\n", n);
+	TxPrintf("Scaled magic input cell %s geometry by factor of %d\n", 
+		cellDef->cd_name, n);
 
     /*
      * Next, get the paint, subcells, and labels for this cell.
@@ -404,6 +407,8 @@ DBCellRead(cellDef, name, ignoreTech)
 	    continue;
 	}
 
+	TTMaskZero(&typemask);
+	rmask = &typemask;
 	type = DBTechNameType(layername);
 	if (type < 0)
 	{
@@ -430,8 +435,11 @@ DBCellRead(cellDef, name, ignoreTech)
 		continue;
 	    }
 	    else if (!strcmp(layername, "end")) goto done;
-	    TxError("Unknown layer %s ignored in %s\n", layername,
-			cellDef->cd_name);
+	    else
+		DBTechNoisyNameMask(layername, rmask);
+
+	    //	TxError("Unknown layer %s ignored in %s\n", layername,
+	    //		cellDef->cd_name);
 	}
 
 	/*
@@ -441,7 +449,10 @@ DBCellRead(cellDef, name, ignoreTech)
 	if (DBPlane(type) > 0)
 	{
 	    if (type < DBNumUserLayers)
+	    {
 		TTMaskSetType(&cellDef->cd_types, type);
+		TTMaskSetType(rmask, type);
+	    }
 	    else
 	    {
 		/* Separate stacked contact types into their components */
@@ -521,30 +532,17 @@ nextrect:
 		/*				   	Tim 7/8/04	*/
 		/*------------------------------------------------------*/
 
-		TileType loctype;
-
-		if (type >= DBNumUserLayers)
+		for (rtype = TT_SPACE + 1; rtype < DBNumUserLayers; rtype++)
 		{
-		    rmask = DBResidueMask(type);
-		    for (rtype = TT_SPACE + 1; rtype < DBNumUserLayers; rtype++)
-			if (TTMaskHasType(rmask, rtype))
-			{
-			    loctype = rtype;
+		    if (TTMaskHasType(rmask, rtype))
+		    {
+			loctype = rtype;
 #ifdef NONMANHATTAN
-			    if (dinfo & TT_SIDE) loctype <<= 14;
-			    loctype |= dinfo;
+			if (dinfo & TT_SIDE) loctype <<= 14;
+			loctype |= dinfo;
 #endif
-			    DBPaint(cellDef, rp, loctype);
-			}
-		}
-		else
-		{
-		    loctype = type;
-#ifdef NONMANHATTAN
-		    if (dinfo & TT_SIDE) loctype <<= 14;
-		    loctype |= dinfo;
-#endif
-		    DBPaint(cellDef, rp, loctype);
+			DBPaint(cellDef, rp, loctype);
+		    }
 		}
 	    }
 	}
@@ -1016,6 +1014,10 @@ dbReadProperties(cellDef, line, len, f)
 	    }
 	    storedvalue = StrDup((char **)NULL, propertyvalue);
 	    (void) DBPropPut(cellDef, propertyname, storedvalue);
+
+	    /* Go ahead and process the vendor GDS property */
+	    if (!strcmp(propertyname, "GDS_FILE"))
+		cellDef->cd_flags |= CDVENDORGDS;
 	}
 
 nextproperty:
@@ -1986,15 +1988,23 @@ DBCellWrite(cellDef, fileName)
 	{
 	    result = FALSE;
 	    perror("rename");
-	    TxError(
-"\
-ATTENTION: Magic was unable to rename file %s to %s.\n\
-If the file %s exists, it is the old copy of the cell %s.\n\
-The new copy is in the file %s.  Please copy this file\n\
-to a safe place before executing any more Magic commands.\n",
-		tmpname, expandname, expandname, cellDef->cd_name, tmpname);
+	    TxError("ATTENTION: Magic was unable to rename file %s to %s.\n"
+		    "If the file %s exists, it is the old copy of the cell %s.\n"
+		    "The new copy is in the file %s.  Please copy this file\n"
+		    "to a safe place before executing any more Magic commands.\n",
+		    tmpname, expandname, expandname, cellDef->cd_name, tmpname);
 	    goto cleanup;
 	}
+	else
+	{
+	    /* mkstemp() generates a file with permissions 0600 */
+	    /* This is okay for the temporary file, but we want	*/
+	    /* to respect the user's umask value when renaming	*/
+	    /* the file to ".mag".				*/
+	    mode_t userumask = umask((mode_t)0);
+	    umask(userumask);
+	    chmod(expandname, (mode_t)0666 & ~userumask);
+	}
     }
     else
     {
