@@ -17,14 +17,14 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBlabel.c,v 1.4 2001/07/23 17:49:51 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBlabel.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/malloc.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -47,7 +47,7 @@ static TileType DBPickLabelLayer(/* CellDef *def, Label *lab, int noreconnect */
  * that the CellDef should be treated as a subcircuit.
  *
  * Results:
- *	True if CellDef contains ports, False otherwise.
+ *	TRUE if CellDef contains ports, FALSE otherwise.
  *
  * Side effects:
  *	None.
