@@ -23,14 +23,14 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtcontact.c,v 1.2 2001/01/12 22:12:21 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBtcontact.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
 #include "utils/malloc.h"
@@ -38,7 +38,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtcontact.c,v 1.
 #include "utils/hash.h"
 #include "database/database.h"
 #include "database/databaseInt.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "textio/textio.h"
 
 /* type-to-bitmask conversion (1 bit per entry) */
@@ -807,11 +807,8 @@ DBTechFinalContact()
 	    {
 		lp = &dbLayerInfo[primaryType];
 
-		if (primaryType < DBNumUserLayers)
-		    for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
-			if (PlaneMaskHasPlane(lp->l_pmask, pNum))
-			    DBTypePlaneMaskTbl[primaryType] |=
-					PlaneNumToMaskBit(pNum);
+		/* if (primaryType < DBNumUserLayers) */
+		    DBTypePlaneMaskTbl[primaryType] |= lp->l_pmask;
 
 		for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 		    if (PlaneMaskHasPlane(lp->l_pmask, pNum))
