@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtechname.c,v 1.3 2001/11/29 20:08:51 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBtechname.c,v 1.2 2005/06/01 21:41:40 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -30,14 +30,14 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtechname.c,v 1.
 #endif
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "database/databaseInt.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "textio/textio.h"
 #include "utils/malloc.h"
 
@@ -106,7 +106,7 @@ DBTechNameType(typename)
  */
 
 TileType
-DBTechNameTypes(typename,bitmask)
+DBTechNameTypes(typename, bitmask)
     char *typename;	/* The name of the type */
     TileTypeBitMask	*bitmask;
 {
@@ -119,7 +119,23 @@ DBTechNameTypes(typename,bitmask)
     slash = index(typename, '/');
     if (slash != NULL) *slash = 0;
     type = (TileType) dbTechNameLookup(typename, &dbTypeNameLists);
-    if (type < 0) return type;
+    if (type < 0)
+    {
+	HashEntry *he;
+
+	/* Check against the alias table */
+	he = HashLookOnly(&DBTypeAliasTable, typename);
+	if (he)
+	{
+	    TTMaskSetMask(bitmask, (TileTypeBitMask *)HashGetValue(he));
+	    for (type = TT_TECHDEPBASE; type < DBNumUserLayers; type++)
+		if (TTMaskHasType(bitmask, type))
+		    return type;
+	    type = -2;
+	}
+	return type;
+    }
+
     if (slash == NULL)
     {
     	 TTMaskSetType(bitmask,type);
@@ -138,6 +154,7 @@ DBTechNameTypes(typename,bitmask)
     	 TTMaskSetType(bitmask, type);
     	 return type;
     } 
+
     return -2;
 }
 
@@ -514,13 +531,18 @@ DBTechPrintCanonicalType(layername)
  *	a parenthesized group.  For example, ~(x,y)/foo refers to
  *	all layers on plane "foo" except "x" and "y".
  *
- *	Finally, magic version 7.3 defines *x to mean "x and all
+ *	Magic version 7.3 defines *x to mean "x and all
  *	contact layers containing x as a residue".  Thus, for example,
  *	*metal1 expands to m1,pc,ndc,pdc,nsc,psc,m2c,...  Because many
  *	connect, cif, extract, and drc rules list all these layers,
  *	the asterisk notation makes the techfile shorter and more
  *	readable.
  *
+ *	Magic version 7.3 also defines a hash table "DBTypeAliasTable"
+ *	that allows macros to be embedded in the technology file.
+ *	Macros are expanded first, prior to applying the parsing
+ *	described above.
+ *
  * Results:
  *	Returns a plane mask of all the planes specified.
  *
@@ -616,26 +638,39 @@ DBTechNoisyNameMask(layers, mask)
 	    }
 	    else if (strcmp(p, "0") != 0)
 	    {
+		HashEntry *he;
+
 		save = *p2;
-		*p2 = 0;
+		*p2 = '\0';
 
-		/* Check for asterisk notation, meaning to include	*/
-		/* all types which have this type as a residue.		*/
-		if (*p == '*')
+		/* Check the alias table for macro definitions */
+		he = HashLookOnly(&DBTypeAliasTable, p);
+		if (he)
 		{
-		    allResidues = TRUE;
-		    p++;
+		    TileTypeBitMask *amask;
+		    amask = (TileTypeBitMask *)HashGetValue(he);
+		    TTMaskSetMask(&m2, amask);
 		}
+		else
+		{
+		    /* Check for asterisk notation, meaning to include	*/
+		    /* all types which have this type as a residue.	*/
+		    if (*p == '*')
+		    {
+			allResidues = TRUE;
+			p++;
+		    }
 
-		t = DBTechNoisyNameType(p);
-		if (t >= 0) m2 = DBLayerTypeMaskTbl[t];
+		    t = DBTechNoisyNameType(p);
+		    if (t >= 0) m2 = DBLayerTypeMaskTbl[t];
+
+		    /* Include all types which have t as a residue */
+		    if (allResidues)
+			for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
+			    if (TTMaskHasType(DBResidueMask(rtype), t))
+				TTMaskSetType(&m2, rtype);
+		}
 		*p2 = save;
-
-		/* Include all types which have t as a residue */
-		if (allResidues)
-		    for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
-			if (TTMaskHasType(DBResidueMask(rtype), t))
-			    TTMaskSetType(&m2, rtype);
 	    }
 	    p = p2;
 	}
@@ -719,7 +754,7 @@ DBTechMinSetPlanes(typeMask, newTypes)
     TTMaskZero(newTypes);
     for (i = TT_SELECTBASE; i < DBNumTypes; i++)
     {
-	if (!TTMaskHasType(&typeMask, i) || DBConnPlanes[i] != 0)
+	if (!TTMaskHasType(&typeMask, i) || DBIsContact(i))
 	    continue;
 	TTMaskSetType(newTypes, i);
 	planes |= PlaneNumToMaskBit(DBPlane(i));
@@ -727,12 +762,12 @@ DBTechMinSetPlanes(typeMask, newTypes)
 
     /*
      * Now go through each contact type.  If one of the contact's planes
-     * is already necessary, use it.  Otherwise, add the plane for
+     * is already necessary, use it.  Otherwise, add the home plane for
      * the contact.
      */
     for (i = TT_SELECTBASE; i < DBNumTypes; i++)
     {
-	if (!TTMaskHasType(&typeMask, i) || DBConnPlanes[i] == 0)
+	if (!TTMaskHasType(&typeMask, i) || !DBIsContact(i))
 	    continue;
 	okPlanes = DBConnPlanes[i] & planes;
 	if (okPlanes != 0)
