@@ -17,21 +17,21 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtpaint.c,v 1.5 2001/01/12 22:12:22 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBtpaint.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>	/* for memset(), memcpy() */
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "database/databaseInt.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "textio/textio.h"
 
     /* Painting and erasing tables */
@@ -63,6 +63,8 @@ Rule dbSavedRules[NT];
 
 extern void dbTechBitTypeInit();
 
+bool dbTechAddPaintErase();
+bool dbTechSaveCompose();
 
 /*
  * ----------------------------------------------------------------------------
@@ -377,9 +379,8 @@ DBTechAddCompose(sectionName, argc, argv)
     switch (ruleType)
     {
 	case RULE_PAINT:
-	    return (dbTechAddPaint(sectionName, argc, argv));
 	case RULE_ERASE:
-	    return (dbTechAddErase(sectionName, argc, argv));
+	    return (dbTechAddPaintErase(ruleType, sectionName, argc, argv));
     }
 
     /* Compose or decompose rule: find result type and then skip over it */
@@ -498,8 +499,12 @@ dbTechSaveCompose(ruleType, t, argc, argv)
 		return FALSE;
 	    }
 
-	    if (!dbTechCheckImages(t, r, s) || !dbTechCheckImages(t, s, r))
-		return FALSE;
+	    /* The following restriction has been lifted due to	*/
+	    /* the recursive plane painting method added to	*/
+	    /* routine DBPaint().	(Tim, 5/11/04)		*/
+
+//	    if (!dbTechCheckImages(t, r, s) || !dbTechCheckImages(t, s, r))
+//		return FALSE;
 	}
 
 	rp->r_pairs[rp->r_npairs].rp_a = r;
@@ -510,6 +515,8 @@ dbTechSaveCompose(ruleType, t, argc, argv)
     return TRUE;
 }
 
+
+#if 0	/* deprecated function (5/11/04) */
 /*
  * ----------------------------------------------------------------------------
  *
@@ -555,13 +562,15 @@ dbTechCheckImages(t, r, s)
 
     return TRUE;
 }
+#endif	/* 0 */
+
 
 /*
  * ----------------------------------------------------------------------------
  *
- * dbTechAddPaint --
+ * dbTechAddPaintErase --
  *
- * Add a new entry to the paint table.
+ * Add a new entry to the paint or erase table.
  * The semantics is that painting a tile of type1 with paint type2
  * yields a tile of typeres.
  *
@@ -575,12 +584,13 @@ dbTechCheckImages(t, r, s)
  */
 
 bool
-dbTechAddPaint(sectionName, argc, argv)
+dbTechAddPaintErase(type, sectionName, argc, argv)
+    int type;
     char *sectionName;
     int argc;
     char *argv[];
 {
-    int pNum;
+    int pNum, pMask, rMask;
     TileType t1, t2, tres;
 
     if (argc < 3)
@@ -602,108 +612,68 @@ dbTechAddPaint(sectionName, argc, argv)
 			argv[0], argv[1], argv[2]);
 	    return FALSE;
 	}
-	pNum = DBPlane(t1);
-    }
-    else if ((pNum = DBTechNoisyNamePlane(argv[3])) < 0) return FALSE;
-
-    if ((tres != TT_SPACE) && (DBPlane(tres) != pNum))
-    {
-	if (DBIsContact(tres))
-	{
-	    TileTypeBitMask *rmask = DBResidueMask(tres);
-	    TileType rtype;
-	    for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
-		if (TTMaskHasType(rmask, rtype))
-		    dbSetPaintEntry(t1, t2, DBPlane(rtype), tres);
-	}
-	else if (argc == 3)
-	    TechError("<%s, %s, %s>:\n"
-		"Warning: type %s (%d) belongs on plane %s (%d), "
-		"not %s (%d)\n",
-		argv[0], argv[1], argv[2],
-		argv[2], tres,
-		DBPlaneLongName(DBPlane(tres)), DBPlane(tres),
-		argv[0], pNum);
 	else
-	    TechError("<%s, %s, %s, %s>:\n"
-		"Warning: type %s (%d) belongs on plane %s (%d), "
-		"not %s (%d)\n",
-		argv[0], argv[1], argv[2], argv[3],
-		argv[2], tres,
-		DBPlaneLongName(DBPlane(tres)), DBPlane(tres),
-		argv[3], pNum);
+	   pMask = PlaneMask(t1);
     }
+    else
+    {
+	if ((pNum = DBTechNoisyNamePlane(argv[3])) < 0) return FALSE;
+	else
+	    pMask = PlaneNumToMaskBit(pNum);
+    }
+    rMask = PlaneMask(tres);
 
-    dbSetPaintEntry(t1, t2, pNum, tres);
+    /* (The plane mask of "tres" may be a subset of t1, but NOT vice	*/
+    /* versa. Otherwise we will end up creating rules that are not	*/
+    /* dependent on t1 in some planes.)					*/
+
+    /* This restriction lifted 5/11/04 with code in DBPaint() that	*/
+    /* recursively paints image types onto planes where they may be	*/
+    /* missed by the inability to generate the correct paint table for	*/
+    /* the operation.							*/
+
+//  if (rMask & ~pMask)
+//  {
+//	TechError("Planes of result type must be a subset of those of have-type\n");
+//	return FALSE;
+//  }
+
+    pMask &= ~rMask;
+
+    if (type == RULE_PAINT)
+    {
+	/* Apply to all planes of rMask. */
+
+	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+	    if (PlaneMaskHasPlane(rMask, pNum))
+		dbSetPaintEntry(t1, t2, pNum, tres);
+
+	/* For all planes of pMask which are not in rMask, result is space */
+
+
+	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+	    if (PlaneMaskHasPlane(pMask, pNum))
+		dbSetPaintEntry(t1, t2, pNum, TT_SPACE);
+    }
+    else	/* (type == RULE_ERASE) */
+    {
+	/* Apply to all planes of rMask. */
+
+	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+	    if (PlaneMaskHasPlane(rMask, pNum))
+		dbSetEraseEntry(t1, t2, pNum, tres);
+
+	/* For all planes of pMask which are not in rMask, result is space */
+
+
+	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+	    if (PlaneMaskHasPlane(pMask, pNum))
+		dbSetEraseEntry(t1, t2, pNum, TT_SPACE);
+    }
     TTMaskSetType(&dbNotDefaultPaintTbl[t1], t2);
     return TRUE;
 }
-
-/*
- * ----------------------------------------------------------------------------
- *
- * dbTechAddErase --
- *
- * Add a new entry to the erase table.
- * The semantics is that erasing paint of type2 from a tile of type1
- * yields a tile of typeres.
- *
- * Results:
- *	Returns TRUE if successful, FALSE on error.
- *
- * Side effects:
- *	Updates the database technology variables.
- *
- * ----------------------------------------------------------------------------
- */
 
-    /*ARGSUSED*/
-bool
-dbTechAddErase(sectionName, argc, argv)
-    char *sectionName;
-    int argc;
-    char *argv[];
-{
-    TileType t1, t2, tres;
-    int pNum;
-
-    if (argc < 3)
-    {
-	TechError("Line must contain at least 3 types\n");
-	return FALSE;
-    }
-
-    if ((t1 = DBTechNoisyNameType(argv[0])) < 0) return FALSE;
-    if ((t2 = DBTechNoisyNameType(argv[1])) < 0) return FALSE;
-    if ((tres = DBTechNoisyNameType(argv[2])) < 0) return FALSE;
-
-    if (argc == 3)
-    {
-	if (t1 == TT_SPACE)
-	{
-	    TechError("<%s, %s, %s>:\n"
-			"Must specify plane in paint table for erasing space\n",
-			argv[0], argv[1], argv[2]);
-	    return FALSE;
-	}
-	pNum = DBPlane(t1);
-    }
-    else if ((pNum = DBTechNoisyNamePlane(argv[3])) < 0) return FALSE;
-
-    if (tres != TT_SPACE && DBPlane(tres) != pNum)
-    {
-	TechError("<%s, %s, %s, %s>:\n"
-		"Warning: type %s (%d) belongs on plane %s (%d), not %s (%d)\n",
-		argv[0], argv[1], argv[2], argv[3],
-		argv[2], tres,
-		DBPlaneLongName(DBPlane(tres)), DBPlane(tres),
-		argv[3], pNum);
-    }
-
-    dbSetEraseEntry(t1, t2, pNum, tres);
-    TTMaskSetType(&dbNotDefaultPaintTbl[t1], t2);
-    return TRUE;
-}
 
 /*
  * ----------------------------------------------------------------------------
