@@ -18,19 +18,20 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtpaint2.c,v 1.4 2001/01/12 22:12:22 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBtpaint2.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "database/databaseInt.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "textio/textio.h"
 
 #define	SETPAINT(have,paint,plane,get) \
@@ -110,7 +111,9 @@ DBTechFinalCompose()
  * dbTechPaintErasePlanes --
  *
  * Fill in the tables telling which planes get affected
- * by painting and erasing.
+ * by painting and erasing.  One may take the naive view that only
+ * the planes on which a type is defined can be affected, but then
+ * one can't define arbitrary composite types.
  *
  * Results:
  *	None.
@@ -136,13 +139,13 @@ dbTechPaintErasePlanes()
     {
 	DBTypePaintPlanesTbl[t] = DBTypeErasePlanesTbl[t] = 0;
 
-	for (pNum = PL_PAINTBASE; pNum < DBNumTypes; pNum++)
+	for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	    for (s = 0; s < DBNumTypes; s++)
 	    {
 		if (DBStdPaintEntry(s, t, pNum) != s)
 		    DBTypePaintPlanesTbl[t] |= PlaneNumToMaskBit(pNum);
 		if (DBStdEraseEntry(s, t, pNum) != s)
-		DBTypeErasePlanesTbl[t] |= PlaneNumToMaskBit(pNum);
+		    DBTypeErasePlanesTbl[t] |= PlaneNumToMaskBit(pNum);
 	    }
     }
 }
@@ -311,7 +314,9 @@ void
 dbComposeContacts()
 {
     LayerInfo *lpImage, *lpPaint;
-    int n, m;
+    int m, pNum;
+    TileTypeBitMask *rmask;
+    TileType n, ttype, itype, presult, eresult;
 
     for (m = 0; m < dbNumContacts; m++)
     {
@@ -324,6 +329,31 @@ dbComposeContacts()
 	    dbComposeEraseContact(lpImage, lpPaint);
 	}
     }
+
+    /* For stacking types, determine the result of painting or erasing	*/
+    /* each stacking type by applying the paint and erase results of	*/
+    /* each of its residues in sequence.				*/
+
+    for (itype = 0; itype < DBNumTypes; itype++)
+    {
+	for (n = DBNumUserLayers; n < DBNumTypes; n++)
+	{
+	    lpPaint = &dbLayerInfo[n];	/* Layer being painted or erased */
+	    rmask = &lpPaint->l_residues;
+	    for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+	    {
+		presult = eresult = itype;
+		for (ttype = TT_TECHDEPBASE; ttype < DBNumUserLayers; ttype++)
+		    if (TTMaskHasType(rmask, ttype))
+		    {
+			presult = DBStdPaintEntry(presult, ttype, pNum);
+			eresult = DBStdEraseEntry(eresult, ttype, pNum);
+		    }
+		SETPAINT(itype, n, pNum, presult);
+		SETERASE(itype, n, pNum, eresult);
+	    }
+	}
+    }
 }
 
 
@@ -377,11 +407,19 @@ dbComposePaintContact(lpImage, lpPaint)
 	    TTMaskZero(&rmask);
 	    TTMaskSetMask3(&rmask, &lpImage->l_residues, &lpPaint->l_residues);
 	    dbTechMatchResidues(&rmask, &cmask, TRUE);
-	    if (TTMaskIsZero(&cmask))
+
+	    /* Implicitly-defined stacking types override any	*/
+	    /* explicitly-defined types, or havoc results.	*/
+	    /* This allows one to create a type such as "pad"	*/
+	    /* having the same residues as "m123c" without	*/
+	    /* magic confusing them in the paint tables.	*/
+
+	    newtype = DBTechFindStacking(lpImage->l_type, lpPaint->l_type);
+
+	    if (TTMaskIsZero(&cmask) || (newtype != -1))
 	    {
 		/* If there is a stacking contact type, use it */
 
-		newtype = DBTechFindStacking(lpImage->l_type, lpPaint->l_type);
 		if (newtype >= DBNumUserLayers)
 		{
 		    pshared = lpImage->l_pmask & lpPaint->l_pmask;
@@ -391,16 +429,16 @@ dbComposePaintContact(lpImage, lpPaint)
 					pNum, newtype);
 		}
 
-		else if (lpPaint->l_isContact &&
-			((lpImage->l_type < DBNumUserLayers) ||
-			!TTMaskHasType(&lpImage->l_residues, lpPaint->l_type)))
+		else if (lpPaint->l_isContact && (lpImage->l_type < DBNumUserLayers))
 		{
 		    /* Original contact is replaced by the new one where
 		     * the planes overlap, and is dissolved into its
 		     * residues where they don't.
+		     * In this condition, the residues of image must be
+		     * non-contact types.
 		     */
 
-		    for (itype = TT_TECHDEPBASE; itype < DBNumTypes; itype++)
+		    for (itype = TT_TECHDEPBASE; itype < DBNumUserLayers; itype++)
 		    {
 			if (TTMaskHasType(&lpImage->l_residues, itype))
 			{
@@ -417,6 +455,49 @@ dbComposePaintContact(lpImage, lpPaint)
 			}
 		    }
 		}
+		else if (lpPaint->l_isContact &&
+			!TTMaskHasType(&lpImage->l_residues, lpPaint->l_type))
+		{
+		    /* Original contact is replaced by the new one where
+		     * the planes overlap, and is dissolved into its
+		     * residues where they don't.
+		     * In this condition, the residues of image are contact
+		     * types.
+		     */
+
+		    /* Temporary:  equals the condition below.  HOWEVER, */
+		    /* we should consider the case here where a type may */
+		    /* change due to incompatible residues, e.g., paint	 */
+		    /* pc over pd12c (pd12c becomes pm12c).  But maybe	 */
+		    /* this should be treated by the "compose" section	 */
+		    /* rules.						 */
+		
+/*
+		    SETPAINT(lpImage->l_type, lpPaint->l_type,
+				DBPlane(lpImage->l_type), lpImage->l_type);
+*/
+
+		    /* Do nothing?  These conditions don't seem to apply. */
+
+		    for (itype = TT_TECHDEPBASE; itype < DBNumUserLayers; itype++)
+		    {
+			if (TTMaskHasType(&lpImage->l_residues, itype))
+			{
+		            if (TTMaskHasType(&lpPaint->l_residues, itype))
+			    {
+				SETPAINT(lpImage->l_type, lpPaint->l_type,
+						DBPlane(itype), lpPaint->l_type);
+			    }
+/*
+			    else
+			    {
+				SETPAINT(lpImage->l_type, lpPaint->l_type,
+						DBPlane(itype), itype);
+			    }
+*/
+			}
+		    }
+		}
 		else
 		{
 		    /* Painting a residue type on top of a contact	*/
