@@ -17,26 +17,26 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/database/DBundo.c,v 1.3 2001/03/29 19:50:06 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/database/DBundo.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <string.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/malloc.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "database/databaseInt.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "utils/utils.h"
 #include "drc/drc.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 
 /***
  *** Identifiers for each of the clients defined here
@@ -553,8 +553,7 @@ dbUndoLabelBack(up)
 	 * The remainder contains a copy of the important
 	 * information from the CellUse.
 	 */
-	unsigned	 cue_expandMask;
-	unsigned	 cue_flags;
+	unsigned int	 cue_expandMask;
 	Transform	 cue_transform;
 	ArrayInfo	 cue_array;
 	CellDef		*cue_def;
@@ -612,7 +611,6 @@ DBUndoCellUse(use, action)
     up->cue_array = use->cu_array;
     up->cue_def = use->cu_def;
     up->cue_parent = use->cu_parent;
-    up->cue_flags = use->cu_flags;
     up->cue_expandMask = use->cu_expandMask;
     up->cue_bbox = use->cu_bbox;
     strcpy(up->cue_id, use->cu_id);
@@ -647,7 +645,6 @@ dbUndoCellForw(up)
 	    use = DBCellNewUse(up->cue_def, (char *) NULL);
 	    use->cu_transform = up->cue_transform;
 	    use->cu_array = up->cue_array;
-	    use->cu_flags = up->cue_flags;
 	    use->cu_expandMask = up->cue_expandMask;
 	    use->cu_bbox = up->cue_bbox;
 	    use->cu_id = StrDup((char **) NULL, up->cue_id);
@@ -708,7 +705,6 @@ dbUndoCellBack(up)
 	    use = DBCellNewUse(up->cue_def, (char *) NULL);
 	    use->cu_transform = up->cue_transform;
 	    use->cu_array = up->cue_array;
-	    use->cu_flags = up->cue_flags;
 	    use->cu_expandMask = up->cue_expandMask;
 	    use->cu_bbox = up->cue_bbox;
 	    use->cu_id = StrDup((char **) NULL, up->cue_id);
