@@ -17,18 +17,13 @@
  *
  * Needs to include: magic.h, tile.h, database.h
  *
- * rcsid $Header: /ufs/repository/magic/database/databaseInt.h,v 1.4 2001/04/16 14:31:40 tim Exp $
+ * rcsid $Header: /usr/cvsroot/magic/database/databaseInt.h,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $
  */
 
-#ifndef	_MAGIC
-int err0 = Need_to_include_misc/magic_header;
-#endif	/* _MAGIC */
-#ifndef	_TILES
-int err0 = Need_to_include_tiles/tile_header;
-#endif	/* _TILES */
-#ifndef	_DATABASE
-int err0 = Need_to_include_database/database_header;
-#endif	/* _DATABASE */
+#ifndef _DATABASEINT_H
+#define _DATABASEINT_H
+
+#include "database/database.h"
 
 /* ----------- Argument to area search when writing out cell ---------- */
 
@@ -199,6 +194,7 @@ extern void DBUndoPutLabel();
 extern void DBUndoEraseLabel();
 extern void DBUndoCellUse();
 extern void DBStampMismatch();
+extern void DBTechAddNameToType();
 
 extern void dbComputeBbox();
 extern void dbFreeCellPlane();
@@ -238,3 +234,5 @@ extern TileTypeBitMask dbNotDefaultPaintTbl[];
 	((t) != TT_SPACE && DBStdPaintEntry((t), (s), DBPlane(t)) != (t))
 #define	ERASEAFFECTS(t, s) \
 	((t) != TT_SPACE && DBStdEraseEntry((t), (s), DBPlane(t)) != (t))
+
+#endif /* _DATABASEINT_H */
