@@ -17,18 +17,19 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdINTERP.c,v 1.8 2001/03/29 19:50:05 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/dbwind/DBWcommands.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "textio/textio.h"
 #include "textio/txcommands.h"
@@ -39,14 +40,14 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdINTERP.c,v 1.8
 
 extern void CmdAddPath(), CmdArray();
 extern void CmdBox(), CmdCellname();
-extern void CmdClockwise(), CmdCopy(), CmdCorner();
+extern void CmdClockwise(), CmdCopy(), CmdCorner(), CmdCrosshair();
 extern void CmdDelete(), CmdDown(), CmdDrc(), CmdDump();
 extern void CmdEdit(), CmdElement(), CmdErase(), CmdExpand(), CmdExtract();
 extern void CmdFeedback(), CmdFill(), CmdFindBox(), CmdFindLabel(), CmdFlush();
 extern void CmdGetcell(), CmdGrid(), CmdIdentify();
 extern void CmdLabel(), CmdLoad();
-extern void CmdMove(), CmdPaint(), CmdPath(), CmdPlow();
-extern void CmdPort(), CmdProperty();
+extern void CmdMove(), CmdNetlist(), CmdPaint(), CmdPath(), CmdPlow();
+extern void CmdPolygon(), CmdPort(), CmdProperty();
 extern void CmdSave(), CmdScaleGrid(), CmdSee();
 extern void CmdSelect(), CmdSideways();
 extern void CmdShell(), CmdSnap();
@@ -254,6 +255,9 @@ DBWInitCommands()
 	"corner d1 d2 [layers]	make L-shaped wires inside box, filling\n"
 	"			first in direction d1, then in d2",
 	CmdCorner, FALSE);
+    WindAddCommand(DBWclientID,
+	"crosshair x y | off	enable and move or disable the screen crosshair",
+	CmdCrosshair, FALSE);		
     WindAddCommand(DBWclientID,
 	"delete			delete everything in selection",
 	CmdDelete, FALSE);
@@ -278,7 +282,7 @@ DBWInitCommands()
 	"element option	add a generic drawing element to the layout",
 	CmdElement, FALSE);
     WindAddCommand(DBWclientID,
-	"erase [layers]		erase mask information",
+	"erase [layers]|cursor		erase mask information",
 	CmdErase, FALSE);
     WindAddCommand(DBWclientID,
 	"expand [toggle]		expand everything under box, or toggle\n\
@@ -330,6 +334,9 @@ DBWInitCommands()
 	"grid [xSpacing [ySpacing [xOrigin yOrigin]]]\n\
 			toggle grid on/off (and set parameters)",
 	CmdGrid, FALSE);
+    WindAddCommand(DBWclientID,
+	"id	OR",
+	CmdIdentify, FALSE);
     WindAddCommand(DBWclientID,
 	"identify use_id		set the use identifier of the selected cell",
 	CmdIdentify, FALSE);
@@ -355,8 +362,14 @@ DBWInitCommands()
 			down at cursor position; second form will\n\
 			put box at location x y",
 	CmdMove, FALSE);
+    /* Added by NP 10/28/2004 */
     WindAddCommand(DBWclientID,
-	"paint layers		paint mask information",
+	"netlist option		netlist operation; type \"netlist help\"\n\
+			for information on options",
+	CmdNetlist, FALSE);
+
+    WindAddCommand(DBWclientID,
+	"paint layers|cursor		paint mask information",
 	CmdPaint, FALSE);
     WindAddCommand(DBWclientID,
 	"path [search|cell|sys] [[+]path]\n\
@@ -370,6 +383,10 @@ DBWInitCommands()
 	"port [index] [direction...]\n\
 			declare a label to be a subcircuit port",
 	CmdPort, FALSE);
+    WindAddCommand(DBWclientID,
+	"polygon type x1 y1 x2 y2 [x3 y3. . .] xn yn\n\
+			draw a polygon from a list of point pairs",
+	CmdPolygon, FALSE);
     WindAddCommand(DBWclientID,
 	"property [name] [value]\n\
 			add a property (string) to the edit cell",
@@ -411,10 +428,12 @@ DBWInitCommands()
 	"tech option	technology handling; type \"techinfo help\"\n\
 			for information on options",
 	CmdTech, FALSE);
+#ifndef MAGIC_WRAPPER
     WindAddCommand(DBWclientID,
 	"tool [name|info]	change layout tool or print info about what\n\
 			buttons mean for current tool",
 	CmdTool, FALSE);
+#endif
     WindAddCommand(DBWclientID,
 	"unexpand		unexpand subcells under box",
 	CmdUnexpand, FALSE);
