@@ -18,34 +18,34 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/dbwind/DBWprocs.c,v 1.4 2001/01/12 22:12:24 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/dbwind/DBWprocs.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <string.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "windows/windows.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "dbwind/dbwind.h"
 #include "graphics/graphics.h"
 #include "textio/textio.h"
 #include "textio/txcommands.h"
 #include "utils/utils.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "graphics/glyphs.h"
 #include "utils/malloc.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 
 global WindClient DBWclientID;
-static dbwBitMask = 0;
-#define MAX_BITMASK	0177777
-#define MAX_BITS_IN_MASK 16
+static int dbwBitMask = 0;
+#define MAX_BITMASK MAXINT
+#define MAX_BITS_IN_MASK (sizeof(unsigned int) * 8 - 1)
 
 extern void DBWredisplay();		/* Defined in DBWdisplay.c */
 
@@ -232,6 +232,7 @@ dbwLoadFunc(w, clientData)
  * ----------------------------------------------------------------------------
  */
 
+void
 DBWreload(name)
     char *name;
 {
@@ -241,6 +242,7 @@ DBWreload(name)
 		dbwReloadFunc, (ClientData) name);
 }
 
+int
 dbwReloadFunc(w, name)
     MagWindow *w;
     char *name;
@@ -496,7 +498,10 @@ DBWcommands(w, cmd)
     }
 
     UndoNext();
-    DBFixMismatch();		/* Update timestamps now that it's safe. */
+
+    /* Update timestamps now that it's safe */
+
+    DBFixMismatch();
 }
 
 /*
@@ -629,5 +634,9 @@ DBWinit()
     ASSERT(initialWindow != (MagWindow *) NULL, "DBWinit");
 #endif
 
+    /* Other initialization routines in the DBW package */
+    dbwElementInit();
+    dbwCrosshairInit();
+
     UndoEnable();
 }
