@@ -20,18 +20,18 @@
  */
 
 #ifndef	lint
-static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCbasic.c,v 1.5 2001/11/29 20:50:09 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/drc/DRCbasic.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif	/* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "drc/drc.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 
 int dbDRCDebug = 0;
 
@@ -65,6 +65,8 @@ extern int drcTile();
  *
  * Results:
  *	Zero (so that the search will continue).
+ *	1 if the rule is a trigger rule and an error rectangle is present
+ *	(the trigger rule does not create error paint).
  *
  * Side effects:
  *      Applies the function passed as an argument.
@@ -79,9 +81,13 @@ areaCheck(tile, arg)
 {
     Rect rect;		/* Area where error is to be recorded. */
 
-    /* If the tile has a legal type, then return. */
-
 #ifdef NONMANHATTAN
+
+    /* If the tile has a legal type, then return.  This is where we	*/
+    /* would want to add extra processing to check whether the half of	*/
+    /* the tile that is of an illegal type is inside or outside the	*/
+    /* area of interest.						*/
+
     if (IsSplit(tile))
     {
 	if (TTMaskHasType(&arg->dCD_cptr->drcc_mask, SplitLeftType(tile)) &&
@@ -92,12 +98,15 @@ areaCheck(tile, arg)
 #endif
     if (TTMaskHasType(&arg->dCD_cptr->drcc_mask, TiGetType(tile))) return 0;
 
+    TiToRect(tile, &rect);
+
     /* Only consider the portion of the suspicious tile that overlaps
-     * the clip area for errors.
+     * the clip area for errors, unless this is a trigger rule.
      */
 
-    TiToRect(tile, &rect);
-    GeoClip(&rect, arg->dCD_clip);
+    if (!(arg->dCD_cptr->drcc_flags & DRC_TRIGGER))
+	GeoClip(&rect, arg->dCD_clip);
+
     GeoClip(&rect, arg->dCD_constraint);
     if ((rect.r_xbot >= rect.r_xtop) || (rect.r_ybot >= rect.r_ytop))
 	return 0;
@@ -133,10 +142,13 @@ areaCheck(tile, arg)
 	}
     }
 
+    if (arg->dCD_cptr->drcc_flags & DRC_TRIGGER)
+	return 1;
+
     (*(arg->dCD_function)) (arg->dCD_celldef, &rect,
 	arg->dCD_cptr, arg->dCD_clientData);
     (*(arg->dCD_errors))++;
-    return (0);
+    return 0;
 }
 
 /*
@@ -253,6 +265,8 @@ drcTile (tile, arg)
     Rect errRect;		/* Area checked for an individual rule */
     TileTypeBitMask tmpMask, *rMask;
     unsigned char stackpending = (char)0;
+    bool triggered;		/* Hack for widespacing rule */
+    int cdist, ccdist, result;
 
     arg->dCD_constraint = &errRect;
     arg->dCD_radial = 0; 
@@ -391,11 +405,19 @@ drcTile (tile, arg)
 	    {
 		if (stackpending && (to == tt)) goto hnextstacktype;
 		
+		triggered = FALSE;
 		for (cptr = DRCRulesTbl[to][tt]; cptr != (DRCCookie *) NULL;
 			cptr = cptr->drcc_next)
 		{
+		    if (cptr->drcc_flags & DRC_CATECORNER)
+		    {
+			ccdist = cptr->drcc_cdist;
+			cdist = cptr->drcc_dist;
+		    }
+		    else
+			cdist = cptr->drcc_cdist;
 
-		    /* Contact images exist in multple planes; only check	*/
+		    /* Contact images exist in multiple planes; only check	*/
 		    /* rules that are for the plane being checked.		*/
 		    /* NOTE:  Ideally, the plane of the edge should be recorded	*/
 		    /* in the rule; i.e., we should have cptr->drcc_edgeplane	*/
@@ -403,11 +425,32 @@ drcTile (tile, arg)
 		    /* following eliminates most of the redundant checks.	*/
 		
 		    if ((!(cptr->drcc_flags & DRC_XPLANE)) &&
-				(arg->dCD_plane != cptr->drcc_plane)) continue;
+				(arg->dCD_plane != cptr->drcc_plane))
+		    {
+			if (cptr->drcc_flags & DRC_TRIGGER) cptr = cptr->drcc_next;
+			continue;
+		    }
 
 		    DRCstatRules++;
-		    errRect.r_ytop = edgeTop;
-		    errRect.r_ybot = edgeBot;
+		    if (triggered)
+		    {
+			/* Widespacing hack---rule is triggered by MAXWIDTH	*/
+			/* algorithm, but we only want to look in the region	*/
+			/* of the maxwidth rectangle.  Otherwise, a violation	*/
+			/* is incorrectly flagged on thin wires that happen to	*/
+			/* share a common edge with the maxwidth region.	*/
+
+			errRect.r_ytop += cdist;
+			errRect.r_ybot -= cdist;
+			if (errRect.r_ytop > edgeTop) errRect.r_ytop = edgeTop;
+			if (errRect.r_ybot < edgeBot) errRect.r_ybot = edgeBot;
+			triggered = FALSE;
+		    }
+		    else
+		    {
+			errRect.r_ytop = edgeTop;
+			errRect.r_ybot = edgeBot;
+		    }
 
 #ifdef DRC_EXTENSIONS
 		    if (cptr->drcc_flags & DRC_AREA)
@@ -415,9 +458,41 @@ drcTile (tile, arg)
 			drcCheckArea(tile,arg,cptr);
 			continue;
 		    }
-		    if (cptr->drcc_flags & DRC_MAXWIDTH)
+		    if (cptr->drcc_flags & (DRC_MAXWIDTH | DRC_BENDS))
 		    {
-			drcCheckMaxwidth(tile,arg,cptr);
+			/* New algorithm --- Tim 3/6/05 */
+			Rect *lr;
+
+			if (cptr->drcc_flags & DRC_REVERSE)
+			    lr = drcCanonicalMaxwidth(tpleft, GEO_WEST, arg, cptr);
+			else
+			    lr = drcCanonicalMaxwidth(tile, GEO_EAST, arg, cptr);
+		        if (cptr->drcc_flags & DRC_TRIGGER)
+			{
+			    if (lr)
+			    {
+			        errRect = *lr;
+				triggered = TRUE;
+			    }
+			    else
+				cptr = cptr->drcc_next;
+			}
+			else if (lr)
+			{
+		            GeoClip(lr, arg->dCD_clip);
+			    if (!GEO_RECTNULL(lr))  
+			    {
+				(*(arg->dCD_function)) (arg->dCD_celldef,
+					lr, cptr, arg->dCD_clientData);
+				(*(arg->dCD_errors))++;
+			    }
+			}
+			continue;
+		    }
+		    else if (cptr->drcc_flags & DRC_MAXWIDTH)
+		    {
+			/* bends_illegal option only */
+			drcCheckMaxwidth(tile, arg, cptr);
 			continue;
 		    }
 		    if (cptr->drcc_flags & DRC_RECTSIZE)
@@ -431,16 +506,6 @@ drcTile (tile, arg)
 			continue;
 		    }
 #endif
-		    /* The widespacing rule needs to check if the	*/
-		    /* triggering edge is narrow to avoid generating	*/
-		    /* errors on long, narrow wires.			*/
-
-		    if (cptr->drcc_flags & DRC_TRIGGER)
-			if ((edgeTop - edgeBot) < cptr->drcc_dist)
-			{
-			    cptr = cptr->drcc_next;
-			    continue;
-			}
 
 		    if (cptr->drcc_flags & DRC_REVERSE)
 		    {
@@ -452,7 +517,7 @@ drcTile (tile, arg)
 			    /* Nothing */;
 			if (TTMaskHasType(&cptr->drcc_corner, TiGetTopType(tp)))
 		 	{
-			    errRect.r_ybot -= cptr->drcc_cdist;
+			    errRect.r_ybot -= cdist;
 			    if (DRCEuclidean)
 				arg->dCD_radial |= 0x1000;
 			}
@@ -468,7 +533,7 @@ drcTile (tile, arg)
 				    /* Nothing */;
 			    if (TTMaskHasType(&cptr->drcc_corner, TiGetBottomType(tp)))
 			    {
-				errRect.r_ytop += cptr->drcc_cdist;
+				errRect.r_ytop += cdist;
 				if (DRCEuclidean)
 				    arg->dCD_radial |= 0x2000;
 			    }
@@ -485,7 +550,11 @@ drcTile (tile, arg)
 				&& !(cptr->drcc_flags & DRC_XPLANE)
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetRightType(tpleft)))
+			{
+			    if (cptr->drcc_flags & DRC_TRIGGER)
+				cptr = cptr->drcc_next;
 			    continue;
+			}
 			errRect.r_xtop = edgeX;
 			arg->dCD_initial = tile;
 #ifdef DRC_EXTENSIONS
@@ -502,7 +571,7 @@ drcTile (tile, arg)
 			    /* Nothing */;
 			if (TTMaskHasType(&cptr->drcc_corner, TiGetBottomType(tp)))
 			{
-			    errRect.r_ytop += cptr->drcc_cdist;
+			    errRect.r_ytop += cdist;
 			    if (DRCEuclidean)
 				arg->dCD_radial |= 0x8000;
 			}
@@ -518,7 +587,7 @@ drcTile (tile, arg)
 				    /* Nothing */;
 			    if (TTMaskHasType(&cptr->drcc_corner, TiGetTopType(tp)))
 			    {
-				errRect.r_ybot -= cptr->drcc_cdist;
+				errRect.r_ybot -= cdist;
 				if (DRCEuclidean)
 				    arg->dCD_radial |= 0x4000;
 			    }
@@ -535,7 +604,11 @@ drcTile (tile, arg)
 				&& !(cptr->drcc_flags & DRC_XPLANE)
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetLeftType(tile)))
+			{
+			    if (cptr->drcc_flags & DRC_TRIGGER)
+				cptr = cptr->drcc_next;
 			    continue;
+			}
 			errRect.r_xbot = edgeX;
 			arg->dCD_initial= tpleft;
 #ifdef DRC_EXTENSIONS
@@ -545,26 +618,44 @@ drcTile (tile, arg)
 		    if (arg->dCD_radial)
 		    {
 			arg->dCD_radial &= 0xf000;
-			arg->dCD_radial |= (0xfff & cptr->drcc_cdist);
-		    }
-
-		    /* Trigger rule:  If rule check passed without	*/
-		    /* errors, do the next rule.  Otherwise, skip	*/
-		    /* it.						*/
-
-		    if (cptr->drcc_flags & DRC_TRIGGER)
-		    {
-			if (!GEO_RECTNULL(&errRect))
-			    cptr = cptr->drcc_next;  /* skip triggered rule */
-			continue;	/* don't generate any errors on trigger rule */
+			arg->dCD_radial |= (0xfff & cdist);
 		    }
 
 		    DRCstatSlow++;
 		    arg->dCD_cptr = cptr;
 		    TTMaskCom2(&tmpMask, &cptr->drcc_mask);
-		    (void) DBSrPaintArea((Tile *) NULL,
+
+		    if (cptr->drcc_flags & DRC_CATECORNER)
+		    {
+			int tmpy = errRect.r_ybot;
+			errRect.r_ybot = edgeTop + ccdist;
+			if (errRect.r_ybot < errRect.r_ytop)
+			{
+			    result = 0; /* to be finished */
+			}
+			errRect.r_ybot = tmpy;
+			tmpy = errRect.r_ytop;
+			errRect.r_ytop = edgeBot - ccdist;
+			if (errRect.r_ytop > errRect.r_ybot)
+			{
+			    result = 0; /* to be finished */
+			}
+
+			/* (to be done) */
+		    }
+		    else 
+			result = DBSrPaintArea((Tile *) NULL,
 				arg->dCD_celldef->cd_planes[cptr->drcc_plane],
 				&errRect, &tmpMask, areaCheck, (ClientData) arg);
+
+		    if (result == 0)
+		    {
+		        /* Trigger rule:  If rule check found errors,	*/
+		        /* do the next rule.  Otherwise, skip it.	*/
+
+			if (cptr->drcc_flags & DRC_TRIGGER)
+			    cptr = cptr->drcc_next;
+		    }
 		}
 		DRCstatEdges++;
 
@@ -724,38 +815,83 @@ checkbottom:
 	    {
 		if (stackpending && (to == tt)) goto vnextstacktype;
 
+		triggered = FALSE;
 		for (cptr = DRCRulesTbl[to][tt]; cptr != (DRCCookie *) NULL;
 				cptr = cptr->drcc_next)
 		{
+		    if (cptr->drcc_flags & DRC_CATECORNER)
+		    {
+			ccdist = cptr->drcc_cdist;
+			cdist = cptr->drcc_dist;
+		    }
+		    else
+			cdist = cptr->drcc_cdist;
+
 		    /* Contact images exist in multple planes; only check	*/
 		    /* rules that are for the plane being checked.		*/
 		    if ((!(cptr->drcc_flags & DRC_XPLANE)) &&
-				(arg->dCD_plane != cptr->drcc_plane)) continue;
+				(arg->dCD_plane != cptr->drcc_plane))
+		    {
+			if (cptr->drcc_flags & DRC_TRIGGER) cptr = cptr->drcc_next;
+			continue;
+		    }
 
 		    DRCstatRules++;
-		    errRect.r_xbot = edgeLeft;
-		    errRect.r_xtop = edgeRight;
+		    if (triggered)
+		    {
+			errRect.r_xtop += cdist;
+			errRect.r_xbot -= cdist;
+			if (errRect.r_xtop > edgeRight) errRect.r_xtop = edgeRight;
+			if (errRect.r_xbot < edgeLeft) errRect.r_xbot = edgeLeft;
+			triggered = FALSE;
+		    }
+		    else
+		    {
+			errRect.r_xbot = edgeLeft;
+			errRect.r_xtop = edgeRight;
+		    }
 
 		    /* top to bottom */
 
 #ifdef DRC_EXTENSIONS
-		    if (cptr->drcc_flags & (DRC_AREA|DRC_MAXWIDTH|DRC_RECTSIZE))
+		    if (cptr->drcc_flags & (DRC_MAXWIDTH | DRC_BENDS))
+		    {
+			/* New algorithm --- Tim 3/6/05 */
+			Rect *lr;
+
+			if (cptr->drcc_flags & DRC_REVERSE)
+			    lr = drcCanonicalMaxwidth(tpbot, GEO_SOUTH, arg, cptr);
+			else
+			    lr = drcCanonicalMaxwidth(tile, GEO_NORTH, arg, cptr);
+		        if (cptr->drcc_flags & DRC_TRIGGER)
+			{
+			    if (lr)
+			    {
+				errRect = *lr;
+				triggered = TRUE;
+			    }
+			    else
+				cptr = cptr->drcc_next;
+			}
+			else if (lr)
+			{
+		            GeoClip(lr, arg->dCD_clip);
+			    if (!GEO_RECTNULL(lr))  
+			    {
+				(*(arg->dCD_function)) (arg->dCD_celldef,
+					lr, cptr, arg->dCD_clientData);
+				(*(arg->dCD_errors))++;
+			    }           
+			}
+			continue;
+		    }
+		    else if (cptr->drcc_flags & (DRC_AREA|DRC_RECTSIZE|DRC_MAXWIDTH))
 		    {
 			/* only have to do these checks in one direction */
 			if (cptr->drcc_flags & DRC_TRIGGER) cptr = cptr->drcc_next;
 			continue;
 		    }
 #endif
-		    /* The widespacing rule needs to check if the	*/
-		    /* triggering edge is narrow to avoid generating	*/
-		    /* errors on long, narrow wires.			*/
-
-		    if (cptr->drcc_flags & DRC_TRIGGER)
-			if ((edgeRight - edgeLeft) < cptr->drcc_dist)
-			{
-			    cptr = cptr->drcc_next;
-			    continue;
-			}
 
 		    if (cptr->drcc_flags & DRC_REVERSE)
 		    {
@@ -768,7 +904,7 @@ checkbottom:
 				/* Nothing */;
 		 	if (TTMaskHasType(&cptr->drcc_corner, TiGetLeftType(tp)))
 			{
-			    errRect.r_xtop += cptr->drcc_cdist; 	
+			    errRect.r_xtop += cdist;
 			    if (DRCEuclidean)
 				arg->dCD_radial |= 0x4000;
 			}
@@ -783,7 +919,7 @@ checkbottom:
 				/* Nothing */;
 			    if (TTMaskHasType(&cptr->drcc_corner, TiGetRightType(tp)))
 			    {
-				errRect.r_xbot -= cptr->drcc_cdist; 	
+				errRect.r_xbot -= cdist;
 				if (DRCEuclidean)
 				    arg->dCD_radial |= 0x1000;
 			    }
@@ -800,7 +936,11 @@ checkbottom:
 				&& !(cptr->drcc_flags & DRC_XPLANE)
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetTopType(tpbot)))
+			{
+			    if (cptr->drcc_flags & DRC_TRIGGER)
+				cptr = cptr->drcc_next;
 			    continue;
+			}
 			errRect.r_ytop = edgeY;
 			arg->dCD_initial = tile;
 #ifdef DRC_EXTENSIONS
@@ -817,9 +957,9 @@ checkbottom:
 			    for (tp = BL(tp); TOP(tp) < edgeY; tp = RT(tp))
 				/* Nothing */;
 
-			if (TTMaskHasType(&cptr->drcc_corner, TiGetTopType(tp)))
+			if (TTMaskHasType(&cptr->drcc_corner, TiGetRightType(tp)))
 			{
-			    errRect.r_xbot -= cptr->drcc_cdist;
+			    errRect.r_xbot -= cdist;
 			    if (DRCEuclidean)
 				arg->dCD_radial |= 0x2000;
 			}
@@ -832,9 +972,9 @@ checkbottom:
 			     */
 			    for (tp=tpbot; RIGHT(tp) <= errRect.r_xtop; tp=TR(tp))
 				/* Nothing */;
-			    if (TTMaskHasType(&cptr->drcc_corner, TiGetTopType(tp)))
+			    if (TTMaskHasType(&cptr->drcc_corner, TiGetLeftType(tp)))
 			    {
-				errRect.r_xtop += cptr->drcc_cdist;
+				errRect.r_xtop += cdist;
 				if (DRCEuclidean)
 				    arg->dCD_radial |= 0x8000;
 			    }
@@ -851,7 +991,11 @@ checkbottom:
 				&& !(cptr->drcc_flags & DRC_XPLANE)
 				&& TTMaskHasType(&cptr->drcc_mask,
 				TiGetBottomType(tile)))
+			{
+			    if (cptr->drcc_flags & DRC_TRIGGER)
+				cptr = cptr->drcc_next;
 			    continue;
+			}
 			errRect.r_ybot = edgeY;
 			arg->dCD_initial = tpbot;
 #ifdef DRC_EXTENSIONS
@@ -861,25 +1005,29 @@ checkbottom:
 		    if (arg->dCD_radial)
 		    {
 			arg->dCD_radial &= 0xf000;
-			arg->dCD_radial |= (0xfff & cptr->drcc_cdist);
-		    }
-		    /* Trigger rule:  If trigger check passed, do	*/
-		    /* the next rule.  Otherwise, skip it.		*/
-
-		    if (cptr->drcc_flags & DRC_TRIGGER)
-		    {
-			if (!GEO_RECTNULL(&errRect))
-			    cptr = cptr->drcc_next; /* skip triggered rule */
-			continue;	/* don't generate any errors on trigger rule */
+			arg->dCD_radial |= (0xfff & cdist);
 		    }
 
 		    DRCstatSlow++;
 		    arg->dCD_cptr = cptr;
 		    TTMaskCom2(&tmpMask, &cptr->drcc_mask);
-		    (void) DBSrPaintArea((Tile *) NULL,
+		    if (cptr->drcc_flags & DRC_CATECORNER)
+		    {
+			/* (to be done) */
+		    }
+		    else 
+			result = DBSrPaintArea((Tile *) NULL,
 				arg->dCD_celldef->cd_planes[cptr->drcc_plane],
 				&errRect, &tmpMask, areaCheck, (ClientData) arg);
 
+		    if (result == 0)
+		    {
+		        /* Trigger rule:  If rule check found errors,	*/
+		        /* do the next rule.  Otherwise, skip it.	*/
+
+			if (cptr->drcc_flags & DRC_TRIGGER)
+			    cptr = cptr->drcc_next;
+		    }
 		}
 		DRCstatEdges++;
 
