@@ -20,14 +20,14 @@
  */
 
 #ifndef	lint
-static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCcontin.c,v 1.2 2001/01/12 22:12:29 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/drc/DRCcontin.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
 #include <sys/types.h>
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "textio/textio.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -36,12 +36,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCcontin.c,v 1.2 2001
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "dbwind/dbwtech.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "drc/drc.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "graphics/graphics.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "utils/malloc.h"
 
 #ifdef MAGIC_WRAPPER
@@ -312,6 +312,31 @@ DRCRemovePending(def)
     }
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DRCPreInit --
+ *
+ *	Initialization stuff that has to come before TechLoad() when
+ *	magic starts up.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	None (memory should be nulled; this is a safeguard.)
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DRCPreInit()
+{
+    /* Make sure that the rule tables are initialized to NULL */
+
+    drcInitRulesTbl();
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -352,7 +377,7 @@ DRCInit()
     }
     DRCuse = DBCellNewUse (DRCdef, (char *) NULL);
     DBSetTrans (DRCuse, &GeoIdentityTransform);
-    DRCuse->cu_expandMask = -1;	/* This is always expanded. */
+    DRCuse->cu_expandMask = CU_DESCEND_SPECIAL;	/* This is always expanded. */
 
     /* Create a dummy cell use to use for passing to procedures
      * that need a use when all we've got is a def.
@@ -380,7 +405,7 @@ DRCInit()
     if (DRCStepSize == 0)
     {
 	if (TechHalo == 0) DRCStepSize = 64;
-	else DRCStepSize = 16*TechHalo;
+	else DRCStepSize = 16 * TechHalo;
     }
 
     /* Initialize mask of DRC layer types. */
@@ -479,7 +504,8 @@ DRCContinuous ()
     if (DRCStatus != DRC_NOT_RUNNING) return;	/* Avoid infinitely recursive loop */
     GrFlush();
     DRCStatus = DRC_IN_PROGRESS;
-    TxSetPrompt(']');
+    Tcl_EvalEx(magicinterp, "after idle magic::drcstate busy", -1, 0);
+    if (TxInputRedirect != TX_INPUT_REDIRECTED) TxSetPrompt(']');
     /* fprintf(stderr, "Starting DRC\n"); fflush(stderr); */
 #endif
 
@@ -532,8 +558,9 @@ DRCContinuous ()
 
 #ifdef MAGIC_WRAPPER
     DRCStatus = DRC_NOT_RUNNING;
+    Tcl_EvalEx(magicinterp, "after idle magic::drcstate idle", -1, 0);
     /* fprintf(stderr, "DRC is finished\n"); fflush(stderr); */
-    TxSetPrompt('%');
+    if (TxInputRedirect != TX_INPUT_REDIRECTED) TxSetPrompt('%');
 #endif
 
 checkDone:
