@@ -17,21 +17,21 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCtech.c,v 1.4 2001/01/12 22:12:29 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/drc/DRCtech.c,v 1.2 2005/06/01 21:41:40 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "drc/drc.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "textio/textio.h"
 #include "utils/malloc.h"
 #ifdef DRC_CIF_EXTENSIONS
@@ -42,7 +42,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCtech.c,v 1.4 2001/0
 /*
  * Largest DRC interaction radius in the given technology
  */
-global int TechHalo;
+global int TechHalo, MaxHalo;
+
 #ifdef DRC_CIF_EXTENSIONS
 CIFStyle        *drcCifStyle=NULL;
 #endif
@@ -208,6 +209,35 @@ drcFindBucket(i, j, distance)
     return dp;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * drcInitRulesTbl --
+ *
+ * First and only initialization to ensure that the DRCRulesTbl array is
+ * NULL on startup.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Memory is zeroed.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+drcInitRulesTbl()
+{
+    int i, j;
+
+    for (i = 0; i < TT_MAXTYPES; i++)
+	for (j = 0; j < TT_MAXTYPES; j++)
+	    DRCRulesTbl[i][j] = (DRCCookie *)NULL;
+
+    drcWhyList = NULL;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -230,7 +260,7 @@ DRCTechInit()
     DRCCookie *dp;
     PaintResultType result;
 
-    TechHalo = 0;
+    TechHalo = MaxHalo = 0;
     drcRulesOptimized = 0;
     drcRulesSpecified = 0;
     TTMaskZero(&DRCExactOverlapTypes);
@@ -258,6 +288,7 @@ DRCTechInit()
 	    dp = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 	    dp->drcc_dist = -1;
 	    dp->drcc_next = (DRCCookie *) NULL;
+	    TTMaskZero(&dp->drcc_mask);
 	    DRCRulesTbl[i][j] = dp;
 	}
     }
@@ -407,7 +438,7 @@ DRCTechAddRule(sectionName, argc, argv)
 #ifdef DRC_EXTENSIONS
         "area",		 5,	5,	drcArea,
     "layers area horizon why",
-        "maxwidth",	 5,	5,	drcMaxwidth,
+        "maxwidth",	 4,	5,	drcMaxwidth,
     "layers maxwidth bends why",
         "cifstyle",	 2,	2,	drcSetCifStyle,
     "cif_style",
@@ -543,11 +574,18 @@ drcWidth(argc, argv)
  * Process an area rule.
  * This is of the form:
  *
- *	area layers distance why
+ *	area layers area horizon why
  *
  * e.g,
  *
- *	area pmc 4 "poly contact area must be at least 4"
+ *	area pmc 4 2 "poly contact area must be at least 4"
+ *
+ * "area" is the total area in lambda^2.
+ *
+ * "horizon" is the halo distance for the check.  Normally, this would
+ * be the area (in lambda^2) divided by the minimum width rule (in
+ * lambda).  Anything larger would not be a violation as long as the
+ * minimum width rule is satisfied.
  *
  * Results:
  *	Returns distance.
@@ -620,10 +658,16 @@ drcArea(argc, argv)
  * Process a maxwidth rule.
  * This is of the form:
  *
- *	maxwidth layers distance bends why
+ *	maxwidth layers distance [bends] why
  *
- * e.g,
+ * This routine was updated 3/6/05 to match the "canonical" definition of
+ * a maxwidth region, which is any rectangle containing <layers> that is
+ * at least <distance> in both width and height.  If the keyword
+ * "bends_illegal" is present, then the definition reverts to the original
+ * (see below) for backwards-compatibility.  Otherwise ("bends_ok" or
+ * nothing), the new routine is used.
  *
+ *	maxwidth metal1 389 "metal1 width > 35um must be slotted"
  *	maxwidth pmc 4 bend_illegal "poly contact area must be at least 4"
  *	maxwidth trench 4 bend_ok "poly contact area must be at least 4"
  *
@@ -631,8 +675,9 @@ drcArea(argc, argv)
  *  		point in the region.  This is used for emitters and contacts
  *		that are rectangular (so we can't generate them with the
  *		squares command) and some exact width in one direction.
- *	bend_ok - used for things like trench, where the width is some fixed
- *		value:
+ *	bend_ok - Used mainly for wide metal rules where metal greater than
+ *		some given width must be slotted.  Also, used for things
+ *		like trench, where the width is some fixed value:
  *
  *			XXXXX		XXXXXX
  *			X   X		XXXXXX
@@ -658,7 +703,7 @@ drcMaxwidth(argc, argv)
     char *layers = argv[1];
     int distance = atoi(argv[2]);
     char *bends = argv[3];
-    char *why = drcWhyDup(argv[4]);
+    char *why;
     TileTypeBitMask set, setC, tmp1;
     DRCCookie *dp, *dpnew;
     TileType i, j;
@@ -676,13 +721,24 @@ drcMaxwidth(argc, argv)
 	return (0);
     }
 
-    if (strcmp(bends,"bend_illegal") == 0) bend =0;
-    else if (strcmp(bends,"bend_ok") == 0) bend =DRC_BENDS;
+    if (argc == 3)
+    {
+	bend = DRC_BENDS;
+	why = drcWhyDup(argv[3]);
+    }
     else
     {
-    	 TechError("unknown bend option %s\n",bends);
-	 return (0);
+	if (strcmp(bends,"bend_illegal") == 0) bend = 0;
+	else if (strcmp(bends,"bend_ok") == 0) bend = DRC_BENDS;
+	else
+	{
+	    TechError("unknown bend option %s\n",bends);
+	    return (0);
+	}
+	why = drcWhyDup(argv[4]);
     }
+
+    
     set = tmp1;
     TTMaskCom2(&setC, &set);
 
@@ -702,7 +758,7 @@ drcMaxwidth(argc, argv)
 		dp = drcFindBucket(i, j, distance);
 		dpnew = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
 		ASSIGN(dpnew, distance, dp->drcc_next, set, set, why,
-			    distance, DRC_MAXWIDTH|bend, plane);
+			    distance, DRC_MAXWIDTH | bend, plane);
 
 		dp->drcc_next = dpnew;
 	    }
@@ -724,6 +780,10 @@ drcMaxwidth(argc, argv)
  *
  *	spacing layers1 layers2 distance adjacency why
  *
+ * or (added 3/28/05)
+ *
+ *	spacing layers1 layers2 distance catecorner dist2 why
+ *
  * e.g,
  *
  *	spacing metal,pmc/m,dmc/m metal,pmc/m,dmc/m 4 touching_ok \
@@ -734,6 +794,16 @@ drcMaxwidth(argc, argv)
  * immediately adjacent to types in layers2.  In the second case,
  * such adjacency causes a violation.
  *
+ * The "catecorner" option is designed for two purposes: 1) to catch
+ * rule violations that are caused by a cifoutput grow rules followed
+ * by a shrink rule.  If a tile is in the catecorner position between
+ * distances (G-W) and G, where G is the amount grown, and W is the
+ * minimum width of the CIF layer, then a strip of less than W would
+ * result, and is therefore a DRC violation.  2) with dist2=0, can be
+ * used to indicate that a layer can have no outside corners; that is,
+ * corners must be beveled with a non-manhattan (usually 45 degree)
+ * angle.
+ *
  * Results:
  *	Returns distance.
  *
@@ -778,6 +848,7 @@ drcSpacing(argc, argv)
 
     int wwidth;
     bool widerule;
+    bool catecorner = FALSE;
 
     if (argc == 7)
     {
@@ -798,12 +869,29 @@ drcSpacing(argc, argv)
 	adjacency = argv[4];
 	why = drcWhyDup(argv[5]);
 	wwidth = distance;
+	if (!strcmp(adjacency, "catecorner"))
+	{
+	    wwidth = atoi(argv[5]);
+	    catecorner = TRUE;
+	}
 	widerule = FALSE;
     }
 
     DBTechNoisyNameMask(layers1, &set1);
-    planes1 = DBTechMinSetPlanes(set1, &set1);
     DBTechNoisyNameMask(layers2, &set2);
+
+    /* Add stacking types */
+    for (i = DBNumUserLayers; i < DBNumTypes; i++)
+    {
+	TileTypeBitMask *rmask = DBResidueMask(i);
+	TTMaskAndMask3(&tmp1, &set1, rmask);
+	if (!TTMaskIsZero(&tmp1))
+	    TTMaskSetType(&set1, i);
+	TTMaskAndMask3(&tmp2, &set2, rmask);
+	if (!TTMaskIsZero(&tmp2))
+	    TTMaskSetType(&set2, i);
+    }
+    planes1 = DBTechMinSetPlanes(set1, &set1);
     planes2 = DBTechMinSetPlanes(set2, &set2);
 
     /* Look for plane that contains image of all types in both
@@ -817,13 +905,17 @@ drcSpacing(argc, argv)
 	    break;
     }
 
-    if (strcmp (adjacency, "touching_ok") == 0)
+    if (!strcmp(adjacency, "touching_ok") || catecorner)
     {
 	/* If touching is OK, everything must fall in the same plane. */
 	if (plane == DBNumPlanes)
 	{
-	    TechError(
-		"Spacing check with touching ok must all be in one plane.\n");
+	    if (catecorner)
+	        TechError("Catecorner spacing check must have"
+			" all types in one plane.\n");
+	    else
+		TechError("Spacing check with touching ok must have"
+			" all types in one plane.\n");
 	    return (0);
 	}
 
@@ -854,7 +946,7 @@ drcSpacing(argc, argv)
 	if (!TTMaskEqual(&set1, &set2))
 	    needReverse = TRUE;
     }
-    else if (strcmp (adjacency, "touching_illegal") == 0)
+    else if (!strcmp(adjacency, "touching_illegal"))
     {
 	/* If touching is illegal, set1 and set2 should not intersect 
 	 * (adjacencies between types on the same plane would be missed)
@@ -918,23 +1010,22 @@ drcSpacing(argc, argv)
 			/* Create two contiguous rules, one for spacing */
 			/* and one for width.  These are created in	*/
 			/* reverse order due to the stack property of	*/
-			/* the linked list.  The spacing rule does not	*/
-			/* do corner extensions; otherwise, the wide	*/
-			/* spacing rule gets triggered at endcaps of	*/
-			/* long, narrow wires.				*/
+			/* the linked list.				*/
 
 		        ASSIGN(dpnew, distance, dp->drcc_next, tmp1, tmp2,
 				why, distance, DRC_FORWARD, curPlane);
-
-			dptrig = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
+			dptrig = (DRCCookie *) mallocMagic((unsigned)
+				(sizeof (DRCCookie)));
 		        ASSIGN(dptrig, wwidth, dpnew, set1, set1, why,
-				wwidth, DRC_REVERSE | DRC_TRIGGER, curPlane);
+				wwidth, DRC_REVERSE | DRC_MAXWIDTH | DRC_TRIGGER,
+				curPlane);
+
 			dp->drcc_next = dptrig;
 		    }
 		    else
 		    {
 			ASSIGN(dpnew, distance, dp->drcc_next,
-				tmp1, tmp2, why, distance, DRC_FORWARD,
+				tmp1, tmp2, why, wwidth, DRC_FORWARD,
 				curPlane);
 			dp->drcc_next = dpnew;
 		    }
@@ -953,6 +1044,9 @@ drcSpacing(argc, argv)
 		    if (needReverse)
 		        dpnew->drcc_flags |= DRC_BOTHCORNERS;
 
+		    if (catecorner)
+			dpnew->drcc_flags |= DRC_CATECORNER;
+
 		    if (needReverse)
 		    {
 			/* Add check in reverse direction, 
@@ -974,15 +1068,17 @@ drcSpacing(argc, argv)
 			    ASSIGN(dpnew, distance, dp->drcc_next,
 				tmp1, tmp2, why, distance,
 				DRC_REVERSE | DRC_BOTHCORNERS, curPlane);
-			    dptrig = (DRCCookie *) mallocMagic((unsigned) (sizeof (DRCCookie)));
+			    dptrig = (DRCCookie *) mallocMagic((unsigned)
+				(sizeof (DRCCookie)));
 			    ASSIGN(dptrig, wwidth, dpnew, set1, set1, why,
-				wwidth, DRC_FORWARD | DRC_TRIGGER, curPlane);
+				wwidth, DRC_FORWARD | DRC_MAXWIDTH | DRC_TRIGGER,
+				curPlane);
 			    dp->drcc_next = dptrig;
 			}
 			else
 			{
 			    ASSIGN(dpnew,distance,dp->drcc_next,
-				tmp1, tmp2, why, distance, 
+				tmp1, tmp2, why, wwidth, 
 				DRC_REVERSE | DRC_BOTHCORNERS, curPlane);
 			    dp->drcc_next = dpnew;
 			}
@@ -1030,6 +1126,9 @@ drcSpacing(argc, argv)
 		    if (needReverse)
 		        dpnew->drcc_flags |= DRC_BOTHCORNERS;
 
+		    if (catecorner)
+			dpnew->drcc_flags |= DRC_CATECORNER;
+
 		    dp->drcc_next = dpnew;		    
 
 		    if (needReverse)
@@ -2143,15 +2242,22 @@ DRCTechFinal()
 		    DBTypeShortName(i), DBTypeShortName(j),
 		    next->drcc_why, dp->drcc_why);
 		*/
-		dp2back = &(DRCRulesTbl [i][j]);
+		dp2back = &(DRCRulesTbl[i][j]);
 		while (*dp2back != dp)
 		    dp2back = &(*dp2back)->drcc_next;
 
 		/* Trigger rules */
 		if (dptrig != NULL)
 		{
+		    dptrig = dp->drcc_next;
 		    freeMagic((char *)dp->drcc_next);
 		    *dp2back = dp->drcc_next->drcc_next;
+
+		    /* Replace this entry so on the next cycle	*/
+		    /* dp will be the next rule.  This works	*/
+		    /* even though dp is free'd (below), due to	*/
+		    /* the one-delayed free mechanism.		*/
+		    dp->drcc_next = *dp2back;
 		}
 		else
 		    *dp2back = dp->drcc_next;
@@ -2188,6 +2294,9 @@ DRCTechFinal()
 			}
 		}
     }
+
+    /* Set maximum halo */
+    MaxHalo = TechHalo;
 }
 
 
@@ -2288,6 +2397,9 @@ DRCTechScale(scalen, scaled)
 
     TechHalo *= scaled;
     TechHalo /= scalen;
+
+    MaxHalo *= scaled;
+    MaxHalo /= scalen;
 }
 
 /* The following routines are used by the "tech" command (and in other places,
