@@ -13,7 +13,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/ext2sim/ext2sim.c,v 1.5 2001/07/06 17:58:30 rajit Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/ext2sim/ext2sim.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -23,7 +23,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/ext2sim/ext2sim.c,v 1.5 20
 #include <math.h>	/* for sqrt() in bipolar L,W calculation */
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
 #include "utils/utils.h"
@@ -45,6 +45,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/ext2sim/ext2sim.c,v 1.5 20
 #ifdef MAGIC_WRAPPER
 void CmdExtToSim();
 #endif
+bool simnAP();
+bool simnAPHier();
 int simmainArgs();
 int simdevVisit(), simresistVisit(), simcapVisit(), simnodeVisit();
 int simmergeVisit();
@@ -432,7 +434,7 @@ CmdExtToSim(w, cmd)
 	    if (EFSearchPath)
 	    {
 		freeMagic(EFSearchPath);
-		EFArgTech = NULL;
+		EFSearchPath = NULL;
 	    }
 	    break;
 	case EXTTOSIM_RUN:
@@ -451,6 +453,7 @@ usage:
 runexttosim:
 
     esDevsMerged = 0;
+    esFMIndex = 0;
 
     EFInit();
 
@@ -1020,8 +1023,11 @@ simdevVisit(dev, hierName, trans)
 	    break;
     }
 
-    /* Output gate, source, and drain node names */
-    simdevOutNode(hierName, gate->dterm_node->efnode_name->efnn_hier, name, esSimF);
+    /* Output gate node name.  Resistor devices don't generate this node. */
+    if (dev->dev_class != DEV_RES)
+       simdevOutNode(hierName, gate->dterm_node->efnode_name->efnn_hier, name, esSimF);
+
+    /* Output source and drain node names */
     if (dev->dev_nterm > 1)
 	simdevOutNode(hierName, source->dterm_node->efnode_name->efnn_hier, name, esSimF);
 
@@ -1049,8 +1055,8 @@ simdevVisit(dev, hierName, trans)
     GeoTransRect(trans, &dev->dev_rect, &r);
     scale = GeoScale(trans);
 
-    if (dev->dev_class == DEV_BJT || EFDevTypes[dev->dev_type][0] == 'b') {
-
+    if (dev->dev_class == DEV_BJT || EFDevTypes[dev->dev_type][0] == 'b')
+    {
        /* Bipolar sim format:  We don't have the length and width
 	* of the collector well, but we can get it from the area
 	* and perimeter measurements; hopefully any strict netlist
@@ -1223,7 +1229,7 @@ FILE *outf;
  * ----------------------------------------------------------------------------
  */
  
-int simnAP(node, resClass, scale, outf)
+bool simnAP(node, resClass, scale, outf)
 EFNode *node;
 int  resClass, scale;
 FILE *outf;
@@ -1246,7 +1252,7 @@ FILE *outf;
 	return TRUE;
 }
 
-int simnAPHier(dterm, hierName, resClass, scale, outf)
+bool simnAPHier(dterm, hierName, resClass, scale, outf)
     DevTerm *dterm;
     HierName *hierName;
     int  resClass, scale;
@@ -1602,17 +1608,16 @@ Dev    *dev;
  * ----------------------------------------------------------------------------
  */
 int
-simmergeVisit(dev, hierName, trans, l, w)
+simmergeVisit(dev, hierName, trans)
 Dev *dev;		/* Dev to examine */
 HierName *hierName;	/* Hierarchical path down to this dev */
 Transform *trans;	/* Coordinate transform not used */
-int l, w;
 {
 	DevTerm *gate, *source, *drain;
 	Dev     *cf;
 	DevTerm *cg, *cs, *cd;
 	EFNode	*subnode, *snode, *dnode, *gnode;
-	int      scale, pmode;
+	int      scale, pmode, l, w;
     	Rect	 r;
 	float	 m;
 	devMerge *fp, *cfp;
@@ -1634,6 +1639,7 @@ int l, w;
 
     	GeoTransRect(trans, &dev->dev_rect, &r);
     	scale = GeoScale(trans);
+	EFGetLengthAndWidth(dev, &l, &w);
 	fp = simmkDevMerge(l*scale, w*scale, gnode, snode, 
 			dnode, subnode, hierName, dev);
 
