@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFbuild.c,v 1.2 2001/01/12 22:12:34 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/extflat/EFbuild.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -26,7 +26,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFbuild.c,v 1.2 20
 #include <string.h>
 #endif
 #include <stdlib.h>		/* for atof() */
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
 #include "utils/utils.h"
@@ -700,8 +701,15 @@ efBuildDevNode(def, name, isSubsNode)
 	if (isSubsNode)
 	{
 	    if (!EFHNIsGlob(nn->efnn_hier))
-		efReadError("Default dev substrate node %s is not a global\n",
-		    name);
+	    {
+#ifdef MAGIC_WRAPPER
+		if ((name[0] == '$') && (name[1] != '$'))
+		    efReadError("Substrate node is an undefined Tcl variable.\n");
+		else
+#endif
+		    efReadError("Default device substrate node"
+				" \"%s\" is not a global\n", name);
+	    }
 	    nn->efnn_node->efnode_flags |= EF_DEVTERM;
 	}
     }
@@ -729,8 +737,8 @@ efBuildDevNode(def, name, isSubsNode)
 int
 efBuildAddStr(table, pMax, size, str)
     char *table[];	/* Table to search */
-    int *pMax;			/* Increment this if we add an entry */
-    int size;			/* Maximum size of table */
+    int *pMax;		/* Increment this if we add an entry */
+    int size;		/* Maximum size of table */
     char *str;		/* String to add */
 {
     int n, max;
