@@ -18,11 +18,12 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtCouple.c,v 1.3 2001/07/23 17:49:52 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/extract/ExtCouple.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
@@ -91,7 +92,7 @@ void extNregAdjustCap(nr, c, str)
 {
     char *name;
     name = extNodeName((LabRegion *) nr);
-    TxError("CapDebug: %s += %f (%s)\n", name, c, str);
+    fprintf(stderr, "CapDebug: %s += %f (%s)\n", name, c, str);
 }
 
 void extAdjustCouple(he, c, str)
@@ -105,7 +106,7 @@ void extAdjustCouple(he, c, str)
     ck = (CoupleKey *) he->h_key.h_words;
     name1 = extNodeName((LabRegion *) ck->ck_1);
     name2 = extNodeName((LabRegion *) ck->ck_2);
-    TxError("CapDebug: %s-%s += %f (%s)\n", name1, name2, c, str);
+    fprintf(stderr, "CapDebug: %s-%s += %f (%s)\n", name1, name2, c, str);
 }
 
 
@@ -264,14 +265,25 @@ extBasicOverlap(tile, ecs)
     Tile *tile;
     extCapStruct *ecs;
 {
-    int thisType = TiGetType(tile);
-    int pNum, pMask = ExtCurStyle->exts_overlapOtherPlanes[thisType];
-    TileTypeBitMask *tMask = &ExtCurStyle->exts_overlapOtherTypes[thisType];
+    int thisType;
+    int pNum, pMask;
+    TileTypeBitMask *tMask;
     Rect r;
     CellDef *def = ecs->def;
     int thisPlane = ecs->plane;
     extCoupleStruct ecpls;
 
+#ifdef NONMANHATTAN
+    if (IsSplit(tile))
+	thisType = (SplitSide(tile)) ? SplitRightType(tile) :
+		SplitLeftType(tile);
+    else
+#endif
+    thisType = TiGetTypeExact(tile);
+
+    pMask = ExtCurStyle->exts_overlapOtherPlanes[thisType];
+    tMask = &ExtCurStyle->exts_overlapOtherTypes[thisType];
+
     TITORECT(tile, &r);
     extOverlapDef = def;
     if (extCoupleSearchArea)
@@ -410,8 +422,8 @@ extAddOverlap(tbelow, ecpls)
     /* If any capacitance remains, add this record to the table */
     if (ov.o_area > 0)
     {
-	int oa = ExtCurStyle->exts_planeOrder[DBPlane(ta)];
-	int ob = ExtCurStyle->exts_planeOrder[DBPlane(tb)];
+	int oa = ExtCurStyle->exts_planeOrder[ecpls->plane_of_tile];
+	int ob = ExtCurStyle->exts_planeOrder[ecpls->plane_checked];
 	if (oa > ob)
 	{
 	    /*
@@ -539,6 +551,7 @@ extSubtractOverlap2(tile, ov)
  * ----------------------------------------------------------------------------
  */
 
+int
 extBasicCouple(tile, ecs)
     Tile *tile;
     extCapStruct *ecs;
@@ -642,7 +655,7 @@ extAddCouple(bp, ecs)
     }
 
     if (extCoupleList)
-	(void) DBSrPaintArea((Tile *) NULL, def->cd_planes[DBPlane(tin)],
+	(void) DBSrPaintArea((Tile *) NULL, def->cd_planes[ecs->plane],
 		&r, &ExtCurStyle->exts_sideCoupleOtherEdges[tin][tout],
 		proc, (ClientData) bp);
 
@@ -794,8 +807,8 @@ extSideOverlap(tp, esws)
     }
     else
     {
-	int oa = ExtCurStyle->exts_planeOrder[DBPlane(ta)];
-	int ob = ExtCurStyle->exts_planeOrder[DBPlane(tb)];
+	int oa = ExtCurStyle->exts_planeOrder[esws->plane_of_boundary];
+	int ob = ExtCurStyle->exts_planeOrder[esws->plane_checked];
 	if (oa > ob)
 	{
 	    /* If the overlapped tile is between the substrate and the boundary
