@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtLength.c,v 1.2 2001/01/12 22:12:37 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/extract/ExtLength.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -31,7 +31,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtLength.c,v 1.2
 #include <ctype.h>
 #include <math.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
@@ -42,12 +42,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtLength.c,v 1.2
 #include "debug/debug.h"
 #include "extract/extract.h"
 #include "extract/extractInt.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "utils/stack.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "utils/utils.h"
 
 /* Temporary cell for holding an entire flattened net */
@@ -90,6 +90,7 @@ static Label *extLengthLabelList;
 struct extPathArg
 {
     int		 epa_min, epa_max;
+    int		 epa_pNum;
     Label	*epa_lab1, *epa_lab2;
 };
 
@@ -648,9 +649,12 @@ extPathPairDistance(lab1, lab2, pMin, pMax)
     pMask = DBTechTypesToPlanes(&mask);
     for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
 	if (PlaneMaskHasPlane(pMask, pNum))
+	{
+	    epa.epa_pNum = pNum;
 	    (void) DBSrPaintClient((Tile *) NULL, extPathDef->cd_planes[pNum],
 		    &r, &mask, (ClientData) MINFINITY,
 		    extPathPairFunc, (ClientData) &epa);
+	}
 
     /* Pass the min and max delay back to our caller */
     *pMin = epa.epa_min;
@@ -765,7 +769,6 @@ extPathFlood(tile, p, distance, epa)
 				 */
 {
     TileType type = TiGetType(tile);
-    int tilePlaneNum = DBPlane(type);
     Label *lab2 = epa->epa_lab2;
     int pMask, pNum, newdistance;
     Tile *tp;
@@ -835,8 +838,12 @@ extPathFlood(tile, p, distance, epa)
     /* Try connections to other planes */
     if (DBIsContact(type))
     {
-	for (pNum = tilePlaneNum - 1; pNum <= tilePlaneNum + 1; pNum += 2)
-	    if (PlaneMaskHasPlane(DBConnPlanes[type], pNum))
+	int saveplane = epa->epa_pNum;
+	int pMask = DBConnPlanes[type];
+	pMask &= ~(PlaneNumToMaskBit(epa->epa_pNum));
+
+	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+	    if (PlaneMaskHasPlane(pMask, pNum))
 	    {
 		Plane *plane = extPathDef->cd_planes[pNum];
 		 
@@ -849,9 +856,11 @@ extPathFlood(tile, p, distance, epa)
 		if (tp->ti_client == (ClientData) MINFINITY
 			&& DBConnectsTo(type, TiGetType(tp)))
 		{
+		    epa->epa_pNum = pNum;
 		    extPathFlood(tp, p, distance, epa);
 		}
 	    }
+	epa->epa_pNum = saveplane;
     }
 
     /*
@@ -865,6 +874,7 @@ extPathFlood(tile, p, distance, epa)
      */
     if (pMask = DBAllConnPlanes[type])
     {
+	int saveplane = epa->epa_pNum;
 	struct extPathFloodArg epfa;
 	Rect biggerArea;
 
@@ -875,11 +885,15 @@ extPathFlood(tile, p, distance, epa)
 	epfa.epfa_srcPoint = p;
 	epfa.epfa_srcTile = tile;
 	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
-	    if (pNum != tilePlaneNum && PlaneMaskHasPlane(pMask, pNum))
+	    if (pNum != epa->epa_pNum && PlaneMaskHasPlane(pMask, pNum))
+	    {
+		epa->epa_pNum = pNum;
 		(void) DBSrPaintClient((Tile *) NULL,
 			extPathDef->cd_planes[pNum], &biggerArea,
 			&DBConnectTbl[type], (ClientData) MINFINITY,
 			extPathFloodFunc, (ClientData) &epfa);
+	    }
+	epa->epa_pNum = saveplane;
     }
 }
 
