@@ -28,9 +28,10 @@ static char sccsid[] = "@(#)ExtTech.c	4.8 MAGIC (Berkeley) 10/26/85";
 #include <string.h>
 #endif
 #include <stdlib.h>		/* for strtod() */
+#include <ctype.h>		/* for isspace() */
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/utils.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -39,7 +40,7 @@ static char sccsid[] = "@(#)ExtTech.c	4.8 MAGIC (Berkeley) 10/26/85";
 #include "database/databaseInt.h"
 #include "utils/malloc.h"
 #include "textio/textio.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "debug/debug.h"
 #include "extract/extract.h"
 #include "extract/extractInt.h"
@@ -54,6 +55,7 @@ ExtKeep *ExtAllStyles = NULL;
 /* Forward declarations */
 void extTechFinalStyle();
 void ExtLoadStyle();
+void ExtTechScale(int, int);
 
 /*
  * Table used for parsing the extract section of a .tech file
@@ -345,7 +347,7 @@ ExtGetZAxis(tile, height, thick)
 
     ttype = TiGetLeftType(tile);        /* Ignore non-Manhattan for now */
 
-    /* Note that w_scale is multiplied by 4096 to get fixed-point accuracy.     */
+    /* Note that w_scale is multiplied by SUBPIXEL to get fixed-point accuracy. */
     /* However, we downshift by only 9 (divide by 512) so that heights are      */
     /* exaggerated in the layout by a factor of 8 (height exaggeration is       */
     /* standard practice for VLSI cross-sections).                              */
@@ -878,7 +880,7 @@ ExtTechLine(sectionName, argc, argv)
     int argc;
     char *argv[];
 {
-    int n, l, size, val, p1, p2, p3, nterm, pov, pshield, class;
+    int n, l, j, size, val, p1, p2, p3, nterm, pov, pshield, class;
     CapValue capVal, gscap, gccap;
     TileTypeBitMask types1, types2, near, far, ov, shield, subsTypes;
     char *subsName, *transName, *cp, *endptr;
@@ -925,7 +927,7 @@ ExtTechLine(sectionName, argc, argv)
 	for (newStyle = ExtAllStyles; newStyle != NULL;
 		newStyle = newStyle->exts_next)
 	{
-	    if (!strcmp(newStyle->exts_name, argv[1]))
+	    if (!strncmp(newStyle->exts_name, argv[1], l))
 		break;
 	}
 	if (newStyle == NULL)
@@ -1059,7 +1061,12 @@ ExtTechLine(sectionName, argc, argv)
 	while (*tptr != '\0')
 	{
 	    cptr = strchr(tptr, ',');
-	    if (cptr != NULL) *cptr = '\0';
+	    if (cptr != NULL) 
+	    {
+		*cptr = '\0';
+		for (j = 1; isspace(*(cptr - j)); j++)
+		    *(cptr - j) = '\0';
+	    }
 
 	    if (*tptr == '*')	/* Wildcard for "all variants" */
 	    {
@@ -1824,7 +1831,7 @@ zinit:
  * ----------------------------------------------------------------------------
  */
 
-int
+void
 ExtTechScale(scalen, scaled)
     int scalen;			/* Scale numerator */
     int scaled;			/* Scale denominator */
@@ -1873,4 +1880,6 @@ ExtTechScale(scalen, scaled)
 	    }
 	}
     }
+
+    return;
 }
