@@ -18,17 +18,17 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtTest.c,v 1.2 2001/01/12 22:12:38 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/extract/ExtTest.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/utils.h"
 #include "utils/geometry.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
@@ -36,7 +36,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtTest.c,v 1.2 20
 #include "windows/windows.h"
 #include "graphics/graphics.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "commands/commands.h"
 #include "textio/textio.h"
 #include "textio/txcommands.h"
@@ -76,6 +76,13 @@ int extEdgePixels = 4;
 int extShowInter();
 void extShowTech();
 void extDispInit();
+bool extShowRect();
+void extMore();
+
+void extShowTrans(char *, TileTypeBitMask *, FILE *);
+void extShowMask(TileTypeBitMask *, FILE *);
+void extShowPlanes(int, FILE *);
+void extShowConnect(char *, TileTypeBitMask *, FILE *);
 
 
 /*
@@ -290,6 +297,7 @@ ExtractTest(w, cmd)
     }
 }
 
+int
 extShowInter(tile)
     Tile *tile;
 {
@@ -345,112 +353,112 @@ extShowTech(name)
 
     extShowTrans("Transistor", &ExtCurStyle->exts_transMask, out);
 
-    (void) fprintf(out, "\nNode resistance and capacitance:\n");
-    (void) fprintf(out, "type     R-ohm/sq  AreaC-ff/l**2\n");
+    fprintf(out, "\nNode resistance and capacitance:\n");
+    fprintf(out, "type     R-ohm/sq  AreaC-ff/l**2\n");
     for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
-	(void) fprintf(out, "%-8.8s %8d      %9lf\n",
+	fprintf(out, "%-8.8s %8d      %9lf\n",
 		    DBTypeShortName(t),
 		    ExtCurStyle->exts_resistByResistClass[
 			ExtCurStyle->exts_typeToResistClass[t]],
 		    ExtCurStyle->exts_areaCap[t]);
 
-    (void) fprintf(out, "\nTypes contributing to resistive perimeter:\n");
-    (void) fprintf(out, "type     R-type boundary types\n");
+    fprintf(out, "\nTypes contributing to resistive perimeter:\n");
+    fprintf(out, "type     R-type boundary types\n");
     for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
     {
-	(void) fprintf(out, "%-8.8s ", DBTypeShortName(t));
-	(void) fprintf(out, "%7d ", ExtCurStyle->exts_typeToResistClass[t]);
+	fprintf(out, "%-8.8s ", DBTypeShortName(t));
+	fprintf(out, "%7d ", ExtCurStyle->exts_typeToResistClass[t]);
 	extShowMask(&ExtCurStyle->exts_typesResistChanged[t], out);
-	(void) fprintf(out, "\n");
+	fprintf(out, "\n");
     }
 
-    (void) fprintf(out, "\nSidewall capacitance:\n");
+    fprintf(out, "\nSidewall capacitance:\n");
     for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 	for (s = 0; s < DBNumTypes; s++)
 	    if (ExtCurStyle->exts_perimCap[t][s] != (CapValue) 0)
-		(void) fprintf(out, "    %-8.8s %-8.8s %8lf\n",
+		fprintf(out, "    %-8.8s %-8.8s %8lf\n",
 			DBTypeShortName(t), DBTypeShortName(s),
 			ExtCurStyle->exts_perimCap[t][s]);
 
-    (void) fprintf(out, "\nInternodal overlap capacitance:\n");
-    (void) fprintf(out, "\n  (by plane)\n");
+    fprintf(out, "\nInternodal overlap capacitance:\n");
+    fprintf(out, "\n  (by plane)\n");
     for (p = PL_TECHDEPBASE; p < DBNumPlanes; p++)
     {
 	if (PlaneMaskHasPlane(ExtCurStyle->exts_overlapPlanes	, p))
 	{
-	    (void) fprintf(out, "    %-10.10s: types=", DBPlaneShortName(p));
+	    fprintf(out, "    %-10.10s: types=", DBPlaneShortName(p));
 	    extShowMask(&ExtCurStyle->exts_overlapTypes	[p], out);
-	    (void) fprintf(out, "\n");
+	    fprintf(out, "\n");
 	}
     }
-    (void) fprintf(out, "\n  (by type)\n");
+    fprintf(out, "\n  (by type)\n");
     for (t = 0; t < DBNumTypes; t++)
 	if (!TTMaskIsZero(&ExtCurStyle->exts_overlapOtherTypes[t]))
 	{
-	    (void) fprintf(out, "    %-10.10s: planes=", DBTypeShortName(t));
+	    fprintf(out, "    %-10.10s: planes=", DBTypeShortName(t));
 	    extShowPlanes(ExtCurStyle->exts_overlapOtherPlanes[t], out);
-	    (void) fprintf(out, "\n      overlapped types=");
+	    fprintf(out, "\n      overlapped types=");
 	    extShowMask(&ExtCurStyle->exts_overlapOtherTypes[t], out);
-	    (void) fprintf(out, "\n");
+	    fprintf(out, "\n");
 	    for (s = 0; s < DBNumTypes; s++)
 		if (ExtCurStyle->exts_overlapCap[t][s] != (CapValue) 0)
-		    (void) fprintf(out, "              %-10.10s: %8lf\n",
+		    fprintf(out, "              %-10.10s: %8lf\n",
 				DBTypeShortName(s), ExtCurStyle->exts_overlapCap[t][s]);
 	}
 
-    (void) fprintf(out, "\nSidewall-coupling/sidewall-overlap capacitance:\n");
-    (void) fprintf(out, "\n  (by plane)\n");
+    fprintf(out, "\nSidewall-coupling/sidewall-overlap capacitance:\n");
+    fprintf(out, "\n  (by plane)\n");
     for (p = PL_TECHDEPBASE; p < DBNumPlanes; p++)
     {
 	if (PlaneMaskHasPlane(ExtCurStyle->exts_sidePlanes, p))
 	{
-	    (void) fprintf(out, "    %-10.10s: ", DBPlaneShortName(p));
+	    fprintf(out, "    %-10.10s: ", DBPlaneShortName(p));
 	    extShowMask(&ExtCurStyle->exts_sideTypes[p], out);
-	    (void) fprintf(out, "\n");
+	    fprintf(out, "\n");
 	}
     }
-    (void) fprintf(out, "\n  (by type)\n");
+    fprintf(out, "\n  (by type)\n");
     for (s = 0; s < DBNumTypes; s++)
 	if (!TTMaskIsZero(&ExtCurStyle->exts_sideEdges[s]))
 	{
-	    (void) fprintf(out, "    %-10.10s: ", DBTypeShortName(s));
+	    fprintf(out, "    %-10.10s: ", DBTypeShortName(s));
 	    extShowMask(&ExtCurStyle->exts_sideEdges[s], out);
-	    (void) fprintf(out, "\n");
+	    fprintf(out, "\n");
 	    for (t = 0; t < DBNumTypes; t++)
 	    {
 		if (!TTMaskIsZero(&ExtCurStyle->exts_sideCoupleOtherEdges[s][t]))
 		{
-		    (void) fprintf(out, "                edge mask=");
+		    fprintf(out, "                edge mask=");
 		    extShowMask(&ExtCurStyle->exts_sideCoupleOtherEdges[s][t], out);
-		    (void) fprintf(out, "\n");
+		    fprintf(out, "\n");
 		}
 		if (!TTMaskIsZero(&ExtCurStyle->exts_sideOverlapOtherTypes[s][t]))
 		{
-		    (void) fprintf(out, "                overlap mask=");
+		    fprintf(out, "                overlap mask=");
 		    extShowMask(&ExtCurStyle->exts_sideOverlapOtherTypes[s][t],
 				out);
-		    (void) fprintf(out, "\n");
+		    fprintf(out, "\n");
 		}
 		if (e = ExtCurStyle->exts_sideCoupleCap[s][t])
 		    for ( ; e; e = e->ec_next)
 		    {
-			(void) fprintf(out, "                COUPLE: ");
+			fprintf(out, "                COUPLE: ");
 			extShowMask(&e->ec_near, out);
-			(void) fprintf(out, " || ");
+			fprintf(out, " || ");
 			extShowMask(&e->ec_far, out);
-			(void) fprintf(out, ": %lf\n", e->ec_cap);
+			fprintf(out, ": %lf\n", e->ec_cap);
 		    }
 		if (e = ExtCurStyle->exts_sideOverlapCap[s][t])
 		    for ( ; e; e = e->ec_next)
 		    {
-			(void) fprintf(out, "                OVERLAP: ");
+			fprintf(out, "                OVERLAP: ");
 			extShowMask(&e->ec_near, out);
-			(void) fprintf(out, ": %lf\n", e->ec_cap);
+			fprintf(out, ": %lf\n", e->ec_cap);
 		    }
 	    }
 	}
 
-    (void) fprintf(out, "\n\nSidewall coupling halo = %d\n", ExtCurStyle->exts_sideCoupleHalo	);
+    fprintf(out, "\n\nSidewall coupling halo = %d\n", ExtCurStyle->exts_sideCoupleHalo	);
 
     extShowConnect("\nNode connectivity", ExtCurStyle->exts_nodeConn, out);
     extShowConnect("\nResistive region connectivity", ExtCurStyle->exts_resistConn, out);
@@ -460,6 +468,7 @@ extShowTech(name)
 	(void) fclose(out);
 }
 
+void
 extShowTrans(name, mask, out)
     char *name;
     TileTypeBitMask *mask;
@@ -467,22 +476,23 @@ extShowTrans(name, mask, out)
 {
     TileType t;
 
-    (void) fprintf(out, "%s types: ", name);
+    fprintf(out, "%s types: ", name);
     extShowMask(mask, out);
-    (void) fprintf(out, "\n");
+    fprintf(out, "\n");
 
     for (t = 0; t < DBNumTypes; t++)
 	if (TTMaskHasType(mask, t))
 	{
-	    (void) fprintf(out, "    %-8.8s  %d terminals: ",
+	    fprintf(out, "    %-8.8s  %d terminals: ",
 			DBTypeShortName(t), ExtCurStyle->exts_transSDCount[t]);
 	    extShowMask(&ExtCurStyle->exts_transSDTypes[t], out);
-	    (void) fprintf(out, "\n\tcap (gate-sd/gate-ch) = %lf/%lf\n",
+	    fprintf(out, "\n\tcap (gate-sd/gate-ch) = %lf/%lf\n",
 			ExtCurStyle->exts_transSDCap[t],
 			ExtCurStyle->exts_transGateCap[t]);
 	}
 }
 
+void
 extShowConnect(hdr, connectsTo, out)
     char *hdr;
     TileTypeBitMask *connectsTo;
@@ -490,16 +500,17 @@ extShowConnect(hdr, connectsTo, out)
 {
     TileType t;
 
-    (void) fprintf(out, "%s\n", hdr);
+    fprintf(out, "%s\n", hdr);
     for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 	if (!TTMaskEqual(&connectsTo[t], &DBZeroTypeBits))
 	{
-	    (void) fprintf(out, "    %-8.8s: ", DBTypeShortName(t));
+	    fprintf(out, "    %-8.8s: ", DBTypeShortName(t));
 	    extShowMask(&connectsTo[t], out);
-	    (void) fprintf(out, "\n");
+	    fprintf(out, "\n");
 	}
 }
 
+void
 extShowMask(m, out)
     TileTypeBitMask *m;
     FILE *out;
@@ -511,12 +522,13 @@ extShowMask(m, out)
 	if (TTMaskHasType(m, t))
 	{
 	    if (!first)
-		(void) fprintf(out, ",");
+		fprintf(out, ",");
 	    first = FALSE;
-	    (void) fprintf(out, "%s", DBTypeShortName(t));
+	    fprintf(out, "%s", DBTypeShortName(t));
 	}
 }
 
+void
 extShowPlanes(m, out)
     int m;
     FILE *out;
@@ -528,9 +540,9 @@ extShowPlanes(m, out)
 	if (PlaneMaskHasPlane(m, pNum))
 	{
 	    if (!first)
-		(void) fprintf(out, ",");
+		fprintf(out, ",");
 	    first = FALSE;
-	    (void) fprintf(out, "%s", DBPlaneShortName(pNum));
+	    fprintf(out, "%s", DBPlaneShortName(pNum));
 	}
 }
 
@@ -684,6 +696,7 @@ extShowRect(r, style)
     return (TRUE);
 }
 
+void
 extMore()
 {
     char line[100];
