@@ -3,7 +3,7 @@
  *
  * Defines the exported interface to the circuit extractor.
  *
- * rcsid "$Header: /ufs/repository/magic/extract/extract.h,v 1.1.1.1 2000/03/29 18:35:39 rajit Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic/extract/extract.h,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $"
  *
  *     ********************************************************************* 
  *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
@@ -22,7 +22,7 @@
 #ifndef _EXTRACT_H
 #define _EXTRACT_H
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 
 /* Extractor warnings */
 #define	EXTWARN_DUP	0x01	/* Warn if two nodes have the same name */
