@@ -4,7 +4,7 @@
  * Defines things shared internally by the extract module of Magic,
  * but not generally needed outside the extract module.
  *
- * rcsid "$Header: /ufs/repository/magic/extract/extractInt.h,v 1.3 2001/07/23 17:49:52 tim Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic/extract/extractInt.h,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $"
  *
  *     ********************************************************************* 
  *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
@@ -23,23 +23,10 @@
  *
  */
 
-#define _EXTINT
+#ifndef _EXTRACTINT_H
+#define _EXTRACTINT_H
 
-#ifndef	_MAGIC
-int err0 = Need_to_include_misc/magic_header;
-#endif	/* _MAGIC */
-#ifndef	_DATABASE
-int err1 = Need_to_include_database/database_header;
-#endif	/* _DATABASE */
-#ifndef	_TILES
-int err2 = Need_to_include_tiles/tile_header;
-#endif	/* _TILES */
-#ifndef	_HASH
-int err3 = Need_to_include_utils/hash_header;
-#endif	/* _HASH */
-#ifndef	_GEOMETRY
-int err4 = Need_to_include_utils/geometry_header;
-#endif	/* _GEOMETRY */
+#include "database/database.h"
 
 #undef	NT
 #define	NT	TT_MAXTYPES
@@ -1005,3 +992,4 @@ extern struct stack *extNodeStack;
 
 #include "extDebugInt.h"
 
+#endif /* _EXTRACTINT_H */
