@@ -17,29 +17,29 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/garouter/gaChannel.c,v 1.2 2001/01/12 22:12:39 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/garouter/gaChannel.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "tiles/tile.h"
 #include "database/database.h"
 #include "windows/windows.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "dbwind/dbwind.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "netmenu/netmenu.h"
 #include "gcr/gcr.h"
 #include "router/router.h"
 #include "grouter/grouter.h"
 #include "garouter/garouter.h"
-#include "netlist/netlist.h"
+#include "utils/netlist.h"
 #include "textio/textio.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "debug/debug.h"
 
 /* List of all active channels */
