@@ -17,17 +17,18 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/garouter/gaTest.c,v 1.3 2001/01/12 22:12:40 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/garouter/gaTest.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
 #include "utils/heap.h"
 #include "tiles/tile.h"
 #include "database/database.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "textio/textio.h"
 #include "debug/debug.h"
 #include "gcr/gcr.h"
@@ -38,10 +39,10 @@ static char rcsid[] = "$Header: /ufs/repository/magic/garouter/gaTest.c,v 1.3 20
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "textio/txcommands.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "utils/utils.h"
 #include "commands/commands.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 
 bool gaInitialized = FALSE;
 
