@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/graphics/grMain.c,v 1.8 2001/04/13 20:02:59 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/graphics/grMain.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 /*
@@ -57,8 +57,8 @@ static char rcsid[]="$Header: /ufs/repository/magic/graphics/grMain.c,v 1.8 2001
 #include <errno.h>
 #include <ctype.h>
 
-#include "misc/magic.h"
-#include "misc/magsgtty.h"
+#include "utils/magic.h"
+#include "utils/magsgtty.h"
 #include "textio/textio.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
@@ -145,8 +145,6 @@ bool (*GrInitPtr)()		= NULL;
 void (*GrClosePtr)()		= NULL;
 void (*GrSetCMapPtr)()		= NULL;
 
-void (*GrEnableTabletPtr)()	= NULL;
-void (*GrDisableTabletPtr)()	= NULL;
 void (*GrSetCursorPtr)()	= NULL;
 void (*GrTextSizePtr)()		= NULL;
 void (*GrDrawGlyphPtr)()	= NULL;
@@ -165,6 +163,14 @@ int (*GrWindowIdPtr)()   	= NULL;
 char *(*GrWindowNamePtr)()	= NULL;
 bool (*GrGetCursorPosPtr)()   	= NULL;
 
+void (*GrEnableTabletPtr)()	= NULL;
+void (*GrDisableTabletPtr)()	= NULL;
+
+bool (*GrGetBackingStorePtr)() 	  = NULL;
+bool (*GrScrollBackingStorePtr)() = NULL;
+void (*GrPutBackingStorePtr)() 	  = NULL;
+void (*GrFreeBackingStorePtr)()   = NULL;
+void (*GrCreateBackingStorePtr)() = NULL;
 
 /* variables similar to the above, except that they are only used
  * internal to the graphics package
@@ -173,6 +179,7 @@ void (*grPutTextPtr)()		= NULL;
 void (*grGetCharSizePtr)()	= NULL;
 void (*grSetSPatternPtr)()	= NULL;
 void (*grDefineCursorPtr)()	= NULL;
+void (*grFreeCursorPtr)()	= NULL;
 bool (*grDrawGridPtr)()		= NULL;
 void (*grDrawLinePtr)()		= NULL;
 void (*grSetWMandCPtr)()	= NULL;
@@ -196,15 +203,6 @@ char *grDStyleType = "7bit";
 char *grCMapType = "7bit";
 char *grCursorType = "bw";
 
-/* The following variable is a device-dependent limit on how many
- * stipples are permitted.  It defaults to something, but may be reset
- * by device initializers to a different number, depending
- * on what the devices can actually support.  Must not be greater
- * than GR_NUM_STIPPLES.
- */
-
-int grMaxStipples = GR_NUM_STIPPLES;
-
 int grNumBitPlanes = 0;      /* Number of bit-planes we are using. */
 int grBitPlaneMask = 0;      /* Mask of the valid bit-plane bits. */
 
