@@ -20,21 +20,21 @@
 #include <unistd.h>
 #include <signal.h>
 
-#include "misc/magic.h"
-#include "misc/magsgtty.h"
+#include "utils/magic.h"
+#include "utils/magsgtty.h"
 #include "utils/geometry.h"
 #include "graphics/graphics.h"
 #include "windows/windows.h"
 #include "graphics/graphicsInt.h"
 #include "textio/textio.h"
 #include "textio/txcommands.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "utils/utils.h"
 #include "utils/hash.h"
 #include "grOGLInt.h"
-#include "misc/paths.h"
+#include "utils/paths.h"
 
-GLubyte 	*grOGLStipples[GR_NUM_STIPPLES];
+GLubyte 	**grOGLStipples;
 HashTable	grOGLWindowTable;
 Display 	*grXdpy;
 GLXContext	grXcontext;
@@ -59,11 +59,11 @@ OGL_CURRENT oglCurrent= {(XFontStruct *)NULL, 0,0,0,0, (Window)0, (MagWindow *)N
  * for details on this.
  */
 
-extern Void GrOGLClose(), GrOGLFlush(), GrOGLInit();
-extern Void GrOGLDelete(),GrOGLConfigure(),GrOGLRaise(),GrOGLLower();
-extern Void GrOGLLock(),GrOGLUnlock(),GrOGLIconUpdate();
-extern bool GrOGLCreate();
-extern Void grOGLWStdin(int , ClientData );
+extern void GrOGLClose(), GrOGLFlush();
+extern void GrOGLDelete(), GrOGLConfigure(), GrOGLRaise(), GrOGLLower();
+extern void GrOGLLock(), GrOGLUnlock(), GrOGLIconUpdate();
+extern bool GrOGLInit(), GrOGLCreate();
+extern void grOGLWStdin();
 
 
 /*---------------------------------------------------------
@@ -79,7 +79,7 @@ extern Void grOGLWStdin(int , ClientData );
  *---------------------------------------------------------
  */
 
-Void
+void
 groglSetWMandC (mask, c)
     int mask;			/* New value for write mask */
     int c;			/* New value for current color */
@@ -134,7 +134,7 @@ groglSetWMandC (mask, c)
  *---------------------------------------------------------
  */
 
-Void
+void
 groglSetLineStyle (style)
     int style;			/* New stipple pattern for lines. */
 {
@@ -161,9 +161,9 @@ groglSetLineStyle (style)
 
 /*---------------------------------------------------------
  * groglSetSPattern:
- *	xSetSPattern associates a stipple pattern with a given
- *	stipple number.  This is a local routine called from
- *	grStyle.c .
+ *	xSetSPattern associates stipple patterns with
+ *	OpenGL stipples.  This is a local routine
+ *	called from grStyle.c.
  *
  * Results:	None.
  *
@@ -171,24 +171,28 @@ groglSetLineStyle (style)
  *---------------------------------------------------------
  */
 
-Void
-groglSetSPattern (stipple, pattern)
-    int stipple;			/* The stipple number, 1-15. */
-    int pattern[8];			/* 32 x 32 bit pattern */
+void
+groglSetSPattern (sttable, numstipples)
+    int **sttable;			/* The table of patterns */
+    int numstipples;			/* Number of stipples */
 {
-    int i, j, n = 0;
+    int i, j, k, n;
     GLubyte *pdata;
 
-    pdata = (GLubyte *)mallocMagic(128 * sizeof(GLubyte));
+    grOGLStipples = (GLubyte **)mallocMagic(numstipples * sizeof(GLubyte *));
+    for (k = 0; k < numstipples; k++)
+    {
+	pdata = (GLubyte *)mallocMagic(128 * sizeof(GLubyte));
+	n = 0;
 
-    /* expand magic's default 8x8 stipple to OpenGL's 32x32 */
+	/* expand magic's default 8x8 stipple to OpenGL's 32x32 */
 
-    for (i = 0; i < 32; i++)
-       for (j = 0; j < 4; j++) {
-          pdata[n++] = (GLubyte)pattern[i % 8];
-       }
+	for (i = 0; i < 32; i++)
+	    for (j = 0; j < 4; j++)
+		pdata[n++] = (GLubyte)sttable[k][i % 8];
 
-    grOGLStipples[stipple] = pdata;
+	grOGLStipples[k] = pdata;
+    }
 }
 
 
@@ -204,7 +208,7 @@ groglSetSPattern (stipple, pattern)
  *---------------------------------------------------------
  */
 
-Void
+void
 groglSetStipple (stipple)
     int stipple;			/* The stipple number to be used. */
 {
@@ -212,7 +216,7 @@ groglSetStipple (stipple)
     if (stipple == oldStip) return;
     oldStip = stipple;
     GR_X_FLUSH_BATCH();
-    if (stipple == 0 || stipple > GR_NUM_STIPPLES) {
+    if (stipple == 0 || stipple > grNumStipples) {
 	glDisable(GL_POLYGON_STIPPLE);
     } else {
 	if (grOGLStipples[stipple] == (GLubyte *)NULL) MainExit(1);
@@ -221,7 +225,6 @@ groglSetStipple (stipple)
     }
 }
 
-bool
 
 /*---------------------------------------------------------
  * GrOGLInit:
@@ -232,7 +235,8 @@ bool
  *---------------------------------------------------------
  */
 
-GrOGLInit ()
+bool
+GrOGLInit()
 {
     XVisualInfo *grVisualInfo;
     static int	attributeList[] = { GLX_RGBA, None, None };
@@ -249,7 +253,7 @@ GrOGLInit ()
     if (grXdpy == NULL)
     {   
 	TxError("Couldn't open display; check DISPLAY variable\n");
-	return false;
+	return FALSE;
     }
     grXscrn = DefaultScreen(grXdpy);
     grVisualInfo = glXChooseVisual(grXdpy, grXscrn, attributeList);
@@ -268,13 +272,19 @@ GrOGLInit ()
     grVisual = grVisualInfo->visual;
     oglCurrent.depth = grVisualInfo->depth;
 
-    grXcontext = glXCreateContext(grXdpy, grVisualInfo, NULL, GL_TRUE);
+    /* Note: The last parameter is GL_TRUE for direct rendering. 	*/
+    /* Direct rendering has a speedup advantange for raw rendering, but	*/
+    /* disallows X11 operations such as XCopyArea, which prevents the	*/
+    /* implementation of backing store and fast screen refreshes.  So	*/
+    /* we force an indirect rendering context through the X server.	*/
+
+    grXcontext = glXCreateContext(grXdpy, grVisualInfo, NULL, GL_FALSE);
 
     /* Basic GL parameters */
 
     glLineWidth(1.0);
     glShadeModel (GL_FLAT);
-    glPixelStorei(GL_PACK_LSB_FIRST, true);
+    glPixelStorei(GL_PACK_LSB_FIRST, TRUE);
 
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
@@ -292,14 +302,16 @@ GrOGLInit ()
 /*---------------------------------------------------------
  * GrOGLClose:
  *
- * Results:		None.
+ * Results:
+ *	None.
  *
- * Side Effects:	None.
+ * Side Effects:
+ *	Helper process/thread killed.
  *---------------------------------------------------------
  */
 
-Void
-GrOGLClose ()
+void
+GrOGLClose()
 {
     if (grXdpy == NULL) return;
     TxDelete1InputDevice(pipeRead);
@@ -328,8 +340,8 @@ GrOGLClose ()
  *---------------------------------------------------------
  */
 
-Void
-GrOGLFlush ()
+void
+GrOGLFlush()
 {
    GR_X_FLUSH_BATCH();
    glFlush();
@@ -344,7 +356,8 @@ GrOGLFlush ()
 #define glTransYs(n) (DisplayHeight(grXdpy, grXscrn)-(n))
 
 /*
-int glTransYs(int wy)
+int
+glTransYs(int wy)
 {
    int my;
    GLint vparms[4];
@@ -361,7 +374,7 @@ int glTransYs(int wy)
  * Set the OpenGL viewport (projection matrix) for the current window
  *----------------------------------------------------------------------
  */
-Void
+int
 oglSetProjection(llx, lly, width, height)
     int llx, lly, width, height;
 {
@@ -399,9 +412,9 @@ oglSetProjection(llx, lly, width, height)
 /* is a macro and can be re-implemented using XNextEvent() calls.	*/
 /*----------------------------------------------------------------------*/
 
-void pipehandler()
+void
+pipehandler()
 {
-
    TxInputEvent	*event;
    XEvent	xevent;
    HashEntry	*entry;
@@ -453,6 +466,12 @@ void pipehandler()
 	    case Button3:
 	    	event->txe_button = TX_RIGHT_BUTTON;
 	    	break;
+	    case Button4:
+	    	event->txe_button = TX_BUTTON_4;
+	    	break;
+	    case Button5:
+	    	event->txe_button = TX_BUTTON_5;
+	    	break;
    	}
 	switch(ButtonEvent->type) {
 	    case ButtonRelease:
@@ -463,7 +482,6 @@ void pipehandler()
 	    	break;
 	}
 
-
 	event->txe_p.p_x = ButtonEvent->x;
 	event->txe_p.p_y = glTransY(mw, ButtonEvent->y);
 	event->txe_wid = mw ? mw->w_wid : WIND_UNKNOWN_WINDOW;
@@ -499,6 +517,33 @@ void pipehandler()
 
 	} break;
 
+	case VisibilityNotify:		{
+	    XVisibilityEvent *VisEvent = (XVisibilityEvent*) &xevent;
+
+	    entry = HashLookOnly(&grOGLWindowTable, VisEvent->window);
+	    mw = (entry)?(MagWindow *)HashGetValue(entry):0;
+
+	    switch(VisEvent->state)
+	    {
+		case VisibilityUnobscured:
+		    mw->w_flags &= ~WIND_OBSCURED;
+		    if (mw->w_backingStore == (ClientData)NULL)
+		    {
+			groglCreateBackingStore(mw);
+			if (mw->w_backingStore != (ClientData)NULL)
+			{
+			    WindAreaChanged(mw, &mw->w_allArea);
+			    WindUpdate();
+			}
+		    }
+		    break;
+		case VisibilityPartiallyObscured:
+		case VisibilityFullyObscured:
+		    mw->w_flags |= WIND_OBSCURED;
+		    break;
+	    }
+	} break;
+
       case Expose:	{	/* (Re)Display Callback Function */
 
 	Rect		screenRect;
@@ -514,7 +559,16 @@ void pipehandler()
 	screenRect.r_ybot = mw->w_allArea.r_ytop -
 			(ExposeEvent->y + ExposeEvent->height);
 
-	WindAreaChanged(mw, &screenRect);
+	if (mw->w_backingStore != (ClientData)NULL)
+	{
+	    Rect surface;
+	    (*GrGetBackingStorePtr)(mw, &screenRect);
+	    WindScreenToSurface(mw, &screenRect, &surface);
+	    DBWHLRedrawPrepWindow(mw, &surface);
+	    WindDrawBorder(mw, &screenRect);
+	}	
+	else
+	    WindAreaChanged(mw, &screenRect);
 	WindUpdate(mw);
 
 	} break;
@@ -543,7 +597,8 @@ void pipehandler()
  *---------------------------------------------------------
  */
 
-bool oglSetDisplay (dispType, outFileName, mouseFileName)
+bool
+oglSetDisplay (dispType, outFileName, mouseFileName)
     char *dispType;		/* arguments not used by X */
     char *outFileName;
     char *mouseFileName;
@@ -552,7 +607,7 @@ bool oglSetDisplay (dispType, outFileName, mouseFileName)
     char	*planecount;
     char 	*fullname;
     FILE* 	f;
-    bool	execFailed = false;
+    bool	execFailed = FALSE;
 
     WindPackageType = WIND_X_WINDOWS;	/* This works okay. */
 
@@ -584,7 +639,7 @@ bool oglSetDisplay (dispType, outFileName, mouseFileName)
             X11HELP_PROG, HELPER_PATH);
         error = 0;
         write(fildes[1], &error, 4);
-        return false;
+        return FALSE;
     }
     else {
         fclose(f);
@@ -598,7 +653,7 @@ bool oglSetDisplay (dispType, outFileName, mouseFileName)
         sprintf(argv[1], "%d %d", fildes2[0],fildes[1]);
         if (execl(argv[0], argv[0], argv[1], 0) != 0)
         {
-            execFailed = true;
+            execFailed = TRUE;
             TxError("Couldn't execute helper process \"%s\".\n", fullname);
             TxFlush();
             /* we're the child process -- don't muck things up by returning */
@@ -632,12 +687,18 @@ bool oglSetDisplay (dispType, outFileName, mouseFileName)
     GrOverWindowPtr = GrOGLRaise;
     GrUnderWindowPtr = GrOGLLower;
     GrUpdateIconPtr = GrOGLIconUpdate; 
+    GrBitBltPtr = GrOGLBitBlt;
+
+    GrFreeBackingStorePtr = groglFreeBackingStore;
+    GrCreateBackingStorePtr = groglCreateBackingStore;
+    GrGetBackingStorePtr = groglGetBackingStore;
+    GrPutBackingStorePtr = groglPutBackingStore;
+    GrScrollBackingStorePtr = groglScrollBackingStore;
 
     /* local indirections */
     grSetSPatternPtr = groglSetSPattern;
     grPutTextPtr = groglPutText;
     grDefineCursorPtr = groglDefineCursor;
-    GrBitBltPtr = GrOGLBitBlt;
     grDrawGridPtr = groglDrawGrid;
     grDrawLinePtr = groglDrawLine;
     grSetWMandCPtr = groglSetWMandC;
@@ -645,27 +706,26 @@ bool oglSetDisplay (dispType, outFileName, mouseFileName)
     grSetStipplePtr = groglSetStipple;
     grSetLineStylePtr = groglSetLineStyle;
     grSetCharSizePtr = groglSetCharSize;
-    grMaxStipples = 32;
 #ifdef NONMANHATTAN
     grFillPolygonPtr = groglFillPolygon;
 #endif
 
     if (execFailed) {
         TxError("Execution failed!\n");
-        return false;
+        return FALSE;
     }
 
     TxAdd1InputDevice(fileno(stdin), grOGLWStdin, (ClientData) NULL);
 
     if(!GrOGLInit()){
-	return false;
+	return FALSE;
     }
     GrScreenRect.r_xbot = 0;
     GrScreenRect.r_ybot = 0;
     GrScreenRect.r_xtop = DisplayWidth(grXdpy,grXscrn);
     GrScreenRect.r_ytop = DisplayHeight(grXdpy,grXscrn);
 
-    return true;
+    return TRUE;
 }
 
 /*
@@ -684,7 +744,8 @@ bool oglSetDisplay (dispType, outFileName, mouseFileName)
  */
  /*ARGSUSED*/
 
-Void grOGLWStdin(fd, cdata)
+void
+grOGLWStdin(fd, cdata)
    int fd;
    ClientData cdata;
 {
@@ -714,10 +775,10 @@ Void grOGLWStdin(fd, cdata)
  *	Bind OpenGL window to Magic Window w.
  *
  * Results:
- *      Window created, window ID send to OGLhelper.
+ *	Success/Fail
  *
  * Side Effects:
- *	None.
+ *      Window created, window ID send to OGLhelper.
  *
  * ----------------------------------------------------------------------------
  */
@@ -733,7 +794,7 @@ GrOGLCreate(w, name)
    XSizeHints	*xsh;
    char		*windowplace;
    char		*option = (firstWindow)?"window":"newwindow";
-   bool		result = True;
+   bool		result = TRUE;
    int		x     = w->w_frameArea.r_xbot;
    int		y     = glTransYs(w->w_frameArea.r_ytop);
    int		width = w->w_frameArea.r_xtop - w->w_frameArea.r_xbot;
@@ -807,13 +868,13 @@ GrOGLCreate(w, name)
 	xevent.x = xsh->x;
 	xevent.y = xsh->y;
 	xevent.window = wind;
-	XSendEvent(grXdpy, wind, false, StructureNotifyMask, (XEvent *)&xevent);
+	XSendEvent(grXdpy, wind, FALSE, StructureNotifyMask, (XEvent *)&xevent);
 	XFree(xsh);
 	return result;
     }
     else {
 	TxError("Could not open new X window\n");
-	result = False;
+	result = FALSE;
     }
     return result;
 }
@@ -825,12 +886,15 @@ GrOGLCreate(w, name)
  *      Destroy an X window.
  *
  * Results:
- *      Window destroyed.
+ *	None.
  *
+ * Side effects:
+ *      Window destroyed.
  *
  * ----------------------------------------------------------------------------
  */
 
+void
 GrOGLDelete(w)
     MagWindow *w;
 {
@@ -851,15 +915,15 @@ GrOGLDelete(w)
  *      Full Screen function
  *
  * Results:
- *      Window reconfigured to w->w_frameArea.
+ *      None.
  *
  * Side Effects:
- *      None.
+ *      Window reconfigured to w->w_frameArea.
  *
  * ----------------------------------------------------------------------------
  */
- /*ARGSUSED*/
 
+void
 GrOGLConfigure(w)
     MagWindow *w;
 {
@@ -877,14 +941,15 @@ GrOGLConfigure(w)
  *	obscures it.
  *
  * Results:
- *      Window raised.
+ *      None.
  *
  * Side Effects:
- *      None.
+ *      Window raised.
  *
  * ----------------------------------------------------------------------------
  */
 
+void
 GrOGLRaise(w)
     MagWindow *w;
 {
@@ -899,14 +964,15 @@ GrOGLRaise(w)
  *	obscures it.
  *
  * Results:
- *      Window lowered.
+ *      None.
  *
  * Side Effects:
- *      None.
+ *      Window lowered.
  *
  * ----------------------------------------------------------------------------
  */
 
+void
 GrOGLLower(w)
     MagWindow *w;
 {
@@ -921,14 +987,15 @@ GrOGLLower(w)
  *	and "oglCurrent.mw" to reference the locked window.
  *
  * Results:
- *      Window locked.
+ *	None.
  *
  * Side Effects:
- *	None.
+ *      Window locked.
  *
  * ----------------------------------------------------------------------------
  */
 
+void
 GrOGLLock(w, flag)
     MagWindow *w;
     bool flag;
@@ -952,21 +1019,23 @@ GrOGLLock(w, flag)
  *      Unlock a window, flushing stuff out to the display.
  *
  * Results:
- *      Window unlocked.
+ *	Display update.
  *
  * Side Effects:
- *	Display update.
+ *      Window unlocked.
  *
  * ----------------------------------------------------------------------------
  */
 
+void
 GrOGLUnlock(w)
     MagWindow *w;
 {
-    GR_X_FLUSH_BATCH();
+    GrOGLFlush();
     grSimpleUnlock(w);
 }
 
+
 /*
  *-------------------------------------------------------------------------
  *
@@ -978,10 +1047,11 @@ GrOGLUnlock(w)
  *
  *-------------------------------------------------------------------------
  */
-Void GrOGLIconUpdate(w,text)
+
+void
+GrOGLIconUpdate(w,text)
 	MagWindow	*w;
 	char		*text;
-
 {
      Window wind = (Window) w->w_grdata;
      XClassHint	class;
