@@ -27,26 +27,27 @@
 #include <GL/glx.h>
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
-#include "misc/magsgtty.h"
+#include "utils/magic.h"
+#include "utils/malloc.h"
+#include "utils/magsgtty.h"
 #include "utils/geometry.h"
 #include "windows/windows.h"
 #include "graphics/graphics.h"
 #include "graphics/graphicsInt.h"
 #include "textio/textio.h"
 #include "textio/txcommands.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "drc/drc.h"
-#include "macros/macros.h"
+#include "utils/macros.h"
 #include "graphics/grTOGLInt.h"
-#include "misc/paths.h"
+#include "utils/paths.h"
 #include "graphics/grTkCommon.h"
 
-GLubyte		*grTOGLStipples[GR_NUM_STIPPLES];
+GLubyte		**grTOGLStipples;
 HashTable	grTOGLWindowTable;
 GLXContext	grXcontext;
 XVisualInfo	*grVisualInfo;
@@ -187,24 +188,27 @@ grtoglSetLineStyle (style)
  */
 
 void
-grtoglSetSPattern (stipple, pattern)
-    int stipple;			/* The stipple number, 1-15. */
-    int pattern[8];			/* 8 8-bit patterns integers */
+grtoglSetSPattern (sttable, numstipples)
+    int **sttable;			/* The table of patterns */
+    int numstipples;			/* Number of stipples */
 {
-    int i, j, n = 0;
+    int i, j, k, n;
     GLubyte *pdata;
 
-    pdata = (GLubyte *)mallocMagic(128 * sizeof(GLubyte));
+    grTOGLStipples = (GLubyte **)mallocMagic(numstipples * sizeof(GLubyte *));
+    for (k = 0; k < numstipples; k++)
+    {
+	pdata = (GLubyte *)mallocMagic(128 * sizeof(GLubyte));
+	n = 0;
 
-    /* expand magic's default 8x8 stipple to OpenGL's 32x32 */
+	/* expand magic's default 8x8 stipple to OpenGL's 32x32 */
 
-    for (i = 0; i < 32; i++)
-	for (j = 0; j < 4; j++)
-	{
-	    pdata[n++] = (GLubyte)pattern[i % 8];
-	}
+	for (i = 0; i < 32; i++)
+	    for (j = 0; j < 4; j++)
+		pdata[n++] = (GLubyte)sttable[k][i % 8];
 
-    grTOGLStipples[stipple] = pdata;
+	grTOGLStipples[k] = pdata;
+    }
 }
 
 
@@ -228,7 +232,7 @@ grtoglSetStipple (stipple)
     if (stipple == oldStip) return;
     oldStip = stipple;
     GR_TOGL_FLUSH_BATCH();
-    if (stipple == 0 || stipple > GR_NUM_STIPPLES) {
+    if (stipple == 0 || stipple > grNumStipples) {
 	glDisable(GL_POLYGON_STIPPLE);
     } else {
 	if (grTOGLStipples[stipple] == (GLubyte *)NULL) MainExit(1);
@@ -268,7 +272,7 @@ GrTOGLInit ()
     if (toglCurrent.window == NULL)
     {
     	 TxError("No Top-Level Tk window available. . . is Tk running?\n");
-	 return false;
+	 return FALSE;
     }
 
     toglCurrent.windowid = Tk_WindowId(toglCurrent.window);
@@ -291,7 +295,7 @@ GrTOGLInit ()
 	if (!grVisualInfo)
 	{
 	    TxError("No suitable visual!\n");
-	    return false;
+	    return FALSE;
 	}
     }
     grXscrn = grVisualInfo->screen;
@@ -305,7 +309,7 @@ GrTOGLInit ()
 
     glLineWidth(1.0);
     glShadeModel (GL_FLAT);
-    glPixelStorei(GL_PACK_LSB_FIRST, true);
+    glPixelStorei(GL_PACK_LSB_FIRST, TRUE);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
 
@@ -397,7 +401,10 @@ toglSetProjection(llx, lly, width, height)
     glEnableClientState(GL_VERTEX_ARRAY);
 #endif
 
-    glEnable(GL_LINE_SMOOTH);
+    /* Because this tends to result in thick lines, it has been moved	*/
+    /* the line drawing routine so it can be enabled for individual	*/
+    /* lines.								*/
+    /* glEnable(GL_LINE_SMOOTH); */
 
     /* Force draw to front buffer (in case of double-buffered config) */
     glDrawBuffer(GL_FRONT);
@@ -443,6 +450,11 @@ TOGLEventProc(clientData, xevent)
     Tk_Window tkwind = (Tk_Window)clientData;
     Window wind;
     MagWindow *mw;
+    unsigned char LocRedirect = TxInputRedirect;
+
+    XKeyPressedEvent *KeyPressedEvent = (XKeyPressedEvent *) xevent;
+    KeySym keysym;
+    int nbytes;
 
     /* Keys and Buttons:  Determine expansion of macros or redirect
      * keys to the terminal or console.
@@ -451,71 +463,49 @@ TOGLEventProc(clientData, xevent)
     switch (xevent->type) 
     {
 	case ButtonPress:
-	case ButtonRelease:
 	    {
 		XButtonEvent *ButtonEvent = (XButtonEvent *) xevent;
-		int oldButtons = TxCurButtons;
+		int txbutton;
 
 		switch (ButtonEvent->button) {
 		case Button1:
-		    TxCurCommand.tx_button = TX_LEFT_BUTTON;
+		    txbutton = TX_LEFT_BUTTON;
+		    keysym = XK_Pointer_Button1;
 		    break;
 		case Button2:
-		    TxCurCommand.tx_button = TX_MIDDLE_BUTTON;
+		    txbutton = TX_MIDDLE_BUTTON;
+		    keysym = XK_Pointer_Button2;
 		    break;
 		case Button3:
-		    TxCurCommand.tx_button = TX_RIGHT_BUTTON;
+		    txbutton = TX_RIGHT_BUTTON;
+		    keysym = XK_Pointer_Button3;
 		    break;
-		}
-		switch(xevent->type) {
-		case ButtonRelease:
-		    TxCurCommand.tx_buttonAction = TX_BUTTON_UP;
-		    TxCurButtons &= ~(TxCurCommand.tx_button);
+		case Button4:
+		    txbutton = TX_BUTTON_4;
+		    keysym = XK_Pointer_Button4;
 		    break;
-		case ButtonPress:
-		    TxCurCommand.tx_buttonAction = TX_BUTTON_DOWN;
-		    TxCurButtons |= TxCurCommand.tx_button;
+		case Button5:
+		    txbutton = TX_BUTTON_5;
+		    keysym = XK_Pointer_Button5;
 		    break;
 		}
+		nbytes = 0;
 
-		if (oldButtons != TxCurButtons)
-		{
-		    entry = HashLookOnly(&grTOGLWindowTable, tkwind);
-		    mw = (entry)?(MagWindow *)HashGetValue(entry):0;
+		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
-		    TxCurCommand.tx_argc = 0;
-		    TxCurCommand.tx_argv[0] = NULL;
-		    TxCurCommand.tx_p.p_x = ButtonEvent->x;
-		    TxCurCommand.tx_p.p_y = glTransY(mw, ButtonEvent->y);
-		    TxCurCommand.tx_wid = mw->w_wid;
+		if (mw && (mw->w_flags & WIND_SCROLLBARS))
+		    if (WindButtonInFrame(mw, ButtonEvent->x,
+				grXtransY(mw, ButtonEvent->y),
+				txbutton))
+			break;
 
-		    DRCBreak();
-
-		    SigIOReady = false;
-		    SigInterruptOnSigIO = true;
-		    SigInterruptPending = false;
-
-		    (void) WindSendCommand(mw, &TxCurCommand);
-		    WindUpdate();
-
-		    if (SigInterruptPending)
-			TxPrintf("[Redisplay Interrupted]\n");
-
-		    TxCommandNumber++;	/* for proper "select" behavior */
-		    Tcl_DoWhenIdle(DRCContinuous, NULL);
-
-		    SigInterruptPending = false;
-		    SigInterruptOnSigIO = false;
-		    SigIOReady = false;
-		}
+		goto keys_and_buttons;
 	    }
 	    break;
 	case KeyPress:
 	    {
-		XKeyPressedEvent *KeyPressedEvent = (XKeyPressedEvent *) xevent;
-
-		KeySym keysym;
-		int nbytes, keywstate, keymod, idx, idxmax;
+		int keywstate, keymod, idx, idxmax;
 		char inChar[10];
 		Tcl_Channel outChannel = Tcl_GetStdChannel(TCL_STDOUT);
 
@@ -527,6 +517,8 @@ TOGLEventProc(clientData, xevent)
 		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
+keys_and_buttons:
+
 		keymod = (LockMask | ControlMask | ShiftMask | Mod1Mask)
 				& KeyPressedEvent->state;
 
@@ -571,24 +563,42 @@ TOGLEventProc(clientData, xevent)
 			keywstate = (keymod << 16) | ((int)inChar[idx] & 0xff);
 		    }
 
-		    if (TxInputRedirect && TxTkConsole)
+		    /* Allow buttons to bypass the console and be	*/
+		    /* treated as macros.				*/
+
+		    if (LocRedirect == TX_INPUT_REDIRECTED)
+		    {
+			switch (keysym)
+			{
+			    case XK_Pointer_Button1:
+			    case XK_Pointer_Button2:
+			    case XK_Pointer_Button3:
+			    case XK_Pointer_Button4:
+			    case XK_Pointer_Button5:
+				LocRedirect = TX_INPUT_NORMAL;;
+				break;
+			}
+		    }
+
+		    if ((LocRedirect == TX_INPUT_REDIRECTED) && TxTkConsole)
 		    {
 			Tcl_SavedResult state;
 			static char outstr[] = "::tkcon::Insert .text \"x\" ";
+
 			switch (keysym)
 			{
 			    case XK_Return:
 			        TxSetPoint(KeyPressedEvent->x,
-					glTransY(mw, KeyPressedEvent->y),
+					grXtransY(mw, KeyPressedEvent->y),
 					mw->w_wid);
-				TxInputRedirect = False;
+				TxInputRedirect = TX_INPUT_NORMAL;
 				Tcl_EvalEx(consoleinterp, "::tkcon::Eval .text",
 						19, 0);
 				TxSetPrompt('%');
 
 				Tcl_SaveResult(magicinterp, &state);
 				Tcl_EvalEx(magicinterp, "history event 0", 15, 0);
-			        MacroDefine((int)'.',
+			        MacroDefine(mw->w_client, (int)'.',
 					Tcl_GetStringResult(magicinterp),
 					FALSE);
 				Tcl_RestoreResult(magicinterp, &state);
@@ -601,6 +611,14 @@ TOGLEventProc(clientData, xevent)
 				Tcl_EvalEx(consoleinterp, "::tkcon::Event 1",
 					16, 0);
 				break;
+			    case XK_Left:
+				Tcl_EvalEx(consoleinterp, ".text mark set insert "
+					"insert-1c ; .text see insert", 50, 0);
+				break;
+			    case XK_Right:
+				Tcl_EvalEx(consoleinterp, ".text mark set insert "
+					"insert+1c ; .text see insert", 50, 0);
+				break;
 			    case XK_BackSpace: case XK_Delete:
 				Tcl_EvalEx(consoleinterp, ".text delete insert-1c ;"
 					".text see insert", 40, 0);
@@ -609,7 +627,7 @@ TOGLEventProc(clientData, xevent)
 				outstr[23] = '\\';
 				outstr[24] = inChar[idx];
 				outstr[25] = '\"';
-				Tcl_EvalEx(consoleinterp, outstr, 25, 0);
+				Tcl_EvalEx(consoleinterp, outstr, 26, 0);
 				outstr[24] = '\"';
 				outstr[25] = '\0';
 			    default:
@@ -618,7 +636,7 @@ TOGLEventProc(clientData, xevent)
 				break;
 			}
 		    }
-		    else if (TxInputRedirect) {
+		    else if (LocRedirect == TX_INPUT_REDIRECTED) {
 			int tl;
 			if (TxBuffer == NULL)
 			{
@@ -648,10 +666,11 @@ TOGLEventProc(clientData, xevent)
 			{
 			    *(TxBuffer + tl) = '\n';
 			    *(TxBuffer + tl + 1) = '\0';
-			    if (tl != 0) MacroDefine(XK_period, TxBuffer, FALSE);
-			    TxInputRedirect = False;
+			    if (tl != 0) MacroDefine(mw->w_client,
+					XK_period, TxBuffer, FALSE);
+			    TxInputRedirect = TX_INPUT_NORMAL;
 			    TxSetPoint(KeyPressedEvent->x,
-					glTransY(mw, KeyPressedEvent->y),
+					grXtransY(mw, KeyPressedEvent->y),
 					mw->w_wid);
 			    TxPrintf("\n");
 			    TxFlushOut();
@@ -670,7 +689,7 @@ TOGLEventProc(clientData, xevent)
 				|| (keywstate == (int)TX_LONG_CMD2))
 		    {
 			/* Redirect input into the interpreter's terminal window */
-			TxInputRedirect = True;
+			TxInputRedirect = TX_INPUT_REDIRECTED;
 			if (TxTkConsole)
 			    TxSetPrompt(':');
 			else
@@ -684,7 +703,8 @@ TOGLEventProc(clientData, xevent)
 			bool iMacro;
 			char *macroDef;
 
-			macroDef = MacroRetrieve(keywstate, &iMacro);
+			macroDef = MacroRetrieve(mw->w_client, keywstate, &iMacro);
+			macroDef = MacroSubstitute(macroDef, "%W", Tk_PathName(tkwind));
 
 			if (macroDef == NULL)
 			{
@@ -721,23 +741,18 @@ TOGLEventProc(clientData, xevent)
 				    TxPrintf("%s", macroDef);
 				    TxFlushOut();
 				}
-				TxInputRedirect = True;
+				TxInputRedirect = TX_INPUT_REDIRECTED;
 			    }
 			    else
 			    {
-				/* TxParseString is defined by magic/magicwrap.i */
-				/* and calls Tcl_Eval()				 */
+				/* TxParseString is defined by tcltk/tclmagic.c
+				 * and calls Tcl_Eval()
+				 */
 
-				char *outstring = Tcl_Alloc(sl +
-					 strlen(Tk_PathName(tkwind)) + 2);
-				sprintf(outstring, "%s %s", 
-					Tk_PathName(tkwind),
-					macroDef);
 				TxSetPoint(KeyPressedEvent->x,
-					glTransY(mw, KeyPressedEvent->y),
+					grXtransY(mw, KeyPressedEvent->y),
 					mw->w_wid);
-				TxParseString(outstring, NULL, NULL);
-				Tcl_Free(outstring);
+				TxParseString(macroDef, NULL, NULL);
 			    }
 			    freeMagic(macroDef);
 			}
@@ -770,6 +785,34 @@ TOGLEventProc(clientData, xevent)
 
             }
             break;
+	case VisibilityNotify:
+	    {
+		XVisibilityEvent *VisEvent = (XVisibilityEvent*) xevent;
+
+		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
+
+		switch(VisEvent->state)
+		{
+		    case VisibilityUnobscured:
+			mw->w_flags &= ~WIND_OBSCURED;
+			if (mw->w_backingStore == (ClientData)NULL)
+			{
+			    grtkCreateBackingStore(mw);
+			    if (mw->w_backingStore != (ClientData)NULL)
+			    {
+				WindAreaChanged(mw, &mw->w_allArea);
+				WindUpdate();
+			    }
+			}
+			break;
+		    case VisibilityPartiallyObscured:
+		    case VisibilityFullyObscured:
+			mw->w_flags |= WIND_OBSCURED;
+			break;
+		}
+	    }
+	    break;
 	case Expose:
 	    {
 		XExposeEvent *ExposeEvent = (XExposeEvent*) xevent;
@@ -784,7 +827,16 @@ TOGLEventProc(clientData, xevent)
             	screenRect.r_ybot = mw->w_allArea.r_ytop - 
 				(ExposeEvent->y + ExposeEvent->height);
 
-		WindAreaChanged(mw, &screenRect);
+		if (mw->w_backingStore != (ClientData)NULL)
+		{
+		    Rect surface;
+		    (*GrGetBackingStorePtr)(mw, &screenRect);
+		    WindScreenToSurface(mw, &screenRect, &surface);
+		    DBWHLRedrawPrepWindow(mw, &surface);
+		    WindDrawBorder(mw, &screenRect);
+		}
+		else
+		    WindAreaChanged(mw, &screenRect);
 		WindUpdate();
             }
 	    break;
@@ -832,11 +884,11 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     char *planecount;
     char *fullname;
     FILE* f;
-    bool execFailed = false;
+    bool execFailed = FALSE;
     int x, y, width, height;
 
     WindPackageType = WIND_X_WINDOWS;  /* to be changed? */
-    TxInputRedirect = False;
+    TxInputRedirect = TX_INPUT_NORMAL;
 
     grCursorType = "color";
     WindScrollBarWidth = 14;
@@ -874,6 +926,7 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     grSetSPatternPtr = grtoglSetSPattern;
     grPutTextPtr = grtoglPutText;
     grDefineCursorPtr = grTkDefineCursor;
+    grFreeCursorPtr = grTkFreeCursors;
     GrBitBltPtr = GrTOGLBitBlt;
     grDrawGridPtr = grtoglDrawGrid;
     grDrawLinePtr = grtoglDrawLine;
@@ -882,18 +935,23 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     grSetStipplePtr = grtoglSetStipple;
     grSetLineStylePtr = grtoglSetLineStyle;
     grSetCharSizePtr = grtoglSetCharSize;
-    grMaxStipples = 32;
 #ifdef NONMANHATTAN
     grFillPolygonPtr = grtoglFillPolygon;
 #endif
+
+    GrFreeBackingStorePtr = grtkFreeBackingStore;
+    GrCreateBackingStorePtr = grtkCreateBackingStore;
+    GrGetBackingStorePtr = grtkGetBackingStore;
+    GrPutBackingStorePtr = grtkPutBackingStore;
+    GrScrollBackingStorePtr = grtkScrollBackingStore;
     
     if (execFailed) {
 	TxError("Execution failed!\n");
-	return false;
+	return FALSE;
     }
 
     if(!GrTOGLInit()){
-	return false;
+	return FALSE;
     };
 
     Tk_GetVRootGeometry(Tk_MainWindow(magicinterp), &x, &y, &width, &height);
@@ -902,7 +960,7 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     GrScreenRect.r_xtop = width + x;
     GrScreenRect.r_ytop = height + y;
 
-    return Tk_MainWindow(magicinterp) ? true : false;
+    return Tk_MainWindow(magicinterp) ? TRUE : FALSE;
 }
 
 extern void MakeWindowCommand();
@@ -1040,7 +1098,7 @@ GrTOGLCreate(w, name)
 	/* set up Tk event handler to start processing */
 
 	Tk_CreateEventHandler(tkwind, ExposureMask | StructureNotifyMask
-		| ButtonPressMask | ButtonReleaseMask | KeyPressMask,
+		| ButtonPressMask | KeyPressMask | VisibilityChangeMask,
 		 (Tk_EventProc *)TOGLEventProc, (ClientData) tkwind);
 
 	/* set up commands to be passed expressly to this window */  
