@@ -23,24 +23,25 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 
-#include "magic/tclmagic.h"
-#include "misc/magic.h"
-#include "misc/magsgtty.h"
+#include "tcltk/tclmagic.h"
+#include "utils/magic.h"
+#include "utils/malloc.h"
+#include "utils/magsgtty.h"
 #include "utils/geometry.h"
 #include "windows/windows.h"
 #include "graphics/graphics.h"
 #include "graphics/graphicsInt.h"
 #include "textio/textio.h"
 #include "textio/txcommands.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "tiles/tile.h"
 #include "utils/utils.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "drc/drc.h"
-#include "macros/macros.h"
+#include "utils/macros.h"
 #include "grTkInt.h"
-#include "misc/paths.h"
+#include "utils/paths.h"
 #include "graphics/grTkCommon.h"
 
 GR_CURRENT grCurrent = {
@@ -60,7 +61,7 @@ unsigned long grPixels[256];
 unsigned long grPlanes[256];
 XColor   colors[256*3];	/* Unique colors used by Magic */
 
-Pixmap grTkStipples[GR_NUM_STIPPLES];
+Pixmap *grTkStipples;
 HashTable	grTkWindowTable;
 /* locals */
 
@@ -259,13 +260,14 @@ grtkSetLineStyle (style)
  */
 
 void
-grtkSetSPattern (stipple, pattern)
-    int stipple;			/* The stipple number, 1-15. */
-    int pattern[8];			/* 8 8-bit patterns integers */
+grtkSetSPattern (sttable, numstipples)
+    int **sttable;			/* The table of patterns */
+    int numstipples;			/* Number of stipples */
 {
     Tk_Window tkwind;
+    Window xwid;
     Pixmap p;
-    int x,y,pat;
+    int i, x, y, pat;
 
     tkwind = Tk_MainWindow(magicinterp);
 
@@ -275,21 +277,26 @@ grtkSetSPattern (stipple, pattern)
 
     if (tkwind == 0 || Tk_WindowId(tkwind) == 0)
 	Tk_MakeWindowExist(tkwind);
+    xwid = Tk_WindowId(tkwind);
 
-    p = Tk_GetPixmap(grXdpy, Tk_WindowId(tkwind), 8, 8, 1);
-    if (grGCStipple == 0) {
-	/* grGCStipple = Tk_GetGC(tkwind, 0, 0); */
-	grGCStipple = XCreateGC(grXdpy, p, 0, 0);
-    }
-    for (y = 0; y < 8; y++) {
-	pat = pattern[y];
-	for (x = 0; x < 8; x++) {
-	    XSetForeground(grXdpy, grGCStipple, pat & 1);
-	    XDrawPoint(grXdpy, p, grGCStipple, x, y);
-	    pat >>= 1;
+    grTkStipples = (Pixmap *)mallocMagic(numstipples * sizeof(Pixmap));
+    for (i = 0; i < numstipples; i++)
+    {
+	p = Tk_GetPixmap(grXdpy, xwid, 8, 8, 1);
+	if (grGCStipple == NULL) {
+	    /* grGCStipple = Tk_GetGC(tkwind, 0, 0); */
+	    grGCStipple = XCreateGC(grXdpy, p, 0, 0);
 	}
+	for (y = 0; y < 8; y++) {
+	    pat = sttable[i][y];
+	    for (x = 0; x < 8; x++) {
+		XSetForeground(grXdpy, grGCStipple, pat & 1);
+		XDrawPoint(grXdpy, p, grGCStipple, x, y);
+		pat >>= 1;
+	    }
+	}
+	grTkStipples[i] = p;
     }
-    grTkStipples[stipple] = p;
 }
 
 
@@ -313,7 +320,7 @@ grtkSetStipple (stipple)
     if (stipple == oldStip) return;
     oldStip = stipple;
     GR_TK_FLUSH_BATCH();
-    if (stipple == 0 || stipple > GR_NUM_STIPPLES) {
+    if (stipple == 0 || stipple > grNumStipples) {
 	XSetFillStyle(grXdpy, grGCFill, FillSolid);
     } else {
 	if (grTkStipples[stipple] == 0) MainExit(1);
@@ -371,7 +378,7 @@ GrTkInit(dispType)
     if (grCurrent.window == NULL)
     {
     	 TxError("No Top-Level Tk window available. . . is Tk running?\n");
-	 return false;
+	 return FALSE;
     }
 
     grXdpy = Tk_Display(grCurrent.window);
@@ -554,7 +561,7 @@ GrTkInit(dispType)
     {
 	status = 0;
 	if (grClass != 4)
-	    status= XAllocColorCells(grXdpy, grXcmap, True, grDisplay.planes,	
+	    status= XAllocColorCells(grXdpy, grXcmap, TRUE, grDisplay.planes,	
 			grDisplay.planeCount, &grDisplay.basepixel, 1); 
 	if (status == 0) 
 	{
@@ -582,7 +589,7 @@ GrTkInit(dispType)
 		TxPrintf("Unable to allocate %d planes in default colormap; "
 			 "making a new one.\n", grDisplay.planeCount);
 #ifdef MAGIC_WRAPPER
-	    if (grClass == 3) GrTkInstalledCMap = True;
+	    if (grClass == 3) GrTkInstalledCMap = TRUE;
 #endif
 	    if (grDisplay.planeCount <= 8)
 	    {
@@ -671,7 +678,8 @@ GrTkClose ()
 
     if (grXdpy == NULL) return;
 
-    grGCStipple = 0;
+    XFreeGC(grXdpy, grGCStipple);
+    grGCStipple = NULL;
 
     /* Free Tk fonts from the font cache */
     grTkFreeFonts();
@@ -725,6 +733,11 @@ MagicEventProc(clientData, xevent)
     HashEntry	*entry;
     Tk_Window wind = (Tk_Window)clientData;
     MagWindow *mw;
+    unsigned char LocRedirect = TxInputRedirect;
+
+    XKeyPressedEvent *KeyPressedEvent = (XKeyPressedEvent *) xevent;
+    KeySym keysym;
+    int nbytes;
 
     /* Keys and Buttons:  Determine expansion of macros or redirect
      * keys to the terminal/console.
@@ -733,73 +746,51 @@ MagicEventProc(clientData, xevent)
     switch (xevent->type) 
     {
 	case ButtonPress:
-	case ButtonRelease:
 	    {
 		XButtonEvent *ButtonEvent = (XButtonEvent *) xevent;
-		int oldButtons = TxCurButtons;
+		int txbutton;
 
 		switch (ButtonEvent->button) {
 		case Button1:
-		    TxCurCommand.tx_button = TX_LEFT_BUTTON;
+		    txbutton = TX_LEFT_BUTTON;
+		    keysym = XK_Pointer_Button1;
 		    break;
 		case Button2:
-		    TxCurCommand.tx_button = TX_MIDDLE_BUTTON;
+		    txbutton = TX_MIDDLE_BUTTON;
+		    keysym = XK_Pointer_Button2;
 		    break;
 		case Button3:
-		    TxCurCommand.tx_button = TX_RIGHT_BUTTON;
+		    txbutton = TX_RIGHT_BUTTON;
+		    keysym = XK_Pointer_Button3;
 		    break;
-		}
-		switch(xevent->type) {
-		case ButtonRelease:
-		    TxCurCommand.tx_buttonAction = TX_BUTTON_UP;
-	 	    TxCurButtons &= ~(TxCurCommand.tx_button);
+		case Button4:
+		    txbutton = TX_BUTTON_4;
+		    keysym = XK_Pointer_Button4;
 		    break;
-		case ButtonPress:
-		    TxCurCommand.tx_buttonAction = TX_BUTTON_DOWN;
-	 	    TxCurButtons |= TxCurCommand.tx_button;
+		case Button5:
+		    txbutton = TX_BUTTON_5;
+		    keysym = XK_Pointer_Button5;
 		    break;
 		}
+		nbytes = 0;
 
-		if (oldButtons != TxCurButtons)
-		{
-		    entry = HashLookOnly(&grTkWindowTable, wind);
-		    mw = (entry)?(MagWindow *)HashGetValue(entry):0;
-	            grCurrent.window = wind;
-		    grCurrent.windowid = Tk_WindowId(wind);
-	            grCurrent.mw = mw;
+		entry = HashLookOnly(&grTkWindowTable, wind);
+		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
-		    TxCurCommand.tx_argc = 0;
-		    TxCurCommand.tx_argv[0] = NULL;
-		    TxCurCommand.tx_p.p_x = ButtonEvent->x;
-		    TxCurCommand.tx_p.p_y = grXToMagic(ButtonEvent->y);
-		    TxCurCommand.tx_wid = mw->w_wid;
+		/* Hack---non-wrapper versions of magic have	*/
+		/* fixed handlers for the frame.		*/
+		if (mw && (mw->w_flags & WIND_SCROLLBARS))
+		    if (WindButtonInFrame(mw, ButtonEvent->x,
+				grXToMagic(ButtonEvent->y),
+				txbutton))
+			break;
 
-		    DRCBreak();
-
-		    SigIOReady = false;
-		    SigInterruptOnSigIO = true;
-		    SigInterruptPending = false;
-
-		    (void) WindSendCommand(mw, &TxCurCommand);
-		    WindUpdate();
-
-		    if (SigInterruptPending)
-			TxPrintf("[Redisplay Interrupted]\n");
-
-		    TxCommandNumber++;	/* for proper "select" behavior */
-		    Tcl_DoWhenIdle(DRCContinuous, NULL);
-
-		    SigInterruptPending = false;
-		    SigInterruptOnSigIO = false;
-		    SigIOReady = false;
-		}
+		goto keys_and_buttons;
 	    }
 	    break;
 	case KeyPress:
 	    {
-		XKeyPressedEvent *KeyPressedEvent = (XKeyPressedEvent *) xevent;
-		KeySym keysym;
-		int nbytes, keywstate, keymod, idx, idxmax;
+		int keywstate, keymod, idx, idxmax;
 		char inChar[10];
 		Tcl_Channel outChannel = Tcl_GetStdChannel(TCL_STDOUT);
 
@@ -810,6 +801,8 @@ MagicEventProc(clientData, xevent)
 
 		entry = HashLookOnly(&grTkWindowTable, wind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
+
+keys_and_buttons:
 		grCurrent.window = wind;
 		grCurrent.windowid = Tk_WindowId(wind);
 		grCurrent.mw = mw;
@@ -862,51 +855,84 @@ MagicEventProc(clientData, xevent)
 			keywstate = (keymod << 16) | ((int)inChar[idx] & 0xff);
 		    }
 
-		    if (TxInputRedirect && TxTkConsole)
+		    /* Allow buttons to bypass the console and be	*/
+		    /* treated as macros.				*/
+ 
+		    if (LocRedirect == TX_INPUT_REDIRECTED)
+		    {  
+			switch (keysym)
+			{
+			    case XK_Pointer_Button1:
+			    case XK_Pointer_Button2:
+			    case XK_Pointer_Button3:
+			    case XK_Pointer_Button4:
+			    case XK_Pointer_Button5:
+				LocRedirect = TX_INPUT_NORMAL;;
+				break;
+			}
+		    }
+
+		    if ((LocRedirect == TX_INPUT_REDIRECTED) && TxTkConsole)
 		    {
 			Tcl_SavedResult state;
 			static char outstr[] = "::tkcon::Insert .text \"x\" ";
+			/* Translate Control-H to BackSpace */
+			if ((keymod & ControlMask) && (keysym == XK_h))
+			    keysym = XK_BackSpace;
+
 			switch (keysym)
 			{
 			    case XK_Return:
 			        TxSetPoint(KeyPressedEvent->x,
 					grXToMagic(KeyPressedEvent->y),
 					grCurrent.mw->w_wid);
-				TxInputRedirect = False;
-				Tcl_Eval(consoleinterp, "::tkcon::Eval .text");
+				TxInputRedirect = TX_INPUT_NORMAL;
+				Tcl_EvalEx(consoleinterp, "::tkcon::Eval .text",
+						19, 0);
 				TxSetPrompt('%');
 
 				Tcl_SaveResult(magicinterp, &state);
-				Tcl_Eval(magicinterp, "history event 0");
-				MacroDefine((int)'.',
+				Tcl_EvalEx(magicinterp, "history event 0", 15,
+						0);
+				MacroDefine(mw->w_client, (int)'.',
 					Tcl_GetStringResult(magicinterp),
 					FALSE);
 				Tcl_RestoreResult(magicinterp, &state);
 				break;
 			    case XK_Up:
-				Tcl_Eval(consoleinterp, "::tckon::Event -1");
+				Tcl_EvalEx(consoleinterp, "::tckon::Event -1",
+						17, 0);
 				break;
 			    case XK_Down:
-				Tcl_Eval(consoleinterp, "::tckon::Event 1");
+				Tcl_EvalEx(consoleinterp, "::tckon::Event 1",
+						16, 0);
+				break;
+			    case XK_Left:
+				Tcl_EvalEx(consoleinterp, ".text mark set insert "
+					"insert-1c ; .text see insert", 50, 0);
+				break;
+			    case XK_Right:
+				Tcl_EvalEx(consoleinterp, ".text mark set insert "
+					"insert+1c ; .text see insert", 50, 0);
 				break;
 			    case XK_BackSpace: case XK_Delete:
-				Tcl_Eval(consoleinterp, ".text delete insert-1c ; "
-					".text see insert");
+				Tcl_EvalEx(consoleinterp, ".text delete insert-1c ; "
+					".text see insert", 41, 0);
 				break;
 			    case XK_quotedbl: case XK_backslash: case XK_bracketleft:
 				outstr[23] = '\\';
 				outstr[24] = inChar[idx];
 				outstr[25] = '\"';
-				Tcl_Eval(consoleinterp, outstr);
+				Tcl_EvalEx(consoleinterp, outstr, 26, 0);
 				outstr[24] = '\"';
 				outstr[25] = '\0';
 			    default:
 				outstr[23] = inChar[idx];
-				Tcl_Eval(consoleinterp, outstr);
+				Tcl_EvalEx(consoleinterp, outstr, 25, 0);
 				break;
 			}
 		    }
-		    else if (TxInputRedirect)
+		    else if (LocRedirect == TX_INPUT_REDIRECTED)
 		    {
 			int tl;	
 			if (TxBuffer == NULL)
@@ -937,8 +963,9 @@ MagicEventProc(clientData, xevent)
 			{
 			    *(TxBuffer + tl) = '\n';
 			    *(TxBuffer + tl + 1) = '\0';
-			    if (tl != 0) MacroDefine((int)'.', TxBuffer, FALSE);
-			    TxInputRedirect = False;
+			    if (tl != 0) MacroDefine(mw->w_client, (int)'.',
+					TxBuffer, FALSE);
+			    TxInputRedirect = TX_INPUT_NORMAL;
 			    TxSetPoint(KeyPressedEvent->x,
 					grXToMagic(KeyPressedEvent->y),
 					grCurrent.mw->w_wid);
@@ -959,7 +986,7 @@ MagicEventProc(clientData, xevent)
 				|| (keywstate == (int)TX_LONG_CMD2))
 		    {
 			/* Redirect input into the interpreter's terminal window */
-			TxInputRedirect = True;
+			TxInputRedirect = TX_INPUT_REDIRECTED;
 			if (TxTkConsole)
 			    TxSetPrompt(':');
 			else
@@ -973,7 +1000,8 @@ MagicEventProc(clientData, xevent)
 			bool iMacro;
 			char *macroDef;
 
-			macroDef = MacroRetrieve(keywstate, &iMacro);
+			macroDef = MacroRetrieve(mw->w_client, keywstate, &iMacro);
+			macroDef = MacroSubstitute(macroDef, "%W", Tk_PathName(wind));
 
 			if (macroDef == NULL)
 			{
@@ -999,7 +1027,7 @@ MagicEventProc(clientData, xevent)
 				    char *outstring = Tcl_Alloc(sl + 20);
 				    sprintf(outstring, ".text insert end \"%s\"",
 						macroDef);
-				    Tcl_Eval(consoleinterp, outstring);
+				    Tcl_EvalEx(consoleinterp, outstring, -1, 0);
 				    Tcl_Free(outstring);
 				}
 				else
@@ -1009,21 +1037,18 @@ MagicEventProc(clientData, xevent)
 				    TxPrintf("%s", macroDef);
 				    TxFlushOut();
 				}
-				TxInputRedirect = True;
+				TxInputRedirect = TX_INPUT_REDIRECTED;
 			    }
 			    else
 			    {
-				/* TxParseString is defined by magic/magicwrap.i */
-				/* and calls Tcl_Eval()				 */
-				char *outstring = Tcl_Alloc(sl +
-					strlen(Tk_PathName(wind)) + 2);
-				sprintf(outstring, "%s %s",
-					Tk_PathName(wind), macroDef);
+				/* TxParseString is defined by tcltk/tclmagic.c
+				 * and calls Tcl_Eval().
+				 */
+
 				TxSetPoint(KeyPressedEvent->x,
 					grXToMagic(KeyPressedEvent->y),
 					grCurrent.mw->w_wid);
-				TxParseString(outstring, NULL, NULL);
-				Tcl_Free(outstring);
+				TxParseString(macroDef, NULL, NULL);
 			    }
 			    freeMagic(macroDef);
 			}
@@ -1031,6 +1056,34 @@ MagicEventProc(clientData, xevent)
 		}
 	    } 
 	    break;
+	case VisibilityNotify:
+	    {
+		XVisibilityEvent *VisEvent = (XVisibilityEvent*) xevent;
+
+		entry = HashLookOnly(&grTkWindowTable, wind);
+		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
+
+		switch(VisEvent->state)
+		{
+		    case VisibilityUnobscured:
+			mw->w_flags &= ~WIND_OBSCURED;
+			if (mw->w_backingStore == (ClientData)NULL)
+			{
+			    grtkCreateBackingStore(mw);
+			    if (mw->w_backingStore != (ClientData)NULL)
+			    {
+				WindAreaChanged(mw, &mw->w_allArea);
+				WindUpdate();
+			    }
+			}
+			break;
+		    case VisibilityPartiallyObscured:
+		    case VisibilityFullyObscured:
+			mw->w_flags |= WIND_OBSCURED;
+			break;
+		}
+	    }
+	    break;
 	case Expose:
 	    {
 		XExposeEvent *ExposeEvent = (XExposeEvent*) xevent;
@@ -1048,7 +1101,16 @@ MagicEventProc(clientData, xevent)
             	screenRect.r_ybot = mw->w_allArea.r_ytop - 
 				(ExposeEvent->y + ExposeEvent->height);
 
-		WindAreaChanged(mw, &screenRect);
+		if (mw->w_backingStore != (ClientData)NULL)
+		{
+		    Rect surface;
+		    (*GrGetBackingStorePtr)(mw, &screenRect);
+		    WindScreenToSurface(mw, &screenRect, &surface);
+		    DBWHLRedrawPrepWindow(mw, &surface);
+		    WindDrawBorder(mw, &screenRect);
+		}
+		else
+		    WindAreaChanged(mw, &screenRect);
 		WindUpdate();
             }
 	    break;
@@ -1116,12 +1178,12 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     char *planecount;
     char *fullname;
     FILE* f;
-    bool execFailed = false;
+    bool execFailed = FALSE;
     int x, y, width, height;
 
     WindPackageType = WIND_X_WINDOWS;
-    TxInputRedirect = False;
-    grCursorType = "bw";
+    TxInputRedirect = TX_INPUT_NORMAL;
+    grCursorType = "color";
     WindScrollBarWidth = 14;
 
     /* Set up the procedure values in the indirection table. */
@@ -1155,6 +1217,7 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     grSetSPatternPtr = grtkSetSPattern;
     grPutTextPtr = grtkPutText;
     grDefineCursorPtr = grTkDefineCursor;
+    grFreeCursorPtr = grTkFreeCursors;
     GrBitBltPtr = GrTkBitBlt;
     grDrawGridPtr = grtkDrawGrid;
     grDrawLinePtr = grtkDrawLine;
@@ -1163,18 +1226,23 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     grSetStipplePtr = grtkSetStipple;
     grSetLineStylePtr = grtkSetLineStyle;
     grSetCharSizePtr = grtkSetCharSize;
-    grMaxStipples = 32;
 #ifdef NONMANHATTAN
     grFillPolygonPtr = grtkFillPolygon;
 #endif
+
+    GrFreeBackingStorePtr = grtkFreeBackingStore;
+    GrCreateBackingStorePtr = grtkCreateBackingStore;
+    GrGetBackingStorePtr = grtkGetBackingStore;
+    GrPutBackingStorePtr = grtkPutBackingStore;
+    GrScrollBackingStorePtr = grtkScrollBackingStore;
     
     if (execFailed) {
 	TxError("Execution failed!\n");
-	return false;
+	return FALSE;
     }
 
     if (!GrTkInit(dispType)) {
-	return false;
+	return FALSE;
     };
 
     Tk_GetVRootGeometry(Tk_MainWindow(magicinterp), &x, &y, &width, &height);
@@ -1183,7 +1251,7 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     GrScreenRect.r_xtop = x + width;
     GrScreenRect.r_ytop = y + height;
 
-    return Tk_MainWindow(magicinterp) ? true : false;
+    return Tk_MainWindow(magicinterp) ? TRUE : FALSE;
 }
 
 extern void MakeWindowCommand();
@@ -1324,7 +1392,6 @@ GrTkCreate(w, name)
         Tk_DefineCursor(tkwind, grCurrent.cursor);
 	GrTkIconUpdate(w, w->w_caption); 
 
-
 	/*----------------------------------------------------------------------*/
 	/* If we're using TkCon in a PseudoColor visual, we need to set the	*/
 	/* console's colormap to match the magic window.  TkCon need know	*/
@@ -1336,7 +1403,7 @@ GrTkCreate(w, name)
 	if (TxTkConsole && !WindowNumber)
 	{
 	    Window parent, rret, *clist;
-	    int iret;
+	    unsigned int iret;
 
 	    if (Tk_Visual(tktop) == Tk_Visual(tkwind))
 	    {
@@ -1353,15 +1420,15 @@ GrTkCreate(w, name)
 	    else
 		GrTkInstalledCMap = FALSE;
 
-	    Tcl_Eval(consoleinterp, "catch repaintconsole");
+	    Tcl_EvalEx(consoleinterp, "catch repaintconsole", 20, 0);
 	}
 	WindowNumber++;
 
 	/* set up Tk event handler to start processing */
 
 	Tk_CreateEventHandler(tkwind, ExposureMask | StructureNotifyMask
-		| ButtonPressMask | ButtonReleaseMask | KeyPressMask,
-		 (Tk_EventProc *)MagicEventProc, (ClientData) tkwind);
+		| ButtonPressMask | KeyPressMask | VisibilityChangeMask,
+		(Tk_EventProc *)MagicEventProc, (ClientData) tkwind);
 
 	/* set up commands to be passed expressly to this window */
 
@@ -1378,7 +1445,7 @@ GrTkCreate(w, name)
 	{
 	    Tk_Window tklayout = tkwind;
 	    Window parent, rret, *clist;
-	    int iret;
+	    unsigned int iret;
 
 	    while (!Tk_IsTopLevel(tklayout)) tklayout = Tk_Parent(tklayout);
 	    XQueryTree(grXdpy, Tk_WindowId(tklayout), &rret, &parent, &clist, &iret);
@@ -1598,7 +1665,7 @@ GrTkUnlock(w)
  *	in the queue.
  *
  * Results:
- *	True if an event is in the event queue.
+ *	TRUE if an event is in the event queue.
  *
  * Side effects:
  *	Hopefully none (put back the event!)
@@ -1614,6 +1681,8 @@ GrTkEventPending()
    XEvent	genEvent;
    bool		retval;
 
+   if (wind == 0) return FALSE;
+
    retval = XCheckWindowEvent(grXdpy, wind, ExposureMask
 		| StructureNotifyMask | ButtonPressMask
 		| KeyPressMask, &genEvent); 
