@@ -9,16 +9,17 @@
  */
 
 #include <stdio.h>
+#include <string.h>
 #include <X11/Xlib.h>
 
-#include "magic/tclmagic.h"
-#include "misc/magic.h"
+#include "tcltk/tclmagic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "windows/windows.h"
 #include "graphics/graphics.h"
 #include "graphics/graphicsInt.h"
 #include "textio/textio.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "utils/utils.h"
 #include "utils/hash.h"
 #include "graphics/grTkInt.h"
@@ -44,7 +45,7 @@ grtkDrawGrid (prect, outline, clip)
 			         * for the grid.  Note:  in order to maintain
 			         * precision for the grid, the rectangle
 			         * coordinates are specified in units of
-			         * screen coordinates multiplied by 4096.
+			         * screen coordinates multiplied by SUBPIXEL.
 			         */
     int outline;		/* the outline style */
     Rect *clip;			/* a clipping rectangle */
@@ -61,19 +62,19 @@ grtkDrawGrid (prect, outline, clip)
 	return FALSE;
     
     xstart = prect->r_xbot % xsize;
-    while (xstart < clip->r_xbot<<12) xstart += xsize;
+    while (xstart < clip->r_xbot << SUBPIXELBITS) xstart += xsize;
     ystart = prect->r_ybot % ysize;
-    while (ystart < clip->r_ybot<<12) ystart += ysize;
+    while (ystart < clip->r_ybot << SUBPIXELBITS) ystart += ysize;
     
     grtkSetLineStyle(outline);
 
     snum = 0;
     low = grMagicToX(clip->r_ybot);
     hi = grMagicToX(clip->r_ytop);
-    for (x = xstart; x < (clip->r_xtop+1)<<12; x += xsize)
+    for (x = xstart; x < (clip->r_xtop + 1) << SUBPIXELBITS; x += xsize)
     {
         if (snum >= GR_NUM_GRIDS) break;
-	shifted = x >> 12;
+	shifted = x >> SUBPIXELBITS;
 	seg[snum].x1 = shifted;
 	seg[snum].y1 = low;
 	seg[snum].x2 = shifted;
@@ -85,10 +86,10 @@ grtkDrawGrid (prect, outline, clip)
     snum = 0;
     low = clip->r_xbot;
     hi = clip->r_xtop;
-    for (y = ystart; y < (clip->r_ytop+1)<<12; y += ysize)
+    for (y = ystart; y < (clip->r_ytop + 1) << SUBPIXELBITS; y += ysize)
     {
         if (snum >= GR_NUM_GRIDS) break;
-	shifted = grMagicToX(y >> 12);
+	shifted = grMagicToX(y >> SUBPIXELBITS);
 	seg[snum].x1 = low;
 	seg[snum].y1 = shifted;
 	seg[snum].x2 = hi;
@@ -96,7 +97,7 @@ grtkDrawGrid (prect, outline, clip)
 	snum++;
     }
     XDrawSegments(grXdpy, grCurrent.windowid, grGCDraw, seg, snum);
-    return true;
+    return TRUE;
 }
 
 
