@@ -19,8 +19,8 @@
 #include <X11/keysym.h>
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
-#include "misc/styles.h"
+#include "utils/magic.h"
+#include "utils/styles.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
 #include "utils/malloc.h"
@@ -29,7 +29,7 @@
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwtech.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "graphics/graphics.h"
 #include "graphics/graphicsInt.h"
 #include "dbwind/dbwind.h"
@@ -126,6 +126,29 @@ grTkFreeFonts()
         Tk_FreeFont(grTkFonts[i]);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * grTkFreeCursors:
+ *
+ *	Remove cursors from the Tk cursor cache.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Tk forgets about the glyph cursors.
+ * ----------------------------------------------------------------------------
+ */
+
+void
+grTkFreeCursors(glyphs)
+    GrGlyphs *glyphs;
+{
+    int i;
+    for (i = 0; i < glyphs->gr_num; i++)
+	Tk_FreeCursor(grXdpy, grCursors[i]);
+}
+
 /*
  * ----------------------------------------------------------------------------
  * grTkDefineCursor:
@@ -161,7 +184,7 @@ grTkDefineCursor(glyphs)
 
     if (glyphs->gr_num > MAX_CURSORS)
     {
-	TxError("X only has room for %d cursors\n", MAX_CURSORS);
+	TxError("magic/Tk only has room for %d cursors\n", MAX_CURSORS);
 	return;
     }
 
@@ -288,6 +311,273 @@ GrTkWindowName(mw)
     return Tk_PathName(tkwind);
 }  
 
+/*
+ * ----------------------------------------------------------------------------
+ * grtkFreeBackingStore --
+ *	Free up Pixmap memory for a backing store cell.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	memory Free'd
+ * ----------------------------------------------------------------------------
+ */
+
+void
+grtkFreeBackingStore(MagWindow *window)
+{
+    Pixmap pmap = (Pixmap)window->w_backingStore;
+    if (pmap == (Pixmap)NULL) return;
+    XFreePixmap(grXdpy, pmap);
+    window->w_backingStore = (ClientData)NULL;
+    /* TxPrintf("grtkFreeBackingStore called\n"); */
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * grtkCreateBackingStore --
+ *	Create Pixmap memory for a backing store cell and copy data
+ *	from the window into it.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	memory Allocated.
+ * 
+ * ----------------------------------------------------------------------------
+ */
+
+void
+grtkCreateBackingStore(MagWindow *w)
+{
+    Pixmap pmap;
+    Tk_Window tkwind = (Tk_Window)w->w_grdata;
+    Window wind;
+    unsigned int width, height;
+
+    /* ignore all windows other than layout */
+    if (w->w_client != DBWclientID) return;
+
+    /* deferred */
+    if (tkwind == NULL) return;
+
+    wind = Tk_WindowId(tkwind);
+    width = w->w_screenArea.r_xtop - w->w_screenArea.r_xbot;
+    height = w->w_screenArea.r_ytop - w->w_screenArea.r_ybot;
+
+    if (w->w_backingStore != (ClientData)NULL) grtkFreeBackingStore(w);
+
+    pmap = XCreatePixmap(grXdpy, wind, width, height, Tk_Depth(tkwind));
+    w->w_backingStore = (ClientData)pmap;
+
+    /* TxPrintf("grtkCreateBackingStore area %d %d %d %d\n",
+	w->w_screenArea.r_xbot, w->w_screenArea.r_ybot,
+	w->w_screenArea.r_xtop, w->w_screenArea.r_ytop); */
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * grtkGetBackingStore --
+ *	Copy data from a backing store Pixmap into the indicated window.
+ *
+ * Results:
+ *	TRUE if backing store was copied successfully, FALSE if not.
+ *
+ * Side effects:
+ *	Data copied into Pixmap memory.
+ * 
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+grtkGetBackingStore(MagWindow *w, Rect *area)
+{
+    Pixmap pmap;
+    Tk_Window tkwind = (Tk_Window)w->w_grdata;
+    Window wind = Tk_WindowId(tkwind);
+    unsigned int width, height;
+    int ybot;
+    int xoff, yoff;
+    GC gc;
+    XGCValues gcValues;
+    Rect r;
+
+    pmap = (Pixmap)w->w_backingStore;
+    if (pmap == (Pixmap)NULL)
+	return FALSE;
+
+    gcValues.graphics_exposures = FALSE;
+    gc = Tk_GetGC(tkwind, GCGraphicsExposures, &gcValues);
+
+    /* Make a local copy of area so we don't disturb the	*/
+    /* original.  Expand by one pixel to deal with different	*/
+    /* boundary	conditions between X11 and OpenGL.  The redraw	*/
+    /* mechanism allows for at least this much slop.		*/
+
+    GEO_EXPAND(area, 1, &r);
+    GeoClip(&r, &(w->w_screenArea));
+
+    width = r.r_xtop - r.r_xbot;
+    height = r.r_ytop - r.r_ybot;
+    ybot = grXtransY(w, r.r_ytop);
+
+    xoff = w->w_screenArea.r_xbot - w->w_allArea.r_xbot;
+    yoff = w->w_allArea.r_ytop - w->w_screenArea.r_ytop;
+
+    XCopyArea(grXdpy, pmap, wind, gc, r.r_xbot - xoff,
+		ybot - yoff, width, height, r.r_xbot, ybot);
+
+    /* This is really only necessary for OpenGL, to avoid event */
+    /* timing conflicts between OpenGL calls and XCopyArea.	*/
+    /* It does not make sense for this to come AFTER the	*/
+    /* XCopyArea() command.  Yet that is what works.  So I'm	*/
+    /* sure that I do not entirely understand the situation.	*/
+
+    (*GrFlushPtr)();
+
+    /* TxPrintf("grtkGetBackingStore %d %d %d %d\n",
+		r.r_xbot, r.r_ybot, r.r_xtop, r.r_ytop); */
+    return TRUE;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * grtkScrollBackingStore --
+ *	Enable fast scrolling by shifting part of the backing store
+ *	from one position to another, with the amount of shift indicated
+ *	by the X and/or Y value of the indicated point.
+ *
+ * Results:
+ *	TRUE on success, FALSE on failure.
+ *
+ * Side effects:
+ *	Data shifted in Pixmap memory.
+ * 
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+grtkScrollBackingStore(MagWindow *w, Point *shift)
+{
+    Pixmap pmap;
+    Tk_Window tkwind = (Tk_Window)w->w_grdata;
+    unsigned int width, height;
+    int xorigin, yorigin, xshift, yshift;
+    GC gc;
+    XGCValues gcValues;
+
+    pmap = (Pixmap)w->w_backingStore;
+    if (pmap == (Pixmap)NULL)
+    {
+	TxPrintf("grtkScrollBackingStore %d %d failure\n",
+		shift->p_x, shift->p_y);
+	return FALSE;
+    }
+
+    gcValues.graphics_exposures = FALSE;
+    gc = Tk_GetGC(tkwind, GCGraphicsExposures, &gcValues);
+
+    width = w->w_screenArea.r_xtop - w->w_screenArea.r_xbot;
+    height = w->w_screenArea.r_ytop - w->w_screenArea.r_ybot;
+    xorigin = 0;
+    yorigin = 0;
+    xshift = shift->p_x;
+    yshift = -shift->p_y;
+
+    if (xshift > 0)
+	width -= xshift;
+    else if (xshift < 0)
+    {
+	width += xshift;
+	xorigin = -xshift;
+	xshift = 0;
+    }
+    if (yshift > 0)
+	height -= yshift;
+    else if (yshift < 0)
+    {
+	height += yshift;
+	yorigin = -yshift;
+	yshift = 0;
+    }
+
+    XCopyArea(grXdpy, pmap, pmap, gc, xorigin, yorigin, width, height,
+		xshift, yshift);
+
+    /* TxPrintf("grtkScrollBackingStore %d %d\n", shift->p_x, shift->p_y); */
+    return TRUE;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * grtkPutBackingStore --
+ *	Copy data from the window into backing store.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Graphics drawing into the window.
+ * ----------------------------------------------------------------------------
+ */
+
+void
+grtkPutBackingStore(MagWindow *w, Rect *area)
+{
+    Pixmap pmap = (Pixmap)w->w_backingStore;
+    Tk_Window tkwind = (Tk_Window)w->w_grdata;
+    Window wind = Tk_WindowId(tkwind);
+    unsigned int width, height;
+    int ybot, xbot, xoff, yoff;
+    GC gc;
+    XGCValues gcValues;
+
+    if (pmap == (Pixmap)NULL) return;
+
+    /* Attempting to write backing store into an obscured	*/
+    /* window immediately invalidates everything in backing	*/
+    /* store.  This is extreme, but is much simpler and under	*/
+    /* normal conditions faster than tracking all obscured	*/
+    /* areas separately.					*/
+
+    if (w->w_flags & WIND_OBSCURED)
+    {
+	grtkFreeBackingStore(w);
+	w->w_backingStore = (ClientData)NULL;
+	return;
+    }
+
+    width = area->r_xtop - area->r_xbot;
+    height = area->r_ytop - area->r_ybot;
+    ybot = grXtransY(w, area->r_ytop);
+    xbot = area->r_xbot;
+
+    gcValues.graphics_exposures = FALSE;
+    gc = Tk_GetGC(tkwind, GCGraphicsExposures, &gcValues);
+
+    xoff = w->w_screenArea.r_xbot - w->w_allArea.r_xbot;
+    yoff = w->w_allArea.r_ytop - w->w_screenArea.r_ytop;
+
+    /* This area may need to be expanded by a pixel and/or	*/
+    /* expanded by GrPixelCorrect to compensate for the OpenGL	*/
+    /* coordinate system.					*/
+
+    if (GrPixelCorrect == 0)
+    {
+	height--;
+	width--;
+	xbot++;
+    }
+
+    XCopyArea(grXdpy, wind, pmap, gc, xbot, ybot, width, height,
+		xbot - xoff, ybot - yoff);
+
+    /* TxPrintf("grtkPutBackingStore %d %d %d %d\n",
+		xbot, area->r_ybot, area->r_xtop, area->r_ytop); */
+}
+
 /*
  *---------------------------------------------------------
  * GrTkGetColorByName --
@@ -754,7 +1044,7 @@ ImgLayerConfigureInstance(instancePtr)
 	r.r_xtop = masterPtr->width;
 	r.r_ytop = masterPtr->height;
 
-	gcValues.graphics_exposures = False;
+	gcValues.graphics_exposures = FALSE;
 	gcmask = GCGraphicsExposures;
 	gc = Tk_GetGC(tkwind, gcmask, &gcValues);
 
