@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/lef/defRead.c,v 1.0 2003/05/01 10:37:00 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/lef/defRead.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -22,17 +22,17 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/defRead.c,v 1.0 2003/0
 #include <stdarg.h>
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "utils/malloc.h"
 #include "graphics/graphics.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "cif/cif.h"
 #include "lef/lefInt.h"
 
@@ -68,7 +68,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
     bool valid = FALSE;		/* is there a valid reference point? */
     bool initial = TRUE;
     Rect locarea;
-    float x, y, w;
+    float x, y, z, w;
     int routeWidth, paintWidth, saveWidth;
     TileType routeLayer, paintLayer;
     HashEntry *he;
@@ -91,7 +91,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 	    if (he != NULL)
 	    {
 		lefl = (lefLayer *)HashGetValue(he);
-		routeLayer = lefl->type;
+		if (lefl) routeLayer = lefl->type;
 	    }
 	    else
 	    {
@@ -294,11 +294,18 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 		goto endCoord;
 	    }
 
+	    /* Extension is half-width for regular nets, 0 for special nets */
+	    /* 0 for special nets is *not* how the 5.3 spec reads, but it   */
+	    /* is apparently how everyone interprets it, and is true for    */
+	    /* 5.6 spec.						    */
+
+	    z = (special) ? 0 : paintWidth;
 	    token = LefNextToken(f, TRUE);
 	    if (*token != ')')
 	    {
 		/* non-default route extension */
-		/* (to be done) */
+		if (sscanf(token, "%f", &z) != 1)
+		    LefError("Can't parse route extension value.\n");
 	    }
 
 	    /* Indicate that we have a valid reference point */
@@ -329,10 +336,27 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 
 		GeoCanonicalRect(&locarea, &newRoute->area);
 
-		newRoute->area.r_xbot -= paintWidth;
-		newRoute->area.r_ybot -= paintWidth;
-		newRoute->area.r_xtop += paintWidth;
-		newRoute->area.r_ytop += paintWidth;
+		if (newRoute->area.r_xbot == newRoute->area.r_xtop)
+		{
+		    newRoute->area.r_xbot -= paintWidth;
+		    newRoute->area.r_xtop += paintWidth;
+		}
+		else
+		{
+		    newRoute->area.r_xbot -= z;
+		    newRoute->area.r_xtop += z;
+		}
+
+		if (newRoute->area.r_ybot == newRoute->area.r_ytop)
+		{
+		    newRoute->area.r_ybot -= paintWidth;
+		    newRoute->area.r_ytop += paintWidth;
+		}
+		else
+		{
+		    newRoute->area.r_ybot -= z;
+		    newRoute->area.r_ytop += z;
+		}
 
 		/* If we don't have integer units here, we should	*/
 		/* rescale the magic internal grid.			*/
@@ -714,11 +738,12 @@ DefReadVias(f, sname, oscale, total)
 		    lefl = (lefLayer *)mallocMagic(sizeof(lefLayer));
 		    lefl->type = -1;
 		    lefl->obsType = -1;
-		    lefl->is_via = TRUE;
+		    lefl->lefClass = CLASS_VIA;
 		    lefl->info.via.area = GeoNullRect;
 		    lefl->info.via.cell = (CellDef *)NULL;
 		    lefl->info.via.lr = (linkedRect *)NULL;
 		    HashSetValue(he, lefl);
+		    lefl->canonName = (char *)he->h_key.h_name;
 		}
 
 		/* Now do a search through the line for "+" entries	*/
@@ -868,14 +893,24 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 		token = LefNextToken(f, TRUE);
 
 		/* Find the corresponding macro definition */
+		defUse = NULL;
 		defMacro = DBCellLookDef(token);
+
 		if (defMacro == (CellDef *)NULL)
 		{
-		    LefError("Cell %s is not defined.  Maybe you have not "
+		    /* Before giving up, assume that this cell has a	*/
+		    /* magic .mag layout file.				*/
+		    defMacro = DBCellNewDef(token, (char *)NULL);
+		    defMacro->cd_flags &= ~CDNOTFOUND;
+		    if (!DBCellRead(defMacro, (char *)NULL, TRUE))
+		    {
+		        LefError("Cell %s is not defined.  Maybe you have not "
 				"read the corresponding LEF file?\n",
 				token);
-		    LefEndStatement(f);
-		    break;
+		        LefEndStatement(f);
+			DBCellDeleteDef(defMacro);
+			defMacro = NULL;
+		    }
 		}
 
 		/* Create a use for this celldef in the edit cell */
@@ -972,7 +1007,7 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 enum def_sections {DEF_VERSION = 0, DEF_NAMESCASESENSITIVE,
 	DEF_UNITS, DEF_DESIGN, DEF_REGIONS, DEF_ROW, DEF_TRACKS,
 	DEF_GCELLGRID, DEF_DIVIDERCHAR, DEF_BUSBITCHARS,
-	DEF_PROPERTYDEFINITIONS, DEF_DEFAULTCAP,
+	DEF_PROPERTYDEFINITIONS, DEF_DEFAULTCAP, DEF_TECHNOLOGY,
 	DEF_HISTORY, DEF_DIEAREA, DEF_COMPONENTS, DEF_VIAS,
 	DEF_PINS, DEF_PINPROPERTIES, DEF_SPECIALNETS,
 	DEF_NETS, DEF_IOTIMINGS, DEF_SCANCHAINS,
@@ -1003,6 +1038,7 @@ DefRead(inName)
 	"BUSBITCHARS",
 	"PROPERTYDEFINITIONS",
 	"DEFAULTCAP",
+	"TECHNOLOGY",
 	"HISTORY",
 	"DIEAREA",
 	"COMPONENTS",
@@ -1071,6 +1107,16 @@ DefRead(inName)
 	    case DEF_NAMESCASESENSITIVE:
 		LefEndStatement(f);
 		break;
+	    case DEF_TECHNOLOGY:
+		token = LefNextToken(f, TRUE);
+		if (strcmp(token, DBTechName))
+		{
+		    LefError("Warning: DEF technology name \"%s\" does not"
+			" match current magic technology name \"%s\"\n",
+			token, DBTechName);
+		}
+		LefEndStatement(f);
+	 	break;
 	    case DEF_REGIONS:
 		LefSkipSection(f, sections[DEF_REGIONS]);
 		break;
