@@ -14,7 +14,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/lef/lefRead.c,v 1.0 2003/05/01 10:37:00 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/lef/lefRead.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -28,11 +28,11 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/lefRead.c,v 1.0 2003/0
 #include <sys/time.h>
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "graphics/graphics.h"
@@ -42,7 +42,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/lefRead.c,v 1.0 2003/0
 #include "cif/cif.h"
 #include "lef/lefInt.h"
 
-/* ---------------------------------------------------------------------
+/* ---------------------------------------------------------------------*/
 
 /* Current line number for reading */
 int lefCurrentLine;
@@ -502,7 +502,7 @@ LefReadLayer(f, obstruct)
 	if (he)
 	{
 	    lefLayer *lefl = (lefLayer *)HashGetValue(he);
-	    if (obstruct)
+	    if (lefl && obstruct)
 	    {
 		/* Use the obstruction type, if it is defined */
 		curlayer = lefl->obsType;
@@ -1210,11 +1210,12 @@ LefReadLayerSection(f, lname, mode, lefl)
     TileType curlayer = -1;
     float fvalue, oscale;
 
+    /* These are defined in the order of CLASS_* in lefInt.h */
     static char *layer_type_keys[] = {
-	"MASTERSLICE",
-	"OVERLAP",
 	"ROUTING",
 	"CUT",
+	"MASTERSLICE",
+	"OVERLAP",
 	NULL
     };
 
@@ -1256,6 +1257,10 @@ LefReadLayerSection(f, lname, mode, lefl)
 			LefError("Unknown layer type \"%s\" in LEF file; "
 				"ignoring.\n", token);
 		}
+		if (lefl->lefClass != typekey)
+		    LefError("Attempt to reclassify layer %s from %s to %s\n",
+				lname, layer_type_keys[lefl->lefClass],
+				layer_type_keys[typekey]);
 		LefEndStatement(f);
 		break;
 	    case LEF_LAYER_WIDTH:
@@ -1458,12 +1463,13 @@ LefRead(inName, importForeign)
 		    lefl->type = -1;
 		    lefl->obsType = -1;
 		    lefl->refCnt = 1;
-		    lefl->is_via = TRUE;
+		    lefl->lefClass = CLASS_VIA;
 		    lefl->info.via.area = GeoNullRect;
 		    lefl->info.via.cell = (CellDef *)NULL;
 		    lefl->info.via.lr = (linkedRect *)NULL;
 		    HashSetValue(he, lefl);
 		    LefReadLayerSection(f, tsave, keyword, lefl);
+		    lefl->canonName = (char *)he->h_key.h_name;
 		}
 		else
 		    /* If we've already seen this via, don't reprocess. */
@@ -1512,14 +1518,15 @@ LefRead(inName, importForeign)
 			lefl->type = mtype;
 			lefl->obsType = -1;
 			lefl->refCnt = 1;
-			lefl->is_via = DBIsContact(mtype);
+			lefl->lefClass = (DBIsContact(mtype)) ? CLASS_VIA : CLASS_ROUTE;
 			HashSetValue(he, lefl);
+		        lefl->canonName = (char *)he->h_key.h_name;
 		    }
 		}
 		else
 		{
 		    lefl = (lefLayer *)HashGetValue(he);
-		    if (lefl->type < 0)
+		    if (lefl && lefl->type < 0)
 		    {
 			LefError("Layer %s is only defined for obstructions!\n", token);
 			LefSkipSection(f, tsave);
