@@ -13,7 +13,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/lef/lefWrite.c,v 1.0 2003/05/01 10:37:00 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/lef/lefWrite.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -24,25 +24,25 @@ static char rcsid[] = "$Header: /ufs/repository/magic/lef/lefWrite.c,v 1.0 2003/
 #include <errno.h>
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "utils/utils.h"
 #include "utils/malloc.h"
 #include "utils/stack.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "graphics/graphics.h"
-#include "main/main.h"
-#include "undo/undo.h"
+#include "utils/main.h"
+#include "utils/undo.h"
 #include "cif/cif.h"
 #include "lef/lefInt.h"
 
-/* ---------------------------------------------------------------------
+/* ---------------------------------------------------------------------*/
 
 /* Stack of cell definitions */
 Stack *lefDefStack;
@@ -170,11 +170,89 @@ lefWriteHeader(def, f)
     /* scalefactor (conversion to microns) is always 1000.		*/
 
     fprintf(f, "UNITS\n");
-    fprintf(f, "   DATABASE MICRONS 1000\n");
+    fprintf(f, "   DATABASE MICRONS 1000 ;\n");
     fprintf(f, "END UNITS\n");
     fprintf(f, "\n");
 
-    /* Layers, vias, and rules (to be completed, presumably) */
+    /* Layers (minimal information) */
+
+    if (LefInfo.ht_table != (HashEntry **)NULL)
+    {
+	HashSearch hs;
+	HashEntry *he;
+	lefLayer *lefl;
+
+	float oscale = CIFGetOutputScale(1000);	/* lambda->micron conversion */
+
+	HashStartSearch(&hs);
+	while (he = HashNext(&LefInfo, &hs))
+	{
+	    lefl = (lefLayer *)HashGetValue(he);
+	    if (!lefl) continue;
+
+	    if (lefl->refCnt > 0)
+	    {
+		/* Avoid writing more than one entry per defined layer */
+	        if (lefl->refCnt > 1) lefl->refCnt = -lefl->refCnt;
+
+		/* Ignore obstruction-only layers */
+		if (lefl->type == -1) continue;	
+		/* Ignore VIA types, report only CUT types here */
+		else if ((lefl->lefClass == CLASS_VIA)
+			&& lefl->info.via.cell != NULL) continue;
+
+		fprintf(f, "LAYER %s\n", lefl->canonName);
+		if (lefl->lefClass == CLASS_VIA)
+		{
+		    int cutarea;
+		    cutarea = (lefl->info.via.area.r_xtop - lefl->info.via.area.r_xbot);
+		    cutarea *= (lefl->info.via.area.r_ytop - lefl->info.via.area.r_ybot);
+		    fprintf(f, "   TYPE CUT ;\n");
+		    if (cutarea > 0)
+			fprintf(f, "   CUT AREA %f ;\n",
+					(float)cutarea * oscale * oscale);
+		}
+		else if (lefl->lefClass == CLASS_ROUTE)
+		{
+		    fprintf(f, "   TYPE ROUTING ;\n");
+		    if (lefl->info.route.pitch > 0)
+			fprintf(f, "   PITCH %f ;\n", (float)(lefl->info.route.pitch)
+				* oscale);
+		    if (lefl->info.route.width > 0)
+			fprintf(f, "   WIDTH %f ;\n", (float)(lefl->info.route.width)
+				* oscale);
+		    if (lefl->info.route.spacing > 0)
+			fprintf(f, "   SPACING %f ;\n", (float)(lefl->info.route.spacing)
+				* oscale);
+		    /* No sense in providing direction info unless we know the width */
+		    if (lefl->info.route.width > 0)
+			fprintf(f, "   DIRECTION %s ;\n", (lefl->info.route.hdirection)
+				?  "HORIZONTAL" : "VERTICAL");
+		}
+		else if (lefl->lefClass == CLASS_MASTER)
+		{
+		    fprintf(f, "   TYPE MASTERSLICE ;\n");
+		}
+		else if (lefl->lefClass == CLASS_OVERLAP)
+		{
+		    fprintf(f, "   TYPE OVERLAP ;\n");
+		}
+		fprintf(f, "END %s ;\n\n", lefl->canonName);
+	    }
+	}
+
+	/* Return reference counts to normal */
+	HashStartSearch(&hs);
+	while (he = HashNext(&LefInfo, &hs))
+	{
+	    lefl = (lefLayer *)HashGetValue(he);
+	    if (lefl && lefl->refCnt < 0)
+		lefl->refCnt = -lefl->refCnt;
+	}
+    }
+
+    /* Vias (to be completed, presumably) */
+    /* Rules (to be completed, presumably) */
 
     UndoEnable();
 }
@@ -271,7 +349,7 @@ lefWriteGeometry(tile, cdata)
     }
 
 #ifdef NONMANHATTAN
-    if (!IsSplit(tile) || (lefdata->lefMode == LEF_MODE_OBSTRUCT))
+    if (!IsSplit(tile))
 #endif
 
     fprintf(f, "	    RECT %.4f %.4f %.4f %.4f ;\n",
@@ -283,6 +361,20 @@ lefWriteGeometry(tile, cdata)
     return 0;
 }
 
+/*
+ * When called from SimSrConnect(), the callback function needs another
+ * argument for the plane.
+ */
+
+int
+lefWriteGeometry2(tile, plane, cdata)
+    Tile *tile;
+    Plane *plane;
+    ClientData cdata;
+{
+    return lefWriteGeometry(tile, cdata);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -366,6 +458,8 @@ lefWriteMacro(def, f, scale)
     lefClient lc;
     int idx, pNum;
     char *LEFtext;
+    HashSearch hs;
+    HashEntry *he;
 
     UndoDisable();
 
@@ -379,15 +473,21 @@ lefWriteMacro(def, f, scale)
 
     TxPrintf("Diagnostic:  Scale value is %f\n", lc.oscale);
 
-    /* Temporary hack !!  Which layers are routing layers should be	*/
-    /* defined in the tech file.  Here, we just define metal and via	*/
-    /* layers, by their names.						*/
+    /* Which layers are routing layers are defined in the tech file.	*/
 
     TTMaskZero(&lc.rmask);
-    for (ttype = TT_TECHDEPBASE; ttype < DBNumTypes; ttype++)
-	if (strstr(DBTypeLongName(ttype), "metal") ||
-		strstr(DBTypeLongName(ttype), "contact"))
-	    TTMaskSetType(&lc.rmask, ttype);
+    HashStartSearch(&hs);
+    while (he = HashNext(&LefInfo, &hs))
+    {
+	lefLayer *lefl = (lefLayer *)HashGetValue(he);
+	if (lefl && (lefl->lefClass == CLASS_ROUTE || lefl->lefClass == CLASS_VIA))
+	    if (lefl->type != -1)
+	    {
+		TTMaskSetType(&lc.rmask, lefl->type);
+		if (DBIsContact(lefl->type))
+		    TTMaskSetMask(&lc.rmask, DBResidueMask(lefl->type));
+	    }
+    }
 
     /* Any layer which has a port label attached to it should by	*/
     /* necessity be considered a routing layer.	 Usually this will not	*/
@@ -397,8 +497,6 @@ lefWriteMacro(def, f, scale)
 	if (lab->lab_flags & PORT_DIR_MASK)
 	    TTMaskSetType(&lc.rmask, lab->lab_type);
 
-    /* End temporary hack. */
-
     /* NOTE:  This routine corresponds to Envisia LEF/DEF Language	*/
     /* Reference version 5.3 (May 31, 2000)				*/
 
@@ -521,15 +619,31 @@ lefWriteMacro(def, f, scale)
 
 	    for (clab = lab; clab != NULL; clab = clab->lab_next)
 	    {
-		if (clab->lab_flags & PORT_NUM_MASK == idx)
+		if ((clab->lab_flags & PORT_NUM_MASK) == idx)
 		{
+		    Rect labr = clab->lab_rect;
+
+		    /* Deal with degenerate (line or point) labels	*/
+		    /* by growing by 1 in each direction.		*/
+
+		    if (labr.r_xtop - labr.r_xbot == 0)
+		    {
+			labr.r_xtop++;
+			labr.r_xbot--;
+		    }
+		    if (labr.r_ytop - labr.r_ybot == 0)
+		    {
+			labr.r_ytop++;
+			labr.r_ybot--;
+		    }
+
 		    fprintf(f, "      PORT\n");
 
 		    TTMaskSetOnlyType(&lmask, clab->lab_type);
 
 		    ttype = TT_SPACE;
-		    SimSrConnect(def, &clab->lab_rect, &lmask, DBConnectTbl,
-			&TiPlaneRect, lefWriteGeometry, (ClientData) &lc);
+		    SimSrConnect(def, &labr, &lmask, DBConnectTbl,
+			&TiPlaneRect, lefWriteGeometry2, (ClientData) &lc);
 
 		    fprintf(f, "      END\n");	/* end of port geometries */
 		    clab->lab_flags |= PORT_VISITED;
@@ -553,7 +667,6 @@ lefWriteMacro(def, f, scale)
 
     /* Restrict to routing planes only */
 
-//  fprintf(f, "   OBS %s\n", def->cd_name);	/* Error Fixed; Tim 6/16/04 */
     fprintf(f, "   OBS\n");
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	(void) DBSrPaintArea((Tile *)NULL, def->cd_planes[pNum], 
@@ -649,6 +762,7 @@ LefWriteAll(rootUse, writeTopCell)
  * for a subtree rooted at a particular def.
  */
 
+int 
 lefDefInitFunc(def)
     CellDef *def;
 {
@@ -662,6 +776,7 @@ lefDefInitFunc(def)
  * on all that def's children.
  */
 
+int
 lefDefPushFunc(use)
     CellUse *use;
 {
