@@ -21,19 +21,15 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/magic/magicTop.c,v 1.2 2001/01/12 22:12:58 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/magic/magicTop.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/malloc.h"
 
-/* dummy definitions */
-YCRVert() {}
-YCRHoriz() {}
-Yacr() {}
-
 /*---------------------------------------------------------------------------
  * main:
  *
@@ -65,4 +61,5 @@ main(argc, argv)
  */
 
 char *MagicVersion = MAGIC_VERSION;
+char *MagicRevision = MAGIC_REVISION;
 char *MagicCompileTime = MAGIC_DATE;
