@@ -21,14 +21,14 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/plot/plotRutils.c,v 1.3 2001/01/12 22:13:10 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/plot/plotRutils.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
