@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowMain.c,v 1.3 2001/06/15 15:33:39 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/plow/PlowMain.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -27,12 +27,12 @@ static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowMain.c,v 1.3 2001/0
 #include <sys/time.h>
 #include <sys/resource.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "debug/debug.h"
 #include "plow/plow.h"
 #include "plow/plowInt.h"
@@ -40,10 +40,11 @@ static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowMain.c,v 1.3 2001/0
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "drc/drc.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "utils/malloc.h"
-#include "signals/signals.h"
-#include "main/main.h"
+#include "utils/signals.h"
+#include "utils/main.h"
+#include "select/select.h"
 
 #if defined(SYSV) || defined(macosx)
 # define	NO_RUSAGE
@@ -151,6 +152,9 @@ bool plowPropagateSel();
 bool plowPropagateRect();
 PlowRule *plowBuildWidthRules();
 
+void plowMergeBottom(Tile *, Plane *);
+void plowInitRule();
+
 extern void PlowRedrawBound();
 extern void PlowClearBound();
 extern void plowUpdate();
@@ -1054,6 +1058,7 @@ plowSelPaintPlow(rect, type, distance)
     return (0);
 }
 
+int
 plowSelPaintAdd(edge)
     Edge *edge;
 {
@@ -1092,17 +1097,18 @@ plowSelCellPlow(selUse, realUse, transform, distance)
     int distance;		/* Plow distance */
 {
     int plowFindSelCell();
-    int save;
+    ClientData save;
 
     /* Find the cell in the yanked def that has the same use-id as this one */
-    save = realUse->cu_delta;
-    realUse->cu_delta = distance;
-    (void) DBCellEnum(plowYankDef, plowFindSelCell, (ClientData) realUse);
-    realUse->cu_delta = save;
+    save = realUse->cu_client;
+    realUse->cu_client = (ClientData)distance;
+    (void) DBCellEnum(plowYankDef, plowFindSelCell, (ClientData)realUse);
+    realUse->cu_client = save;
 
     return (0);
 }
 
+int
 plowFindSelCell(yankUse, editUse)
     CellUse *yankUse;	/* Cell in the plow yank buffer */
     CellUse *editUse;	/* Cell from the original cell def */
@@ -1118,7 +1124,7 @@ plowFindSelCell(yankUse, editUse)
     edge.e_ytop = yankUse->cu_bbox.r_ytop;
     edge.e_ybot = yankUse->cu_bbox.r_ybot;
     edge.e_x = yankUse->cu_bbox.r_xtop;
-    edge.e_newx = yankUse->cu_bbox.r_xtop + editUse->cu_delta;
+    edge.e_newx = yankUse->cu_bbox.r_xtop + (int)editUse->cu_client;
     edge.e_ltype = PLOWTYPE_CELL;
     edge.e_rtype = PLOWTYPE_CELL;
     (void) plowQueueAdd(&edge);
@@ -1537,7 +1543,7 @@ plowProcessEdge(edge, changedArea)
     plowProcessedEdges++;
     if (edge->e_use)
     {
-	if (amountToMove > edge->e_use->cu_delta)
+	if (amountToMove > (int)edge->e_use->cu_client)
 	{
 	    /* Update area modified by plowing */
 	    (void) GeoInclude(&edge->e_rect, changedArea);
@@ -1557,7 +1563,7 @@ plowProcessEdge(edge, changedArea)
 	     * to update the area changed by the area of the
 	     * cell PLUS the area swept out by its RHS.
 	     */
-	    edge->e_use->cu_delta = amountToMove;
+	    edge->e_use->cu_client = (ClientData)amountToMove;
 	    r = edge->e_use->cu_bbox;
 	    r.r_xbot += amountToMove;
 	    r.r_xtop += amountToMove;
@@ -1987,6 +1993,7 @@ plowMergeTop(tp, plane)
     }
 }
 
+void
 plowMergeBottom(tp, plane)
     Tile *tp;
     Plane *plane;
@@ -2189,6 +2196,7 @@ PlowInit()
     plowYankHalo = TechHalo;
 }
 
+void
 plowInitRule(rtePtr, rteEnd, whichRules, proc, name, ltypes, rtypes)
     RuleTableEntry *rtePtr;	/* Pointer to entry to be added */
     RuleTableEntry *rteEnd;	/* Pointer to one past last entry in table */
