@@ -18,16 +18,17 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowRules2.c,v 1.3 2001/06/15 15:33:40 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/plow/PlowRules2.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "plow/plow.h"
 #include "plow/plowInt.h"
 #include "drc/drc.h"
@@ -64,12 +65,12 @@ int plowDragEdgeProc();
  * ----------------------------------------------------------------------------
  */
 
-Void
+void
 prFixedLHS(edge)
     Edge *edge;			/* Edge being moved */
 {
     int distance = edge->e_newx - edge->e_x;
-    register Tile *tpL;
+    Tile *tpL;
     Point startPoint;
     Rect atomRect;
     Plane *plane;
@@ -97,12 +98,12 @@ restart:
     }
 }
 
-Void
+int
 prFixedRHS(edge)
     Edge *edge;			/* Edge being moved */
 {
     int distance = edge->e_newx - edge->e_x;
-    register Tile *tpR, *tp;
+    Tile *tpR, *tp;
     Point startPoint;
     Rect atomRect;
     Plane *plane;
@@ -132,7 +133,7 @@ restart:
 	/* Move all fixed-width tiles along the top */
 	for (tp = RT(tpR); RIGHT(tp) > LEFT(tpR); tp = BL(tp))
 	{
-	    if (TTMaskHasType(&PlowFixedTypes, TiGetBottomType(tp)))
+	    if (TTMaskHasType(&PlowFixedTypes, TiGetTypeExact(tp)))
 	    {
 		atomRect.r_xbot = LEFT(tp);
 		atomRect.r_xtop = LEFT(tp) + distance;
@@ -151,7 +152,7 @@ restart:
 	/* Move all fixed-width tiles along the bottom */
 	for (tp = LB(tpR); LEFT(tp) < RIGHT(tpR); tp = TR(tp))
 	{
-	    if (TTMaskHasType(&PlowFixedTypes, TiGetTopType(tp)))
+	    if (TTMaskHasType(&PlowFixedTypes, TiGetTypeExact(tp)))
 	    {
 		atomRect.r_xbot = LEFT(tp);
 		atomRect.r_xtop = LEFT(tp) + distance;
@@ -204,20 +205,20 @@ restart:
  * ----------------------------------------------------------------------------
  */
 
-Void
+void
 prFixedPenumbraTop(edge)
-    register Edge *edge;	/* Edge being moved */
+    Edge *edge;	/* Edge being moved */
 {
     struct applyRule ar;
-    register PlowRule *pr;
-    register Tile *tp;
+    PlowRule *pr;
+    Tile *tp;
     Rect searchRect;
     Point p;
 
     p.p_x = edge->e_x - 1;
     p.p_y = edge->e_ytop;
     tp = TiSrPointNoHint(plowYankDef->cd_planes[edge->e_pNum], &p);
-    pr = plowSpacingRulesTbl[edge->e_ltype][TiGetBottomType(tp)];
+    pr = plowSpacingRulesTbl[edge->e_ltype][TiGetTypeExact(tp)];
     if (pr == (PlowRule *) NULL)
 	return;
 
@@ -234,20 +235,20 @@ prFixedPenumbraTop(edge)
     }
 }
 
-Void
+int
 prFixedPenumbraBot(edge)
-    register Edge *edge;	/* Edge being moved */
+    Edge *edge;	/* Edge being moved */
 {
     struct applyRule ar;
-    register PlowRule *pr;
-    register Tile *tp;
+    PlowRule *pr;
+    Tile *tp;
     Rect searchRect;
     Point p;
 
     p.p_x = edge->e_x - 1;
     p.p_y = edge->e_ybot - 1;
     tp = TiSrPointNoHint(plowYankDef->cd_planes[edge->e_pNum], &p);
-    pr = plowSpacingRulesTbl[edge->e_ltype][TiGetTopType(tp)];
+    pr = plowSpacingRulesTbl[edge->e_ltype][TiGetTypeExact(tp)];
     if (pr == (PlowRule *) NULL)
 	return;
 
@@ -285,12 +286,12 @@ prFixedPenumbraBot(edge)
  * ----------------------------------------------------------------------------
  */
 
-Void
+void
 prFixedDragStubs(edge)
-    register Edge *edge;	/* Edge being moved; RHS is fixed-width */
+    Edge *edge;	/* Edge being moved; RHS is fixed-width */
 {
     int distance = edge->e_newx - edge->e_x;
-    register Tile *tpL;
+    Tile *tpL;
     Point startPoint;
     Rect atomRect;
     Plane *plane;
@@ -341,13 +342,13 @@ restart:
 
 int
 plowDragEdgeProc(lhsEdge, movingEdge)
-    register Edge *lhsEdge;	/* Edge on LHS; the caller has already
+    Edge *lhsEdge;	/* Edge on LHS; the caller has already
 				 * determined that this edge has not
 				 * already moved far enough.
 				 */
-    register Edge *movingEdge;	/* RHS of this edge is fixed-width */
+    Edge *movingEdge;	/* RHS of this edge is fixed-width */
 {
-    register PlowRule *pr;
+    PlowRule *pr;
     int xsep, width;
 
     /* Don't move the edge if it isn't to space */
@@ -408,14 +409,15 @@ plowDragEdgeProc(lhsEdge, movingEdge)
  * ----------------------------------------------------------------------------
  */
 
-Void
+void
 prContactLHS(edge)
-    register Edge *edge;	/* Edge being moved (LHS is contact) */
+    Edge *edge;	/* Edge being moved (LHS is contact) */
 {
-    register int pNum, connPlanes = DBConnPlanes[edge->e_ltype];
-    register int pMax;
+    int pNum, connPlanes = DBConnPlanes[edge->e_ltype];
+    int pMax;
 
     /* Add the edges of the contact on its other planes */
+    connPlanes &= ~(edge->e_pNum);
     pMax = DBPlane(edge->e_ltype) + 1;
     for (pNum = pMax - 2; pNum <= pMax; pNum++)
 	if (PlaneMaskHasPlane(connPlanes, pNum))
@@ -423,14 +425,15 @@ prContactLHS(edge)
 				plowPropagateProcPtr, (ClientData) NULL);
 }
 
-Void
+int
 prContactRHS(edge)
-    register Edge *edge;	/* Edge being moved (RHS is contact) */
+    Edge *edge;	/* Edge being moved (RHS is contact) */
 {
-    register pNum, connPlanes = DBConnPlanes[edge->e_rtype];
-    register int pMax;
+    int pNum, connPlanes = DBConnPlanes[edge->e_rtype];
+    int pMax;
 
     /* Add the edges of the contact on its other planes */
+    connPlanes &= ~(edge->e_pNum);
     pMax = DBPlane(edge->e_ltype) + 1;
     for (pNum = pMax - 2; pNum <= pMax; pNum++)
 	if (PlaneMaskHasPlane(connPlanes, pNum))
@@ -458,13 +461,13 @@ prContactRHS(edge)
  * ----------------------------------------------------------------------------
  */
 
-Void
+void
 prCoverTop(edge)
-    register Edge *edge;	/* Edge being moved */
+    Edge *edge;	/* Edge being moved */
 {
     TileType ltype, rtype;
-    register PlowRule *pr;
-    register Tile *tp;
+    PlowRule *pr;
+    Tile *tp;
     struct applyRule ar;
     Point startPoint;
     Rect searchArea;
@@ -472,11 +475,11 @@ prCoverTop(edge)
     startPoint.p_x = edge->e_x - 1;
     startPoint.p_y = edge->e_ytop;
     tp = TiSrPointNoHint(plowYankDef->cd_planes[edge->e_pNum], &startPoint);
-    if (TiGetBottomType(tp) == TT_SPACE)
+    if (TiGetTypeExact(tp) == TT_SPACE)
 	return;
 
     ltype = edge->e_ltype;
-    rtype = TiGetLeftType(tp);
+    rtype = TiGetTypeExact(tp);
     ar.ar_moving = edge;
     ar.ar_rule = (PlowRule *) NULL;
     searchArea.r_xbot = edge->e_x - 1;
@@ -496,13 +499,13 @@ prCoverTop(edge)
     }
 }
 
-Void
+int
 prCoverBot(edge)
-    register Edge *edge;	/* Edge being moved */
+    Edge *edge;	/* Edge being moved */
 {
     TileType ltype, rtype;
-    register PlowRule *pr;
-    register Tile *tp;
+    PlowRule *pr;
+    Tile *tp;
     struct applyRule ar;
     Point startPoint;
     Rect searchArea;
@@ -510,11 +513,11 @@ prCoverBot(edge)
     startPoint.p_x = edge->e_x - 1;
     startPoint.p_y = edge->e_ybot - 1;
     tp = TiSrPointNoHint(plowYankDef->cd_planes[edge->e_pNum], &startPoint);
-    if (TiGetTopType(tp) == TT_SPACE)
+    if (TiGetTypeExact(tp) == TT_SPACE)
 	return;
 
     ltype = edge->e_ltype;
-    rtype = TiGetLeftType(tp);
+    rtype = TiGetTypeExact(tp);
     ar.ar_moving = edge;
     ar.ar_rule = (PlowRule *) NULL;
     searchArea.r_xbot = edge->e_x - 1;
@@ -552,9 +555,9 @@ prCoverBot(edge)
  * ----------------------------------------------------------------------------
  */
 
-Void
+void
 prIllegalTop(edge)
-    register Edge *edge;
+    Edge *edge;
 {
     TileTypeBitMask insideTypes;
     struct applyRule ar;
@@ -582,9 +585,9 @@ prIllegalTop(edge)
 		plowCoverTopProc, (ClientData) &ar);
 }
 
-Void
+int
 prIllegalBot(edge)
-    register Edge *edge;
+    Edge *edge;
 {
     TileTypeBitMask insideTypes;
     struct applyRule ar;
@@ -633,8 +636,8 @@ prIllegalBot(edge)
 
 int
 plowCoverTopProc(outline, ar)
-    register Outline *outline;
-    register struct applyRule *ar;
+    Outline *outline;
+    struct applyRule *ar;
 {
     Edge edge;
     int ret = 0;
@@ -659,8 +662,8 @@ plowCoverTopProc(outline, ar)
 	edge.e_pNum = ar->ar_moving->e_pNum;
 	edge.e_use = (CellUse *) NULL;
 	edge.e_flags = 0;
-	edge.e_ltype = TiGetRightType(outline->o_inside);
-	edge.e_rtype = TiGetLeftType(outline->o_outside);
+	edge.e_ltype = TiGetTypeExact(outline->o_inside);
+	edge.e_rtype = TiGetTypeExact(outline->o_outside);
 	(void) (*plowPropagateProcPtr)(&edge);
     }
 
@@ -669,8 +672,8 @@ plowCoverTopProc(outline, ar)
 
 int
 plowCoverBotProc(outline, ar)
-    register Outline *outline;
-    register struct applyRule *ar;
+    Outline *outline;
+    struct applyRule *ar;
 {
     Edge edge;
     int ret = 0;
@@ -695,8 +698,8 @@ plowCoverBotProc(outline, ar)
 	edge.e_pNum = ar->ar_moving->e_pNum;
 	edge.e_use = (CellUse *) NULL;
 	edge.e_flags = 0;
-	edge.e_ltype = TiGetRightType(outline->o_outside);
-	edge.e_rtype = TiGetLeftType(outline->o_inside);
+	edge.e_ltype = TiGetTypeExact(outline->o_outside);
+	edge.e_rtype = TiGetTypeExact(outline->o_inside);
 	(void) (*plowPropagateProcPtr)(&edge);
     }
 
@@ -732,13 +735,13 @@ plowCoverBotProc(outline, ar)
 
 int
 plowIllegalTopProc(outline, ar)
-    register Outline *outline;
-    register struct applyRule *ar;
+    Outline *outline;
+    struct applyRule *ar;
 {
-    TileType badType = TiGetBottomType(outline->o_inside), leftType;
-    register Edge *movingEdge = ar->ar_moving;
-    register DRCCookie *dp;
-    register PlowRule *pr;
+    TileType badType = TiGetTypeExact(outline->o_inside), leftType;
+    Edge *movingEdge = ar->ar_moving;
+    DRCCookie *dp;
+    PlowRule *pr;
     int width;
 
     if (outline->o_currentDir != GEO_EAST
@@ -763,7 +766,7 @@ found_bad:
 
     ar->ar_slivtype = badType;
     ar->ar_mustmove = outline->o_rect.r_xbot;
-    leftType = TiGetRightType(BL(outline->o_inside));
+    leftType = TiGetTypeExact(BL(outline->o_inside));
     width = 1;
     for (pr = plowSpacingRulesTbl[movingEdge->e_ltype][leftType];
 	    pr; pr = pr->pr_next)
@@ -777,14 +780,14 @@ found_bad:
 
 int
 plowIllegalBotProc(outline, ar)
-    register Outline *outline;
-    register struct applyRule *ar;
+    Outline *outline;
+    struct applyRule *ar;
 {
-    TileType badType = TiGetTopType(outline->o_outside), leftType;
-    register Edge *movingEdge = ar->ar_moving;
-    register DRCCookie *dp;
-    register PlowRule *pr;
-    register Tile *tp;
+    TileType badType = TiGetTypeExact(outline->o_outside), leftType;
+    Edge *movingEdge = ar->ar_moving;
+    DRCCookie *dp;
+    PlowRule *pr;
+    Tile *tp;
     int width;
 
     if (outline->o_currentDir != GEO_EAST
@@ -812,7 +815,7 @@ found_bad:
     for (tp = BL(outline->o_outside);
 	    TOP(tp) < outline->o_rect.r_ybot; tp = RT(tp))
 	/* Nothing */;
-    leftType = TiGetRightType(tp);
+    leftType = TiGetTypeExact(tp);
 
     width = 1;
     for (pr = plowSpacingRulesTbl[movingEdge->e_ltype][leftType];
@@ -842,12 +845,12 @@ found_bad:
  * ----------------------------------------------------------------------------
  */
 
-Void
+void
 prFindCells(edge)
-    register Edge *edge;	/* Edge being moved */
+    Edge *edge;	/* Edge being moved */
 {
-    register Plane *cellPlane = plowYankDef->cd_planes[PL_CELL];
-    register Tile *cellTile = cellPlane->pl_hint;
+    Plane *cellPlane = plowYankDef->cd_planes[PL_CELL];
+    Tile *cellTile = cellPlane->pl_hint;
     struct applyRule ar;
     Rect searchArea;
 
@@ -891,7 +894,7 @@ prFindCells(edge)
  * ----------------------------------------------------------------------------
  */
 
-Void
+void
 prCell(edge)
     Edge *edge;			/* Cell edge being moved */
 {
@@ -965,10 +968,10 @@ prCell(edge)
 
 int
 plowCellDragPaint(tile, ar)
-    register Tile *tile;
-    register struct applyRule *ar;
+    Tile *tile;
+    struct applyRule *ar;
 {
-    register Edge *movingEdge = ar->ar_moving;
+    Edge *movingEdge = ar->ar_moving;
     int distance = movingEdge->e_newx - movingEdge->e_x;
     Rect atomRect;
 
@@ -1091,7 +1094,8 @@ plowFoundCell(cellTile, ar)
 	}
 
 	/* Only queue the edge if the cell has not moved far enough */
-	if (use->cu_delta < xmove)
+	if ((use->cu_client != (ClientData)MINFINITY) &&
+		((int)(use->cu_client) < xmove))
 	{
 	    edge.e_use = use;
 	    edge.e_flags = 0;
