@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResBasic.c,v 1.2 2001/01/12 22:13:15 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/resis/ResBasic.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -11,7 +11,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResBasic.c,v 1.2 200
 #else
 #include <strings.h>
 #endif
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
@@ -23,7 +24,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResBasic.c,v 1.2 200
 #include "extract/extractInt.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "textio/txcommands.h"
 #include "resis/resis.h"
 
