@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMain.c,v 1.2 2001/01/12 22:13:16 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/resis/ResMain.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -10,7 +10,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMain.c,v 1.2 2001
 #else
 #include <strings.h>
 #endif
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
@@ -22,7 +23,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMain.c,v 1.2 2001
 #include "extract/extractInt.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "textio/txcommands.h"
 #include "resis/resis.h"
 
@@ -118,7 +119,7 @@ ResGetReCell()
     }
     ResUse = DBCellNewUse(ResDef, (char *) NULL);
     DBSetTrans(ResUse, &GeoIdentityTransform);
-    ResUse->cu_expandMask = -1;
+    ResUse->cu_expandMask = CU_DESCEND_SPECIAL;
 
 }
  
@@ -137,6 +138,7 @@ ResGetReCell()
  *
  *------------------------------------------------------------------------
  */
+void
 ResDissolveContacts(contacts)
 	ResContactPoint *contacts;
 {
@@ -564,7 +566,7 @@ ResExtractNet(startlist,goodies,cellname)
 	if (def == (CellDef *) NULL)
 	{
 	    TxError("Error:  No such cell \"%s\"\n", cellname);
-	    return 1;
+	    return TRUE;
 	}
 	scx.scx_use = DBCellNewUse(def, (char *)NULL);
 	DBSetTrans (scx.scx_use, &GeoIdentityTransform);
@@ -576,7 +578,7 @@ ResExtractNet(startlist,goodies,cellname)
 	if (w == (MagWindow *) NULL)
 	{
 	    TxError("Sorry, the box must appear in one of the windows.\n");
-	    return 1;
+	    return TRUE;
 	}
 	scx.scx_use = (CellUse *) w->w_surfaceID;
 	scx.scx_trans = GeoIdentityTransform;   
@@ -684,8 +686,8 @@ ResExtractNet(startlist,goodies,cellname)
 #endif
 
     /* do extraction */
-    if (ResProcessTiles(goodies, &startpoint) != 0) return(1);
-    return(0);
+    if (ResProcessTiles(goodies, &startpoint) != 0) return TRUE;
+    return FALSE;
 }
 
 
