@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMakeRes.c,v 1.2 2001/01/12 22:13:16 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/resis/ResMakeRes.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -12,7 +12,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMakeRes.c,v 1.2 2
 #else
 #include <strings.h>
 #endif
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
@@ -24,7 +25,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMakeRes.c,v 1.2 2
 #include "extract/extractInt.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "textio/txcommands.h"
 #include "resis/resis.h"
 
@@ -464,7 +465,7 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
  *	breakpoints, and call either ResCalcEastWest or ResCalcNorthSouth
  *
  * Results:
- *	True if merging occurred, False if not.
+ *	TRUE if merging occurred, FALSE if not.
  *
  * Side Effects: Makes resistors. Frees breakpoints
  *
