@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResPrint.c,v 1.2 2001/01/12 22:13:16 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/resis/ResPrint.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -11,7 +11,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResPrint.c,v 1.2 200
 #else
 #include <strings.h>
 #endif
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
@@ -25,10 +26,11 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResPrint.c,v 1.2 200
 #include "dbwind/dbwind.h"
 #include "utils/utils.h"
 #include "cif/cif.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "textio/txcommands.h"
 #include "utils/stack.h"
-#include	"resis/resis.h"
+#include "utils/styles.h"
+#include "resis/resis.h"
 
 #define MAXNAME			1000
 #define KV_TO_mV		1000000
@@ -47,6 +49,8 @@ extern ResSimNode *ResInitializeNode();
  *
  *-------------------------------------------------------------------------
  */
+
+void
 ResPrintExtRes(outextfile,resistors,nodename)
 	FILE	*outextfile;
 	resResistor *resistors;
@@ -105,6 +109,8 @@ ResPrintExtRes(outextfile,resistors,nodename)
  *
  *-------------------------------------------------------------------------
  */
+
+void
 ResPrintExtTran(outextfile,transistors)
 	FILE		*outextfile;
 	RTran		*transistors;
@@ -153,6 +159,8 @@ ResPrintExtTran(outextfile,transistors)
  *
  *-------------------------------------------------------------------------
  */
+
+void
 ResPrintExtNode(outextfile, nodelist, nodename)
 	FILE	*outextfile;
 	resNode	*nodelist;
@@ -298,7 +306,7 @@ resWriteNodeName(fp, nodeptr)
  */
 
 void
-ResPrintGeoNodes(fp, nodelist, nodename, nidx, celldef)
+ResPrintFHNodes(fp, nodelist, nodename, nidx, celldef)
     FILE	*fp;
     resNode	*nodelist;
     char	*nodename;
@@ -473,13 +481,21 @@ ResPrintGeoNodes(fp, nodelist, nodename, nidx, celldef)
 
 /*
  *-------------------------------------------------------------------------
+ * ResPrintFHRects --
+ *	Generate FastHenry segment output to the FastHenry data file
+ *	"fp".	
  *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Stuff written to the stream file "fp".
  *
  *-------------------------------------------------------------------------
  */
 
 void
-ResPrintGeoRects(fp, reslist, nodename, eidx)
+ResPrintFHRects(fp, reslist, nodename, eidx)
     FILE	*fp;
     resResistor	*reslist;
     char	*nodename;
@@ -618,3 +634,77 @@ ResPrintReference(fp, transistors, cellDef)
     fprintf(fp, "\nNsub x=%1.2f y=%1.2f z=0\n", llx, lly);
     fprintf(fp, ".Equiv Nsub Ngp\n");
 }
+
+/*
+ *-------------------------------------------------------------------------
+ * ResCreateCenterlines --
+ *	Generate centerline markers on the layout that correspond to
+ * 	network routes.  Use the "DBWelement" mechanism.
+ *
+ * Results:
+ *	0 on success, -1 if a window cannot be found.
+ *
+ * Side effects:
+ *	Database "line" elements are generated in the layout.
+ *
+ *-------------------------------------------------------------------------
+ */
+
+int
+ResCreateCenterlines(reslist, nidx, def)
+    resResistor	*reslist;
+    int		*nidx;
+    CellDef *def;
+{
+    resResistor	*resistors;
+    resNode *nodeptr;
+    Rect r, rcanon;
+    MagWindow *w;	/* should be passed from up in CmdExtResis. . . */
+    char name[128];
+
+    w = ToolGetBoxWindow (&r,  (int *)NULL);
+    if (w == (MagWindow *)NULL)
+	windCheckOnlyWindow(&w, DBWclientID);
+    if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID)) {
+        TxError("Put the cursor in a layout window.\n");
+        return -1;
+    }
+
+    for (resistors = reslist; resistors; resistors = resistors->rr_nextResistor)
+    {
+	/* Ignore vias */
+
+	if (!DBIsContact(resistors->rr_tt))
+	{
+	    nodeptr = resistors->rr_connection1;
+	    r.r_xbot = nodeptr->rn_loc.p_x;
+	    r.r_ybot = nodeptr->rn_loc.p_y;
+	    if (nodeptr->rn_name == NULL)
+	    {
+		nodeptr->rn_id = (*nidx);
+		(*nidx)++;
+		sprintf(name, "N%d_", nodeptr->rn_id);
+	    }
+	    else
+		sprintf(name, "N%s_", nodeptr->rn_name);
+
+	    nodeptr = resistors->rr_connection2;
+	    r.r_xtop = nodeptr->rn_loc.p_x;
+	    r.r_ytop = nodeptr->rn_loc.p_y;
+	    GeoCanonicalRect(&r, &rcanon);
+	    if (nodeptr->rn_name == NULL)
+	    {
+		nodeptr->rn_id = (*nidx);
+		(*nidx)++;
+		sprintf(name + strlen(name), "%d", nodeptr->rn_id);
+	    }
+	    else
+		strcat(name, nodeptr->rn_name);
+
+	    /* Note that if any element exists with name "name"	*/
+	    /* it will be deleted (overwritten).		*/ 
+	    DBWElementAddLine(w, name, &rcanon, def, STYLE_YELLOW1);
+	}
+    }
+}
+
