@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResUtils.c,v 1.2 2001/01/12 22:13:17 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/resis/ResUtils.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -11,7 +11,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResUtils.c,v 1.2 200
 #else
 #include <strings.h>
 #endif
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
@@ -24,7 +25,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResUtils.c,v 1.2 200
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "utils/stack.h"
-#include "tech/tech.h"
+#include "utils/tech.h"
 #include "textio/txcommands.h"
 #include "resis/resis.h"
 
@@ -65,7 +66,7 @@ ResFirst(tile, arg)
 #endif
     t = TiGetType(tile);
 
-    if ((DBConnPlanes[t] !=0) && DBIsContact(t))
+    if (DBIsContact(t))
     {
 	    reg = (ResContactPoint *) mallocMagic((unsigned) (sizeof(ResContactPoint)));
 	    reg->cp_center.p_x = (LEFT(tile)+RIGHT(tile))>>1;
@@ -667,6 +668,7 @@ ResPreProcessTransistors(TileList, TransistorList, Def)
  *
  *-------------------------------------------------------------------------
  */
+void
 ResAddToQueue(node,list)
      resNode		*node,**list;
 {
