@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/router/routeCmd.c,v 1.0 2001/03/29 19:50:05 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/router/rtrCmd.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -26,9 +26,9 @@ static char rcsid[] = "$Header: /ufs/repository/magic/router/routeCmd.c,v 1.0 20
 #include <string.h>
 #endif
 
-#include "magic/tclmagic.h"
-#include "misc/magic.h"
-#include "misc/styles.h"
+#include "tcltk/tclmagic.h"
+#include "utils/magic.h"
+#include "utils/styles.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
@@ -36,14 +36,13 @@ static char rcsid[] = "$Header: /ufs/repository/magic/router/routeCmd.c,v 1.0 20
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "textio/txcommands.h"
 #include "select/select.h"
 #include "commands/commands.h"
-#include "utils/doubleint.h"
 #include "utils/heap.h"
 #include "router/router.h"
-#include "netlist/netlist.h"
+#include "utils/netlist.h"
 #include "netmenu/netmenu.h"
 #include "gcr/gcr.h"
 #include "grouter/grouter.h"
@@ -363,10 +362,10 @@ badWarn:
 	    }
 	    else if (cmd->tx_argc == 6 || cmd->tx_argc == 7)
 	    {
-		editArea.r_xbot = cmdParseCoord(cmd->tx_argv[2]);
-		editArea.r_ybot = cmdParseCoord(cmd->tx_argv[3]);
-		editArea.r_xtop = cmdParseCoord(cmd->tx_argv[4]);
-		editArea.r_ytop = cmdParseCoord(cmd->tx_argv[5]);
+		editArea.r_xbot = cmdParseCoord(w, cmd->tx_argv[2], FALSE, TRUE);
+		editArea.r_ybot = cmdParseCoord(w, cmd->tx_argv[3], FALSE, FALSE);
+		editArea.r_xtop = cmdParseCoord(w, cmd->tx_argv[4], FALSE, TRUE);
+		editArea.r_ytop = cmdParseCoord(w, cmd->tx_argv[5], FALSE, FALSE);
 		chanType = CHAN_NORMAL;
 		if (cmd->tx_argc == 7)
 		    channame = cmd->tx_argv[6];
@@ -686,8 +685,8 @@ CmdRoute(w, cmd)
 	    {
 		if (cmd->tx_argc != 4)
 		    goto wrongNumArgs;
-		RtrOrigin.p_x = cmdParseCoord(cmd->tx_argv[2]);
-		RtrOrigin.p_y = cmdParseCoord(cmd->tx_argv[3]);
+		RtrOrigin.p_x = cmdParseCoord(w, cmd->tx_argv[2], FALSE, TRUE);
+		RtrOrigin.p_y = cmdParseCoord(w, cmd->tx_argv[3], FALSE, FALSE);
 		break;
 	    }
 	    TxPrintf("Routing grid origin = (%d,%d)\n",
