@@ -17,12 +17,13 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/router/rtrVia.c,v 1.2 2001/01/12 22:13:19 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/router/rtrVia.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
@@ -30,10 +31,10 @@ static char rcsid[]="$Header: /ufs/repository/magic/router/rtrVia.c,v 1.2 2001/0
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "netmenu/netmenu.h"
 #include "textio/textio.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "utils/malloc.h"
 #include "gcr/gcr.h"
 #include "router/router.h"
