@@ -20,19 +20,19 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/select/selEnum.c,v 1.3 2001/03/29 19:50:11 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/select/selEnum.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <string.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "windows/windows.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "select/select.h"
 #include "select/selInt.h"
 #include "utils/malloc.h"
