@@ -18,22 +18,23 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/select/selOps.c,v 1.4 2001/03/29 19:50:11 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/select/selOps.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
-#include "main/main.h"
+#include "utils/main.h"
 #include "select/select.h"
 #include "select/selInt.h"
 #include "textio/textio.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "plow/plow.h"
 #include "utils/malloc.h"
 #include "drc/drc.h"
@@ -80,12 +81,13 @@ static StretchArea *selStretchList;	/* List of areas to paint. */
  */
 
 void
-SelectDelete(msg)
+SelectDelete(msg, do_clear)
     char *msg;		/* Some information to print in error messages.
 			 * For example, if called as part of a move procedure,
 			 * supply "moved".  This will appear in messages of
 			 * the form "only edit cell information was moved".
 			 */
+    bool do_clear;	/* If TRUE, clear the select def before returning. */
 {
     bool nonEdit;
     Rect editArea;
@@ -121,7 +123,7 @@ SelectDelete(msg)
     DBWAreaChanged(EditCellUse->cu_def, &editArea, DBW_ALLWINDOWS,
 	(TileTypeBitMask *) NULL);
     DRCCheckThis(EditCellUse->cu_def, TT_CHECKPAINT, &editArea);
-    SelectClear();
+    if (do_clear) SelectClear();
 }
 
 /* Search function to delete paint. */
@@ -133,6 +135,24 @@ selDelPaintFunc(rect, type)
 {
     Rect editRect;
 
+#ifdef NONMANHATTAN
+    TileType newtype;
+
+    /* Change diagonal direction according to the transform */
+
+    if (type & TT_DIAGONAL)
+    {
+	TileType dinfo = DBTransformDiagonal(type, &RootToEditTransform);
+
+	if ((type & TT_SIDE) != (dinfo & TT_SIDE))
+	    newtype = dinfo | ((type & TT_LEFTMASK) << 14) |
+                        ((type & TT_RIGHTMASK) >> 14);
+	else
+	    newtype = dinfo | (type & (TT_LEFTMASK | TT_RIGHTMASK));
+	type = newtype;
+    }
+#endif
+
     GeoTransRect(&RootToEditTransform, rect, &editRect);
     DBErase(EditCellUse->cu_def, &editRect, type);
     return 0;
@@ -202,11 +222,12 @@ SelectCopy(transform)
     DBCellClearDef(Select2Def);
     scx.scx_use = SelectUse;
     scx.scx_area = SelectUse->cu_bbox;
-    scx.scx_trans = *transform;
-    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, -1, Select2Use);
-    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, -1, Select2Use,
-	(Rect *) NULL);
-    (void) DBCellCopyAllCells(&scx, -1, Select2Use, (Rect *) NULL);
+    GeoTransTrans(transform, &SelectUse->cu_transform, &scx.scx_trans);
+    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, CU_DESCEND_SPECIAL,
+		Select2Use);
+    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL, Select2Use,
+		(Rect *) NULL);
+    (void) DBCellCopyAllCells(&scx, CU_DESCEND_SPECIAL, Select2Use, (Rect *) NULL);
     DBReComputeBbox(Select2Def);
     UndoEnable();
 
@@ -263,33 +284,28 @@ selTransPaintFunc(rect, type, transform)
     TileType type;		/* Type of paint. */
     Transform *transform;	/* How to change coords before painting. */
 {
-    Rect new;
+    Rect newarea;
 
 #ifdef NONMANHATTAN
+    TileType newtype;
+
     /* Change diagonal direction according to the transform */
 
     if (type & TT_DIAGONAL)
     {
-	int o1, o2, o3, dir, side, oldside;
+	TileType dinfo = DBTransformDiagonal(type, transform);
 
-	o1 = ((transform->t_e > 0) || (transform->t_d > 0)) ? 1 : 0;
-	o2 = ((transform->t_a > 0) || (transform->t_b > 0)) ? 1 : 0;
-	o3 = (transform->t_a != 0) ? 1 : 0;
-	dir = ((type & TT_DIRECTION) ? 1 : 0);
-	oldside = (type & TT_SIDE) ? 1 : 0;
-	side = oldside ^ o2 ^ (dir | o3);
-	dir ^= o1 ^ o2;
-	if (side != oldside)
-	    type = TT_DIAGONAL | ((type & TT_LEFTMASK) << 14) |
+	if ((type & TT_SIDE) != (dinfo & TT_SIDE))
+	    newtype = dinfo | ((type & TT_LEFTMASK) << 14) |
                         ((type & TT_RIGHTMASK) >> 14);
 	else
-	    type &= ~(TT_DIRECTION | TT_SIDE);
-	type |= ((dir) ? TT_DIRECTION : 0) | ((side) ? TT_SIDE : 0);
+	    newtype = dinfo | (type & (TT_LEFTMASK | TT_RIGHTMASK));
+	type = newtype;
     }
 #endif
 
-    GeoTransRect(transform, rect, &new);
-    DBPaint(Select2Def, &new, type);
+    GeoTransRect(transform, rect, &newarea);
+    DBPaint(Select2Def, &newarea, type);
     return 0;
 }
 
@@ -379,6 +395,7 @@ SelectTransform(transform)
 					 * visible) coordinates.
 					 */
 {
+
     /* Copy from SelectDef to Select2Def, transforming along the way. */
 
     selTransTo2(transform);
@@ -387,7 +404,7 @@ SelectTransform(transform)
      * copying into the edit cell along the way.
      */
 
-    SelectDelete("modified");
+    SelectDelete("modified", TRUE);
     SelectAndCopy2(EditRootDef);
 }
 
@@ -445,7 +462,7 @@ selExpandFunc(selUse, use, transform, mask)
      * the one in the layout in order to keep them consistend.
      */
 
-    if (DBIsExpand(use, mask))
+    if (DBDescendSubcell(use, mask))
     {
 	DBExpand(selUse, mask, FALSE);
 	DBExpand(use, mask, FALSE);
@@ -507,7 +524,7 @@ SelectArray(arrayInfo)
      * copying into the edit cell along the way.
      */
     
-    SelectDelete("arrayed");
+    SelectDelete("arrayed", TRUE);
     SelectAndCopy2(EditRootDef);
 }
 
@@ -714,7 +731,7 @@ SelectStretch(x, y)
 
     /* Delete the selection itself. */
 
-    SelectDelete("stretched");
+    SelectDelete("stretched", TRUE);
 
     /* Next, delete all the material in front of each piece of paint in
      * the selection.
@@ -726,7 +743,7 @@ SelectStretch(x, y)
     {
 	(void) DBSrPaintArea((Tile *) NULL, Select2Def->cd_planes[plane],
 		&TiPlaneRect, &DBAllButSpaceAndDRCBits, selStretchEraseFunc,
-		(ClientData) NULL);
+		(ClientData) &plane);
     }
 
     /* To achieve the stretch affect, fill in material behind the selection
@@ -741,7 +758,7 @@ SelectStretch(x, y)
     {
 	(void) DBSrPaintArea((Tile *) NULL, Select2Def->cd_planes[plane],
 		&TiPlaneRect, &DBAllButSpaceAndDRCBits, selStretchFillFunc,
-		(ClientData) NULL);
+		(ClientData) &plane);
     }
 
     /* Paint back the areas in the list. */
@@ -782,12 +799,21 @@ SelectStretch(x, y)
  */
 
 int
-selStretchEraseFunc(tile)
+selStretchEraseFunc(tile, plane)
     Tile *tile;			/* Tile being moved in a stretch operation. */
+    int *plane;			/* Plane of tiles being searched */
 {
     Rect area, editArea;
-    TileType type;
+    int planeNum;
+    TileType type, t;
+    TileTypeBitMask *typeResidueMask, *connectMask, tmpmask, mask;
 
+#ifdef NONMANHATTAN
+    if (IsSplit(tile))
+	type = (SplitSide(tile)) ? SplitRightType(tile) :
+			SplitLeftType(tile);
+    else
+#endif
     type = TiGetType(tile);
 
     TiToRect(tile, &area);
@@ -808,19 +834,117 @@ selStretchEraseFunc(tile)
      * tile's plane.
      */
 
+    GeoTransRect(&RootToEditTransform, &area, &editArea);
+ 
+    /* Was previously DBEraseMask(), and needs to be again to avoid	*/
+    /* quirky behavior.  However, the DBErase() call is worse.  We need	*/
+    /* to erase all types that interact with "type", *not* all types on	*/
+    /* "plane", due to the way stacked contacts are handled.  But	*/
+    /* contacts on different planes may stretch across one another	*/
+    /* without effect, if the types are stackable.  So, getting the	*/
+    /* correct mask is not so simple.					*/
+
+    /* DBEraseMask(EditCellUse->cu_def, &editArea, &DBPlaneTypes[*plane]); */
+    /* DBErase(EditCellUse->cu_def, &editArea, type); */
+
+    /* Routine modified by Nishit; code below written 9/28/2004, added	*/
+    /* 10/14/04								*/
+
     /*
-     * Stacking contact types do not get erased
+     *   Process mask first
      */
 
-    GeoTransRect(&RootToEditTransform, &area, &editArea);
+    planeNum = *plane;
+    typeResidueMask = DBResidueMask(type);
+    mask = DBPlaneTypes[planeNum];
+    connectMask = &DBConnectTbl[type];
+    for (t = TT_SPACE + 1; t < DBNumTypes; t++)
+    {
+	/* Process all tile types that exist on plane
+	 * "planeNum" (except the current one, as we want
+	 *  to erase that type of tile from the layout).
+	 */
+	if (TTMaskHasType(&mask, t) && (t != type))
+	{
+	    if (DBIsContact(t))
+	    {
+	        /* While stretching non-contact material
+	         * we would not process residue type of "t".
+	         * Because that may remove mask of "type" from
+	         * "mask" which would untouch that material in
+	         * stretching path.
+	         */
+		if (DBIsContact(type))
+	        {
+		     tmpmask = *DBResidueMask(t);
+	             if (!TTMaskIsZero(typeResidueMask) &&
+	                	!TTMaskIsZero(&tmpmask))
+                        TTMaskAndMask3(&tmpmask, typeResidueMask, &tmpmask);
 
-    DBErase(EditCellUse->cu_def, &editArea, type);
+	             /* If t is stacked type then we would not
+	              * remove its residue mask from "mask".
+	              */
+	             if (!TTMaskIsZero(&tmpmask) && (t < DBNumUserLayers))
+	                 TTMaskClearMask(&mask, &tmpmask);
+	         }
+	
+                 /* If tile type "t" connects to tile "type"
+                  * remove that type from mask. We would not
+                  * like to erase that type of tiles.
+                  */
+                 if (TTMaskHasType(connectMask, t))
+                 {
+                     /* In case "type" is a stacked type and both "t" and
+		      * "type" are on same plane, erase that tile (e.g.,
+		      * stack of 3 types selected)
+                      */
+                     if ((type < DBNumUserLayers) &&
+                        	(DBPlane(type) != DBPlane(t)))
+                          TTMaskClearType(&mask, t);
+                 }
+             }
+         }
+      }
 
-    /* Was previously DBEraseMask().  Don't know why.  If you trust the */
-    /* paint/erase tables, it should not be necessary to erase anything	*/
-    /* other than the type itself.					*/
+    /* For stacked type tiles do not erase common residue
+     * type of both contacts from layout.
+     */
 
-    /* DBEraseMask(EditCellUse->cu_def, &editArea, &DBPlaneTypes[DBPlane(type)]); */
+    if (type >= DBNumUserLayers)
+    {
+        t = (DBPlaneToResidue(type, planeNum));
+        TTMaskClearType(&mask, t);
+    }
+
+    /*
+     *   Erase all tile types specified in the processed mask.
+     */
+    	
+    /* New DBEraseMask() routine for stretching. */
+    /* TODO : Placed here just for checking funtionality.
+     * Once works properly shift to DBtcontact.c.
+     */
+    {
+       /* Erase contacts first from mask . Because if you
+        * erase one type of tile from contact first then you
+        * can't remove that contact completely as its residue
+        * type remains on other plane. And that will stay for ever !
+        */
+
+       if (DBIsContact(type))
+       {
+           for (t = TT_SPACE + 1; t < DBNumUserLayers; t++)
+	       if (TTMaskHasType(&mask, t) && DBIsContact(t))
+	           DBErase(EditCellUse->cu_def, &editArea, t);
+       }
+       for (t = TT_SPACE + 1; t < DBNumUserLayers; t++)
+           if (TTMaskHasType(&mask, t) && !DBIsContact(t))
+	        DBErase(EditCellUse->cu_def, &editArea, t);
+    }
+
+    /*
+     *   Completed.
+     */
 
     return 0;
 }
@@ -845,8 +969,9 @@ selStretchEraseFunc(tile)
  */
 
 int
-selStretchFillFunc(tile)
+selStretchFillFunc(tile, plane)
     Tile *tile;			/* Tile in the old selection. */
+    int *plane;			/* Plane of tile being searched */
 {
     Rect area;
     extern int selStretchFillFunc2();
@@ -903,7 +1028,7 @@ selStretchFillFunc(tile)
      */
 
     (void) DBSrPaintArea((Tile *) NULL,
-	    Select2Def->cd_planes[DBPlane(selStretchType)], &area,
+	    Select2Def->cd_planes[*plane], &area,
 	    &DBSpaceBits, selStretchFillFunc2, (ClientData) &area);
     
     return 0;
@@ -944,12 +1069,13 @@ selStretchFillFunc2(tile, area)
     GeoTransRect(&RootToEditTransform, &spaceArea, &editArea);
 
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
-	if (DBPaintOnPlane(selStretchType, pNum))
+    {
+	if (DBTypeOnPlane(selStretchType, pNum))
 	    (void) DBSrPaintArea((Tile *) NULL,
 			EditCellUse->cu_def->cd_planes[pNum],
 			&editArea, &DBAllButSpaceAndDRCBits, selStretchFillFunc3,
 			(ClientData) &spaceArea);
-
+    }
     return 0;
 }
 
@@ -965,8 +1091,8 @@ selStretchFillFunc3(tile, area)
     Tile *tile;			/* Tile of edit material that's about to
 				 * be left behind selection.
 				 */
-    Rect *area;			/* Border area we're interested in, in 
-    				 * root coords.
+    Rect *area;			/* The border area we're interested in,
+				 * in root coords.
 				 */
 {
     Rect editArea, rootArea;
@@ -1032,9 +1158,9 @@ selStretchFillFunc3(tile, area)
 		(((stype = DBTechFindStacking(type, selStretchType))
 		< DBNumUserLayers) && (stype >= TT_TECHDEPBASE)))
     {
-	/* If type is a residue of selStretchType then we always fill	*/
-	/* with type.  Otherwise, we check the list of plow types to	*/
-	/* see what is and isn't fixed.					*/
+	/* If type is a residue of selStretchType then we always 	*/
+	/* fill with type.  Otherwise, we check the list of plow	*/
+	/* types to see what is and isn't fixed.			*/
 
 	if (!TTMaskHasType(DBResidueMask(selStretchType), type))
 	    if (TTMaskHasType(&PlowFixedTypes, type)
@@ -1045,8 +1171,11 @@ selStretchFillFunc3(tile, area)
     /* Otherwise, if we have two fixed types, and they both border the	*/
     /* selection area (as opposed to overlapping), and they share a	*/
     /* residue, then the type to paint is the shared residue type.	*/
+    /* We must ignore the case type == selStretchType (fixed by Nishit,	*/
+    /* 6/29/04.								*/
 
-    else if (DBIsContact(type) && DBIsContact(selStretchType))
+    else if (DBIsContact(type) && DBIsContact(selStretchType)
+		&& (type != selStretchType))
     {
 	if (((selStretchX < 0) && (editArea.r_xtop == area->r_xbot)) ||
 	    ((selStretchX > 0) && (editArea.r_xbot == area->r_xtop)) ||
@@ -1109,10 +1238,11 @@ SelectDump(scx)
 
     UndoDisable();
     DBCellClearDef(Select2Def);
-    (void) DBCellCopyAllPaint(scx, &DBAllButSpaceAndDRCBits, -1, Select2Use);
-    (void) DBCellCopyAllLabels(scx, &DBAllTypeBits, -1, Select2Use,
-	    (Rect *) NULL);
-    (void) DBCellCopyAllCells(scx, -1, Select2Use, (Rect *) NULL);
+    (void) DBCellCopyAllPaint(scx, &DBAllButSpaceAndDRCBits, CU_DESCEND_SPECIAL,
+		Select2Use);
+    (void) DBCellCopyAllLabels(scx, &DBAllTypeBits, CU_DESCEND_SPECIAL, Select2Use,
+		(Rect *) NULL);
+    (void) DBCellCopyAllCells(scx, CU_DESCEND_SPECIAL, Select2Use, (Rect *) NULL);
     DBReComputeBbox(Select2Def);
     UndoEnable();
 
