@@ -20,11 +20,12 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/select/selUndo.c,v 1.2 2001/01/12 22:13:23 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/select/selUndo.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
@@ -32,7 +33,7 @@ static char rcsid[]="$Header: /ufs/repository/magic/select/selUndo.c,v 1.2 2001/
 #include "windows/windows.h"
 #include "graphics/graphics.h"
 #include "dbwind/dbwind.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "commands/commands.h"
 #include "textio/textio.h"
 #include "select/select.h"
