@@ -17,13 +17,13 @@
  *     *********************************************************************
  *
  *
- * rcsid="$Header: /ufs/repository/magic/select/select.h,v 1.2 2001/01/12 22:13:23 jsolomon Exp $"
+ * rcsid="$Header: /usr/cvsroot/magic/select/select.h,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $"
  */
 
 #ifndef _SELECT_H
 #define _SELECT_H
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "database/database.h"
 
 /* Procedures to modify the selection. */
