@@ -26,7 +26,7 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
 #include "tiles/tile.h"
@@ -34,7 +34,7 @@
 #include "database/database.h"
 #include "database/databaseInt.h"
 #include "textio/textio.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "utils/malloc.h"
 #include "extract/extractInt.h"
 #include "sim/sim.h"
@@ -42,7 +42,7 @@
 #include "dbwind/dbwind.h"
 #include "commands/commands.h"
 #include "textio/txcommands.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "graphics/graphics.h"
 
 /* The following structure is used to hold several pieces
@@ -816,7 +816,7 @@ SimCellTileSrFunc(scx, fp)
     char *tnext;
 
     ASSERT(def != (CellDef *) NULL, "SimCellTileSrFunc");
-    if (!DBIsExpand(scx->scx_use, fp->tf_xmask))
+    if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask))
 	return 0;
     if ((def->cd_flags & CDAVAILABLE) == 0)
 	if (!DBCellRead(def, (char *) NULL, TRUE)) return 0;
