@@ -31,15 +31,15 @@
 #include <sys/wait.h>
 #include <sys/file.h>
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/stack.h"
 #include "utils/geometry.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "signals/signals.h"
-#include "misc/styles.h"
+#include "utils/signals.h"
+#include "utils/styles.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "sim/sim.h"
@@ -75,6 +75,8 @@ bool	SimRsimRunning = FALSE;
 bool	SimHasCoords = FALSE;
 bool	SimGetReplyLine();
 
+/* Forward declaration */
+void SimStopRsim();
 
 /*
  *-----------------------------------------------------------------------
