@@ -27,7 +27,7 @@
 #include <errno.h>
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"   
+#include "utils/magic.h"   
 #include "utils/geometry.h"
 #include "tiles/tile.h"  
 #include "utils/hash.h"  
@@ -38,7 +38,7 @@
 #include "utils/utils.h"
 #include "textio/textio.h"
 #include "textio/txcommands.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "graphics/graphics.h"
 #include "utils/malloc.h" 
 #include "dbwind/dbwind.h"
@@ -49,6 +49,7 @@
  */
 
 char *MagicVersion = MAGIC_VERSION;
+char *MagicRevision = MAGIC_REVISION;
 char *MagicCompileTime = MAGIC_DATE;
 
 Tcl_Interp *magicinterp;
@@ -283,7 +284,7 @@ AddCommandTag(ClientData clientData,
     }
     else
     {
-	hstring = StrDup(NULL, argv[2]);
+	hstring = StrDup((char **)NULL, argv[2]);
 	HashSetValue(entry, hstring);
     }
     return TCL_OK;
@@ -377,8 +378,13 @@ _tcl_dispatch(ClientData clientData,
     }
     Tcl_ResetResult(interp);
 
-    TxInputRedirect = False;
+    if (TxInputRedirect == TX_INPUT_REDIRECTED)
+	TxInputRedirect = TX_INPUT_PENDING_RESET;
+
     TxTclDispatch(clientData, argc, argv);
+    if (TxInputRedirect == TX_INPUT_PENDING_RESET)
+	TxInputRedirect = TX_INPUT_NORMAL;
+
     return TagCallback(interp, NULL, argc, argv);
 }
 
@@ -460,7 +466,8 @@ static int
 _magic_initialize(ClientData clientData,
         Tcl_Interp *interp, int argc, char *argv[])
 {
-    int n;
+    WindClient client;
+    int n, i;
     char keyword[100];
     char *kwptr = keyword + 7;
     char **commandTable;
@@ -494,23 +501,23 @@ _magic_initialize(ClientData clientData,
     /* Registration of commands is performed after calling the	*/
     /* start function, not after initialization, as the command */
     /* modularization requires magic initialization to get a	*/
-    /* valid DBWclientID and windClientID.			*/
+    /* valid DBWclientID, windClientID, etc.			*/
 
     sprintf(keyword, "magic::");
-    commandTable = WindGetCommandTable(DBWclientID);
-    for (n = 0; commandTable[n] != NULL; n++)
-    {
-	sscanf(commandTable[n], "%s ", kwptr); /* get first word */
-	Tcl_CreateCommand(interp, keyword, (Tcl_CmdProc *)_tcl_dispatch,
-			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
-    }
 
-    commandTable = WindGetCommandTable(windClientID);
-    for (n = 0; commandTable[n] != NULL; n++)
+    /* Work through all the known clients, and register the	*/
+    /* commands of all of them.					*/
+
+    client = (WindClient)NULL;
+    while ((client = WindNextClient(client)) != NULL)
     {
-	sscanf(commandTable[n], "%s ", kwptr); /* get first word */
-	Tcl_CreateCommand(interp, keyword, (Tcl_CmdProc *)_tcl_dispatch,
+	commandTable = WindGetCommandTable(client);
+	for (n = 0; commandTable[n] != NULL; n++)
+	{
+	    sscanf(commandTable[n], "%s ", kwptr); /* get first word */
+	    Tcl_CreateCommand(interp, keyword, (Tcl_CmdProc *)_tcl_dispatch,
 			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
+	}
     }
 
     /* Extra commands provided by the Tk graphics routines	*/
@@ -587,15 +594,20 @@ TxDialog(prompt, responses, defresp)
 
     /* Ensure that use of TxPrintString doesn't overwrite the	*/
     /* value of prompt my making a copy of it.			*/
+    /* 5/11/05---use Tcl_escape() to do the duplication; this	*/
+    /* ensures that cell names with special characters like '$'	*/
+    /* will be handled properly.				*/
 
-    newstr = StrDup(NULL, prompt);
+    newstr = Tcl_escape(prompt);
+    /* newstr = StrDup((char **)NULL, prompt); */
     evalstr = TxPrintString("tk_dialog .dialog \"Dialog\""
 	" \"%s\" {} %d ", newstr, defresp);
-    freeMagic(newstr);
+    /* freeMagic(newstr); */
+    Tcl_Free(newstr);		/* Tcl_escape() uses Tcl_Alloc() */
 
     for (pos = 0; responses[pos] != 0; pos++)
     {
-	newstr = StrDup(NULL, evalstr);
+	newstr = StrDup((char **)NULL, evalstr);
 	evalstr = TxPrintString("%s \"%s\" ", newstr,
 		responses[pos]);
 	freeMagic(newstr);
@@ -674,7 +686,7 @@ TxGetLinePfix(dest, maxChars, prefix)
 	    Tcl_Free(string);
 	}
 	else
-	    Tcl_EvalEx(magicinterp, "magic::dialog", 12, 0);
+	    Tcl_EvalEx(magicinterp, "magic::dialog", 13, 0);
     }
     else
     {
@@ -828,13 +840,14 @@ Tcl_printf(FILE *f, char *fmt, va_list args_in)
 	if (*(outptr + i) == '\"' || *(outptr + i) == '[' ||
 	    	*(outptr + i) == ']' || *(outptr + i) == '\\')
 	    escapes++;
-	else if (*(outptr + i) == '$' && (*outptr + i + 1) == '$')
+	else if (*(outptr + i) == '$' && *(outptr + i + 1) == '$')
 	    escapes += 2;
     }
 
     if (escapes > 0)
     {
-	finalstr = Tcl_Alloc(nchars + escapes + 26);
+	/* "+ 4" required to process "$$...$$"; haven't figured out why. */ 
+	finalstr = Tcl_Alloc(nchars + escapes + 26 + 4);
 	strncpy(finalstr, outptr, 24);
 	escapes = 0;
 	for (i = 24; *(outptr + i) != '\0'; i++)
@@ -948,7 +961,7 @@ TerminalInputProc(instanceData, buf, toRead, errorCodePtr)
 
     *errorCodePtr = 0;
 
-    TxInputRedirect = False;
+    TxInputRedirect = TX_INPUT_NORMAL;
     if (TxBuffer != NULL) {
        tlen = strlen(TxBuffer);
        if (tlen < toRead) {
@@ -991,14 +1004,6 @@ Tclmagic_Init(interp)
 
     if (Tcl_InitStubs(interp, "8.1", 0) == NULL) return TCL_ERROR;
 
-    /* Set initial command structure, so that magic doesn't	 */
-    /* fault when running commands before magic::start is called */
-
-    TxCurCommand.tx_p.p_x = 20;
-    TxCurCommand.tx_p.p_y = 20;
-    TxCurCommand.tx_wid = WIND_UNKNOWN_WINDOW;
-
-
     /* Initialization and Startup commands */
     Tcl_CreateCommand(interp, "magic::initialize", (Tcl_CmdProc *)_magic_initialize,
 			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
