@@ -42,24 +42,6 @@ set Glyph(zoom) [image create bitmap \
 	-file ${CAD_HOME}/lib/magic/tcl/bitmaps/zoom.xbm \
 	-background gray -foreground steelblue]
 
-# Handle the kludgy macros-call-Tcl-procedures setup by creating
-# a command "gmacro" (general-purpose macro).  The idea is that
-# macros only call magic (not Tcl) commands, but that tag callbacks
-# can call anything.  So "gmacro" sets up a dummy keyword that
-# is not a command, and attaches a tag callback function to it.
-# The tag callback implements the actual macro.
-
-proc magic::gmacro {key value} {
-   global Opts
-   if {[catch {set Opts(gmacrokeys)}]} {
-      set Opts(gmacrokeys) 0
-   } else {
-      incr Opts(gmacrokeys)
-   }
-   magic::macro $key gmkey$Opts(gmacrokeys)
-   magic::tag gmkey$Opts(gmacrokeys) $value
-}
-
 # Menu button callback functions
 
 proc magic::promptload {type} {
@@ -213,6 +195,37 @@ proc magic::instcallback {command} {
    }
 }
 
+# Callback functions used by the DRC.
+
+proc magic::drcupdate { option } {
+   global Opts
+   if {[info level] <= 1} {
+      switch $option {
+         on {set Opts(drc) 1}
+         off {set Opts(drc) 0}
+      }
+   }
+}
+
+proc magic::drcstate { status } {
+   set winlist [*bypass windownames layout]
+   foreach lwin $winlist {
+      set framename [winfo parent $lwin]
+      switch $status {
+         idle { 
+		set dct [*bypass drc count total]
+		if {$dct > 0} {
+	           ${framename}.titlebar.drcbutton configure -selectcolor red
+		} else {
+	           ${framename}.titlebar.drcbutton configure -selectcolor green
+		}
+	        ${framename}.titlebar.drcbutton configure -text "DRC=$dct"
+	      }
+         busy { ${framename}.titlebar.drcbutton configure -selectcolor yellow }
+      }
+   }
+}
+
 # Create the menu of windows.  This is kept separate from the cell manager,
 # and linked into it by the "clone" command.
 
@@ -495,16 +508,22 @@ proc magic::techmanager {{option "update"}} {
 }
 
 proc magic::caption {winpath} {
+   global Opts
+
    set framename [winfo parent $winpath]
    set caption [$winpath windowcaption]
    set subcaption1 [lindex $caption 0]
+   if {[catch {set Opts(tool)}]} {
+      set Opts(tool) unknown
+   }
    if {[lindex $caption 1] == "EDITING"} {
       set subcaption2 [lindex $caption 2]
    } else {
-      set subcaption2 [join [lrange $caption 1 end]]
+      # set subcaption2 [join [lrange $caption 1 end]]
+      set subcaption2 $caption
    }
    ${framename}.titlebar.caption configure -text \
-	"Loaded: ${subcaption1} Editing: ${subcaption2}"
+	"Loaded: ${subcaption1} Editing: ${subcaption2} Tool: $Opts(tool)"
 }
 
 # Allow captioning in the title window by tagging the "load" and "edit" commands
@@ -515,7 +534,7 @@ magic::tag load "magic::cellmanager; magic::caption %W"
 magic::tag getcell "magic::cellmanager"
 magic::tag edit "magic::caption %W"
 magic::tag save "magic::caption %W"
-magic::tag box "magic::boxview %W"
+magic::tag box "magic::boxview %W %1"
 magic::tag move "magic::boxview %W"
 magic::tag scroll "magic::scrollupdate %W"
 magic::tag view "magic::scrollupdate %W"
@@ -523,6 +542,7 @@ magic::tag zoom "magic::scrollupdate %W"
 magic::tag findbox "magic::scrollupdate %W"
 magic::tag see "magic::toolupdate %W %1 %2"
 magic::tag tech "magic::techrebuild %W %1"
+magic::tag drc "magic::drcupdate %1"
 if {$MagicBLT} {
    magic::tag cellname "magic::mgrupdate %W %1"
    magic::tag select "magic::mgrselect %R"
@@ -536,6 +556,11 @@ set windowsopen 0
 set Opts(cellmgr) 0
 set Opts(techmgr) 0
 set Opts(target)  default
+set Opts(netlist) 0
+set Opts(colormap) 0
+set Opts(wind3d) 0
+set Opts(crosshair) 0
+set Opts(drc) 1
 
 # Set default width and height to be 3/4 of the screen size.
 set Opts(geometry) \
@@ -574,9 +599,10 @@ proc magic::repaintwrapper { win } {
 # Because "box" calls "box", use the "info level" command to avoid
 # infinite recursion.
 
-proc magic::boxview {win} {
-   if {[info level] <= 1} {
-
+proc magic::boxview {win {cmdstr ""}} {
+   if {${cmdstr} == "exists" || ${cmdstr} == "help" || ${cmdstr} == ""} {
+      # do nothing. . . informational only, no change to the box
+   } elseif {[info level] <= 1} {
       # For NULL window, find all layout windows and apply update to each.
       if {$win == {}} {
          set winlist [magic::windownames layout]
@@ -604,11 +630,13 @@ proc magic::boxview {win} {
 }
 
 proc magic::cursorview {win} {
+   global Opts
    if {$win == {}} {
       return
    }
    set framename [winfo parent $win]
    set lambda [${win} tech lambda]
+   if {[llength $lambda] != 2} {return}
    set lr [expr {(0.0 + [lindex $lambda 0]) / [lindex $lambda 1]}]
    set olst [${win} cursor]
    set olstx [expr {[lindex $olst 0] * $lr }]
@@ -616,6 +644,10 @@ proc magic::cursorview {win} {
    if {[expr {$olstx == int($olstx)}]} {set olstx [expr {int($olstx)}]}
    if {[expr {$olsty == int($olsty)}]} {set olsty [expr {int($olsty)}]}
 
+   if {$Opts(crosshair)} {
+      *bypass crosshair ${olstx}i ${olsty}i
+   }
+
    if {[${win} box exists]} {
       set dlst [${win} box position]
       set dx [expr {([lindex $olst 0] - [lindex $dlst 0]) * $lr }]
@@ -628,6 +660,7 @@ proc magic::cursorview {win} {
    } else {
       ${framename}.titlebar.pos configure -text "($olstx $olsty) lambda"
    }
+
 }
 
 proc magic::toolupdate {win {yesno "yes"} {layerlist "none"}} {
@@ -921,14 +954,14 @@ proc magic::scrollview { w win orient } {
       set bwidth [expr {[lindex $bvalues 2] - [lindex $bvalues 0]}]
       set wwidth [expr {0.000 + [lindex $wvalues 2] - [lindex $wvalues 0]}]
       set xscale [expr {$bwidth / $wwidth}]
-      ${win} scroll e [expr {$delta * $xscale}]
+      ${win} scroll e [expr {$delta * $xscale}]i
 
    } else {
 
       set bheight [expr {[lindex $bvalues 3] - [lindex $bvalues 1]}]
       set wheight [expr {0.000 + [lindex $wvalues 3] - [lindex $wvalues 1]}]
       set yscale [expr {$bheight / $wheight}]
-      ${win} scroll s [expr {$delta * $yscale}]
+      ${win} scroll s [expr {$delta * $yscale}]i
    }
 }
 
@@ -1040,7 +1073,7 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
    bind ${framename}.zb <Button-2> "${winname} view"
 
    frame ${framename}.titlebar
-   label ${framename}.titlebar.caption -text "Loaded: none Editing: none" \
+   label ${framename}.titlebar.caption -text "Loaded: none Editing: none Tool: box" \
 	-foreground white -background sienna4 -anchor w -padx 15
    label ${framename}.titlebar.message -text "" -foreground white \
 	-background sienna4 -anchor w -padx 5
@@ -1054,6 +1087,12 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
    menubutton ${framename}.titlebar.optsbutton -text Options -relief raised \
 		-menu ${framename}.titlebar.optsbutton.optsmenu -borderwidth 2
 
+   # DRC status button
+   checkbutton ${framename}.titlebar.drcbutton -text "DRC" -anchor w \
+	-borderwidth 2 -variable Opts(drc) \
+	-foreground white -background sienna4 -selectcolor green \
+	-command [subst { if { \$Opts(drc) } { drc on } else { drc off } }]
+
    magic::openwindow $cell $winname
 
    # Create toolbar frame.  Make sure it has the same visual and depth as
@@ -1077,17 +1116,22 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 
    grid ${framename}.titlebar.filebutton -row 0 -column 0 -sticky news
    grid ${framename}.titlebar.optsbutton -row 0 -column 1 -sticky news
-   grid ${framename}.titlebar.caption -row 0 -column 2 -sticky news
-   grid ${framename}.titlebar.message -row 0 -column 3 -sticky news
-   grid ${framename}.titlebar.pos -row 0 -column 4 -sticky news
-   grid columnconfigure ${framename}.titlebar 2 -weight 1
+   grid ${framename}.titlebar.drcbutton -row 0 -column 2 -sticky news
+   grid ${framename}.titlebar.caption -row 0 -column 3 -sticky news
+   grid ${framename}.titlebar.message -row 0 -column 4 -sticky news
+   grid ${framename}.titlebar.pos -row 0 -column 5 -sticky news
+   grid columnconfigure ${framename}.titlebar 3 -weight 1
 
    bind $winname <Enter> {focus %W}
 
-   # Button press events bypass the normal command mechanism, so any tag features
-   # required on button presses/releases need to be duplicated.
+   # Note: Tk binding bypasses the event proc, so it is important to
+   # set the current point;  otherwise, the cursor will report the
+   # wrong position and/or the wrong window.  HOWEVER we should wrap
+   # this command with the "bypass" command such that it does not
+   # reset any current input redirection to the terminal.
 
-   bind ${winname} <Motion> "::magic::cursorview ${winname}"
+   bind ${winname} <Motion> "*bypass setpoint %x %y ${winname}; \
+	magic::cursorview ${winname}"
 
    # Resize the window
    if {[catch {wm geometry ${framename} $Winopts(${framename},geometry)}]} {
@@ -1095,6 +1139,7 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
    }
 
    set Winopts(${framename},toolbar) 0
+   set Winopts(${framename},cmdentry) 0
 
    # Generate the file menu
 
@@ -1157,6 +1202,40 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 		wm withdraw .padlist } }]
    }
 
+   $m add check -label "Netlist Window" -variable Opts(netlist) \
+	-command [subst { if { \[windownames netlist\] != {}} { \
+		   set Opts(netlist) 0 ; closewindow \[windownames netlist\] \
+		} else { \
+		   set Opts(netlist) 1 ; specialopen netlist \
+		} }]
+
+   $m add check -label "Colormap Window" -variable Opts(colormap) \
+	-command [subst { if { \[windownames color\] != {}} { \
+		   set Opts(colormap) 0 ; closewindow \[windownames color\] \
+		} else { \
+		   set Opts(colormap) 1 ; specialopen color \
+		} }]
+
+   $m add check -label "3D Display" -variable Opts(wind3d) \
+	-command [subst { if { \[windownames wind3d\] != {}} { \
+		   set Opts(wind3d) 0 ; .render.magic closewindow ; \
+		   destroy .render \
+		} else { \
+		   set Opts(wind3d) 1 ; \
+		   magic::render3d \[${winname} cellname list window\] \
+		} }]
+
+   $m add check -label "Window Command Entry" \
+	-variable Winopts(${framename},cmdentry) \
+	-command [subst { if { \$Winopts(${framename},cmdentry) } { \
+		addcommandentry $framename \
+		} else { \
+		deletecommandentry $framename } }]
+
+   $m add check -label "Crosshair" \
+	-variable Opts(crosshair) \
+	-command "if {$Opts(crosshair) == 0} {crosshair off}"
+
    # Update the cell manager (if it exists)
    magic::cellmanager update
 
@@ -1167,10 +1246,10 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 
    magic::caption ${winname}
 
-   # Rename "open" macro by rebinding using the "gmacro" procedure
-   # (see above for the gmacro definition)
-   magic::gmacro o {magic::openwrapper}
-   magic::gmacro O {magic::closewrapper}
+   # Remove "open" and "close" macros so they don't generate non-GUI
+   # windows or (worse) blow away the window inside the GUI frame
+   magic::macro o ""
+   magic::macro O ""
 
    return ${winname}
 }
@@ -1214,20 +1293,6 @@ proc magic::addcommandentry { framename } {
    bind ${framename}.eval.text <Enter> {focus %W}
 }
 
-# Check for presence of padlist manager script and include it
-
-if {[file exists ${CAD_HOME}/lib/magic/tcl/padlist.tcl]} {
-   source ${CAD_HOME}/lib/magic/tcl/padlist.tcl
-   set Opts(padlist) 0
-}
-
-# Check for presence of the miscellaneous tools script and include it
-
-if {[file exists ${CAD_HOME}/lib/magic/tcl/tools.tcl]} {
-   source ${CAD_HOME}/lib/magic/tcl/tools.tcl
-   set Opts(tools) 0
-}
-
 # Remove the command entry window from the bottom of a frame.
 
 proc magic::deletecommandentry { framename } {
@@ -1239,16 +1304,32 @@ puts "Use openwrapper to create a new GUI-based layout window"
 namespace import magic::closewrapper
 puts "Use closewrapper to remove a new GUI-based layout window"
 
-namespace import magic::gmacro
-
 # Create a simple wrapper for the 3D window. . . this can be
 # greatly expanded upon.
 
-proc magic::render {{cell ""}} {
+proc magic::render3d {{cell ""}} {
+   global Opts
+
    toplevel .render
    tkwait visibility .render
    magic::specialopen wind3d $cell .render.magic
-   pack .render.magic
+   .render.magic cutbox box
+   set Opts(cutbox) 1
+
+   frame .render.title
+   pack .render.title -expand true -fill x -side top
+   checkbutton .render.title.cutbox -text "Cutbox" -variable Opts(cutbox) \
+	-foreground white -background sienna4 -anchor w -padx 15 \
+	-command [subst { if { \$Opts(cutbox) } { .render.magic cutbox box \
+	} else { \
+	.render.magic cutbox none } }]
+	
+   if {$cell == ""} {set cell default}
+   label .render.title.msg -text "3D Rendering window  Cell: $cell" \
+	-foreground white -background sienna4 -anchor w -padx 15
+   pack .render.title.cutbox -side left
+   pack .render.title.msg -side right -fill x -expand true
+   pack .render.magic -expand true -fill both -side bottom
    bind .render.magic <Enter> {focus %W}
 }
 
