@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/textio/txInput.c,v 1.11 2001/08/23 13:48:57 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/textio/txInput.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -31,15 +31,15 @@ static char rcsid[]="$Header: /ufs/repository/magic/textio/txInput.c,v 1.11 2001
 #endif
 
 
-#include "misc/magsgtty.h"
-#include "misc/magic.h"
-#include "misc/magsgtty.h"
+#include "utils/magsgtty.h"
+#include "utils/magic.h"
+#include "utils/magsgtty.h"
 #include "textio/textio.h"
 #include "utils/geometry.h"
 #include "textio/txcommands.h"
 #include "textio/textioInt.h"
 #include "utils/dqueue.h"
-#include "macros/macros.h"
+#include "utils/macros.h"
 #include "utils/hash.h"
 #include "tiles/tile.h"
 #include "database/database.h"
@@ -800,7 +800,7 @@ ostyle_completion_function(char *text, int state)
  *
  * cellname_completion_function --
  *
- *	Cell name completion function for use with "readline" package.
+  name completion function for use with "readline" package.
  *
  * ----------------------------------------------------------------------------
  */
@@ -850,28 +850,37 @@ cellname_completion_function(char *text, int state)
 char *
 macro_completion_function(char *text, int state)
 {
-    static macrodef *macro;
+    extern HashTable MacroClients;
+    static HashSearch hs, mc_hs;
     static int len;
+    macrodef *macro;
+    HashEntry	*he;
 
     if (state == 0)
     {
-	macro = mactable;
 	len = strlen(text);
+	HashStartSearch(&hs);
     }
 
-    while (macro != NULL)
+    while ((he = HashNext(&MacroClients, &hs)) != (HashEntry *)NULL )
     {
-	if ((macro->interactive == TRUE && interactive_flag == 1) ||
-        	(macro->interactive == FALSE && interactive_flag == 0))
+	HashTable *ht = (HashTable *)HashGetValue(he);
+	HashEntry *hm;
+
+	if (state == 0)
+	    HashStartSearch(&mc_hs);
+
+	while ((hm = HashNext(ht, &mc_hs)) != (HashEntry *)NULL )
 	{
-	    char *macro_name = MacroName(macro->keyvalue);
-	    if (strncmp(macro_name, text, len) == 0)
+	    macro = (macrodef *)HashGetValue(hm);
+	    if ((macro->interactive == TRUE && interactive_flag == 1) ||
+        		(macro->interactive == FALSE && interactive_flag == 0))
 	    {
-		macro = macro->nextmacro;
-		return strdup(macro_name);
+		char *macro_name = MacroName(macro->macrotext);
+		if (strncmp(macro_name, text, len) == 0)
+		    return strdup(macro_name);
 	    }
 	}
-	macro = macro->nextmacro;
     }
     return (char *)NULL;
 }
