@@ -17,15 +17,35 @@
  */
 
 #ifndef	lint
-static char rcsid[] = "$Header: /ufs/repository/magic/tiles/tile.c,v 1.2 2001/01/12 22:13:27 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/tiles/tile.c,v 1.1.1.1 2005/05/09 23:29:23 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "utils/malloc.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 
+/*
+ * Debugging version of TiSetBody() macro in tile.h
+ * Includes sanity check that a tile at "infinity"
+ * is not being set to a type other than space.
+ */
+/*
+void
+TiSetBody(tp, b)
+   Tile *tp;
+   ClientData b;
+{
+   if (b != (ClientData)0 && b != (ClientData)(-1))
+	if (RIGHT(tp) == INFINITY || TOP(tp) == INFINITY ||
+		LEFT(tp) == MINFINITY || BOTTOM(tp) == MINFINITY)
+	    TxError("Error:  Tile at infinity set to non-space value %d\n", (int)b);
+   tp->ti_body = b;
+}
+*/
+
 /*
  * Rectangle that defines the maximum extent of any plane.
  * No tile created by the user should ever extend outside of
@@ -213,8 +233,8 @@ TiSplitX(tile, x)
     ASSERT(x > LEFT(tile) && x < RIGHT(tile), "TiSplitX");
 
     newtile = (Tile *) mallocMagic((unsigned) (sizeof (Tile)));
-    newtile->ti_client = (ClientData) MINFINITY;
-    newtile->ti_body = (ClientData) 0;
+    TiSetClient(newtile, MINFINITY);
+    TiSetBody(newtile, 0);
 
     LEFT(newtile) = x;
     BOTTOM(newtile) = BOTTOM(tile);
@@ -285,8 +305,8 @@ TiSplitY(tile, y)
     ASSERT(y > BOTTOM(tile) && y < TOP(tile), "TiSplitY");
 
     newtile = (Tile *) mallocMagic((unsigned) (sizeof (Tile)));
-    newtile->ti_client = (ClientData) MINFINITY;
-    newtile->ti_body = (ClientData) 0;
+    TiSetClient(newtile, MINFINITY);
+    TiSetBody(newtile, 0);
 
     LEFT(newtile) = LEFT(tile);
     BOTTOM(newtile) = y;
@@ -358,8 +378,8 @@ TiSplitX_Left(tile, x)
     ASSERT(x > LEFT(tile) && x < RIGHT(tile), "TiSplitX");
 
     newtile = (Tile *) mallocMagic((unsigned) (sizeof (Tile)));
-    newtile->ti_client = (ClientData) MINFINITY;
-    newtile->ti_body = (ClientData) 0;
+    TiSetClient(newtile, MINFINITY);
+    TiSetBody(newtile, 0);
 
     LEFT(newtile) = LEFT(tile);
     LEFT(tile) = x;
@@ -421,8 +441,8 @@ TiSplitY_Bottom(tile, y)
     ASSERT(y > BOTTOM(tile) && y < TOP(tile), "TiSplitY");
 
     newtile = (Tile *) mallocMagic((unsigned) (sizeof (Tile)));
-    newtile->ti_client = (ClientData) MINFINITY;
-    newtile->ti_body = (ClientData) 0;
+    TiSetClient(newtile, MINFINITY);
+    TiSetBody(newtile, 0);
 
     LEFT(newtile) = LEFT(tile);
     BOTTOM(newtile) = BOTTOM(tile);
@@ -628,8 +648,8 @@ TiAlloc()
     Tile *newtile;
 
     newtile = (Tile *) mallocMagic((unsigned) (sizeof (Tile)));
-    newtile->ti_client = (ClientData) MINFINITY;
-    newtile->ti_body = (ClientData) 0;
+    TiSetClient(newtile, MINFINITY);
+    TiSetBody(newtile, 0);
     return (newtile);
 }
 
@@ -646,6 +666,7 @@ TiFree(tp)
 /*									*/
 /* ==================================================================== */
 
+void
 tiPrint(tp)
     Tile *tp;
 {
@@ -654,6 +675,7 @@ tiPrint(tp)
     printf("BL=0x%x LB=0x%x RT=0x%x TR=0x%x\n", BL(tp), LB(tp), RT(tp), TR(tp));
 }
 
+void
 tiPrintAll(tp)
     Tile *tp;
 {
