@@ -5,7 +5,7 @@
  *  Pasadena, CA 91125.
  *  All Rights Reserved
  *
- *  $Id: child.c,v 1.3 2001/07/23 17:49:53 tim Exp $
+ *  $Id: child.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $
  *
  *************************************************************************/
 #include <stdio.h>
@@ -169,7 +169,7 @@ Wait (status)
   int pid;
   int p_status = 0;
 
-  pid = get_next (p_status);
+  pid = get_next(&p_status);
   if (pid != -1) {
     if (status)
       *status = p_status;
