@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/utils/flock.c,v 1.10 2001/08/24 16:51:25 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/utils/flock.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -26,7 +26,9 @@ static char rcsid[] = "$Header: /ufs/repository/magic/utils/flock.c,v 1.10 2001/
 #ifdef __STDC__
 #include <unistd.h>
 #endif
+#include <string.h>
 
+#include <sys/param.h>
 #include <sys/types.h>
 #include <sys/file.h>
 #include <sys/stat.h>
@@ -35,7 +37,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/utils/flock.c,v 1.10 2001/
 
 #ifdef FILE_LOCKS
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/hash.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -97,6 +99,37 @@ global int flock_flag;  /* This is not good, but it is the only way
  *-------------------------------------------------------------------------
  */
 
+/*-------------------------------------------------------------------------
+ * path_check --
+ *
+ *	Check a filename to see if it has path components.
+ *-------------------------------------------------------------------------
+ */
+
+int path_check(char *lfilename)
+{
+    char  *pwd_buf;
+    char  *temp;
+
+    if (strpbrk(lfilename, "/\\") == 0) return(0);
+    pwd_buf = getenv("PWD");
+    if (pwd_buf != NULL)
+    {
+	if (strncmp(pwd_buf, lfilename, strlen(pwd_buf)) == 0)
+	{
+	    temp = (char *)mallocMagic(strlen(lfilename+1));
+	    strcpy(temp, lfilename+strlen(pwd_buf)+1);
+	    strcpy(lfilename, temp);
+	    freeMagic(temp);
+	    return(0);
+	}
+	else
+	    return(1);
+    }
+    else
+	return(0);
+}
+
 /*-------------------------------------------------------------------------
  * flock_init --
  *
@@ -420,7 +453,7 @@ flock_test_set(lfilename, f)
 
     /* Test for Lock set by another Magic and set lock or reopen file read-only */
 
-    if (strpbrk(lfilename, "/\\") == 0)
+    if (path_check(lfilename) == 0)
     {
 	locfile = (char *) mallocMagic((unsigned) (strlen(LOCK_DIR) + strlen(lfilename) +
 			strlen("lock") + 4));
@@ -545,10 +578,10 @@ flock_test_set(lfilename, f)
     }
     else
     {
-	flock_flag = 2;
-	TxPrintf("File <%s> not local to this directory. Read-only mode set.\n",
+      flock_flag = 2;
+      TxPrintf("File <%s> not local to this directory. Read-only mode set.\n",
 		lfilename);
-	f = fopen(lfilename, "r");
+      f = fopen(lfilename, "r");
     }
     return(f);
 }
@@ -567,7 +600,7 @@ flock_open(filename, ext, mode)
     FILE *f;
     int ptr, n=0, dup=0;
     char *locfile, *recname;
-    char lfilename[1024], tempfilename[1024];
+    char lfilename[MAXPATHLEN], tempfilename[MAXPATHLEN];
     int readlen;
     struct stat filestat;
     flock_list *fptr = f_list_ptr, *fnul =  NULL;
@@ -611,8 +644,11 @@ flock_open(filename, ext, mode)
           else { TxPrintf("Readlink failed!\n");
        }
     }
-    if(strpbrk(filename, "/\\") == 0) {
-       recname = (char *) mallocMagic((unsigned) (strlen(LOCK_DIR)+strlen(filename)+strlen("lock")+4));
+    
+    if (path_check(filename) == 0)
+    {
+       recname = (char *) mallocMagic((unsigned) (strlen(LOCK_DIR)
+		+ strlen(filename) + strlen("lock") + 4));
        sprintf(recname, "%s/%s.%s", LOCK_DIR, filename, "lock" );
 
        /* Test for "rb" is the other half of the kludge found in DBio.c */
