@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /ufs/repository/magic/utils/path.c,v 1.5 2001/01/12 22:13:32 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/utils/path.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -28,7 +28,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/utils/path.c,v 1.5 2001/01
 #include <sys/param.h>
 
 #include "tcltk/tclmagic.h"
-#include "misc/magic.h"
+#include "utils/magic.h"
 #include "utils/hash.h"
 #include "utils/malloc.h"
 #include "utils/utils.h"
