@@ -16,12 +16,12 @@
  * Magic utility routines.
  */
 
-/* rcsid "$Header: /ufs/repository/magic/utils/utils.h,v 1.5 2001/08/23 13:48:58 tim Exp $" */
+/* rcsid "$Header: /usr/cvsroot/magic/utils/utils.h,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $" */
 
 #ifndef _UTILS_H
 #define _UTILS_H 1
 
-#include "misc/magic.h"
+#include "utils/magic.h"
 
 /*
  * Cast second argument to LookupStruct() to (LookupTable *) to
@@ -73,7 +73,7 @@ extern int flock_flag;
  */
 #define	LAST_BIT_OF(x)	((x) & ~((x) - 1))
 
-extern double MagAtof();
+extern float MagAtof();
 
 extern int Wait();
 extern int WaitPid();
