@@ -17,11 +17,12 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/windows/windClient.c,v 1.3 2001/01/12 22:13:34 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic/windows/windClient.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include "misc/magic.h"
+
+#include "utils/magic.h"
 #include "textio/textio.h"
 #include "utils/geometry.h"
 #include "windows/windows.h"
@@ -30,14 +31,14 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windClient.c,v 1.3 2
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "main/main.h"
-#include "macros/macros.h"
+#include "utils/main.h"
+#include "utils/macros.h"
 #include "utils/utils.h"
 #include "utils/malloc.h"
 #include "graphics/graphics.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "textio/txcommands.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 
 /* The following defines are used to indicate corner positions
  * of the box:
@@ -59,12 +60,13 @@ extern int windQuitCmd(), windRedrawCmd();
 extern int windResetCmd(), windSpecialOpenCmd();
 extern int windOverCmd(), windUnderCmd(), windDebugCmd();
 extern int windDumpCmd(), windHelpCmd();
+extern int windMacroCmd(), windIntMacroCmd();
 extern int windLogCommandsCmd(), windUpdateCmd(), windSleepCmd();
 extern int windSourceCmd();
 extern int windSetpointCmd();
 extern int windPushbuttonCmd(), windSendCmd();
 extern int windPauseCmd(), windGrstatsCmd();
-extern int windEchoCmd(), windGrowCmd(), windMacroCmd(), windIntMacroCmd();
+extern int windEchoCmd(), windGrowCmd();
 extern int windUndoCmd(), windRedoCmd();
 extern int windCenterCmd(), windScrollCmd();
 extern int windVersionCmd(), windViewCmd(), windXviewCmd(), windZoomCmd();
@@ -72,6 +74,7 @@ extern int windScrollBarsCmd(), windPositionsCmd();
 
 #ifdef MAGIC_WRAPPER
 extern int windNamesCmd();
+extern int windBypassCmd();
 #endif
 
 static Rect windFrameRect;
@@ -366,8 +369,118 @@ windFrameUp(w, cmd)
     }
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * windFrameButtons --
+ *
+ *	Handle button pushes to the window frame area (zoom and scroll)
+ *	Always handle scroll bars ourselves, even if there is an external
+ *	window package.  BUT---in the Tcl/Tk version of Magic, the
+ *	window does not set WIND_SCROLLBARS, causing this routine to be
+ *	bypassed.
+ *
+ * Results:
+ *	TRUE if the button was pushed in the frame and handled by one
+ *	of the button handler routines, FALSE if not.
+ *
+ * Side effects:
+ *	Depends upon where the button was pushed.
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+windFrameButtons(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    extern void windBarLocations();
+
+    Rect leftBar, botBar, up, down, right, left, zoom;
+    Point p;
+
+    if (w == NULL) return FALSE;
+    p.p_x = w->w_screenArea.r_xtop - w->w_screenArea.r_xbot;
+    p.p_y = w->w_screenArea.r_ytop - w->w_screenArea.r_ybot;
+
+    if ((w->w_flags & WIND_SCROLLBARS) != 0)
+    {
+	windBarLocations(w, &leftBar, &botBar, &up, &down, &right, &left, &zoom);
+	if (cmd->tx_button == TX_MIDDLE_BUTTON) {
+	    if (GEO_ENCLOSE(&cmd->tx_p, &leftBar))
+	    {
+		/* move elevator */
+		p.p_x = 0;
+		p.p_y = w->w_bbox->r_ybot + 
+			((w->w_bbox->r_ytop - w->w_bbox->r_ybot) * 
+			(cmd->tx_p.p_y - leftBar.r_ybot))
+			/ (leftBar.r_ytop - leftBar.r_ybot) -
+			(w->w_surfaceArea.r_ytop + w->w_surfaceArea.r_ybot)/2;
+		WindScroll(w, &p, (Point *) NULL);
+		return TRUE;
+	    }
+	    else if (GEO_ENCLOSE(&cmd->tx_p, &botBar))
+	    {
+		/* move elevator */
+		p.p_y = 0;
+		p.p_x = w->w_bbox->r_xbot + 
+			((w->w_bbox->r_xtop - w->w_bbox->r_xbot) *
+			(cmd->tx_p.p_x - botBar.r_xbot))
+			/ (botBar.r_xtop - botBar.r_xbot) -
+			(w->w_surfaceArea.r_xtop + w->w_surfaceArea.r_xbot)/2;
+		WindScroll(w, &p, (Point *) NULL);
+		return TRUE;
+	    }
+	    else if (GEO_ENCLOSE(&cmd->tx_p, &up))
+	    {
+		/* scroll up */
+		p.p_y = -p.p_y;
+		p.p_x = 0;
+		WindScroll(w, (Point *) NULL, &p);
+		return TRUE;
+	    }
+	    else if (GEO_ENCLOSE(&cmd->tx_p, &down))
+	    {
+		/* scroll down */
+		p.p_x = 0;
+		WindScroll(w, (Point *) NULL, &p);
+		return TRUE;
+	    }
+	    else if (GEO_ENCLOSE(&cmd->tx_p, &right))
+	    {
+		/* scroll right */
+		p.p_x = -p.p_x;
+		p.p_y = 0;
+		WindScroll(w, (Point *) NULL, &p);
+		return TRUE;
+	    }
+	    else if (GEO_ENCLOSE(&cmd->tx_p, &left))
+	    {
+		/* scroll left */
+		p.p_y = 0;
+		WindScroll(w, (Point *) NULL, &p);
+		return TRUE;
+	    }
+	}
+	if (GEO_ENCLOSE(&cmd->tx_p, &zoom)) {
+	    /* zoom in, out, or view */
+	    switch (cmd->tx_button)
+	    {
+		case TX_LEFT_BUTTON:
+		    WindZoom(w, 2.0);
+		    break;
+		case TX_MIDDLE_BUTTON:
+		    WindView(w);
+		    break;
+		case TX_RIGHT_BUTTON:
+		    WindZoom(w, 0.5);
+		    break;
+	    }
+	    return TRUE;
+	}
+    }
+    return FALSE;
+}
 
-
 /*
  * ----------------------------------------------------------------------------
  * windClientButtons --
@@ -387,8 +500,6 @@ windClientButtons(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    extern void windBarLocations();
-
     /*
      * Is this an initial 'down push' in a non-iconic window?  If so, we
      * will initiate some user-interaction sequence, such as moving the corner
@@ -399,10 +510,12 @@ windClientButtons(w, cmd)
      * in the future it might happen in other cases too.)
      *
      */
-    if ((WindOldButtons == 0) && ((w->w_flags & WIND_ISICONIC) == 0)) {   
+
+    if ((WindOldButtons == 0) && ((w->w_flags & WIND_ISICONIC) == 0))
+    {
 	/* single button down */
 	Point p;
-	Rect caption, leftBar, botBar, up, down, right, left, zoom;
+	Rect caption;
 
 	windFrameWindow = NULL;
 	if (w == NULL) return;
@@ -415,92 +528,19 @@ windClientButtons(w, cmd)
 	    caption.r_ybot = caption.r_ytop;
 
 	/* Handle 'grow' for our window package. */
-	if (WindPackageType == WIND_MAGIC_WINDOWS) {
+	if (WindPackageType == WIND_MAGIC_WINDOWS)
+	{
 	    if ((cmd->tx_button == TX_MIDDLE_BUTTON) && 
-		GEO_ENCLOSE(&cmd->tx_p, &caption)) {
-		    WindFullScreen(w);
-		    return;
-	    };
-	}
-
-	/* Always handle scroll bars ourselves, even if there is an external
-	 * window package.
-	 */
-	if ((w->w_flags & WIND_SCROLLBARS) != 0) {
-	    windBarLocations(w, &leftBar, &botBar, &up, &down, 
-		&right, &left, &zoom);
-	    if (cmd->tx_button == TX_MIDDLE_BUTTON) {
-		if (GEO_ENCLOSE(&cmd->tx_p, &leftBar)) {
-		    /* move elevator */
-		    p.p_x = 0;
-		    p.p_y = 
-			w->w_bbox->r_ybot + 
-			((w->w_bbox->r_ytop - w->w_bbox->r_ybot) * 
-			(cmd->tx_p.p_y - leftBar.r_ybot))
-			/ (leftBar.r_ytop - leftBar.r_ybot) -
-			(w->w_surfaceArea.r_ytop + w->w_surfaceArea.r_ybot)/2;
-		    WindScroll(w, &p, (Point *) NULL);
-		    return;
-		}
-		else if (GEO_ENCLOSE(&cmd->tx_p, &botBar)) {
-		    /* move elevator */
-		    p.p_y = 0;
-		    p.p_x = 
-			w->w_bbox->r_xbot + 
-			((w->w_bbox->r_xtop - w->w_bbox->r_xbot) *
-			(cmd->tx_p.p_x - botBar.r_xbot))
-			/ (botBar.r_xtop - botBar.r_xbot) -
-			(w->w_surfaceArea.r_xtop + w->w_surfaceArea.r_xbot)/2;
-		    WindScroll(w, &p, (Point *) NULL);
-		    return;
-		}
-		else if (GEO_ENCLOSE(&cmd->tx_p, &up)) {
-		    /* scroll up */
-		    p.p_y = -p.p_y;
-		    p.p_x = 0;
-		    WindScroll(w, (Point *) NULL, &p);
-		    return;
-		}
-		else if (GEO_ENCLOSE(&cmd->tx_p, &down)) {
-		    /* scroll down */
-		    p.p_x = 0;
-		    WindScroll(w, (Point *) NULL, &p);
-		    return;
-		}
-		else if (GEO_ENCLOSE(&cmd->tx_p, &right)) {
-		    /* scroll right */
-		    p.p_x = -p.p_x;
-		    p.p_y = 0;
-		    WindScroll(w, (Point *) NULL, &p);
-		    return;
-		}
-		else if (GEO_ENCLOSE(&cmd->tx_p, &left)) {
-		    /* scroll left */
-		    p.p_y = 0;
-		    WindScroll(w, (Point *) NULL, &p);
-		    return;
-		}
-	    }
-	    if (GEO_ENCLOSE(&cmd->tx_p, &zoom)) {
-		/* zoom in, out, or view */
-		switch (cmd->tx_button)
-		{
-		    case TX_LEFT_BUTTON:
-			WindZoom(w, 2.0);
-			break;
-		    case TX_MIDDLE_BUTTON:
-			WindView(w);
-			break;
-		    case TX_RIGHT_BUTTON:
-			WindZoom(w, 0.5);
-			break;
-		};
+				GEO_ENCLOSE(&cmd->tx_p, &caption))
+	    {
+		WindFullScreen(w);
 		return;
 	    }
-	};
-
-	/* else fall through */
-    };
+	}
+	if (windFrameButtons(w, cmd)) return;
+	    
+	/* Otherwise, continue onward */
+    }
 
     /*
      * At this point, we have decided that the button was not an initial
@@ -531,12 +571,50 @@ windClientButtons(w, cmd)
 		    windFrameUp(w, cmd);
 		    break;
 		default:
-		    TxError("windClientButtons failed!!.\n");
+		    TxError("windClientButtons() failed!\n");
 		    break;
 	    }
     }
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * WindButtonInFrame --
+ *
+ *	To be called from the graphics packages. Handle button pushes to
+ *	the window frame area (zoom and scroll) by calling windFrameButtons()
+ *	if it appears to be appropriate for this MagWindow structure.
+ *	This bypasses the key macro handler, thus hard-coding the button
+ *	actions for the frame.  
+ *
+ * Results:
+ *	TRUE if the button was pushed in the frame and handled by one
+ *	of the button handler routines, FALSE if not.
+ *
+ * Side effects:
+ *	Depends upon where the button was pushed.
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+WindButtonInFrame(w, x, y, b)
+    MagWindow *w;
+    int x;
+    int y;
+    int b;
+{
+    TxCommand cmd;
+    cmd.tx_p.p_x = x;
+    cmd.tx_p.p_y = y;
+    cmd.tx_button = b;
+    if (windFrameButtons(w, &cmd))
+    {
+	WindUpdate();
+	return TRUE;
+    }
+    return FALSE;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -606,6 +684,11 @@ windClientInit()
 
     /* Set up all of the window commands */
 
+#ifdef MAGIC_WRAPPER
+    WindAddCommand(windClientID,
+	"*bypass command	run command independently of the command line",
+	windBypassCmd, FALSE);
+#endif
     WindAddCommand(windClientID,
 	"*crash			cause a core dump",
 	windCrashCmd, FALSE);
@@ -643,15 +726,15 @@ windClientInit()
 			containing pattern)",
 	windHelpCmd, FALSE);
     WindAddCommand(windClientID,
-	"imacro [char [string]]	define or print an interactive macro called char",
-	windIntMacroCmd, FALSE);
+        "imacro [char [string]] define or print an interactive macro called char",
+        windIntMacroCmd, FALSE);
     WindAddCommand(windClientID,
 	"logcommands [file [update]]\n\
 			log all commands into a file",
 	windLogCommandsCmd, FALSE);
     WindAddCommand(windClientID,
-	"macro [char [string]]	define or print a macro called char",
-	windMacroCmd, FALSE);
+        "macro [char [string]]  define or print a macro called char",
+        windMacroCmd, FALSE);
     WindAddCommand(windClientID,
 	"openwindow [cell][name]\n\
 			open a new window with indicated name, bound to indicated cell",
