@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/windows/windCmdAM.c,v 1.7 2001/04/25 14:38:06 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/windows/windCmdAM.c,v 1.1.1.1 2005/05/09 23:29:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -29,17 +29,17 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windCmdAM.c,v 1.7 20
 #include <sys/stat.h>
 #include <errno.h>
 
-#include "magic/tclmagic.h"
-#include "misc/magic.h"
+#include "tcltk/tclmagic.h"
+#include "utils/magic.h"
 #include "textio/textio.h"
 #include "utils/geometry.h"
 #include "windows/windows.h"
 #include "utils/malloc.h"
 #include "utils/runstats.h"
-#include "macros/macros.h"
-#include "signals/signals.h"
+#include "utils/macros.h"
+#include "utils/signals.h"
 #include "graphics/graphics.h"
-#include "misc/styles.h"
+#include "utils/styles.h"
 #include "textio/txcommands.h"
 #include "graphics/glyphs.h"
 #include "windows/windInt.h"
@@ -49,6 +49,9 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windCmdAM.c,v 1.7 20
 #include "dbwind/dbwind.h"
 #include "utils/utils.h"
 
+/* Forward declarations */
+void windDoMacro();
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -316,6 +319,23 @@ windCloseCmd(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
+    if ((cmd->tx_argc == 2) && GrWindowNamePtr)
+    {
+	char *mwname;
+
+	for (w = windTopWindow; w != (MagWindow *)NULL; w = w->w_nextWindow)
+	{
+	    mwname = (*GrWindowNamePtr)(w);
+	    if (!strcmp(mwname, cmd->tx_argv[1]))
+		break;
+	}
+	if (w == NULL)
+	{
+	    TxError("Window named %s cannot be found\n", cmd->tx_argv[1]);
+	    return;
+	}
+    }
+
     if (w == (MagWindow *) NULL)
     {
 	TxError("Point to a window first\n");
@@ -331,7 +351,43 @@ windCloseCmd(w, cmd)
 #endif
 }
 
-
+#ifdef MAGIC_WRAPPER
+/*
+ * ----------------------------------------------------------------------------
+ * windBypassCmd --
+ *
+ *	Run a magic command independently of the command line.  That is,
+ *	if a command is being typed on the command line, the input
+ *	redirection will not be reset by the execution of this command.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+windBypassCmd(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    if (cmd->tx_argc == 1)
+    {
+	TxError("Usage:  *bypass <command>\n");
+	return;
+    }
+
+    /* Dispatch the referenced command */
+    TxTclDispatch((ClientData)w, cmd->tx_argc - 1, cmd->tx_argv + 1);
+    if (TxInputRedirect == TX_INPUT_PENDING_RESET)
+	TxInputRedirect = TX_INPUT_REDIRECTED;
+}
+
+#endif	/* MAGIC_WRAPPER */
+
 /*
  * ----------------------------------------------------------------------------
  * windCrashCmd --
@@ -369,6 +425,8 @@ windCrashCmd(w, cmd)
  * windCursorCmd --
  *
  *	Report the cursor position in Magic (internal) coordinates
+ *	If an argument of a number is given, then the cursor icon
+ *	is changed to the glyph of that number.
  *
  * Results:
  *	None.
@@ -390,6 +448,18 @@ windCursorCmd(w, cmd)
     Tcl_Obj *listxy;
 #endif
 
+    if (cmd->tx_argc == 2)
+    {
+	if (!StrIsInt(cmd->tx_argv[1]))
+	{
+	    TxError("Usage: cursor [glyphnum]\n");
+	    return;
+	}
+	if (GrSetCursorPtr != NULL)
+	    (*GrSetCursorPtr)(atoi(cmd->tx_argv[1]));
+	return;
+    }
+
     if (GrGetCursorPosPtr == NULL)
 	return;
 
@@ -518,6 +588,7 @@ windEchoCmd(w, cmd)
  */
  /*ARGSUSED*/
 
+void
 windFilesCmd(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
@@ -721,131 +792,6 @@ windHelpCmd(w, cmd)
     ASSERT(FALSE, windHelpCmd);
 }
 
-
-/*
- * ----------------------------------------------------------------------------
- *
- * windDoMacro --
- *
- *	Working function for windIntMacroCmd and windMacroCmd
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	Causes the macro package to define a new macro.
- *
- * ----------------------------------------------------------------------------
- */
-
-void
-windDoMacro(cmd, interactive)
-    TxCommand *cmd;
-    bool interactive;
-{
-    macrodef *cmacro;
-    char *cp, *cn;
-    char ch;
-    int ct, iReturn;
-    bool any;
-
-    if (cmd->tx_argc == 1)
-    {
-	any = FALSE;
-	ch = 0;
-	for (cmacro = mactable; cmacro != NULL; cmacro = cmacro->nextmacro)
-	{
-	    cp = cmacro->macrodef;
-	    cn = MacroName(cmacro->keyvalue);
-
-	    if (cmacro->interactive)
-	    {
-		TxPrintf("Interactive macro '%s' contains \"%s\"\n",
-			     cn, cp);
-	    }
-	    else
-	    {
-		TxPrintf("Macro '%s' contains \"%s\"\n",
-			     cn, cp);
-	    }
-	    freeMagic(cn);
-	    any = TRUE;
-	}
-	if (!any)
-	    TxPrintf("No macros are defined.\n");
-	return;
-    }
-    else if (cmd->tx_argc == 2)
-    {
-	int verbose;
-	ct = MacroKey(cmd->tx_argv[1], &verbose);
-	if (ct == 0)
-	{
-        if( verbose ) TxError("Unrecognized macro name %s\n", cmd->tx_argv[1]);
-	    return;
-	}
-	cp = MacroRetrieve(ct, &iReturn);
-	if (cp != NULL)
-	{
-	    cn = MacroName(ct);
-	    if (iReturn)
-	    {
-		TxPrintf("Interactive macro '%s' contains \"%s\"\n",
-			 cn, cp);
-	    }
-	    else
-	    {
-		TxPrintf("Macro '%s' contains \"%s\"\n",
-			 cn, cp);
-	    }
-	    freeMagic(cp);
-	    freeMagic(cn);
-	}
-	return;
-    }
-    else if (cmd->tx_argc == 3)
-    {
-	int verbose;
-	ct = MacroKey(cmd->tx_argv[1], &verbose);
-	if (ct == 0)
-	{
-        if( verbose ) TxError("Unrecognized macro name %s\n", cmd->tx_argv[1]);
-	    return;
-	}
-	if (cmd->tx_argv[2][0] == '\0') MacroDelete(ct);
-	else if (interactive) MacroDefine(ct, cmd->tx_argv[2], TRUE);
-	else MacroDefine(ct, cmd->tx_argv[2], FALSE);
-	return;
-    }
-
-    TxError("Usage: %s [macro_name [string]]\n", cmd->tx_argv[0]);
-}
-
-/*
- * ----------------------------------------------------------------------------
- *
- * windIntMacroCmd --
- *
- *	Define a new interactive macro.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	Calls windDoMacro.
- *
- * ----------------------------------------------------------------------------
- */
-
-void
-windIntMacroCmd(w, cmd)
-    MagWindow *w;
-    TxCommand *cmd;
-{
-    windDoMacro(cmd, TRUE);
-}
-
-
 static char *logKeywords[] =
     {
 	"update",
@@ -897,7 +843,30 @@ usage:
     TxError("Usage: %s [filename [update]]\n", cmd->tx_argv[0]);
 }
 
-
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * windIntMacroCmd --
+ *
+ *	Define a new interactive macro.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Calls windDoMacro.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+windIntMacroCmd(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    windDoMacro(w, cmd, TRUE);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -919,7 +888,164 @@ windMacroCmd(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    windDoMacro(cmd, FALSE);
+    windDoMacro(w, cmd, FALSE);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * windDoMacro --
+ *
+ *	Working function for CmdIntMacro and CmdMacro
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Causes the macro package to define a new macro.
+ *
+ * ----------------------------------------------------------------------------
+ */
 
+void
+windDoMacro(w, cmd, interactive)
+    MagWindow *w;
+    TxCommand *cmd;
+    bool interactive;
+{
+    char *cp, *cn;
+    char ch;
+    int ct, argstart;
+    bool any, iReturn;
+    macrodef *cMacro;
+    HashTable *clienttable;
+    HashEntry *h;
+    HashSearch hs;
+    WindClient wc;
+
+    /* If the second argument is a window name, we attempt to	*/
+    /* retrieve a client ID from it.  This overrides the actual	*/
+    /* window the command was called from, so technically we	*/
+    /* can define macros for clients from inside other clients.	*/
+    /* The main use, though, is to define macros for a client	*/
+    /* from a script, rc file, or command-line.			*/
+    /* Default to the layout window if the command has no	*/
+    /* associated window.					*/
+
+    argstart = 1;
+    if (cmd->tx_argc == 1) wc = DBWclientID;  /* Added by NP 11/15/04 */
+
+    if (cmd->tx_argc > 1)
+    {
+	wc = WindGetClient(cmd->tx_argv[1], TRUE);
+	if (wc == (WindClient)NULL)
+	    if (cmd->tx_argc == 4)
+	    {
+		/* This may look odd, but it has a reason.  If	*/
+		/* we don't register a client, such as wind3d 	*/
+		/* when running a non-OpenGL magic, then we	*/
+		/* shuffle off macros for that window type to	*/
+		/* a "fake" NULL client, where it won't be seen	*/
+		/* again, but won't cause an error.		*/
+		wc = 0;
+		argstart = 2;
+	    }
+	    else if (w != NULL)
+		wc = w->w_client;
+	    else
+		wc = DBWclientID;
+	else
+	    argstart = 2;
+    }
+
+    if (cmd->tx_argc == argstart)
+    {
+	h = HashLookOnly(&MacroClients, wc);
+	if (h == NULL)
+	    return;
+	else
+	{
+	    clienttable = (HashTable *)HashGetValue(h);
+	    if (clienttable == (HashTable *)NULL)
+	    {
+		TxError("No such client.\n");
+		return;
+	    }
+	}
+
+	any = FALSE;
+	ch = 0;
+	HashStartSearch(&hs);
+	while (TRUE)
+	{
+	    h = HashNext(clienttable, &hs);
+	    if (h == NULL) break;
+	    cMacro = (macrodef *) HashGetValue(h);
+	    cp = cMacro->macrotext;
+	    cn = MacroName((int)h->h_key.h_ptr);
+
+	    if (cMacro->interactive)
+	    {
+		TxPrintf("Interactive macro '%s' contains \"%s\"\n",
+			     cn, cp);
+	    }
+	    else
+	    {
+		TxPrintf("Macro '%s' contains \"%s\"\n",
+			     cn, cp);
+	    }
+	    freeMagic(cn);
+	    any = TRUE;
+	}
+	if (!any)
+	    TxPrintf("No macros are defined for this client.\n");
+	return;
+    }
+    else if (cmd->tx_argc == (argstart + 1))
+    {
+	int verbose;
+	ct = MacroKey(cmd->tx_argv[argstart], &verbose);
+	if (ct == 0)
+	{
+	    if (verbose)
+		TxError("Unrecognized macro name %s\n", cmd->tx_argv[argstart]);
+	    return;
+	}
+	cp = MacroRetrieve(wc, ct, &iReturn);
+	if (cp != NULL)
+	{
+	    cn = MacroName(ct);
+	    if (iReturn)
+	    {
+		TxPrintf("Interactive macro '%s' contains \"%s\"\n",
+			 cn, cp);
+	    }
+	    else
+	    {
+		TxPrintf("Macro '%s' contains \"%s\"\n",
+			 cn, cp);
+	    }
+	    freeMagic(cp);
+	    freeMagic(cn);
+	}
+	return;
+    }
+    else if (cmd->tx_argc == (argstart + 2))
+    {
+	int verbose;
+	ct = MacroKey(cmd->tx_argv[argstart], &verbose);
+	if (ct == 0)
+	{
+	    if (verbose)
+		TxError("Unrecognized macro name %s\n", cmd->tx_argv[argstart]);
+	    return;
+	}
+	argstart++;
+	if (cmd->tx_argv[argstart][0] == '\0') MacroDelete(wc, ct);
+	else if (interactive) MacroDefine(wc, ct, cmd->tx_argv[argstart], TRUE);
+	else MacroDefine(wc, ct, cmd->tx_argv[argstart], FALSE);
+	return;
+    }
+
+    TxError("Usage: %s [macro_name [string]]\n", cmd->tx_argv[0]);
+}
