@@ -17,31 +17,31 @@
  */
 
 #ifndef lint
-static char rcsid[]="$Header: /ufs/repository/magic/windows/windCmdSZ.c,v 1.3 2001/01/12 22:13:34 jsolomon Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic/windows/windCmdSZ.c,v 1.2 2005/05/11 17:39:32 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>		/* for sleep() */
 #include <string.h>
 #include <math.h>
 
-#include "magic/tclmagic.h"
-#include "misc/magic.h"
+#include "tcltk/tclmagic.h"
+#include "utils/magic.h"
 #include "textio/textio.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
 #include "windows/windows.h"
 #include "graphics/glyphs.h"
 #include "windows/windInt.h"
-#include "undo/undo.h"
+#include "utils/undo.h"
 #include "utils/utils.h"
-#include "signals/signals.h"
+#include "utils/signals.h"
 #include "textio/txcommands.h"
 #include "utils/hash.h"
 #include "database/database.h"
 #include "dbwind/dbwind.h"
 #include "graphics/graphics.h"
-#include "misc/patchlevel.h"
 
 
 /*
@@ -139,7 +139,8 @@ windScrollCmd(w, cmd)
     else
     {
 	/* Alternate syntax: parse for integer coordinate amount to scroll */
-	xsize = ysize = cmdParseCoord(cmd->tx_argv[2]);
+	xsize = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
+	ysize = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
     }
 
     p.p_x = 0;
@@ -222,13 +223,27 @@ windSetpointCmd(w, cmd)
     if ((cmd->tx_argc != 1) && !
 	(StrIsInt(cmd->tx_argv[1]) && StrIsInt(cmd->tx_argv[2])) )
 	goto usage;
-    if ((cmd->tx_argc == 4) && !StrIsInt(cmd->tx_argv[3]))
-	goto usage;
 
-    if (cmd->tx_argc == 4) 
-	wid = atoi(cmd->tx_argv[3]);
-    else
-	wid = WIND_UNKNOWN_WINDOW;
+    if (cmd->tx_argc == 4) {
+	if (StrIsInt(cmd->tx_argv[3]))
+	   wid = atoi(cmd->tx_argv[3]);
+	else if (GrWindowIdPtr)
+	   wid = (*GrWindowIdPtr)(cmd->tx_argv[3]);
+	else
+	   wid = WIND_UNKNOWN_WINDOW;
+    }
+    else if (w != NULL)
+	wid = w->w_wid;
+    else {
+	windCheckOnlyWindow(&w, DBWclientID);
+	if (w != NULL)
+	   wid = w->w_wid;
+	else
+	   wid = WIND_UNKNOWN_WINDOW;
+    }
+
+    /* Ensure a valid window, if possible */
+    if (w == NULL) w = WindSearchWid(wid);
 
     if (cmd->tx_argc == 1)
     {
@@ -253,12 +268,24 @@ windSetpointCmd(w, cmd)
 	}
     }
     else {
-	TxSetPoint(atoi(cmd->tx_argv[1]), atoi(cmd->tx_argv[2]), wid);
+	int yval;
+
+	yval = atoi(cmd->tx_argv[2]);
+
+	/* Reinterpret coordinates according to the graphics package */ 
+	switch (WindPackageType)
+	{
+	    case WIND_X_WINDOWS:
+		/* Windows have origin at lower-left corner */
+		yval = w->w_allArea.r_ytop - yval;
+		break;
+	}
+	TxSetPoint(atoi(cmd->tx_argv[1]), yval, wid);
     }
     return;
 
 usage:
-    TxError("Usage: %s [x y [window ID]]\n", cmd->tx_argv[0]);
+    TxError("Usage: %s [x y [window ID|name]]\n", cmd->tx_argv[0]);
 }
 
 int
@@ -395,7 +422,7 @@ windSpecialOpenCmd(w, cmd)
     else
 	client = cmd->tx_argv[1];
 
-    wc = WindGetClient(client);
+    wc = WindGetClient(client, FALSE);
     /* clients whose names begin with '*' are hidden */
     if ((wc == (WindClient) NULL) || (client[0] == '*')) goto usage;
 
@@ -451,7 +478,7 @@ windNamesCmd(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    bool doforall = False;
+    bool doforall = FALSE;
     WindClient wc = NULL;
     MagWindow *sw;
 
@@ -463,10 +490,10 @@ windNamesCmd(w, cmd)
     if (cmd->tx_argc == 2)
     {
 	if (!strncmp(cmd->tx_argv[1], "all", 3))
-	    doforall = True;
+	    doforall = TRUE;
 	else
 	{
-	    wc = WindGetClient(cmd->tx_argv[1]);
+	    wc = WindGetClient(cmd->tx_argv[1], FALSE);
 	    if (wc == (WindClient) NULL)
 	    {
 		TxError("Usage:  windownames [all | client_type]\n");
@@ -474,7 +501,7 @@ windNamesCmd(w, cmd)
 		WindPrintClientList(FALSE);
 		return;
 	    }
-	    doforall = True;
+	    doforall = TRUE;
 	}
     }
 
@@ -483,10 +510,10 @@ windNamesCmd(w, cmd)
 	wc = DBWclientID;
 	windCheckOnlyWindow(&w, wc);
 	if (w == (MagWindow *)NULL)
-	    doforall = True;
+	    doforall = TRUE;
     }
 
-    if (doforall == True)
+    if (doforall == TRUE)
     {
 	for (sw = windTopWindow; sw != NULL; sw = sw->w_nextWindow)
 	    if ((wc == NULL) || (sw->w_client == wc))
@@ -562,18 +589,35 @@ windUndoCmd(w, cmd)
 {
     int count;
 
-    if (cmd->tx_argc > 2)
+    if (cmd->tx_argc > 3)
     {
 	TxError("Usage: undo [count]\n");
 	return;
     }
-
-    if (cmd->tx_argc == 2)
+    else if (cmd->tx_argc == 3)
+    {
+	if (strncmp(cmd->tx_argv[1], "print", 5))
+	{
+	    TxError("Usage: undo print count\n");
+	    return;
+	}
+	else if (!StrIsInt(cmd->tx_argv[2]))
+	{
+	    TxError("Usage: undo print count\n");
+	    return;
+	}
+	else
+	{
+	    /* Implement undo stack trace */
+	    UndoStackTrace((-1) - atoi(cmd->tx_argv[2]));
+	    return;
+	}
+    }
+    else if (cmd->tx_argc == 2)
     {
 	if (!StrIsInt(cmd->tx_argv[1]))
 	{
-	    TxError("Count must be numeric\n");
-	    return;
+	    TxError("Option must be a count (integer)\n");
 	}
 	count = atoi(cmd->tx_argv[1]);
 	if (count < 0)
@@ -617,13 +661,26 @@ windUpdateCmd(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    if (cmd->tx_argc != 1)
-    {
-	TxError("Usage: %s\n", cmd->tx_argv[0]);
-	return;
-    }
+    if (cmd->tx_argc == 1)
+	WindUpdate();
+#ifdef MAGIC_WRAPPER
+    else if (cmd->tx_argc > 2)
+	goto badusage;
+    else if (!strcmp(cmd->tx_argv[1], "suspend"))
+	GrDisplayStatus = DISPLAY_SUSPEND;
+    else if (!strcmp(cmd->tx_argv[1], "resume"))
+	GrDisplayStatus = DISPLAY_IDLE;
+    else
+	goto badusage;
+#else
+    else if (cmd->tx_argc >= 2)
+	goto badusage;
+#endif
+    return;
 
-    WindUpdate();
+badusage:
+    TxError("Usage: %s [suspend | resume]\n", cmd->tx_argv[0]);
+    return;
 }
 
 /*
@@ -655,8 +712,8 @@ windVersionCmd(w, cmd)
 	return;
     }
 
-    TxPrintf("Version %s.  Compiled on %s.\n", MagicVersion, MagicCompileTime);
-    TxPrintf("Patch level %d.\nPatch history: %s\n", PATCHLEVEL, PatchNames());
+    TxPrintf("Version %s revision %s.  Compiled on %s.\n",
+		MagicVersion, MagicRevision, MagicCompileTime);
 }
 
 /*
@@ -745,10 +802,10 @@ windViewCmd(w, cmd)
     else if (cmd->tx_argc == 5)
     {
 	Rect r;
-	r.r_xbot = cmdParseCoord(cmd->tx_argv[1]);
-        r.r_ybot = cmdParseCoord(cmd->tx_argv[2]);
-        r.r_xtop = cmdParseCoord(cmd->tx_argv[3]);
-        r.r_ytop = cmdParseCoord(cmd->tx_argv[4]);
+	r.r_xbot = cmdParseCoord(w, cmd->tx_argv[1], FALSE, TRUE);
+        r.r_ybot = cmdParseCoord(w, cmd->tx_argv[2], FALSE, FALSE);
+        r.r_xtop = cmdParseCoord(w, cmd->tx_argv[3], FALSE, TRUE);
+        r.r_ytop = cmdParseCoord(w, cmd->tx_argv[4], FALSE, FALSE);
 
 	/* Redisplay */
 	WindMove(w, &r);
@@ -911,7 +968,7 @@ windSendCmd(w, cmd)
     if (cmd->tx_argc < 3) goto usage;
     if (cmd->tx_argv[1][0] == '*') goto usage; /* hidden window client */
 
-    client = WindGetClient(cmd->tx_argv[1]);
+    client = WindGetClient(cmd->tx_argv[1], FALSE);
     if (client == (WindClient) NULL) goto usage;
     toWindow = (MagWindow *) NULL;
     (void) WindSearch(client, (ClientData) NULL, (Rect *) NULL, 
